@@ -53,7 +53,6 @@ struct module_state {
     PyObject* BSONInt64;
     PyObject* Decimal128;
     PyObject* Mapping;
-    PyObject* CodecOptions;
 };
 
 /* The Py_TYPE macro was introduced in CPython 2.6 */
@@ -371,11 +370,10 @@ static int _load_python_objects(PyObject* module) {
         _load_object(&state->Decimal128, "bson.decimal128", "Decimal128") ||
         _load_object(&state->UUID, "uuid", "UUID") ||
 #if PY_MAJOR_VERSION >= 3
-        _load_object(&state->Mapping, "collections.abc", "Mapping") ||
+        _load_object(&state->Mapping, "collections.abc", "Mapping")) {
 #else
-        _load_object(&state->Mapping, "collections", "Mapping") ||
+        _load_object(&state->Mapping, "collections", "Mapping")) {
 #endif
-        _load_object(&state->CodecOptions, "bson.codec_options", "CodecOptions")) {
         return 1;
     }
     /* Reload our REType hack too. */
@@ -537,26 +535,6 @@ int convert_codec_options(PyObject* options_obj, void* p) {
     return 1;
 }
 
-/* Fill out a codec_options_t* with default options.
- *
- * Return 1 on success.
- * Return 0 on failure.
- */
-int default_codec_options(struct module_state* state, codec_options_t* options) {
-    PyObject* options_obj = NULL;
-    PyObject* codec_options_func = _get_object(
-        state->CodecOptions, "bson.codec_options", "CodecOptions");
-    if (codec_options_func == NULL) {
-        return 0;
-    }
-    options_obj = PyObject_CallFunctionObjArgs(codec_options_func, NULL);
-    Py_DECREF(codec_options_func);
-    if (options_obj == NULL) {
-        return 0;
-    }
-    return convert_codec_options(options_obj, options);
-}
-
 void destroy_codec_options(codec_options_t* options) {
     Py_CLEAR(options->document_class);
     Py_CLEAR(options->tzinfo);
@@ -2675,15 +2653,10 @@ static PyObject* _cbson_element_to_dict(PyObject* self, PyObject* args) {
     PyObject* value;
     PyObject* result_tuple;
 
-    if (!PyArg_ParseTuple(args, "OII|O&", &bson, &position, &max,
+    if (!PyArg_ParseTuple(args, "OIIO&", &bson, &position, &max,
                           convert_codec_options, &options)) {
         return NULL;
     }
-    if (PyTuple_GET_SIZE(args) < 4) {
-        if (!default_codec_options(GETSTATE(self), &options)) {
-            return NULL;
-        }
-    }
 
 #if PY_MAJOR_VERSION >= 3
     if (!PyBytes_Check(bson)) {
@@ -2867,17 +2840,13 @@ static PyObject* _cbson_decode_all(PyObject* self, PyObject* args) {
     PyObject* dict;
     PyObject* result = NULL;
     codec_options_t options;
-    PyObject* options_obj;
-    Py_buffer view;
+    PyObject* options_obj = NULL;
+    Py_buffer view = {0};
 
-    if (!PyArg_ParseTuple(args, "O|O", &bson, &options_obj)) {
+    if (!PyArg_ParseTuple(args, "OO", &bson, &options_obj)) {
         return NULL;
     }
-    if (PyTuple_GET_SIZE(args) < 2) {
-        if (!default_codec_options(GETSTATE(self), &options)) {
-            return NULL;
-        }
-    } else if (!convert_codec_options(options_obj, &options)) {
+    if (!convert_codec_options(options_obj, &options)) {
         return NULL;
     }
 
@@ -2971,7 +2940,7 @@ static PyMethodDef _CBSONMethods[] = {
      "convert a dictionary to a string containing its BSON representation."},
     {"_bson_to_dict", _cbson_bson_to_dict, METH_VARARGS,
      "convert a BSON string to a SON object."},
-    {"decode_all", _cbson_decode_all, METH_VARARGS,
+    {"_decode_all", _cbson_decode_all, METH_VARARGS,
      "convert binary data to a sequence of documents."},
     {"_element_to_dict", _cbson_element_to_dict, METH_VARARGS,
      "Decode a single key, value pair."},
