@@ -1,4 +1,4 @@
-# Copyright 2009-2012 10gen, Inc.
+# Copyright 2009-2015 MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,12 +16,15 @@
 
 from copy import deepcopy
 
+from bson.py3compat import iteritems, string_type
 from bson.son import SON
 
 
 class DBRef(object):
-    """A reference to a document stored in MongoDB.
-    """
+    """A reference to a document stored in MongoDB."""
+
+    # DBRef isn't actually a BSON "type" so this number was arbitrarily chosen.
+    _type_marker = 100
 
     def __init__(self, collection, id, database=None, _extra={}, **kwargs):
         """Initialize a new :class:`DBRef`.
@@ -39,19 +42,12 @@ class DBRef(object):
           - `**kwargs` (optional): additional keyword arguments will
             create additional, custom fields
 
-        .. versionchanged:: 1.8
-           Now takes keyword arguments to specify additional fields.
-        .. versionadded:: 1.1.1
-           The `database` parameter.
-
-        .. mongodoc:: dbrefs
+        .. seealso:: The MongoDB documentation on `dbrefs <https://dochub.mongodb.org/core/dbrefs>`_.
         """
-        if not isinstance(collection, basestring):
-            raise TypeError("collection must be an "
-                            "instance of %s" % (basestring.__name__,))
-        if database is not None and not isinstance(database, basestring):
-            raise TypeError("database must be an "
-                            "instance of %s" % (basestring.__name__,))
+        if not isinstance(collection, string_type):
+            raise TypeError("collection must be an " "instance of %s" % string_type.__name__)
+        if database is not None and not isinstance(database, string_type):
+            raise TypeError("database must be an " "instance of %s" % string_type.__name__)
 
         self.__collection = collection
         self.__id = id
@@ -61,14 +57,12 @@ class DBRef(object):
 
     @property
     def collection(self):
-        """Get the name of this DBRef's collection as unicode.
-        """
+        """Get the name of this DBRef's collection as unicode."""
         return self.__collection
 
     @property
     def id(self):
-        """Get this DBRef's _id.
-        """
+        """Get this DBRef's _id."""
         return self.__id
 
     @property
@@ -76,8 +70,6 @@ class DBRef(object):
         """Get the name of this DBRef's database.
 
         Returns None if this DBRef doesn't specify a database.
-
-        .. versionadded:: 1.1.1
         """
         return self.__database
 
@@ -98,27 +90,22 @@ class DBRef(object):
 
         Generally not needed by application developers
         """
-        doc = SON([("$ref", self.collection),
-                   ("$id", self.id)])
+        doc = SON([("$ref", self.collection), ("$id", self.id)])
         if self.database is not None:
             doc["$db"] = self.database
         doc.update(self.__kwargs)
         return doc
 
     def __repr__(self):
-        extra = "".join([", %s=%r" % (k, v)
-                         for k, v in self.__kwargs.iteritems()])
+        extra = "".join([", %s=%r" % (k, v) for k, v in iteritems(self.__kwargs)])
         if self.database is None:
             return "DBRef(%r, %r%s)" % (self.collection, self.id, extra)
-        return "DBRef(%r, %r, %r%s)" % (self.collection, self.id,
-                                        self.database, extra)
+        return "DBRef(%r, %r, %r%s)" % (self.collection, self.id, self.database, extra)
 
     def __eq__(self, other):
         if isinstance(other, DBRef):
-            us = (self.__database, self.__collection,
-                  self.__id, self.__kwargs)
-            them = (other.__database, other.__collection,
-                    other.__id, other.__kwargs)
+            us = (self.__database, self.__collection, self.__id, self.__kwargs)
+            them = (other.__database, other.__collection, other.__id, other.__kwargs)
             return us == them
         return NotImplemented
 
@@ -126,19 +113,16 @@ class DBRef(object):
         return not self == other
 
     def __hash__(self):
-        """Get a hash value for this :class:`DBRef`.
-
-        .. versionadded:: 1.1
-        """
-        return hash((self.__collection, self.__id, self.__database,
-                     tuple(sorted(self.__kwargs.items()))))
+        """Get a hash value for this :class:`DBRef`."""
+        return hash(
+            (self.__collection, self.__id, self.__database, tuple(sorted(self.__kwargs.items())))
+        )
 
     def __deepcopy__(self, memo):
-        """Support function for `copy.deepcopy()`.
-
-        .. versionadded:: 1.10
-        """
-        return DBRef(deepcopy(self.__collection, memo),
-                     deepcopy(self.__id, memo),
-                     deepcopy(self.__database, memo),
-                     deepcopy(self.__kwargs, memo))
+        """Support function for `copy.deepcopy()`."""
+        return DBRef(
+            deepcopy(self.__collection, memo),
+            deepcopy(self.__id, memo),
+            deepcopy(self.__database, memo),
+            deepcopy(self.__kwargs, memo),
+        )
