@@ -23,14 +23,14 @@ import decimal
 import struct
 import sys
 
-from bson.py3compat import (PY3 as _PY3,
-                            string_type as _string_type)
-
+from bson.py3compat import PY3 as _PY3
+from bson.py3compat import string_type as _string_type
 
 if _PY3:
     _from_bytes = int.from_bytes  # pylint: disable=no-member, invalid-name
 else:
     import binascii
+
     def _from_bytes(value, dummy, _int=int, _hexlify=binascii.hexlify):
         "An implementation of int.from_bytes for python 2.x."
         return _int(_hexlify(value), 16)
@@ -46,8 +46,8 @@ _EXPONENT_MIN = -6143
 _MAX_DIGITS = 34
 
 _INF = 0x7800000000000000
-_NAN = 0x7c00000000000000
-_SNAN = 0x7e00000000000000
+_NAN = 0x7C00000000000000
+_SNAN = 0x7E00000000000000
 _SIGN = 0x8000000000000000
 
 _NINF = (_INF + _SIGN, 0)
@@ -58,24 +58,22 @@ _NSNAN = (_SNAN + _SIGN, 0)
 _PSNAN = (_SNAN, 0)
 
 _CTX_OPTIONS = {
-    'prec': _MAX_DIGITS,
-    'rounding': decimal.ROUND_HALF_EVEN,
-    'Emin': _EXPONENT_MIN,
-    'Emax': _EXPONENT_MAX,
-    'capitals': 1,
-    'flags': [],
-    'traps': [decimal.InvalidOperation,
-              decimal.Overflow,
-              decimal.Inexact]
+    "prec": _MAX_DIGITS,
+    "rounding": decimal.ROUND_HALF_EVEN,
+    "Emin": _EXPONENT_MIN,
+    "Emax": _EXPONENT_MAX,
+    "capitals": 1,
+    "flags": [],
+    "traps": [decimal.InvalidOperation, decimal.Overflow, decimal.Inexact],
 }
 
 try:
     # Python >= 3.3, cdecimal
     decimal.Context(clamp=1)  # pylint: disable=unexpected-keyword-arg
-    _CTX_OPTIONS['clamp'] = 1
+    _CTX_OPTIONS["clamp"] = 1
 except TypeError:
     # Python < 3.3
-    _CTX_OPTIONS['_clamp'] = 1
+    _CTX_OPTIONS["_clamp"] = 1
 
 _DEC128_CTX = decimal.Context(**_CTX_OPTIONS.copy())
 
@@ -85,7 +83,7 @@ def create_decimal128_context():
     for working with IEEE-754 128-bit decimal floating point values.
     """
     opts = _CTX_OPTIONS.copy()
-    opts['traps'] = []
+    opts["traps"] = []
     return decimal.Context(**opts)
 
 
@@ -126,9 +124,9 @@ def _decimal_to_128(value):
     biased_exponent = exponent + _EXPONENT_BIAS
 
     if high >> 49 == 1:
-        high = high & 0x7fffffffffff
+        high = high & 0x7FFFFFFFFFFF
         high |= _EXPONENT_MASK
-        high |= (biased_exponent & 0x3fff) << 47
+        high |= (biased_exponent & 0x3FFF) << 47
     else:
         high |= biased_exponent << 49
 
@@ -232,7 +230,8 @@ class Decimal128(object):
         >>> Decimal('NaN') == Decimal('NaN')
         False
     """
-    __slots__ = ('__high', '__low')
+
+    __slots__ = ("__high", "__low")
 
     _type_marker = 19
 
@@ -241,9 +240,11 @@ class Decimal128(object):
             self.__high, self.__low = _decimal_to_128(value)
         elif isinstance(value, (list, tuple)):
             if len(value) != 2:
-                raise ValueError('Invalid size for creation of Decimal128 '
-                                 'from list or tuple. Must have exactly 2 '
-                                 'elements.')
+                raise ValueError(
+                    "Invalid size for creation of Decimal128 "
+                    "from list or tuple. Must have exactly 2 "
+                    "elements."
+                )
             self.__high, self.__low = value
         else:
             raise TypeError("Cannot convert %r to Decimal128" % (value,))
@@ -257,25 +258,25 @@ class Decimal128(object):
         sign = 1 if (high & _SIGN) else 0
 
         if (high & _SNAN) == _SNAN:
-            return decimal.Decimal((sign, (), 'N'))
+            return decimal.Decimal((sign, (), "N"))
         elif (high & _NAN) == _NAN:
-            return decimal.Decimal((sign, (), 'n'))
+            return decimal.Decimal((sign, (), "n"))
         elif (high & _INF) == _INF:
-            return decimal.Decimal((sign, (), 'F'))
+            return decimal.Decimal((sign, (), "F"))
 
         if (high & _EXPONENT_MASK) == _EXPONENT_MASK:
-            exponent = ((high & 0x1fffe00000000000) >> 47) - _EXPONENT_BIAS
+            exponent = ((high & 0x1FFFE00000000000) >> 47) - _EXPONENT_BIAS
             return decimal.Decimal((sign, (0,), exponent))
         else:
-            exponent = ((high & 0x7fff800000000000) >> 49) - _EXPONENT_BIAS
+            exponent = ((high & 0x7FFF800000000000) >> 49) - _EXPONENT_BIAS
 
         arr = bytearray(15)
-        mask = 0x00000000000000ff
+        mask = 0x00000000000000FF
         for i in range(14, 6, -1):
             arr[i] = (low & mask) >> ((14 - i) << 3)
             mask = mask << 8
 
-        mask = 0x00000000000000ff
+        mask = 0x00000000000000FF
         for i in range(6, 0, -1):
             arr[i] = (high & mask) >> ((6 - i) << 3)
             mask = mask << 8
@@ -284,8 +285,7 @@ class Decimal128(object):
         arr[0] = (high & mask) >> 48
 
         # cdecimal only accepts a tuple for digits.
-        digits = tuple(
-            int(digit) for digit in str(_from_bytes(arr, 'big')))
+        digits = tuple(int(digit) for digit in str(_from_bytes(arr, "big")))
 
         with decimal.localcontext(_DEC128_CTX) as ctx:
             return ctx.create_decimal((sign, digits, exponent))
