@@ -19,15 +19,14 @@ import sys
 PY3 = sys.version_info[0] == 3
 
 if PY3:
-    import codecs
     import _thread as thread
+    import codecs
+    import collections.abc as abc
+    from abc import ABC, abstractmethod
     from io import BytesIO as StringIO
 
-    try:
-        import collections.abc as abc
-    except ImportError:
-        # PyPy3 (based on CPython 3.2)
-        import collections as abc
+    def abstractproperty(func):
+        return property(abstractmethod(func))
 
     MAXSIZE = sys.maxsize
 
@@ -36,7 +35,7 @@ if PY3:
     def b(s):
         # BSON and socket operations deal in binary data. In
         # python 3 that means instances of `bytes`. In python
-        # 2.6 and 2.7 you can create an alias for `bytes` using
+        # 2.7 you can create an alias for `bytes` using
         # the b prefix (e.g. b'foo').
         # See http://python3porting.com/problems.html#nicer-solutions
         return codecs.latin_1_encode(s)[0]
@@ -53,6 +52,9 @@ if PY3:
     def reraise(exctype, value, trace=None):
         raise exctype(str(value)).with_traceback(trace)
 
+    def reraise_instance(exc_instance, trace=None):
+        raise exc_instance.with_traceback(trace)
+
     def _unicode(s):
         return s
 
@@ -61,14 +63,18 @@ if PY3:
     integer_types = int
 else:
     import collections as abc
+    from abc import ABCMeta, abstractproperty
+    from itertools import imap
+
     import thread
 
-    from itertools import imap
     try:
         from cStringIO import StringIO
     except ImportError:
         from StringIO import StringIO
 
+    ABC = ABCMeta("ABC", (object,), {})
+
     MAXSIZE = sys.maxint
 
     def b(s):
@@ -76,7 +82,7 @@ else:
         return s
 
     def bytes_from_hex(h):
-        return h.decode('hex')
+        return h.decode("hex")
 
     def iteritems(d):
         return d.iteritems()
@@ -84,10 +90,18 @@ else:
     def itervalues(d):
         return d.itervalues()
 
+    def reraise(exctype, value, trace=None):
+        _reraise(exctype, str(value), trace)
+
+    def reraise_instance(exc_instance, trace=None):
+        _reraise(exc_instance, None, trace)
+
     # "raise x, y, z" raises SyntaxError in Python 3
-    exec("""def reraise(exctype, value, trace=None):
-    raise exctype, str(value), trace
-""")
+    exec(
+        """def _reraise(exc, value, trace):
+    raise exc, value, trace
+"""
+    )
 
     _unicode = unicode
 
