@@ -1,4 +1,4 @@
-# Copyright 2009-2014 MongoDB, Inc.
+# Copyright 2009-present MongoDB, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,6 +21,8 @@ dictionary."""
 import copy
 import re
 
+from bson.py3compat import abc, iteritems
+
 # This sort of sucks, but seems to be as good as it gets...
 # This is essentially the same as re._pattern_type
 RE_TYPE = type(re.compile(""))
@@ -30,56 +32,8 @@ class SON(dict):
     """SON data.
 
     A subclass of dict that maintains ordering of keys and provides a
-    few extra niceties for dealing with SON. SON objects can be
-    converted to and from BSON.
-
-    The mapping from Python types to BSON types is as follows:
-
-    =======================================  =============  ===================
-    Python Type                              BSON Type      Supported Direction
-    =======================================  =============  ===================
-    None                                     null           both
-    bool                                     boolean        both
-    int [#int]_                              int32 / int64  py -> bson
-    long                                     int64          both
-    float                                    number (real)  both
-    string                                   string         py -> bson
-    unicode                                  string         both
-    list                                     array          both
-    dict / `SON`                             object         both
-    datetime.datetime [#dt]_ [#dt2]_         date           both
-    `bson.regex.Regex` / compiled re [#re]_  regex          both
-    `bson.binary.Binary`                     binary         both
-    `bson.objectid.ObjectId`                 oid            both
-    `bson.dbref.DBRef`                       dbref          both
-    None                                     undefined      bson -> py
-    unicode                                  code           bson -> py
-    `bson.code.Code`                         code           py -> bson
-    unicode                                  symbol         bson -> py
-    bytes (Python 3) [#bytes]_               binary         both
-    =======================================  =============  ===================
-
-    Note that to save binary data it must be wrapped as an instance of
-    `bson.binary.Binary`. Otherwise it will be saved as a BSON string
-    and retrieved as unicode.
-
-    .. [#int] A Python int will be saved as a BSON int32 or BSON int64 depending
-       on its size. A BSON int32 will always decode to a Python int. In Python 2.x
-       a BSON int64 will always decode to a Python long. In Python 3.x a BSON
-       int64 will decode to a Python int since there is no longer a long type.
-    .. [#dt] datetime.datetime instances will be rounded to the nearest
-       millisecond when saved
-    .. [#dt2] all datetime.datetime instances are treated as *naive*. clients
-       should always use UTC.
-    .. [#re] :class:`~bson.regex.Regex` instances and regular expression
-       objects from ``re.compile()`` are both saved as BSON regular expressions.
-       BSON regular expressions are decoded as Python regular expressions by
-       default, or as :class:`~bson.regex.Regex` instances if the ``compile_re``
-       option is set to ``False``.
-    .. [#bytes] The bytes type from Python 3.x is encoded as BSON binary with
-       subtype 0. In Python 3.x it will be decoded back to bytes. In Python 2.x
-       it will be decoded to an instance of :class:`~bson.binary.Binary` with
-       subtype 0.
+    few extra niceties for dealing with SON. SON provides an API
+    similar to collections.OrderedDict from Python 2.7+.
     """
 
     def __init__(self, data=None, **kwargs):
@@ -158,8 +112,7 @@ class SON(dict):
 
     def pop(self, key, *args):
         if len(args) > 1:
-            raise TypeError("pop expected at most 2 arguments, got "\
-                                + repr(1 + len(args)))
+            raise TypeError("pop expected at most 2 arguments, got " + repr(1 + len(args)))
         try:
             value = self[key]
         except KeyError:
@@ -171,9 +124,9 @@ class SON(dict):
 
     def popitem(self):
         try:
-            k, v = self.iteritems().next()
+            k, v = next(self.iteritems())
         except StopIteration:
-            raise KeyError('container is empty')
+            raise KeyError("container is empty")
         del self[k]
         return (k, v)
 
@@ -181,10 +134,10 @@ class SON(dict):
         # Make progressively weaker assumptions about "other"
         if other is None:
             pass
-        elif hasattr(other, 'iteritems'):  # iteritems saves memory and lookups
+        elif hasattr(other, "iteritems"):  # iteritems saves memory and lookups
             for k, v in other.iteritems():
                 self[k] = v
-        elif hasattr(other, 'keys'):
+        elif hasattr(other, "keys"):
             for k in other.keys():
                 self[k] = other[k]
         else:
@@ -223,10 +176,8 @@ class SON(dict):
         def transform_value(value):
             if isinstance(value, list):
                 return [transform_value(v) for v in value]
-            elif isinstance(value, dict):
-                return dict([
-                    (k, transform_value(v))
-                    for k, v in value.iteritems()])
+            elif isinstance(value, abc.Mapping):
+                return dict([(k, transform_value(v)) for k, v in iteritems(value)])
             else:
                 return value
 
