@@ -23,9 +23,9 @@ from bson.tz_util import utc
 
 UPPERBOUND = 4294967296
 
+
 class Timestamp(object):
-    """MongoDB internal timestamps used in the opLog.
-    """
+    """MongoDB internal timestamps used in the opLog."""
 
     _type_marker = 17
 
@@ -65,22 +65,23 @@ class Timestamp(object):
 
     @property
     def time(self):
-        """Get the time portion of this :class:`Timestamp`.
-        """
+        """Get the time portion of this :class:`Timestamp`."""
         return self.__time
 
     @property
     def inc(self):
-        """Get the inc portion of this :class:`Timestamp`.
-        """
+        """Get the inc portion of this :class:`Timestamp`."""
         return self.__inc
 
     def __eq__(self, other):
         if isinstance(other, Timestamp):
-            return (self.__time == other.time and self.__inc == other.inc)
+            return self.__time == other.time and self.__inc == other.inc
         else:
             return NotImplemented
 
+    def __hash__(self):
+        return hash(self.time) ^ hash(self.inc)
+
     def __ne__(self, other):
         return not self == other
 
