@@ -17,26 +17,28 @@
 The :mod:`gridfs` package is an implementation of GridFS on top of
 :mod:`pymongo`, exposing a file-like interface.
 
-.. mongodoc:: gridfs
+.. seealso:: The MongoDB documentation on `gridfs <https://dochub.mongodb.org/core/gridfs>`_.
 """
 
 from bson.py3compat import abc
 from gridfs.errors import NoFile
-from gridfs.grid_file import (GridIn,
-                              GridOut,
-                              GridOutCursor,
-                              DEFAULT_CHUNK_SIZE,
-                              _clear_entity_type_registry)
-from pymongo import (ASCENDING,
-                     DESCENDING)
+from gridfs.grid_file import (
+    DEFAULT_CHUNK_SIZE,
+    GridIn,
+    GridOut,
+    GridOutCursor,
+    _clear_entity_type_registry,
+    _disallow_transactions,
+)
+from pymongo import ASCENDING, DESCENDING
 from pymongo.common import UNAUTHORIZED_CODES, validate_string
 from pymongo.database import Database
 from pymongo.errors import ConfigurationError, OperationFailure
 
 
 class GridFS(object):
-    """An instance of GridFS on top of a single Database.
-    """
+    """An instance of GridFS on top of a single Database."""
+
     def __init__(self, database, collection="fs", disable_md5=False):
         """Create a new instance of :class:`GridFS`.
 
@@ -50,6 +52,10 @@ class GridFS(object):
             computed for uploaded files. Useful in environments where MD5
             cannot be used for regulatory or other reasons. Defaults to False.
 
+        .. versionchanged:: 3.11
+           Running a GridFS operation in a transaction now always raises an
+           error. GridFS does not support multi-document transactions.
+
         .. versionchanged:: 3.1
            Indexes are only ensured on the first write to the DB.
 
@@ -57,7 +63,7 @@ class GridFS(object):
            `database` must use an acknowledged
            :attr:`~pymongo.database.Database.write_concern`
 
-        .. mongodoc:: gridfs
+        .. seealso:: The MongoDB documentation on `gridfs <https://dochub.mongodb.org/core/gridfs>`_.
         """
         if not isinstance(database, Database):
             raise TypeError("database must be an instance of Database")
@@ -65,10 +71,8 @@ class GridFS(object):
         database = _clear_entity_type_registry(database)
 
         if not database.write_concern.acknowledged:
-            raise ConfigurationError('database must use '
-                                     'acknowledged write_concern')
+            raise ConfigurationError("database must use " "acknowledged write_concern")
 
-        self.__database = database
         self.__collection = database[collection]
         self.__files = self.__collection.files
         self.__chunks = self.__collection.chunks
@@ -88,10 +92,7 @@ class GridFS(object):
         :Parameters:
           - `**kwargs` (optional): keyword arguments for file creation
         """
-        # No need for __ensure_index_files_id() here; GridIn ensures
-        # the (files_id, n) index when needed.
-        return GridIn(
-            self.__collection, disable_md5=self.__disable_md5, **kwargs)
+        return GridIn(self.__collection, disable_md5=self.__disable_md5, **kwargs)
 
     def put(self, data, **kwargs):
         """Put data in GridFS as a new file.
@@ -124,8 +125,7 @@ class GridFS(object):
         .. versionchanged:: 3.0
            w=0 writes to GridFS are now prohibited.
         """
-        grid_file = GridIn(
-            self.__collection, disable_md5=self.__disable_md5, **kwargs)
+        grid_file = GridIn(self.__collection, disable_md5=self.__disable_md5, **kwargs)
         try:
             grid_file.write(data)
         finally:
@@ -192,6 +192,7 @@ class GridFS(object):
         if filename is not None:
             query["filename"] = filename
 
+        _disallow_transactions(session)
         cursor = self.__files.find(query, session=session)
         if version < 0:
             skip = abs(version) - 1
@@ -200,8 +201,7 @@ class GridFS(object):
             cursor.limit(-1).skip(version).sort("uploadDate", ASCENDING)
         try:
             doc = next(cursor)
-            return GridOut(
-                self.__collection, file_document=doc, session=session)
+            return GridOut(self.__collection, file_document=doc, session=session)
         except StopIteration:
             raise NoFile("no version %d for filename %r" % (version, filename))
 
@@ -249,6 +249,7 @@ class GridFS(object):
         .. versionchanged:: 3.1
            ``delete`` no longer ensures indexes.
         """
+        _disallow_transactions(session)
         self.__files.delete_one({"_id": file_id}, session=session)
         self.__chunks.delete_many({"files_id": file_id}, session=session)
 
@@ -266,11 +267,12 @@ class GridFS(object):
         .. versionchanged:: 3.1
            ``list`` no longer ensures indexes.
         """
+        _disallow_transactions(session)
         # With an index, distinct includes documents with no filename
         # as None.
         return [
-            name for name in self.__files.distinct("filename", session=session)
-            if name is not None]
+            name for name in self.__files.distinct("filename", session=session) if name is not None
+        ]
 
     def find_one(self, filter=None, session=None, *args, **kwargs):
         """Get a single file from gridfs.
@@ -299,6 +301,7 @@ class GridFS(object):
         if filter is not None and not isinstance(filter, abc.Mapping):
             filter = {"_id": filter}
 
+        _disallow_transactions(session)
         for f in self.find(filter, *args, session=session, **kwargs):
             return f
 
@@ -361,7 +364,7 @@ class GridFS(object):
            Removed the read_preference, tag_sets, and
            secondary_acceptable_latency_ms options.
         .. versionadded:: 2.7
-        .. mongodoc:: find
+        .. seealso:: The MongoDB documentation on `find <https://dochub.mongodb.org/core/find>`_.
         """
         return GridOutCursor(self.__collection, *args, **kwargs)
 
@@ -403,6 +406,7 @@ class GridFS(object):
         .. versionchanged:: 3.6
            Added ``session`` parameter.
         """
+        _disallow_transactions(session)
         if kwargs:
             f = self.__files.find_one(kwargs, ["_id"], session=session)
         else:
@@ -414,9 +418,15 @@ class GridFS(object):
 class GridFSBucket(object):
     """An instance of GridFS on top of a single Database."""
 
-    def __init__(self, db, bucket_name="fs",
-                 chunk_size_bytes=DEFAULT_CHUNK_SIZE, write_concern=None,
-                 read_preference=None, disable_md5=False):
+    def __init__(
+        self,
+        db,
+        bucket_name="fs",
+        chunk_size_bytes=DEFAULT_CHUNK_SIZE,
+        write_concern=None,
+        read_preference=None,
+        disable_md5=False,
+    ):
         """Create a new instance of :class:`GridFSBucket`.
 
         Raises :exc:`TypeError` if `database` is not an instance of
@@ -439,9 +449,13 @@ class GridFSBucket(object):
             computed for uploaded files. Useful in environments where MD5
             cannot be used for regulatory or other reasons. Defaults to False.
 
+        .. versionchanged:: 3.11
+           Running a GridFS operation in a transaction now always raises an
+           error. GridFSBucket does not support multi-document transactions.
+
         .. versionadded:: 3.1
 
-        .. mongodoc:: gridfs
+        .. seealso:: The MongoDB documentation on `gridfs <https://dochub.mongodb.org/core/gridfs>`_.
         """
         if not isinstance(db, Database):
             raise TypeError("database must be an instance of Database")
@@ -450,25 +464,23 @@ class GridFSBucket(object):
 
         wtc = write_concern if write_concern is not None else db.write_concern
         if not wtc.acknowledged:
-            raise ConfigurationError('write concern must be acknowledged')
+            raise ConfigurationError("write concern must be acknowledged")
 
-        self._db = db
         self._bucket_name = bucket_name
         self._collection = db[bucket_name]
         self._disable_md5 = disable_md5
 
         self._chunks = self._collection.chunks.with_options(
-            write_concern=write_concern,
-            read_preference=read_preference)
+            write_concern=write_concern, read_preference=read_preference
+        )
 
         self._files = self._collection.files.with_options(
-            write_concern=write_concern,
-            read_preference=read_preference)
+            write_concern=write_concern, read_preference=read_preference
+        )
 
         self._chunk_size_bytes = chunk_size_bytes
 
-    def open_upload_stream(self, filename, chunk_size_bytes=None,
-                           metadata=None, session=None):
+    def open_upload_stream(self, filename, chunk_size_bytes=None, metadata=None, session=None):
         """Opens a Stream that the application can write the contents of the
         file to.
 
@@ -506,21 +518,20 @@ class GridFSBucket(object):
         """
         validate_string("filename", filename)
 
-        opts = {"filename": filename,
-                "chunk_size": (chunk_size_bytes if chunk_size_bytes
-                               is not None else self._chunk_size_bytes)}
+        opts = {
+            "filename": filename,
+            "chunk_size": (
+                chunk_size_bytes if chunk_size_bytes is not None else self._chunk_size_bytes
+            ),
+        }
         if metadata is not None:
             opts["metadata"] = metadata
 
-        return GridIn(
-            self._collection,
-            session=session,
-            disable_md5=self._disable_md5,
-            **opts)
+        return GridIn(self._collection, session=session, disable_md5=self._disable_md5, **opts)
 
     def open_upload_stream_with_id(
-            self, file_id, filename, chunk_size_bytes=None, metadata=None,
-            session=None):
+        self, file_id, filename, chunk_size_bytes=None, metadata=None, session=None
+    ):
         """Opens a Stream that the application can write the contents of the
         file to.
 
@@ -562,21 +573,21 @@ class GridFSBucket(object):
         """
         validate_string("filename", filename)
 
-        opts = {"_id": file_id,
-                "filename": filename,
-                "chunk_size": (chunk_size_bytes if chunk_size_bytes
-                               is not None else self._chunk_size_bytes)}
+        opts = {
+            "_id": file_id,
+            "filename": filename,
+            "chunk_size": (
+                chunk_size_bytes if chunk_size_bytes is not None else self._chunk_size_bytes
+            ),
+        }
         if metadata is not None:
             opts["metadata"] = metadata
 
-        return GridIn(
-            self._collection,
-            session=session,
-            disable_md5=self._disable_md5,
-            **opts)
+        return GridIn(self._collection, session=session, disable_md5=self._disable_md5, **opts)
 
-    def upload_from_stream(self, filename, source, chunk_size_bytes=None,
-                           metadata=None, session=None):
+    def upload_from_stream(
+        self, filename, source, chunk_size_bytes=None, metadata=None, session=None
+    ):
         """Uploads a user file to a GridFS bucket.
 
         Reads the contents of the user file from `source` and uploads
@@ -612,15 +623,14 @@ class GridFSBucket(object):
         .. versionchanged:: 3.6
            Added ``session`` parameter.
         """
-        with self.open_upload_stream(
-                filename, chunk_size_bytes, metadata, session=session) as gin:
+        with self.open_upload_stream(filename, chunk_size_bytes, metadata, session=session) as gin:
             gin.write(source)
 
         return gin._id
 
-    def upload_from_stream_with_id(self, file_id, filename, source,
-                                   chunk_size_bytes=None, metadata=None,
-                                   session=None):
+    def upload_from_stream_with_id(
+        self, file_id, filename, source, chunk_size_bytes=None, metadata=None, session=None
+    ):
         """Uploads a user file to a GridFS bucket with a custom file id.
 
         Reads the contents of the user file from `source` and uploads
@@ -658,8 +668,8 @@ class GridFSBucket(object):
            Added ``session`` parameter.
         """
         with self.open_upload_stream_with_id(
-                file_id, filename, chunk_size_bytes, metadata,
-                session=session) as gin:
+            file_id, filename, chunk_size_bytes, metadata, session=session
+        ) as gin:
             gin.write(source)
 
     def open_download_stream(self, file_id, session=None):
@@ -746,11 +756,11 @@ class GridFSBucket(object):
         .. versionchanged:: 3.6
            Added ``session`` parameter.
         """
+        _disallow_transactions(session)
         res = self._files.delete_one({"_id": file_id}, session=session)
         self._chunks.delete_many({"files_id": file_id}, session=session)
         if not res.deleted_count:
-            raise NoFile(
-                "no file could be deleted because none matched %s" % file_id)
+            raise NoFile("no file could be deleted because none matched %s" % file_id)
 
     def find(self, *args, **kwargs):
         """Find and return the files collection documents that match ``filter``
@@ -839,9 +849,8 @@ class GridFSBucket(object):
            Added ``session`` parameter.
         """
         validate_string("filename", filename)
-
         query = {"filename": filename}
-
+        _disallow_transactions(session)
         cursor = self._files.find(query, session=session)
         if revision < 0:
             skip = abs(revision) - 1
@@ -850,14 +859,11 @@ class GridFSBucket(object):
             cursor.limit(-1).skip(revision).sort("uploadDate", ASCENDING)
         try:
             grid_file = next(cursor)
-            return GridOut(
-                self._collection, file_document=grid_file, session=session)
+            return GridOut(self._collection, file_document=grid_file, session=session)
         except StopIteration:
-            raise NoFile(
-                "no version %d for filename %r" % (revision, filename))
+            raise NoFile("no version %d for filename %r" % (revision, filename))
 
-    def download_to_stream_by_name(self, filename, destination, revision=-1,
-                                   session=None):
+    def download_to_stream_by_name(self, filename, destination, revision=-1, session=None):
         """Write the contents of `filename` (with optional `revision`) to
         `destination`.
 
@@ -895,8 +901,7 @@ class GridFSBucket(object):
         .. versionchanged:: 3.6
            Added ``session`` parameter.
         """
-        with self.open_download_stream_by_name(
-                filename, revision, session=session) as gout:
+        with self.open_download_stream_by_name(filename, revision, session=session) as gout:
             for chunk in gout:
                 destination.write(chunk)
 
@@ -922,9 +927,12 @@ class GridFSBucket(object):
         .. versionchanged:: 3.6
            Added ``session`` parameter.
         """
-        result = self._files.update_one({"_id": file_id},
-                                        {"$set": {"filename": new_filename}},
-                                        session=session)
+        _disallow_transactions(session)
+        result = self._files.update_one(
+            {"_id": file_id}, {"$set": {"filename": new_filename}}, session=session
+        )
         if not result.matched_count:
-            raise NoFile("no files could be renamed %r because none "
-                         "matched file_id %i" % (new_filename, file_id))
+            raise NoFile(
+                "no files could be renamed %r because none "
+                "matched file_id %i" % (new_filename, file_id)
+            )
