@@ -17,32 +17,38 @@
 .. versionadded:: 2.7
 """
 import copy
-
 from itertools import islice
 
 from bson.objectid import ObjectId
 from bson.raw_bson import RawBSONDocument
 from bson.son import SON
 from pymongo.client_session import _validate_session_write_concern
-from pymongo.common import (validate_is_mapping,
-                            validate_is_document_type,
-                            validate_ok_for_replace,
-                            validate_ok_for_update)
-from pymongo.helpers import _RETRYABLE_ERROR_CODES
 from pymongo.collation import validate_collation_or_none
-from pymongo.errors import (BulkWriteError,
-                            ConfigurationError,
-                            InvalidOperation,
-                            OperationFailure)
-from pymongo.message import (_INSERT, _UPDATE, _DELETE,
-                             _do_batched_insert,
-                             _randint,
-                             _BulkWriteContext,
-                             _EncryptedBulkWriteContext)
+from pymongo.common import (
+    validate_is_document_type,
+    validate_is_mapping,
+    validate_ok_for_replace,
+    validate_ok_for_update,
+)
+from pymongo.errors import (
+    BulkWriteError,
+    ConfigurationError,
+    InvalidOperation,
+    OperationFailure,
+)
+from pymongo.helpers import _RETRYABLE_ERROR_CODES, _get_wce_doc
+from pymongo.message import (
+    _DELETE,
+    _INSERT,
+    _UPDATE,
+    _BulkWriteContext,
+    _do_batched_insert,
+    _EncryptedBulkWriteContext,
+    _randint,
+)
 from pymongo.read_preferences import ReadPreference
 from pymongo.write_concern import WriteConcern
 
-
 _DELETE_ALL = 0
 _DELETE_ONE = 1
 
@@ -51,7 +57,7 @@ _BAD_VALUE = 2
 _UNKNOWN_ERROR = 8
 _WRITE_CONCERN_ERROR = 64
 
-_COMMANDS = ('insert', 'update', 'delete')
+_COMMANDS = ("insert", "update", "delete")
 
 
 # These string literals are used when we create fake server return
@@ -61,11 +67,10 @@ _UOP = u"op"
 
 
 class _Run(object):
-    """Represents a batch of write operations.
-    """
+    """Represents a batch of write operations."""
+
     def __init__(self, op_type):
-        """Initialize a new Run object.
-        """
+        """Initialize a new Run object."""
         self.op_type = op_type
         self.index_map = []
         self.ops = []
@@ -92,8 +97,7 @@ class _Run(object):
 
 
 def _merge_command(run, full_result, offset, result):
-    """Merge a write command result into the full bulk result.
-    """
+    """Merge a write command result into the full bulk result."""
     affected = result.get("n", 0)
 
     if run.op_type == _INSERT:
@@ -110,7 +114,7 @@ def _merge_command(run, full_result, offset, result):
                 doc["index"] = run.index(doc["index"] + offset)
             full_result["upserted"].extend(upserted)
             full_result["nUpserted"] += n_upserted
-            full_result["nMatched"] += (affected - n_upserted)
+            full_result["nMatched"] += affected - n_upserted
         else:
             full_result["nMatched"] += affected
         full_result["nModified"] += result["nModified"]
@@ -126,36 +130,35 @@ def _merge_command(run, full_result, offset, result):
             replacement[_UOP] = run.ops[idx]
             full_result["writeErrors"].append(replacement)
 
-    wc_error = result.get("writeConcernError")
-    if wc_error:
-        full_result["writeConcernErrors"].append(wc_error)
+    wce = _get_wce_doc(result)
+    if wce:
+        full_result["writeConcernErrors"].append(wce)
 
 
 def _raise_bulk_write_error(full_result):
-    """Raise a BulkWriteError from the full bulk api result.
-    """
+    """Raise a BulkWriteError from the full bulk api result."""
     if full_result["writeErrors"]:
-        full_result["writeErrors"].sort(
-            key=lambda error: error["index"])
+        full_result["writeErrors"].sort(key=lambda error: error["index"])
     raise BulkWriteError(full_result)
 
 
 class _Bulk(object):
-    """The private guts of the bulk write API.
-    """
+    """The private guts of the bulk write API."""
+
     def __init__(self, collection, ordered, bypass_document_validation):
-        """Initialize a _Bulk instance.
-        """
+        """Initialize a _Bulk instance."""
         self.collection = collection.with_options(
             codec_options=collection.codec_options._replace(
-                unicode_decode_error_handler='replace',
-                document_class=dict))
+                unicode_decode_error_handler="replace", document_class=dict
+            )
+        )
         self.ordered = ordered
         self.ops = []
         self.executed = False
         self.bypass_doc_val = bypass_document_validation
         self.uses_collation = False
         self.uses_array_filters = False
+        self.uses_hint = False
         self.is_retryable = True
         self.retrying = False
         self.started_retryable_write = False
@@ -171,54 +174,64 @@ class _Bulk(object):
             return _BulkWriteContext
 
     def add_insert(self, document):
-        """Add an insert document to the list of ops.
-        """
+        """Add an insert document to the list of ops."""
         validate_is_document_type("document", document)
         # Generate ObjectId client side.
-        if not (isinstance(document, RawBSONDocument) or '_id' in document):
-            document['_id'] = ObjectId()
+        if not (isinstance(document, RawBSONDocument) or "_id" in document):
+            document["_id"] = ObjectId()
         self.ops.append((_INSERT, document))
 
-    def add_update(self, selector, update, multi=False, upsert=False,
-                   collation=None, array_filters=None):
-        """Create an update document and add it to the list of ops.
-        """
+    def add_update(
+        self,
+        selector,
+        update,
+        multi=False,
+        upsert=False,
+        collation=None,
+        array_filters=None,
+        hint=None,
+    ):
+        """Create an update document and add it to the list of ops."""
         validate_ok_for_update(update)
-        cmd = SON([('q', selector), ('u', update),
-                   ('multi', multi), ('upsert', upsert)])
+        cmd = SON([("q", selector), ("u", update), ("multi", multi), ("upsert", upsert)])
         collation = validate_collation_or_none(collation)
         if collation is not None:
             self.uses_collation = True
-            cmd['collation'] = collation
+            cmd["collation"] = collation
         if array_filters is not None:
             self.uses_array_filters = True
-            cmd['arrayFilters'] = array_filters
+            cmd["arrayFilters"] = array_filters
+        if hint is not None:
+            self.uses_hint = True
+            cmd["hint"] = hint
         if multi:
             # A bulk_write containing an update_many is not retryable.
             self.is_retryable = False
         self.ops.append((_UPDATE, cmd))
 
-    def add_replace(self, selector, replacement, upsert=False,
-                    collation=None):
-        """Create a replace document and add it to the list of ops.
-        """
+    def add_replace(self, selector, replacement, upsert=False, collation=None, hint=None):
+        """Create a replace document and add it to the list of ops."""
         validate_ok_for_replace(replacement)
-        cmd = SON([('q', selector), ('u', replacement),
-                   ('multi', False), ('upsert', upsert)])
+        cmd = SON([("q", selector), ("u", replacement), ("multi", False), ("upsert", upsert)])
         collation = validate_collation_or_none(collation)
         if collation is not None:
             self.uses_collation = True
-            cmd['collation'] = collation
+            cmd["collation"] = collation
+        if hint is not None:
+            self.uses_hint = True
+            cmd["hint"] = hint
         self.ops.append((_UPDATE, cmd))
 
-    def add_delete(self, selector, limit, collation=None):
-        """Create a delete document and add it to the list of ops.
-        """
-        cmd = SON([('q', selector), ('limit', limit)])
+    def add_delete(self, selector, limit, collation=None, hint=None):
+        """Create a delete document and add it to the list of ops."""
+        cmd = SON([("q", selector), ("limit", limit)])
         collation = validate_collation_or_none(collation)
         if collation is not None:
             self.uses_collation = True
-            cmd['collation'] = collation
+            cmd["collation"] = collation
+        if hint is not None:
+            self.uses_hint = True
+            cmd["hint"] = hint
         if limit == _DELETE_ALL:
             # A bulk_write containing a delete_many is not retryable.
             self.is_retryable = False
@@ -250,14 +263,16 @@ class _Bulk(object):
             if run.ops:
                 yield run
 
-    def _execute_command(self, generator, write_concern, session,
-                         sock_info, op_id, retryable, full_result):
-        if sock_info.max_wire_version < 5 and self.uses_collation:
-            raise ConfigurationError(
-                'Must be connected to MongoDB 3.4+ to use a collation.')
+    def _execute_command(
+        self, generator, write_concern, session, sock_info, op_id, retryable, full_result
+    ):
+        if sock_info.max_wire_version < 5:
+            if self.uses_collation:
+                raise ConfigurationError("Must be connected to MongoDB 3.4+ to use a collation.")
+            if self.uses_hint:
+                raise ConfigurationError("Must be connected to MongoDB 3.4+ to use hint.")
         if sock_info.max_wire_version < 6 and self.uses_array_filters:
-            raise ConfigurationError(
-                'Must be connected to MongoDB 3.6+ to use arrayFilters.')
+            raise ConfigurationError("Must be connected to MongoDB 3.6+ to use arrayFilters.")
 
         db_name = self.collection.database.name
         client = self.collection.database.client
@@ -271,32 +286,40 @@ class _Bulk(object):
         # sock_info.write_command.
         sock_info.validate_session(client, session)
         while run:
-            cmd = SON([(_COMMANDS[run.op_type], self.collection.name),
-                       ('ordered', self.ordered)])
-            if not write_concern.is_server_default:
-                cmd['writeConcern'] = write_concern.document
-            if self.bypass_doc_val and sock_info.max_wire_version >= 4:
-                cmd['bypassDocumentValidation'] = True
+            cmd_name = _COMMANDS[run.op_type]
             bwc = self.bulk_ctx_class(
-                db_name, cmd, sock_info, op_id, listeners, session,
-                run.op_type, self.collection.codec_options)
+                db_name,
+                cmd_name,
+                sock_info,
+                op_id,
+                listeners,
+                session,
+                run.op_type,
+                self.collection.codec_options,
+            )
 
             while run.idx_offset < len(run.ops):
+                cmd = SON([(cmd_name, self.collection.name), ("ordered", self.ordered)])
+                if not write_concern.is_server_default:
+                    cmd["writeConcern"] = write_concern.document
+                if self.bypass_doc_val and sock_info.max_wire_version >= 4:
+                    cmd["bypassDocumentValidation"] = True
                 if session:
                     # Start a new retryable write unless one was already
                     # started for this command.
                     if retryable and not self.started_retryable_write:
                         session._start_retryable_write()
                         self.started_retryable_write = True
-                    session._apply_to(cmd, retryable, ReadPreference.PRIMARY)
+                    session._apply_to(cmd, retryable, ReadPreference.PRIMARY, sock_info)
                 sock_info.send_cluster_time(cmd, session, client)
+                sock_info.add_server_api(cmd)
                 ops = islice(run.ops, run.idx_offset, None)
                 # Run as many ops as possible in one command.
-                result, to_send = bwc.execute(ops, client)
+                result, to_send = bwc.execute(cmd, ops, client)
 
                 # Retryable writeConcernErrors halt the execution of this run.
-                wce = result.get('writeConcernError', {})
-                if wce.get('code', 0) in _RETRYABLE_ERROR_CODES:
+                wce = result.get("writeConcernError", {})
+                if wce.get("code", 0) in _RETRYABLE_ERROR_CODES:
                     # Synthesize the full bulk result without modifying the
                     # current one because this write operation may be retried.
                     full = copy.deepcopy(full_result)
@@ -314,14 +337,13 @@ class _Bulk(object):
 
             # We're supposed to continue if errors are
             # at the write concern level (e.g. wtimeout)
-            if self.ordered and full_result['writeErrors']:
+            if self.ordered and full_result["writeErrors"]:
                 break
             # Reset our state
             self.current_run = run = next(generator, None)
 
     def execute_command(self, generator, write_concern, session):
-        """Execute using write commands.
-        """
+        """Execute using write commands."""
         # nModified is only reported for write commands, not legacy ops.
         full_result = {
             "writeErrors": [],
@@ -337,39 +359,51 @@ class _Bulk(object):
 
         def retryable_bulk(session, sock_info, retryable):
             self._execute_command(
-                generator, write_concern, session, sock_info, op_id,
-                retryable, full_result)
+                generator, write_concern, session, sock_info, op_id, retryable, full_result
+            )
 
         client = self.collection.database.client
         with client._tmp_session(session) as s:
-            client._retry_with_session(
-                self.is_retryable, retryable_bulk, s, self)
+            client._retry_with_session(self.is_retryable, retryable_bulk, s, self)
 
         if full_result["writeErrors"] or full_result["writeConcernErrors"]:
             _raise_bulk_write_error(full_result)
         return full_result
 
     def execute_insert_no_results(self, sock_info, run, op_id, acknowledged):
-        """Execute insert, returning no results.
-        """
-        command = SON([('insert', self.collection.name),
-                       ('ordered', self.ordered)])
-        concern = {'w': int(self.ordered)}
-        command['writeConcern'] = concern
-        if self.bypass_doc_val and sock_info.max_wire_version >= 4:
-            command['bypassDocumentValidation'] = True
+        """Execute insert, returning no results."""
         db = self.collection.database
+        concern = {"w": int(self.ordered)}
+        cmd = SON(
+            [("insert", self.collection.name), ("ordered", self.ordered), ("writeConcern", concern)]
+        )
+        if self.bypass_doc_val and sock_info.max_wire_version >= 4:
+            cmd["bypassDocumentValidation"] = True
         bwc = _BulkWriteContext(
-            db.name, command, sock_info, op_id, db.client._event_listeners,
-            None, _INSERT, self.collection.codec_options)
+            db.name,
+            "insert",
+            sock_info,
+            op_id,
+            db.client._event_listeners,
+            None,
+            _INSERT,
+            self.collection.codec_options,
+            cmd_legacy=cmd,
+        )
         # Legacy batched OP_INSERT.
         _do_batched_insert(
-            self.collection.full_name, run.ops, True, acknowledged, concern,
-            not self.ordered, self.collection.codec_options, bwc)
+            self.collection.full_name,
+            run.ops,
+            True,
+            acknowledged,
+            concern,
+            not self.ordered,
+            self.collection.codec_options,
+            bwc,
+        )
 
     def execute_op_msg_no_results(self, sock_info, generator):
-        """Execute write commands with OP_MSG and w=0 writeConcern, unordered.
-        """
+        """Execute write commands with OP_MSG and w=0 writeConcern, unordered."""
         db_name = self.collection.database.name
         client = self.collection.database.client
         listeners = client._event_listeners
@@ -380,23 +414,35 @@ class _Bulk(object):
         run = self.current_run
 
         while run:
-            cmd = SON([(_COMMANDS[run.op_type], self.collection.name),
-                       ('ordered', False),
-                       ('writeConcern', {'w': 0})])
+            cmd_name = _COMMANDS[run.op_type]
             bwc = self.bulk_ctx_class(
-                db_name, cmd, sock_info, op_id, listeners, None,
-                run.op_type, self.collection.codec_options)
+                db_name,
+                cmd_name,
+                sock_info,
+                op_id,
+                listeners,
+                None,
+                run.op_type,
+                self.collection.codec_options,
+            )
 
             while run.idx_offset < len(run.ops):
+                cmd = SON(
+                    [
+                        (cmd_name, self.collection.name),
+                        ("ordered", False),
+                        ("writeConcern", {"w": 0}),
+                    ]
+                )
+                sock_info.add_server_api(cmd)
                 ops = islice(run.ops, run.idx_offset, None)
                 # Run as many ops as possible.
-                to_send = bwc.execute_unack(ops, client)
+                to_send = bwc.execute_unack(cmd, ops, client)
                 run.idx_offset += len(to_send)
             self.current_run = run = next(generator, None)
 
     def execute_command_no_results(self, sock_info, generator):
-        """Execute write commands with OP_MSG and w=0 WriteConcern, ordered.
-        """
+        """Execute write commands with OP_MSG and w=0 WriteConcern, ordered."""
         full_result = {
             "writeErrors": [],
             "writeConcernErrors": [],
@@ -414,24 +460,24 @@ class _Bulk(object):
         op_id = _randint()
         try:
             self._execute_command(
-                generator, write_concern, None,
-                sock_info, op_id, False, full_result)
+                generator, write_concern, None, sock_info, op_id, False, full_result
+            )
         except OperationFailure:
             pass
 
     def execute_no_results(self, sock_info, generator):
-        """Execute all operations, returning no results (w=0).
-        """
+        """Execute all operations, returning no results (w=0)."""
         if self.uses_collation:
-            raise ConfigurationError(
-                'Collation is unsupported for unacknowledged writes.')
+            raise ConfigurationError("Collation is unsupported for unacknowledged writes.")
         if self.uses_array_filters:
-            raise ConfigurationError(
-                'arrayFilters is unsupported for unacknowledged writes.')
+            raise ConfigurationError("arrayFilters is unsupported for unacknowledged writes.")
+        if self.uses_hint:
+            raise ConfigurationError("hint is unsupported for unacknowledged writes.")
         # Cannot have both unacknowledged writes and bypass document validation.
         if self.bypass_doc_val and sock_info.max_wire_version >= 4:
-            raise OperationFailure("Cannot set bypass_document_validation with"
-                                   " unacknowledged write concern")
+            raise OperationFailure(
+                "Cannot set bypass_document_validation with" " unacknowledged write concern"
+            )
 
         # OP_MSG
         if sock_info.max_wire_version > 5:
@@ -455,45 +501,45 @@ class _Bulk(object):
             needs_ack = self.ordered and next_run is not None
             try:
                 if run.op_type == _INSERT:
-                    self.execute_insert_no_results(
-                        sock_info, run, op_id, needs_ack)
+                    self.execute_insert_no_results(sock_info, run, op_id, needs_ack)
                 elif run.op_type == _UPDATE:
                     for operation in run.ops:
-                        doc = operation['u']
+                        doc = operation["u"]
                         check_keys = True
-                        if doc and next(iter(doc)).startswith('$'):
+                        if doc and next(iter(doc)).startswith("$"):
                             check_keys = False
                         coll._update(
                             sock_info,
-                            operation['q'],
+                            operation["q"],
                             doc,
-                            operation['upsert'],
+                            operation["upsert"],
                             check_keys,
-                            operation['multi'],
+                            operation["multi"],
                             write_concern=write_concern,
                             op_id=op_id,
                             ordered=self.ordered,
-                            bypass_doc_val=self.bypass_doc_val)
+                            bypass_doc_val=self.bypass_doc_val,
+                        )
                 else:
                     for operation in run.ops:
-                        coll._delete(sock_info,
-                                     operation['q'],
-                                     not operation['limit'],
-                                     write_concern,
-                                     op_id,
-                                     self.ordered)
+                        coll._delete(
+                            sock_info,
+                            operation["q"],
+                            not operation["limit"],
+                            write_concern,
+                            op_id,
+                            self.ordered,
+                        )
             except OperationFailure:
                 if self.ordered:
                     break
 
     def execute(self, write_concern, session):
-        """Execute operations.
-        """
+        """Execute operations."""
         if not self.ops:
-            raise InvalidOperation('No operations to execute')
+            raise InvalidOperation("No operations to execute")
         if self.executed:
-            raise InvalidOperation('Bulk operations can '
-                                   'only be executed once.')
+            raise InvalidOperation("Bulk operations can " "only be executed once.")
         self.executed = True
         write_concern = write_concern or self.collection.write_concern
         session = _validate_session_write_concern(session, write_concern)
@@ -512,10 +558,9 @@ class _Bulk(object):
 
 
 class BulkUpsertOperation(object):
-    """An interface for adding upsert operations.
-    """
+    """An interface for adding upsert operations."""
 
-    __slots__ = ('__selector', '__bulk', '__collation')
+    __slots__ = ("__selector", "__bulk", "__collation")
 
     def __init__(self, selector, bulk, collation):
         self.__selector = selector
@@ -528,9 +573,9 @@ class BulkUpsertOperation(object):
         :Parameters:
           - `update` (dict): the update operations to apply
         """
-        self.__bulk.add_update(self.__selector,
-                               update, multi=False, upsert=True,
-                               collation=self.__collation)
+        self.__bulk.add_update(
+            self.__selector, update, multi=False, upsert=True, collation=self.__collation
+        )
 
     def update(self, update):
         """Update all documents matching the selector.
@@ -538,9 +583,9 @@ class BulkUpsertOperation(object):
         :Parameters:
           - `update` (dict): the update operations to apply
         """
-        self.__bulk.add_update(self.__selector,
-                               update, multi=True, upsert=True,
-                               collation=self.__collation)
+        self.__bulk.add_update(
+            self.__selector, update, multi=True, upsert=True, collation=self.__collation
+        )
 
     def replace_one(self, replacement):
         """Replace one entire document matching the selector criteria.
@@ -548,15 +593,15 @@ class BulkUpsertOperation(object):
         :Parameters:
           - `replacement` (dict): the replacement document
         """
-        self.__bulk.add_replace(self.__selector, replacement, upsert=True,
-                                collation=self.__collation)
+        self.__bulk.add_replace(
+            self.__selector, replacement, upsert=True, collation=self.__collation
+        )
 
 
 class BulkWriteOperation(object):
-    """An interface for adding update or remove operations.
-    """
+    """An interface for adding update or remove operations."""
 
-    __slots__ = ('__selector', '__bulk', '__collation')
+    __slots__ = ("__selector", "__bulk", "__collation")
 
     def __init__(self, selector, bulk, collation):
         self.__selector = selector
@@ -569,8 +614,7 @@ class BulkWriteOperation(object):
         :Parameters:
           - `update` (dict): the update operations to apply
         """
-        self.__bulk.add_update(self.__selector, update, multi=False,
-                               collation=self.__collation)
+        self.__bulk.add_update(self.__selector, update, multi=False, collation=self.__collation)
 
     def update(self, update):
         """Update all documents matching the selector criteria.
@@ -578,8 +622,7 @@ class BulkWriteOperation(object):
         :Parameters:
           - `update` (dict): the update operations to apply
         """
-        self.__bulk.add_update(self.__selector, update, multi=True,
-                               collation=self.__collation)
+        self.__bulk.add_update(self.__selector, update, multi=True, collation=self.__collation)
 
     def replace_one(self, replacement):
         """Replace one entire document matching the selector criteria.
@@ -587,20 +630,15 @@ class BulkWriteOperation(object):
         :Parameters:
           - `replacement` (dict): the replacement document
         """
-        self.__bulk.add_replace(self.__selector, replacement,
-                                collation=self.__collation)
+        self.__bulk.add_replace(self.__selector, replacement, collation=self.__collation)
 
     def remove_one(self):
-        """Remove a single document matching the selector criteria.
-        """
-        self.__bulk.add_delete(self.__selector, _DELETE_ONE,
-                               collation=self.__collation)
+        """Remove a single document matching the selector criteria."""
+        self.__bulk.add_delete(self.__selector, _DELETE_ONE, collation=self.__collation)
 
     def remove(self):
-        """Remove all documents matching the selector criteria.
-        """
-        self.__bulk.add_delete(self.__selector, _DELETE_ALL,
-                               collation=self.__collation)
+        """Remove all documents matching the selector criteria."""
+        self.__bulk.add_delete(self.__selector, _DELETE_ALL, collation=self.__collation)
 
     def upsert(self):
         """Specify that all chained update operations should be
@@ -610,18 +648,15 @@ class BulkWriteOperation(object):
           - A :class:`BulkUpsertOperation` instance, used to add
             update operations to this bulk operation.
         """
-        return BulkUpsertOperation(self.__selector, self.__bulk,
-                                   self.__collation)
+        return BulkUpsertOperation(self.__selector, self.__bulk, self.__collation)
 
 
 class BulkOperationBuilder(object):
-    """**DEPRECATED**: An interface for executing a batch of write operations.
-    """
+    """**DEPRECATED**: An interface for executing a batch of write operations."""
 
-    __slots__ = '__bulk'
+    __slots__ = "__bulk"
 
-    def __init__(self, collection, ordered=True,
-                 bypass_document_validation=False):
+    def __init__(self, collection, ordered=True, bypass_document_validation=False):
         """**DEPRECATED**: Initialize a new BulkOperationBuilder instance.
 
         :Parameters:
