@@ -37,13 +37,15 @@ the session are causally after previous read and write operations. Using a
 causally consistent session, an application can read its own writes and is
 guaranteed monotonic reads, even when reading from replica set secondaries.
 
-.. mongodoc:: causal-consistency
+.. seealso:: The MongoDB documentation on `causal-consistency <https://dochub.mongodb.org/core/causal-consistency>`_.
 
 .. _transactions-ref:
 
 Transactions
 ============
 
+.. versionadded:: 3.7
+
 MongoDB 4.0 adds support for transactions on replica set primaries. A
 transaction is associated with a :class:`ClientSession`. To start a transaction
 on a session, use :meth:`ClientSession.start_transaction` in a with-statement.
@@ -76,22 +78,56 @@ see the `MongoDB server's documentation for transactions
 A session may only have a single active transaction at a time, multiple
 transactions on the same session can be executed in sequence.
 
-.. versionadded:: 3.7
-
 Sharded Transactions
 ^^^^^^^^^^^^^^^^^^^^
 
+.. versionadded:: 3.9
+
 PyMongo 3.9 adds support for transactions on sharded clusters running MongoDB
-4.2. Sharded transactions have the same API as replica set transactions.
+>=4.2. Sharded transactions have the same API as replica set transactions.
 When running a transaction against a sharded cluster, the session is
 pinned to the mongos server selected for the first operation in the
 transaction. All subsequent operations that are part of the same transaction
 are routed to the same mongos server. When the transaction is completed, by
 running either commitTransaction or abortTransaction, the session is unpinned.
 
-.. versionadded:: 3.9
+.. seealso:: The MongoDB documentation on `transactions <https://dochub.mongodb.org/core/transactions>`_.
 
-.. mongodoc:: transactions
+.. _snapshot-reads-ref:
+
+Snapshot Reads
+==============
+
+.. versionadded:: 3.12
+
+MongoDB 5.0 adds support for snapshot reads. Snapshot reads are requested by
+passing the ``snapshot`` option to
+:meth:`~pymongo.mongo_client.MongoClient.start_session`.
+If ``snapshot`` is True, all read operations that use this session read data
+from the same snapshot timestamp. The server chooses the latest
+majority-committed snapshot timestamp when executing the first read operation
+using the session. Subsequent reads on this session read from the same
+snapshot timestamp. Snapshot reads are also supported when reading from
+replica set secondaries.
+
+.. code-block:: python
+
+  # Each read using this session reads data from the same point in time.
+  with client.start_session(snapshot=True) as session:
+      order = orders.find_one({"sku": "abc123"}, session=session)
+      inventory = inventory.find_one({"sku": "abc123"}, session=session)
+
+Snapshot Reads Limitations
+^^^^^^^^^^^^^^^^^^^^^^^^^^
+
+Snapshot reads sessions are incompatible with ``causal_consistency=True``.
+Only the following read operations are supported in a snapshot reads session:
+
+- :meth:`~pymongo.collection.Collection.find`
+- :meth:`~pymongo.collection.Collection.find_one`
+- :meth:`~pymongo.collection.Collection.aggregate`
+- :meth:`~pymongo.collection.Collection.count_documents`
+- :meth:`~pymongo.collection.Collection.distinct` (on unsharded collections)
 
 Classes
 =======
@@ -105,17 +141,20 @@ from bson.int64 import Int64
 from bson.py3compat import abc, integer_types
 from bson.son import SON
 from bson.timestamp import Timestamp
-
 from pymongo import monotonic
-from pymongo.errors import (ConfigurationError,
-                            ConnectionFailure,
-                            InvalidOperation,
-                            OperationFailure,
-                            PyMongoError,
-                            WTimeoutError)
+from pymongo.cursor import _SocketManager
+from pymongo.errors import (
+    ConfigurationError,
+    ConnectionFailure,
+    InvalidOperation,
+    OperationFailure,
+    PyMongoError,
+    WTimeoutError,
+)
 from pymongo.helpers import _RETRYABLE_ERROR_CODES
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference, _ServerMode
+from pymongo.server_type import SERVER_TYPE
 from pymongo.write_concern import WriteConcern
 
 
@@ -123,22 +162,36 @@ class SessionOptions(object):
     """Options for a new :class:`ClientSession`.
 
     :Parameters:
-      - `causal_consistency` (optional): If True (the default), read
-        operations are causally ordered within the session.
+      - `causal_consistency` (optional): If True, read operations are causally
+        ordered within the session. Defaults to True when the ``snapshot``
+        option is ``False``.
       - `default_transaction_options` (optional): The default
         TransactionOptions to use for transactions started on this session.
+      - `snapshot` (optional): If True, then all reads performed using this
+        session will read from the same snapshot. This option is incompatible
+        with ``causal_consistency=True``. Defaults to ``False``.
+
+    .. versionchanged:: 3.12
+       Added the ``snapshot`` parameter.
     """
-    def __init__(self,
-                 causal_consistency=True,
-                 default_transaction_options=None):
+
+    def __init__(self, causal_consistency=None, default_transaction_options=None, snapshot=False):
+        if snapshot:
+            if causal_consistency:
+                raise ConfigurationError("snapshot reads do not support " "causal_consistency=True")
+            causal_consistency = False
+        elif causal_consistency is None:
+            causal_consistency = True
         self._causal_consistency = causal_consistency
         if default_transaction_options is not None:
             if not isinstance(default_transaction_options, TransactionOptions):
                 raise TypeError(
                     "default_transaction_options must be an instance of "
-                    "pymongo.client_session.TransactionOptions, not: %r" %
-                    (default_transaction_options,))
+                    "pymongo.client_session.TransactionOptions, not: %r"
+                    % (default_transaction_options,)
+                )
         self._default_transaction_options = default_transaction_options
+        self._snapshot = snapshot
 
     @property
     def causal_consistency(self):
@@ -154,10 +207,18 @@ class SessionOptions(object):
         """
         return self._default_transaction_options
 
+    @property
+    def snapshot(self):
+        """Whether snapshot reads are configured.
+
+        .. versionadded:: 3.12
+        """
+        return self._snapshot
+
 
 class TransactionOptions(object):
     """Options for :meth:`ClientSession.start_transaction`.
-    
+
     :Parameters:
       - `read_concern` (optional): The
         :class:`~pymongo.read_concern.ReadConcern` to use for this transaction.
@@ -182,35 +243,41 @@ class TransactionOptions(object):
 
     .. versionadded:: 3.7
     """
-    def __init__(self, read_concern=None, write_concern=None,
-                 read_preference=None, max_commit_time_ms=None):
+
+    def __init__(
+        self, read_concern=None, write_concern=None, read_preference=None, max_commit_time_ms=None
+    ):
         self._read_concern = read_concern
         self._write_concern = write_concern
         self._read_preference = read_preference
         self._max_commit_time_ms = max_commit_time_ms
         if read_concern is not None:
             if not isinstance(read_concern, ReadConcern):
-                raise TypeError("read_concern must be an instance of "
-                                "pymongo.read_concern.ReadConcern, not: %r" %
-                                (read_concern,))
+                raise TypeError(
+                    "read_concern must be an instance of "
+                    "pymongo.read_concern.ReadConcern, not: %r" % (read_concern,)
+                )
         if write_concern is not None:
             if not isinstance(write_concern, WriteConcern):
-                raise TypeError("write_concern must be an instance of "
-                                "pymongo.write_concern.WriteConcern, not: %r" %
-                                (write_concern,))
+                raise TypeError(
+                    "write_concern must be an instance of "
+                    "pymongo.write_concern.WriteConcern, not: %r" % (write_concern,)
+                )
             if not write_concern.acknowledged:
                 raise ConfigurationError(
                     "transactions do not support unacknowledged write concern"
-                    ": %r" % (write_concern,))
+                    ": %r" % (write_concern,)
+                )
         if read_preference is not None:
             if not isinstance(read_preference, _ServerMode):
-                raise TypeError("%r is not valid for read_preference. See "
-                                "pymongo.read_preferences for valid "
-                                "options." % (read_preference,))
+                raise TypeError(
+                    "%r is not valid for read_preference. See "
+                    "pymongo.read_preferences for valid "
+                    "options." % (read_preference,)
+                )
         if max_commit_time_ms is not None:
             if not isinstance(max_commit_time_ms, integer_types):
-                raise TypeError(
-                    "max_commit_time_ms must be an integer or None")
+                raise TypeError("max_commit_time_ms must be an integer or None")
 
     @property
     def read_concern(self):
@@ -224,8 +291,7 @@ class TransactionOptions(object):
 
     @property
     def read_preference(self):
-        """This transaction's :class:`~pymongo.read_preferences.ReadPreference`.
-        """
+        """This transaction's :class:`~pymongo.read_preferences.ReadPreference`."""
         return self._read_preference
 
     @property
@@ -253,14 +319,15 @@ def _validate_session_write_concern(session, write_concern):
                 return None
             else:
                 raise ConfigurationError(
-                    'Explicit sessions are incompatible with '
-                    'unacknowledged write concern: %r' % (
-                        write_concern,))
+                    "Explicit sessions are incompatible with "
+                    "unacknowledged write concern: %r" % (write_concern,)
+                )
     return session
 
 
 class _TransactionContext(object):
     """Internal transaction context manager for start_transaction."""
+
     def __init__(self, session):
         self.__session = session
 
@@ -286,24 +353,56 @@ class _TxnState(object):
 
 class _Transaction(object):
     """Internal class to hold transaction information in a ClientSession."""
-    def __init__(self, opts):
+
+    def __init__(self, opts, client):
         self.opts = opts
         self.state = _TxnState.NONE
         self.sharded = False
         self.pinned_address = None
+        self.sock_mgr = None
         self.recovery_token = None
         self.attempt = 0
+        self.client = client
 
     def active(self):
         return self.state in (_TxnState.STARTING, _TxnState.IN_PROGRESS)
 
+    def starting(self):
+        return self.state == _TxnState.STARTING
+
+    @property
+    def pinned_conn(self):
+        if self.active() and self.sock_mgr:
+            return self.sock_mgr.sock
+        return None
+
+    def pin(self, server, sock_info):
+        self.sharded = True
+        self.pinned_address = server.description.address
+        if server.description.server_type == SERVER_TYPE.LoadBalancer:
+            sock_info.pin_txn()
+            self.sock_mgr = _SocketManager(sock_info, False)
+
+    def unpin(self):
+        self.pinned_address = None
+        if self.sock_mgr:
+            self.sock_mgr.close()
+        self.sock_mgr = None
+
     def reset(self):
+        self.unpin()
         self.state = _TxnState.NONE
         self.sharded = False
-        self.pinned_address = None
         self.recovery_token = None
         self.attempt = 0
 
+    def __del__(self):
+        if self.sock_mgr:
+            # Reuse the cursor closing machinery to return the socket to the
+            # pool soon.
+            self.client._close_cursor_soon(0, None, self.sock_mgr)
+            self.sock_mgr = None
+
 
 def _reraise_with_unknown_commit(exc):
     """Re-raise an exception with the UnknownTransactionCommitResult label."""
@@ -318,10 +417,12 @@ def _max_time_expired_error(exc):
 
 # From the transactions spec, all the retryable writes errors plus
 # WriteConcernFailed.
-_UNKNOWN_COMMIT_ERROR_CODES = _RETRYABLE_ERROR_CODES | frozenset([
-    64,    # WriteConcernFailed
-    50,    # MaxTimeMSExpired
-])
+_UNKNOWN_COMMIT_ERROR_CODES = _RETRYABLE_ERROR_CODES | frozenset(
+    [
+        64,  # WriteConcernFailed
+        50,  # MaxTimeMSExpired
+    ]
+)
 
 # From the Convenient API for Transactions spec, with_transaction must
 # halt retries after 120 seconds.
@@ -347,6 +448,7 @@ class ClientSession(object):
     :class:`ClientSession`, call
     :meth:`~pymongo.mongo_client.MongoClient.start_session`.
     """
+
     def __init__(self, client, server_session, options, authset, implicit):
         # A MongoClient, a _ServerSession, a SessionOptions, and a set.
         self._client = client
@@ -355,9 +457,10 @@ class ClientSession(object):
         self._authset = authset
         self._cluster_time = None
         self._operation_time = None
+        self._snapshot_time = None
         # Is this an implicitly created session?
         self._implicit = implicit
-        self._transaction = _Transaction(None)
+        self._transaction = _Transaction(None, client)
 
     def end_session(self):
         """Finish this session. If a transaction has started, abort it.
@@ -371,6 +474,9 @@ class ClientSession(object):
             try:
                 if self.in_transaction:
                     self.abort_transaction()
+                # It's possible we're still pinned here when the transaction
+                # is in the committed state when the session is discarded.
+                self._unpin()
             finally:
                 self._client._return_server_session(self._server_session, lock)
                 self._server_session = None
@@ -427,8 +533,14 @@ class ClientSession(object):
             return val
         return getattr(self.client, name)
 
-    def with_transaction(self, callback, read_concern=None, write_concern=None,
-                         read_preference=None, max_commit_time_ms=None):
+    def with_transaction(
+        self,
+        callback,
+        read_concern=None,
+        write_concern=None,
+        read_preference=None,
+        max_commit_time_ms=None,
+    ):
         """Execute a callback in a transaction.
 
         This method starts a transaction on this session, executes ``callback``
@@ -516,17 +628,17 @@ class ClientSession(object):
         """
         start_time = monotonic.time()
         while True:
-            self.start_transaction(
-                read_concern, write_concern, read_preference,
-                max_commit_time_ms)
+            self.start_transaction(read_concern, write_concern, read_preference, max_commit_time_ms)
             try:
                 ret = callback(self)
             except Exception as exc:
                 if self.in_transaction:
                     self.abort_transaction()
-                if (isinstance(exc, PyMongoError) and
-                        exc.has_error_label("TransientTransactionError") and
-                        _within_time_limit(start_time)):
+                if (
+                    isinstance(exc, PyMongoError)
+                    and exc.has_error_label("TransientTransactionError")
+                    and _within_time_limit(start_time)
+                ):
                     # Retry the entire transaction.
                     continue
                 raise
@@ -539,14 +651,17 @@ class ClientSession(object):
                 try:
                     self.commit_transaction()
                 except PyMongoError as exc:
-                    if (exc.has_error_label("UnknownTransactionCommitResult")
-                            and _within_time_limit(start_time)
-                            and not _max_time_expired_error(exc)):
+                    if (
+                        exc.has_error_label("UnknownTransactionCommitResult")
+                        and _within_time_limit(start_time)
+                        and not _max_time_expired_error(exc)
+                    ):
                         # Retry the commit.
                         continue
 
-                    if (exc.has_error_label("TransientTransactionError") and
-                            _within_time_limit(start_time)):
+                    if exc.has_error_label("TransientTransactionError") and _within_time_limit(
+                        start_time
+                    ):
                         # Retry the entire transaction.
                         break
                     raise
@@ -554,8 +669,9 @@ class ClientSession(object):
                 # Commit succeeded.
                 return ret
 
-    def start_transaction(self, read_concern=None, write_concern=None,
-                          read_preference=None, max_commit_time_ms=None):
+    def start_transaction(
+        self, read_concern=None, write_concern=None, read_preference=None, max_commit_time_ms=None
+    ):
         """Start a multi-statement transaction.
 
         Takes the same arguments as :class:`TransactionOptions`.
@@ -567,20 +683,23 @@ class ClientSession(object):
         """
         self._check_ended()
 
+        if self.options.snapshot:
+            raise InvalidOperation("Transactions are not supported in " "snapshot sessions")
+
         if self.in_transaction:
             raise InvalidOperation("Transaction already in progress")
 
         read_concern = self._inherit_option("read_concern", read_concern)
         write_concern = self._inherit_option("write_concern", write_concern)
-        read_preference = self._inherit_option(
-            "read_preference", read_preference)
+        read_preference = self._inherit_option("read_preference", read_preference)
         if max_commit_time_ms is None:
             opts = self.options.default_transaction_options
             if opts:
                 max_commit_time_ms = opts.max_commit_time_ms
 
         self._transaction.opts = TransactionOptions(
-            read_concern, write_concern, read_preference, max_commit_time_ms)
+            read_concern, write_concern, read_preference, max_commit_time_ms
+        )
         self._transaction.reset()
         self._transaction.state = _TxnState.STARTING
         self._start_retryable_write()
@@ -600,8 +719,7 @@ class ClientSession(object):
             self._transaction.state = _TxnState.COMMITTED_EMPTY
             return
         elif state is _TxnState.ABORTED:
-            raise InvalidOperation(
-                "Cannot call commitTransaction after calling abortTransaction")
+            raise InvalidOperation("Cannot call commitTransaction after calling abortTransaction")
         elif state is _TxnState.COMMITTED:
             # We're explicitly retrying the commit, move the state back to
             # "in progress" so that in_transaction returns true.
@@ -647,8 +765,7 @@ class ClientSession(object):
         elif state is _TxnState.ABORTED:
             raise InvalidOperation("Cannot call abortTransaction twice")
         elif state in (_TxnState.COMMITTED, _TxnState.COMMITTED_EMPTY):
-            raise InvalidOperation(
-                "Cannot call abortTransaction after calling commitTransaction")
+            raise InvalidOperation("Cannot call abortTransaction after calling commitTransaction")
 
         try:
             self._finish_transaction_with_retry("abortTransaction")
@@ -657,6 +774,7 @@ class ClientSession(object):
             pass
         finally:
             self._transaction.state = _TxnState.ABORTED
+            self._unpin()
 
     def _finish_transaction_with_retry(self, command_name):
         """Run commit or abort with one retry after any retryable error.
@@ -664,8 +782,10 @@ class ClientSession(object):
         :Parameters:
           - `command_name`: Either "commitTransaction" or "abortTransaction".
         """
+
         def func(session, sock_info, retryable):
             return self._finish_transaction(sock_info, command_name)
+
         return self._client._retry_internal(True, func, self, None)
 
     def _finish_transaction(self, sock_info, command_name):
@@ -675,7 +795,7 @@ class ClientSession(object):
         cmd = SON([(command_name, 1)])
         if command_name == "commitTransaction":
             if opts.max_commit_time_ms:
-                cmd['maxTimeMS'] = opts.max_commit_time_ms
+                cmd["maxTimeMS"] = opts.max_commit_time_ms
 
             # Transaction spec says that after the initial commit attempt,
             # subsequent commitTransaction commands should be upgraded to use
@@ -687,14 +807,11 @@ class ClientSession(object):
                 wc = WriteConcern(**wc_doc)
 
         if self._transaction.recovery_token:
-            cmd['recoveryToken'] = self._transaction.recovery_token
+            cmd["recoveryToken"] = self._transaction.recovery_token
 
         return self._client.admin._command(
-            sock_info,
-            cmd,
-            session=self,
-            write_concern=wc,
-            parse_write_concern_error=True)
+            sock_info, cmd, session=self, write_concern=wc, parse_write_concern_error=True
+        )
 
     def _advance_cluster_time(self, cluster_time):
         """Internal cluster time helper."""
@@ -713,8 +830,7 @@ class ClientSession(object):
             another `ClientSession` instance.
         """
         if not isinstance(cluster_time, abc.Mapping):
-            raise TypeError(
-                "cluster_time must be a subclass of collections.Mapping")
+            raise TypeError("cluster_time must be a subclass of collections.Mapping")
         if not isinstance(cluster_time.get("clusterTime"), Timestamp):
             raise ValueError("Invalid cluster_time")
         self._advance_cluster_time(cluster_time)
@@ -736,16 +852,21 @@ class ClientSession(object):
             another `ClientSession` instance.
         """
         if not isinstance(operation_time, Timestamp):
-            raise TypeError("operation_time must be an instance "
-                            "of bson.timestamp.Timestamp")
+            raise TypeError("operation_time must be an instance " "of bson.timestamp.Timestamp")
         self._advance_operation_time(operation_time)
 
     def _process_response(self, reply):
         """Process a response to a command that was run with this session."""
-        self._advance_cluster_time(reply.get('$clusterTime'))
-        self._advance_operation_time(reply.get('operationTime'))
+        self._advance_cluster_time(reply.get("$clusterTime"))
+        self._advance_operation_time(reply.get("operationTime"))
+        if self._options.snapshot and self._snapshot_time is None:
+            if "cursor" in reply:
+                ct = reply["cursor"].get("atClusterTime")
+            else:
+                ct = reply.get("atClusterTime")
+            self._snapshot_time = ct
         if self.in_transaction and self._transaction.sharded:
-            recovery_token = reply.get('recoveryToken')
+            recovery_token = reply.get("recoveryToken")
             if recovery_token:
                 self._transaction.recovery_token = recovery_token
 
@@ -762,6 +883,11 @@ class ClientSession(object):
         """
         return self._transaction.active()
 
+    @property
+    def _starting_transaction(self):
+        """True if this session is starting a multi-statement transaction."""
+        return self._transaction.starting()
+
     @property
     def _pinned_address(self):
         """The mongos address this transaction was created on."""
@@ -769,14 +895,18 @@ class ClientSession(object):
             return self._transaction.pinned_address
         return None
 
-    def _pin_mongos(self, server):
-        """Pin this session to the given mongos Server."""
-        self._transaction.sharded = True
-        self._transaction.pinned_address = server.description.address
+    @property
+    def _pinned_connection(self):
+        """The connection this transaction was started on."""
+        return self._transaction.pinned_conn
 
-    def _unpin_mongos(self):
-        """Unpin this session from any pinned mongos address."""
-        self._transaction.pinned_address = None
+    def _pin(self, server, sock_info):
+        """Pin this session to the given Server or to the given connection."""
+        self._transaction.pin(server, sock_info)
+
+    def _unpin(self):
+        """Unpin this session from any pinned Server."""
+        self._transaction.unpin()
 
     def _txn_read_preference(self):
         """Return read preference of this transaction or None."""
@@ -784,54 +914,60 @@ class ClientSession(object):
             return self._transaction.opts.read_preference
         return None
 
-    def _apply_to(self, command, is_retryable, read_preference):
+    def _apply_to(self, command, is_retryable, read_preference, sock_info):
         self._check_ended()
 
-        self._server_session.last_use = monotonic.time()
-        command['lsid'] = self._server_session.session_id
+        if self.options.snapshot:
+            self._update_read_concern(command, sock_info)
 
-        if not self.in_transaction:
-            self._transaction.reset()
+        self._server_session.last_use = monotonic.time()
+        command["lsid"] = self._server_session.session_id
 
         if is_retryable:
-            command['txnNumber'] = self._server_session.transaction_id
+            command["txnNumber"] = self._server_session.transaction_id
             return
 
         if self.in_transaction:
             if read_preference != ReadPreference.PRIMARY:
                 raise InvalidOperation(
-                    'read preference in a transaction must be primary, not: '
-                    '%r' % (read_preference,))
+                    "read preference in a transaction must be primary, not: "
+                    "%r" % (read_preference,)
+                )
 
             if self._transaction.state == _TxnState.STARTING:
                 # First command begins a new transaction.
                 self._transaction.state = _TxnState.IN_PROGRESS
-                command['startTransaction'] = True
+                command["startTransaction"] = True
 
                 if self._transaction.opts.read_concern:
                     rc = self._transaction.opts.read_concern.document
-                else:
-                    rc = {}
+                    if rc:
+                        command["readConcern"] = rc
+                self._update_read_concern(command, sock_info)
 
-                if (self.options.causal_consistency
-                        and self.operation_time is not None):
-                    rc['afterClusterTime'] = self.operation_time
-
-                if rc:
-                    command['readConcern'] = rc
-
-            command['txnNumber'] = self._server_session.transaction_id
-            command['autocommit'] = False
+            command["txnNumber"] = self._server_session.transaction_id
+            command["autocommit"] = False
 
     def _start_retryable_write(self):
         self._check_ended()
         self._server_session.inc_transaction_id()
 
+    def _update_read_concern(self, cmd, sock_info):
+        if self.options.causal_consistency and self.operation_time is not None:
+            cmd.setdefault("readConcern", {})["afterClusterTime"] = self.operation_time
+        if self.options.snapshot:
+            if sock_info.max_wire_version < 13:
+                raise ConfigurationError("Snapshot reads require MongoDB 5.0 or later")
+            rc = cmd.setdefault("readConcern", {})
+            rc["level"] = "snapshot"
+            if self._snapshot_time is not None:
+                rc["atClusterTime"] = self._snapshot_time
+
 
 class _ServerSession(object):
     def __init__(self, generation):
         # Ensure id is type 4, regardless of CodecOptions.uuid_representation.
-        self.session_id = {'id': Binary(uuid.uuid4().bytes, 4)}
+        self.session_id = {"id": Binary(uuid.uuid4().bytes, 4)}
         self.last_use = monotonic.time()
         self._transaction_id = 0
         self.dirty = False
@@ -865,6 +1001,7 @@ class _ServerSessionPool(collections.deque):
 
     This class is not thread-safe, access it while holding the Topology lock.
     """
+
     def __init__(self, *args, **kwargs):
         super(_ServerSessionPool, self).__init__(*args, **kwargs)
         self.generation = 0
@@ -896,15 +1033,16 @@ class _ServerSessionPool(collections.deque):
         return _ServerSession(self.generation)
 
     def return_server_session(self, server_session, session_timeout_minutes):
-        self._clear_stale(session_timeout_minutes)
-        if not server_session.timed_out(session_timeout_minutes):
-            self.return_server_session_no_lock(server_session)
+        if session_timeout_minutes is not None:
+            self._clear_stale(session_timeout_minutes)
+            if server_session.timed_out(session_timeout_minutes):
+                return
+        self.return_server_session_no_lock(server_session)
 
     def return_server_session_no_lock(self, server_session):
         # Discard sessions from an old pool to avoid duplicate sessions in the
         # child process after a fork.
-        if (server_session.generation == self.generation and
-                not server_session.dirty):
+        if server_session.generation == self.generation and not server_session.dirty:
             self.appendleft(server_session)
 
     def _clear_stale(self, session_timeout_minutes):
