@@ -21,23 +21,26 @@ import warnings
 from bson import SON
 from bson.binary import UuidRepresentation
 from bson.codec_options import CodecOptions, TypeRegistry
-from bson.py3compat import abc, integer_types, iteritems, string_type, PY3
+from bson.py3compat import PY3, abc, integer_types, iteritems, string_type
 from bson.raw_bson import RawBSONDocument
 from pymongo.auth import MECHANISMS
-from pymongo.compression_support import (validate_compressors,
-                                         validate_zlib_compression_level)
+from pymongo.compression_support import (
+    validate_compressors,
+    validate_zlib_compression_level,
+)
 from pymongo.driver_info import DriverInfo
 from pymongo.encryption_options import validate_auto_encryption_opts_or_none
 from pymongo.errors import ConfigurationError
 from pymongo.monitoring import _validate_event_listeners
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import _MONGOS_MODES, _ServerMode
-from pymongo.ssl_support import (validate_cert_reqs,
-                                 validate_allow_invalid_certs)
+from pymongo.server_api import ServerApi
+from pymongo.ssl_support import validate_allow_invalid_certs, validate_cert_reqs
 from pymongo.write_concern import DEFAULT_WRITE_CONCERN, WriteConcern
 
 try:
     from collections import OrderedDict
+
     ORDERED_TYPES = (SON, OrderedDict)
 except ImportError:
     ORDERED_TYPES = (SON,)
@@ -57,9 +60,9 @@ MAX_WRITE_BATCH_SIZE = 1000
 # What this version of PyMongo supports.
 MIN_SUPPORTED_SERVER_VERSION = "2.6"
 MIN_SUPPORTED_WIRE_VERSION = 2
-MAX_SUPPORTED_WIRE_VERSION = 9
+MAX_SUPPORTED_WIRE_VERSION = 13
 
-# Frequency to call ismaster on servers, in seconds.
+# Frequency to call hello on servers, in seconds.
 HEARTBEAT_FREQUENCY = 10
 
 # Frequency to process kill-cursors, in seconds. See MongoClient.close_cursor.
@@ -74,7 +77,7 @@ EVENTS_QUEUE_FREQUENCY = 1
 # longest it is willing to wait for a new primary to be found.
 SERVER_SELECTION_TIMEOUT = 30
 
-# Spec requires at least 500ms between ismaster calls.
+# Spec requires at least 500ms between hello calls.
 MIN_HEARTBEAT_INTERVAL = 0.5
 
 # Spec requires at least 60s between SRV rescans.
@@ -122,22 +125,22 @@ def partition_node(node):
     """Split a host:port string into (host, int(port)) pair."""
     host = node
     port = 27017
-    idx = node.rfind(':')
+    idx = node.rfind(":")
     if idx != -1:
-        host, port = node[:idx], int(node[idx + 1:])
-    if host.startswith('['):
+        host, port = node[:idx], int(node[idx + 1 :])
+    if host.startswith("["):
         host = host[1:-1]
     return host, port
 
 
 def clean_node(node):
-    """Split and normalize a node name from an ismaster response."""
+    """Split and normalize a node name from a hello response."""
     host, port = partition_node(node)
 
     # Normalize hostname to lowercase, since DNS is case-insensitive:
     # http://tools.ietf.org/html/rfc4343
     # This prevents useless rediscovery if "foo.com" is in the seed list but
-    # "FOO.com" is in the ismaster response.
+    # "FOO.com" is in the hello response.
     return host.lower(), port
 
 
@@ -148,11 +151,11 @@ def raise_config_error(key, dummy):
 
 # Mapping of URI uuid representation options to valid subtypes.
 _UUID_REPRESENTATIONS = {
-    'unspecified': UuidRepresentation.UNSPECIFIED,
-    'standard': UuidRepresentation.STANDARD,
-    'pythonLegacy': UuidRepresentation.PYTHON_LEGACY,
-    'javaLegacy': UuidRepresentation.JAVA_LEGACY,
-    'csharpLegacy': UuidRepresentation.CSHARP_LEGACY
+    "unspecified": UuidRepresentation.UNSPECIFIED,
+    "standard": UuidRepresentation.STANDARD,
+    "pythonLegacy": UuidRepresentation.PYTHON_LEGACY,
+    "javaLegacy": UuidRepresentation.JAVA_LEGACY,
+    "csharpLegacy": UuidRepresentation.CSHARP_LEGACY,
 }
 
 
@@ -166,70 +169,60 @@ def validate_boolean(option, value):
 def validate_boolean_or_string(option, value):
     """Validates that value is True, False, 'true', or 'false'."""
     if isinstance(value, string_type):
-        if value not in ('true', 'false'):
-            raise ValueError("The value of %s must be "
-                             "'true' or 'false'" % (option,))
-        return value == 'true'
+        if value not in ("true", "false"):
+            raise ValueError("The value of %s must be " "'true' or 'false'" % (option,))
+        return value == "true"
     return validate_boolean(option, value)
 
 
 def validate_integer(option, value):
-    """Validates that 'value' is an integer (or basestring representation).
-    """
+    """Validates that 'value' is an integer (or basestring representation)."""
     if isinstance(value, integer_types):
         return value
     elif isinstance(value, string_type):
         try:
             return int(value)
         except ValueError:
-            raise ValueError("The value of %s must be "
-                             "an integer" % (option,))
+            raise ValueError("The value of %s must be " "an integer" % (option,))
     raise TypeError("Wrong type for %s, value must be an integer" % (option,))
 
 
 def validate_positive_integer(option, value):
-    """Validate that 'value' is a positive integer, which does not include 0.
-    """
+    """Validate that 'value' is a positive integer, which does not include 0."""
     val = validate_integer(option, value)
     if val <= 0:
-        raise ValueError("The value of %s must be "
-                         "a positive integer" % (option,))
+        raise ValueError("The value of %s must be " "a positive integer" % (option,))
     return val
 
 
 def validate_non_negative_integer(option, value):
-    """Validate that 'value' is a positive integer or 0.
-    """
+    """Validate that 'value' is a positive integer or 0."""
     val = validate_integer(option, value)
     if val < 0:
-        raise ValueError("The value of %s must be "
-                         "a non negative integer" % (option,))
+        raise ValueError("The value of %s must be " "a non negative integer" % (option,))
     return val
 
 
 def validate_readable(option, value):
-    """Validates that 'value' is file-like and readable.
-    """
+    """Validates that 'value' is file-like and readable."""
     if value is None:
         return value
     # First make sure its a string py3.3 open(True, 'r') succeeds
     # Used in ssl cert checking due to poor ssl module error reporting
     value = validate_string(option, value)
-    open(value, 'r').close()
+    open(value, "r").close()
     return value
 
 
 def validate_positive_integer_or_none(option, value):
-    """Validate that 'value' is a positive integer or None.
-    """
+    """Validate that 'value' is a positive integer or None."""
     if value is None:
         return value
     return validate_positive_integer(option, value)
 
 
 def validate_non_negative_integer_or_none(option, value):
-    """Validate that 'value' is a positive integer or 0 or None.
-    """
+    """Validate that 'value' is a positive integer or 0 or None."""
     if value is None:
         return value
     return validate_non_negative_integer(option, value)
@@ -241,21 +234,20 @@ def validate_string(option, value):
     """
     if isinstance(value, string_type):
         return value
-    raise TypeError("Wrong type for %s, value must be "
-                    "an instance of %s" % (option, string_type.__name__))
+    raise TypeError(
+        "Wrong type for %s, value must be " "an instance of %s" % (option, string_type.__name__)
+    )
 
 
 def validate_string_or_none(option, value):
-    """Validates that 'value' is an instance of `basestring` or `None`.
-    """
+    """Validates that 'value' is an instance of `basestring` or `None`."""
     if value is None:
         return value
     return validate_string(option, value)
 
 
 def validate_int_or_basestring(option, value):
-    """Validates that 'value' is an integer or string.
-    """
+    """Validates that 'value' is an integer or string."""
     if isinstance(value, integer_types):
         return value
     elif isinstance(value, string_type):
@@ -263,13 +255,11 @@ def validate_int_or_basestring(option, value):
             return int(value)
         except ValueError:
             return value
-    raise TypeError("Wrong type for %s, value must be an "
-                    "integer or a string" % (option,))
+    raise TypeError("Wrong type for %s, value must be an " "integer or a string" % (option,))
 
 
 def validate_non_negative_int_or_basestring(option, value):
-    """Validates that 'value' is an integer or string.
-    """
+    """Validates that 'value' is an integer or string."""
     if isinstance(value, integer_types):
         return value
     elif isinstance(value, string_type):
@@ -278,13 +268,14 @@ def validate_non_negative_int_or_basestring(option, value):
         except ValueError:
             return value
         return validate_non_negative_integer(option, val)
-    raise TypeError("Wrong type for %s, value must be an "
-                    "non negative integer or a string" % (option,))
+    raise TypeError(
+        "Wrong type for %s, value must be an " "non negative integer or a string" % (option,)
+    )
 
 
 def validate_positive_float(option, value):
     """Validates that 'value' is a float, or can be converted to one, and is
-       positive.
+    positive.
     """
     errmsg = "%s must be an integer or float" % (option,)
     try:
@@ -297,8 +288,7 @@ def validate_positive_float(option, value):
     # float('inf') doesn't work in 2.4 or 2.5 on Windows, so just cap floats at
     # one billion - this is a reasonable approximation for infinity
     if not 0 < value < 1e9:
-        raise ValueError("%s must be greater than 0 and "
-                         "less than one billion" % (option,))
+        raise ValueError("%s must be greater than 0 and " "less than one billion" % (option,))
     return value
 
 
@@ -327,7 +317,7 @@ def validate_timeout_or_zero(option, value):
     config error.
     """
     if value is None:
-        raise ConfigurationError("%s cannot be None" % (option, ))
+        raise ConfigurationError("%s cannot be None" % (option,))
     if value == 0 or value == "0":
         return 0
     return validate_positive_float(option, value) / 1000.0
@@ -352,8 +342,7 @@ def validate_max_staleness(option, value):
 
 
 def validate_read_preference(dummy, value):
-    """Validate a read preference.
-    """
+    """Validate a read preference."""
     if not isinstance(value, _ServerMode):
         raise TypeError("%r is not a read preference." % (value,))
     return value
@@ -372,36 +361,35 @@ def validate_read_preference_mode(dummy, value):
 
 
 def validate_auth_mechanism(option, value):
-    """Validate the authMechanism URI option.
-    """
+    """Validate the authMechanism URI option."""
     # CRAM-MD5 is for server testing only. Undocumented,
     # unsupported, may be removed at any time. You have
     # been warned.
-    if value not in MECHANISMS and value != 'CRAM-MD5':
+    if value not in MECHANISMS and value != "CRAM-MD5":
         raise ValueError("%s must be in %s" % (option, tuple(MECHANISMS)))
     return value
 
 
 def validate_uuid_representation(dummy, value):
-    """Validate the uuid representation option selected in the URI.
-    """
+    """Validate the uuid representation option selected in the URI."""
     try:
         return _UUID_REPRESENTATIONS[value]
     except KeyError:
-        raise ValueError("%s is an invalid UUID representation. "
-                         "Must be one of "
-                         "%s" % (value, tuple(_UUID_REPRESENTATIONS)))
+        raise ValueError(
+            "%s is an invalid UUID representation. "
+            "Must be one of "
+            "%s" % (value, tuple(_UUID_REPRESENTATIONS))
+        )
 
 
 def validate_read_preference_tags(name, value):
-    """Parse readPreferenceTags if passed as a client kwarg.
-    """
+    """Parse readPreferenceTags if passed as a client kwarg."""
     if not isinstance(value, list):
         value = [value]
 
     tag_sets = []
     for tag_set in value:
-        if tag_set == '':
+        if tag_set == "":
             tag_sets.append({})
             continue
         try:
@@ -411,37 +399,41 @@ def validate_read_preference_tags(name, value):
                 tags[unquote_plus(key)] = unquote_plus(val)
             tag_sets.append(tags)
         except Exception:
-            raise ValueError("%r not a valid "
-                             "value for %s" % (tag_set, name))
+            raise ValueError("%r not a valid " "value for %s" % (tag_set, name))
     return tag_sets
 
 
-_MECHANISM_PROPS = frozenset(['SERVICE_NAME',
-                              'CANONICALIZE_HOST_NAME',
-                              'SERVICE_REALM',
-                              'AWS_SESSION_TOKEN'])
+_MECHANISM_PROPS = frozenset(
+    ["SERVICE_NAME", "CANONICALIZE_HOST_NAME", "SERVICE_REALM", "AWS_SESSION_TOKEN"]
+)
 
 
 def validate_auth_mechanism_properties(option, value):
     """Validate authMechanismProperties."""
     value = validate_string(option, value)
     props = {}
-    for opt in value.split(','):
+    for opt in value.split(","):
         try:
-            key, val = opt.split(':')
+            key, val = opt.split(":")
         except ValueError:
             # Try not to leak the token.
-            if 'AWS_SESSION_TOKEN' in opt:
-                opt = ('AWS_SESSION_TOKEN:<redacted token>, did you forget '
-                       'to percent-escape the token with quote_plus?')
-            raise ValueError("auth mechanism properties must be "
-                             "key:value pairs like SERVICE_NAME:"
-                             "mongodb, not %s." % (opt,))
+            if "AWS_SESSION_TOKEN" in opt:
+                opt = (
+                    "AWS_SESSION_TOKEN:<redacted token>, did you forget "
+                    "to percent-escape the token with quote_plus?"
+                )
+            raise ValueError(
+                "auth mechanism properties must be "
+                "key:value pairs like SERVICE_NAME:"
+                "mongodb, not %s." % (opt,)
+            )
         if key not in _MECHANISM_PROPS:
-            raise ValueError("%s is not a supported auth "
-                             "mechanism property. Must be one of "
-                             "%s." % (key, tuple(_MECHANISM_PROPS)))
-        if key == 'CANONICALIZE_HOST_NAME':
+            raise ValueError(
+                "%s is not a supported auth "
+                "mechanism property. Must be one of "
+                "%s." % (key, tuple(_MECHANISM_PROPS))
+            )
+        if key == "CANONICALIZE_HOST_NAME":
             props[key] = validate_boolean_or_string(key, val)
         else:
             props[key] = unquote_plus(val)
@@ -452,17 +444,18 @@ def validate_auth_mechanism_properties(option, value):
 def validate_document_class(option, value):
     """Validate the document_class option."""
     if not issubclass(value, (abc.MutableMapping, RawBSONDocument)):
-        raise TypeError("%s must be dict, bson.son.SON, "
-                        "bson.raw_bson.RawBSONDocument, or a "
-                        "sublass of collections.MutableMapping" % (option,))
+        raise TypeError(
+            "%s must be dict, bson.son.SON, "
+            "bson.raw_bson.RawBSONDocument, or a "
+            "sublass of collections.MutableMapping" % (option,)
+        )
     return value
 
 
 def validate_type_registry(option, value):
     """Validate the type_registry option."""
     if value is not None and not isinstance(value, TypeRegistry):
-        raise TypeError("%s must be an instance of %s" % (
-            option, TypeRegistry))
+        raise TypeError("%s must be an instance of %s" % (option, TypeRegistry))
     return value
 
 
@@ -483,26 +476,32 @@ def validate_list_or_none(option, value):
 def validate_list_or_mapping(option, value):
     """Validates that 'value' is a list or a document."""
     if not isinstance(value, (abc.Mapping, list)):
-        raise TypeError("%s must either be a list or an instance of dict, "
-                        "bson.son.SON, or any other type that inherits from "
-                        "collections.Mapping" % (option,))
+        raise TypeError(
+            "%s must either be a list or an instance of dict, "
+            "bson.son.SON, or any other type that inherits from "
+            "collections.Mapping" % (option,)
+        )
 
 
 def validate_is_mapping(option, value):
     """Validate the type of method arguments that expect a document."""
     if not isinstance(value, abc.Mapping):
-        raise TypeError("%s must be an instance of dict, bson.son.SON, or "
-                        "any other type that inherits from "
-                        "collections.Mapping" % (option,))
+        raise TypeError(
+            "%s must be an instance of dict, bson.son.SON, or "
+            "any other type that inherits from "
+            "collections.Mapping" % (option,)
+        )
 
 
 def validate_is_document_type(option, value):
     """Validate the type of method arguments that expect a MongoDB document."""
     if not isinstance(value, (abc.MutableMapping, RawBSONDocument)):
-        raise TypeError("%s must be an instance of dict, bson.son.SON, "
-                        "bson.raw_bson.RawBSONDocument, or "
-                        "a type that inherits from "
-                        "collections.MutableMapping" % (option,))
+        raise TypeError(
+            "%s must be an instance of dict, bson.son.SON, "
+            "bson.raw_bson.RawBSONDocument, or "
+            "a type that inherits from "
+            "collections.MutableMapping" % (option,)
+        )
 
 
 def validate_appname_or_none(option, value):
@@ -511,7 +510,7 @@ def validate_appname_or_none(option, value):
         return value
     validate_string(option, value)
     # We need length in bytes, so encode utf8 first.
-    if len(value.encode('utf-8')) > 128:
+    if len(value.encode("utf-8")) > 128:
         raise ValueError("%s must be <= 128 bytes" % (option,))
     return value
 
@@ -525,6 +524,15 @@ def validate_driver_or_none(option, value):
     return value
 
 
+def validate_server_api_or_none(option, value):
+    """Validate the server_api keyword arg."""
+    if value is None:
+        return value
+    if not isinstance(value, ServerApi):
+        raise TypeError("%s must be an instance of ServerApi" % (option,))
+    return value
+
+
 def validate_is_callable_or_none(option, value):
     """Validates that 'value' is a callable."""
     if value is None:
@@ -540,8 +548,8 @@ def validate_ok_for_replace(replacement):
     # Replacement can be {}
     if replacement and not isinstance(replacement, RawBSONDocument):
         first = next(iter(replacement))
-        if first.startswith('$'):
-            raise ValueError('replacement can not include $ operators')
+        if first.startswith("$"):
+            raise ValueError("replacement can not include $ operators")
 
 
 def validate_ok_for_update(update):
@@ -549,30 +557,30 @@ def validate_ok_for_update(update):
     validate_list_or_mapping("update", update)
     # Update cannot be {}.
     if not update:
-        raise ValueError('update cannot be empty')
+        raise ValueError("update cannot be empty")
 
     is_document = not isinstance(update, list)
     first = next(iter(update))
-    if is_document and not first.startswith('$'):
-        raise ValueError('update only works with $ operators')
+    if is_document and not first.startswith("$"):
+        raise ValueError("update only works with $ operators")
 
 
-_UNICODE_DECODE_ERROR_HANDLERS = frozenset(['strict', 'replace', 'ignore'])
+_UNICODE_DECODE_ERROR_HANDLERS = frozenset(["strict", "replace", "ignore"])
 
 
 def validate_unicode_decode_error_handler(dummy, value):
-    """Validate the Unicode decode error handler option of CodecOptions.
-    """
+    """Validate the Unicode decode error handler option of CodecOptions."""
     if value not in _UNICODE_DECODE_ERROR_HANDLERS:
-        raise ValueError("%s is an invalid Unicode decode error handler. "
-                         "Must be one of "
-                         "%s" % (value, tuple(_UNICODE_DECODE_ERROR_HANDLERS)))
+        raise ValueError(
+            "%s is an invalid Unicode decode error handler. "
+            "Must be one of "
+            "%s" % (value, tuple(_UNICODE_DECODE_ERROR_HANDLERS))
+        )
     return value
 
 
 def validate_tzinfo(dummy, value):
-    """Validate the tzinfo option
-    """
+    """Validate the tzinfo option"""
     if value is not None and not isinstance(value, datetime.tzinfo):
         raise TypeError("%s must be an instance of datetime.tzinfo" % value)
     return value
@@ -582,15 +590,15 @@ def validate_tzinfo(dummy, value):
 # are lists of aliases for that option. Aliases of option names are assumed
 # to have been deprecated.
 URI_OPTIONS_ALIAS_MAP = {
-    'journal': ['j'],
-    'wtimeoutms': ['wtimeout'],
-    'tls': ['ssl'],
-    'tlsallowinvalidcertificates': ['ssl_cert_reqs'],
-    'tlsallowinvalidhostnames': ['ssl_match_hostname'],
-    'tlscrlfile': ['ssl_crlfile'],
-    'tlscafile': ['ssl_ca_certs'],
-    'tlscertificatekeyfile': ['ssl_certfile'],
-    'tlscertificatekeyfilepassword': ['ssl_pem_passphrase'],
+    "journal": ["j"],
+    "wtimeoutms": ["wtimeout"],
+    "tls": ["ssl"],
+    "tlsallowinvalidcertificates": ["ssl_cert_reqs"],
+    "tlsallowinvalidhostnames": ["ssl_match_hostname"],
+    "tlscrlfile": ["ssl_crlfile"],
+    "tlscafile": ["ssl_ca_certs"],
+    "tlscertificatekeyfile": ["ssl_certfile"],
+    "tlscertificatekeyfilepassword": ["ssl_pem_passphrase"],
 }
 
 # Dictionary where keys are the names of URI options, and values
@@ -598,72 +606,76 @@ URI_OPTIONS_ALIAS_MAP = {
 # alias uses a different validator than its public counterpart, it should be
 # included here as a key, value pair.
 URI_OPTIONS_VALIDATOR_MAP = {
-    'appname': validate_appname_or_none,
-    'authmechanism': validate_auth_mechanism,
-    'authmechanismproperties': validate_auth_mechanism_properties,
-    'authsource': validate_string,
-    'compressors': validate_compressors,
-    'connecttimeoutms': validate_timeout_or_none_or_zero,
-    'directconnection': validate_boolean_or_string,
-    'heartbeatfrequencyms': validate_timeout_or_none,
-    'journal': validate_boolean_or_string,
-    'localthresholdms': validate_positive_float_or_zero,
-    'maxidletimems': validate_timeout_or_none,
-    'maxpoolsize': validate_positive_integer_or_none,
-    'maxstalenessseconds': validate_max_staleness,
-    'readconcernlevel': validate_string_or_none,
-    'readpreference': validate_read_preference_mode,
-    'readpreferencetags': validate_read_preference_tags,
-    'replicaset': validate_string_or_none,
-    'retryreads': validate_boolean_or_string,
-    'retrywrites': validate_boolean_or_string,
-    'serverselectiontimeoutms': validate_timeout_or_zero,
-    'sockettimeoutms': validate_timeout_or_none_or_zero,
-    'ssl_keyfile': validate_readable,
-    'tls': validate_boolean_or_string,
-    'tlsallowinvalidcertificates': validate_allow_invalid_certs,
-    'ssl_cert_reqs': validate_cert_reqs,
-    'tlsallowinvalidhostnames': lambda *x: not validate_boolean_or_string(*x),
-    'ssl_match_hostname': validate_boolean_or_string,
-    'tlscafile': validate_readable,
-    'tlscertificatekeyfile': validate_readable,
-    'tlscertificatekeyfilepassword': validate_string_or_none,
-    'tlsdisableocspendpointcheck': validate_boolean_or_string,
-    'tlsinsecure': validate_boolean_or_string,
-    'w': validate_non_negative_int_or_basestring,
-    'wtimeoutms': validate_non_negative_integer,
-    'zlibcompressionlevel': validate_zlib_compression_level,
+    "appname": validate_appname_or_none,
+    "authmechanism": validate_auth_mechanism,
+    "authmechanismproperties": validate_auth_mechanism_properties,
+    "authsource": validate_string,
+    "compressors": validate_compressors,
+    "connecttimeoutms": validate_timeout_or_none_or_zero,
+    "directconnection": validate_boolean_or_string,
+    "heartbeatfrequencyms": validate_timeout_or_none,
+    "journal": validate_boolean_or_string,
+    "localthresholdms": validate_positive_float_or_zero,
+    "maxidletimems": validate_timeout_or_none,
+    "maxpoolsize": validate_positive_integer_or_none,
+    "maxstalenessseconds": validate_max_staleness,
+    "readconcernlevel": validate_string_or_none,
+    "readpreference": validate_read_preference_mode,
+    "readpreferencetags": validate_read_preference_tags,
+    "replicaset": validate_string_or_none,
+    "retryreads": validate_boolean_or_string,
+    "retrywrites": validate_boolean_or_string,
+    "loadbalanced": validate_boolean_or_string,
+    "serverselectiontimeoutms": validate_timeout_or_zero,
+    "sockettimeoutms": validate_timeout_or_none_or_zero,
+    "ssl_keyfile": validate_readable,
+    "tls": validate_boolean_or_string,
+    "tlsallowinvalidcertificates": validate_allow_invalid_certs,
+    "ssl_cert_reqs": validate_cert_reqs,
+    # Normalized to ssl_match_hostname which is the logical inverse of tlsallowinvalidhostnames
+    "tlsallowinvalidhostnames": lambda *x: not validate_boolean_or_string(*x),
+    "ssl_match_hostname": validate_boolean_or_string,
+    "tlscafile": validate_readable,
+    "tlscertificatekeyfile": validate_readable,
+    "tlscertificatekeyfilepassword": validate_string_or_none,
+    # Normalized to ssl_check_ocsp_endpoint which is the logical inverse of tlsdisableocspendpointcheck
+    "tlsdisableocspendpointcheck": lambda *x: not validate_boolean_or_string(*x),
+    "tlsinsecure": validate_boolean_or_string,
+    "w": validate_non_negative_int_or_basestring,
+    "wtimeoutms": validate_non_negative_integer,
+    "zlibcompressionlevel": validate_zlib_compression_level,
 }
 
 # Dictionary where keys are the names of URI options specific to pymongo,
 # and values are functions that validate user-input values for those options.
 NONSPEC_OPTIONS_VALIDATOR_MAP = {
-    'connect': validate_boolean_or_string,
-    'driver': validate_driver_or_none,
-    'fsync': validate_boolean_or_string,
-    'minpoolsize': validate_non_negative_integer,
-    'socketkeepalive': validate_boolean_or_string,
-    'tlscrlfile': validate_readable,
-    'tz_aware': validate_boolean_or_string,
-    'unicode_decode_error_handler': validate_unicode_decode_error_handler,
-    'uuidrepresentation': validate_uuid_representation,
-    'waitqueuemultiple': validate_non_negative_integer_or_none,
-    'waitqueuetimeoutms': validate_timeout_or_none,
+    "connect": validate_boolean_or_string,
+    "driver": validate_driver_or_none,
+    "server_api": validate_server_api_or_none,
+    "fsync": validate_boolean_or_string,
+    "minpoolsize": validate_non_negative_integer,
+    "socketkeepalive": validate_boolean_or_string,
+    "tlscrlfile": validate_readable,
+    "tz_aware": validate_boolean_or_string,
+    "unicode_decode_error_handler": validate_unicode_decode_error_handler,
+    "uuidrepresentation": validate_uuid_representation,
+    "waitqueuemultiple": validate_non_negative_integer_or_none,
+    "waitqueuetimeoutms": validate_timeout_or_none,
 }
 
 # Dictionary where keys are the names of keyword-only options for the
 # MongoClient constructor, and values are functions that validate user-input
 # values for those options.
 KW_VALIDATORS = {
-    'document_class': validate_document_class,
-    'type_registry': validate_type_registry,
-    'read_preference': validate_read_preference,
-    'event_listeners': _validate_event_listeners,
-    'tzinfo': validate_tzinfo,
-    'username': validate_string_or_none,
-    'password': validate_string_or_none,
-    'server_selector': validate_is_callable_or_none,
-    'auto_encryption_opts': validate_auto_encryption_opts_or_none,
+    "document_class": validate_document_class,
+    "type_registry": validate_type_registry,
+    "read_preference": validate_read_preference,
+    "event_listeners": _validate_event_listeners,
+    "tzinfo": validate_tzinfo,
+    "username": validate_string_or_none,
+    "password": validate_string_or_none,
+    "server_selector": validate_is_callable_or_none,
+    "auto_encryption_opts": validate_auto_encryption_opts_or_none,
 }
 
 # Dictionary where keys are any URI option name, and values are the
@@ -671,16 +683,16 @@ KW_VALIDATORS = {
 # variant need not be included here. Options whose public and internal
 # names are the same need not be included here.
 INTERNAL_URI_OPTION_NAME_MAP = {
-    'j': 'journal',
-    'wtimeout': 'wtimeoutms',
-    'tls': 'ssl',
-    'tlsallowinvalidcertificates': 'ssl_cert_reqs',
-    'tlsallowinvalidhostnames': 'ssl_match_hostname',
-    'tlscrlfile': 'ssl_crlfile',
-    'tlscafile': 'ssl_ca_certs',
-    'tlscertificatekeyfile': 'ssl_certfile',
-    'tlscertificatekeyfilepassword': 'ssl_pem_passphrase',
-    'tlsdisableocspendpointcheck': 'ssl_check_ocsp_endpoint',
+    "j": "journal",
+    "wtimeout": "wtimeoutms",
+    "tls": "ssl",
+    "tlsallowinvalidcertificates": "ssl_cert_reqs",
+    "tlsallowinvalidhostnames": "ssl_match_hostname",
+    "tlscrlfile": "ssl_crlfile",
+    "tlscafile": "ssl_ca_certs",
+    "tlscertificatekeyfile": "ssl_certfile",
+    "tlscertificatekeyfilepassword": "ssl_pem_passphrase",
+    "tlsdisableocspendpointcheck": "ssl_check_ocsp_endpoint",
 }
 
 # Map from deprecated URI option names to a tuple indicating the method of
@@ -693,16 +705,36 @@ URI_OPTIONS_DEPRECATION_MAP = {
     #   preserved for renamed options as they are part of user warnings.
     # - 'removed': <message> may suggest the rationale for deprecating the
     #   option and/or recommend remedial action.
-    'j': ('renamed', 'journal'),
-    'wtimeout': ('renamed', 'wTimeoutMS'),
-    'ssl_cert_reqs': ('renamed', 'tlsAllowInvalidCertificates'),
-    'ssl_match_hostname': ('renamed', 'tlsAllowInvalidHostnames'),
-    'ssl_crlfile': ('renamed', 'tlsCRLFile'),
-    'ssl_ca_certs': ('renamed', 'tlsCAFile'),
-    'ssl_pem_passphrase': ('renamed', 'tlsCertificateKeyFilePassword'),
-    'waitqueuemultiple': ('removed', (
-        'Instead of using waitQueueMultiple to bound queuing, limit the size '
-        'of the thread pool in your application server'))
+    "j": ("renamed", "journal"),
+    "wtimeout": ("renamed", "wTimeoutMS"),
+    "ssl_cert_reqs": ("renamed", "tlsAllowInvalidCertificates"),
+    "ssl_match_hostname": ("renamed", "tlsAllowInvalidHostnames"),
+    "ssl_crlfile": ("renamed", "tlsCRLFile"),
+    "ssl_ca_certs": ("renamed", "tlsCAFile"),
+    "ssl_certfile": (
+        "removed",
+        (
+            "Instead of using ssl_certfile to specify the certificate file, "
+            "use tlsCertificateKeyFile to pass a single file containing both "
+            "the client certificate and the private key"
+        ),
+    ),
+    "ssl_keyfile": (
+        "removed",
+        (
+            "Instead of using ssl_keyfile to specify the private keyfile, "
+            "use tlsCertificateKeyFile to pass a single file containing both "
+            "the client certificate and the private key"
+        ),
+    ),
+    "ssl_pem_passphrase": ("renamed", "tlsCertificateKeyFilePassword"),
+    "waitqueuemultiple": (
+        "removed",
+        (
+            "Instead of using waitQueueMultiple to bound queuing, limit the size "
+            "of the thread pool in your application server"
+        ),
+    ),
 }
 
 # Augment the option validator map with pymongo-specific option information.
@@ -710,8 +742,7 @@ URI_OPTIONS_VALIDATOR_MAP.update(NONSPEC_OPTIONS_VALIDATOR_MAP)
 for optname, aliases in iteritems(URI_OPTIONS_ALIAS_MAP):
     for alias in aliases:
         if alias not in URI_OPTIONS_VALIDATOR_MAP:
-            URI_OPTIONS_VALIDATOR_MAP[alias] = (
-                URI_OPTIONS_VALIDATOR_MAP[optname])
+            URI_OPTIONS_VALIDATOR_MAP[alias] = URI_OPTIONS_VALIDATOR_MAP[optname]
 
 # Map containing all URI option and keyword argument validators.
 VALIDATORS = URI_OPTIONS_VALIDATOR_MAP.copy()
@@ -719,32 +750,29 @@ VALIDATORS.update(KW_VALIDATORS)
 
 # List of timeout-related options.
 TIMEOUT_OPTIONS = [
-    'connecttimeoutms',
-    'heartbeatfrequencyms',
-    'maxidletimems',
-    'maxstalenessseconds',
-    'serverselectiontimeoutms',
-    'sockettimeoutms',
-    'waitqueuetimeoutms',
+    "connecttimeoutms",
+    "heartbeatfrequencyms",
+    "maxidletimems",
+    "maxstalenessseconds",
+    "serverselectiontimeoutms",
+    "sockettimeoutms",
+    "waitqueuetimeoutms",
 ]
 
 
-_AUTH_OPTIONS = frozenset(['authmechanismproperties'])
+_AUTH_OPTIONS = frozenset(["authmechanismproperties"])
 
 
 def validate_auth_option(option, value):
-    """Validate optional authentication parameters.
-    """
+    """Validate optional authentication parameters."""
     lower, value = validate(option, value)
     if lower not in _AUTH_OPTIONS:
-        raise ConfigurationError('Unknown '
-                                 'authentication option: %s' % (option,))
+        raise ConfigurationError("Unknown " "authentication option: %s" % (option,))
     return option, value
 
 
 def validate(option, value):
-    """Generic validation function.
-    """
+    """Generic validation function."""
     lower = option.lower()
     validator = VALIDATORS.get(lower, raise_config_error)
     value = validator(option, value)
@@ -773,8 +801,7 @@ def get_validated_options(options, warn=True):
     for opt, value in iteritems(options):
         normed_key = get_normed_key(opt)
         try:
-            validator = URI_OPTIONS_VALIDATOR_MAP.get(
-                normed_key, raise_config_error)
+            validator = URI_OPTIONS_VALIDATOR_MAP.get(normed_key, raise_config_error)
             value = validator(opt, value)
         except (ValueError, TypeError, ConfigurationError) as exc:
             if warn:
@@ -787,14 +814,7 @@ def get_validated_options(options, warn=True):
 
 
 # List of write-concern-related options.
-WRITE_CONCERN_OPTIONS = frozenset([
-    'w',
-    'wtimeout',
-    'wtimeoutms',
-    'fsync',
-    'j',
-    'journal'
-])
+WRITE_CONCERN_OPTIONS = frozenset(["w", "wtimeout", "wtimeoutms", "fsync", "j", "journal"])
 
 
 class BaseObject(object):
@@ -804,28 +824,32 @@ class BaseObject(object):
     SHOULD NOT BE USED BY DEVELOPERS EXTERNAL TO MONGODB.
     """
 
-    def __init__(self, codec_options, read_preference, write_concern,
-                 read_concern):
+    def __init__(self, codec_options, read_preference, write_concern, read_concern):
 
         if not isinstance(codec_options, CodecOptions):
-            raise TypeError("codec_options must be an instance of "
-                            "bson.codec_options.CodecOptions")
+            raise TypeError(
+                "codec_options must be an instance of " "bson.codec_options.CodecOptions"
+            )
         self.__codec_options = codec_options
 
         if not isinstance(read_preference, _ServerMode):
-            raise TypeError("%r is not valid for read_preference. See "
-                            "pymongo.read_preferences for valid "
-                            "options." % (read_preference,))
+            raise TypeError(
+                "%r is not valid for read_preference. See "
+                "pymongo.read_preferences for valid "
+                "options." % (read_preference,)
+            )
         self.__read_preference = read_preference
 
         if not isinstance(write_concern, WriteConcern):
-            raise TypeError("write_concern must be an instance of "
-                            "pymongo.write_concern.WriteConcern")
+            raise TypeError(
+                "write_concern must be an instance of " "pymongo.write_concern.WriteConcern"
+            )
         self.__write_concern = write_concern
 
         if not isinstance(read_concern, ReadConcern):
-            raise TypeError("read_concern must be an instance of "
-                            "pymongo.read_concern.ReadConcern")
+            raise TypeError(
+                "read_concern must be an instance of " "pymongo.read_concern.ReadConcern"
+            )
         self.__read_concern = read_concern
 
     @property
@@ -846,8 +870,7 @@ class BaseObject(object):
         return self.__write_concern
 
     def _write_concern_for(self, session):
-        """Read only access to the write concern of this instance or session.
-        """
+        """Read only access to the write concern of this instance or session."""
         # Override this operation's write concern with the transaction's.
         if session and session.in_transaction:
             return DEFAULT_WRITE_CONCERN
@@ -863,8 +886,7 @@ class BaseObject(object):
         return self.__read_preference
 
     def _read_preference_for(self, session):
-        """Read only access to the read preference of this instance or session.
-        """
+        """Read only access to the read preference of this instance or session."""
         # Override this operation's read preference with the transaction's.
         if session:
             return session._txn_read_preference() or self.__read_preference
@@ -957,4 +979,4 @@ class _CaseInsensitiveDictionary(abc.MutableMapping):
                 self[key] = other[key]
 
     def cased_key(self, key):
-        return self.__casedkeys[key.lower()]
\ No newline at end of file
+        return self.__casedkeys[key.lower()]
