@@ -15,39 +15,67 @@
 """Cursor class to iterate over Mongo query results."""
 
 import copy
+import threading
 import warnings
-
 from collections import deque
 
-from bson import RE_TYPE
+from bson import RE_TYPE, _convert_raw_document_lists_to_streams
 from bson.code import Code
-from bson.py3compat import (iteritems,
-                            integer_types,
-                            string_type)
+from bson.py3compat import integer_types, iteritems, string_type
 from bson.son import SON
 from pymongo import helpers
-from pymongo.common import validate_boolean, validate_is_mapping
 from pymongo.collation import validate_collation_or_none
-from pymongo.errors import (ConnectionFailure,
-                            InvalidOperation,
-                            NotMasterError,
-                            OperationFailure)
-from pymongo.message import (_CursorAddress,
-                             _GetMore,
-                             _RawBatchGetMore,
-                             _Query,
-                             _RawBatchQuery)
-from pymongo.monitoring import ConnectionClosedReason
+from pymongo.common import validate_boolean, validate_is_mapping
+from pymongo.errors import ConnectionFailure, InvalidOperation, OperationFailure
+from pymongo.message import (
+    _CursorAddress,
+    _GetMore,
+    _Query,
+    _RawBatchGetMore,
+    _RawBatchQuery,
+)
+from pymongo.response import PinnedResponse
 
+# These errors mean that the server has already killed the cursor so there is
+# no need to send killCursors.
+_CURSOR_CLOSED_ERRORS = frozenset(
+    [
+        43,  # CursorNotFound
+        50,  # MaxTimeMSExpired
+        175,  # QueryPlanKilled
+        237,  # CursorKilled
+        # On a tailable cursor, the following errors mean the capped collection
+        # rolled over.
+        # MongoDB 2.6:
+        # {'$err': 'Runner killed during getMore', 'code': 28617, 'ok': 0}
+        28617,
+        # MongoDB 3.0:
+        # {'$err': 'getMore executor error: UnknownError no details available',
+        #  'code': 17406, 'ok': 0}
+        17406,
+        # MongoDB 3.2 + 3.4:
+        # {'ok': 0.0, 'errmsg': 'GetMore command executor error:
+        #  CappedPositionLost: CollectionScan died due to failure to restore
+        #  tailable cursor position. Last seen record id: RecordId(3)',
+        #  'code': 96}
+        96,
+        # MongoDB 3.6+:
+        # {'ok': 0.0, 'errmsg': 'errmsg: "CollectionScan died due to failure to
+        #  restore tailable cursor position. Last seen record id: RecordId(3)"',
+        #  'code': 136, 'codeName': 'CappedPositionLost'}
+        136,
+    ]
+)
 
 _QUERY_OPTIONS = {
     "tailable_cursor": 2,
-    "slave_okay": 4,
+    "secondary_okay": 4,
     "oplog_replay": 8,
     "no_timeout": 16,
     "await_data": 32,
     "exhaust": 64,
-    "partial": 128}
+    "partial": 128,
+}
 
 
 class CursorType(object):
@@ -79,55 +107,73 @@ class CursorType(object):
     """
 
 
-# This has to be an old style class due to
-# http://bugs.jython.org/issue1057
-class _SocketManager:
-    """Used with exhaust cursors to ensure the socket is returned.
-    """
-    def __init__(self, sock, pool):
-        self.sock = sock
-        self.pool = pool
-        self.__closed = False
+class _SocketManager(object):
+    """Used with exhaust cursors to ensure the socket is returned."""
 
-    def __del__(self):
-        self.close()
+    def __init__(self, sock, more_to_come):
+        self.sock = sock
+        self.more_to_come = more_to_come
+        self.closed = False
+        self.lock = threading.Lock()
+
+    def update_exhaust(self, more_to_come):
+        self.more_to_come = more_to_come
 
     def close(self):
-        """Return this instance's socket to the connection pool.
-        """
-        if not self.__closed:
-            self.__closed = True
-            self.pool.return_socket(self.sock)
-            self.sock, self.pool = None, None
+        """Return this instance's socket to the connection pool."""
+        if not self.closed:
+            self.closed = True
+            self.sock.unpin()
+            self.sock = None
 
 
 class Cursor(object):
-    """A cursor / iterator over Mongo query results.
-    """
+    """A cursor / iterator over Mongo query results."""
+
     _query_class = _Query
     _getmore_class = _GetMore
 
-    def __init__(self, collection, filter=None, projection=None, skip=0,
-                 limit=0, no_cursor_timeout=False,
-                 cursor_type=CursorType.NON_TAILABLE,
-                 sort=None, allow_partial_results=False, oplog_replay=False,
-                 modifiers=None, batch_size=0, manipulate=True,
-                 collation=None, hint=None, max_scan=None, max_time_ms=None,
-                 max=None, min=None, return_key=False, show_record_id=False,
-                 snapshot=False, comment=None, session=None,
-                 allow_disk_use=None):
+    def __init__(
+        self,
+        collection,
+        filter=None,
+        projection=None,
+        skip=0,
+        limit=0,
+        no_cursor_timeout=False,
+        cursor_type=CursorType.NON_TAILABLE,
+        sort=None,
+        allow_partial_results=False,
+        oplog_replay=False,
+        modifiers=None,
+        batch_size=0,
+        manipulate=True,
+        collation=None,
+        hint=None,
+        max_scan=None,
+        max_time_ms=None,
+        max=None,
+        min=None,
+        return_key=None,
+        show_record_id=None,
+        snapshot=None,
+        comment=None,
+        session=None,
+        allow_disk_use=None,
+    ):
         """Create a new cursor.
 
         Should not be called directly by application developers - see
         :meth:`~pymongo.collection.Collection.find` instead.
 
-        .. mongodoc:: cursors
+        .. seealso:: The MongoDB documentation on `cursors <https://dochub.mongodb.org/core/cursors>`_.
         """
         # Initialize all attributes used in __del__ before possibly raising
         # an error to avoid attribute errors during garbage collection.
+        self.__collection = collection
         self.__id = None
         self.__exhaust = False
-        self.__exhaust_mgr = None
+        self.__sock_mgr = None
         self.__killed = False
 
         if session:
@@ -147,14 +193,30 @@ class Cursor(object):
         if not isinstance(limit, int):
             raise TypeError("limit must be an instance of int")
         validate_boolean("no_cursor_timeout", no_cursor_timeout)
-        if cursor_type not in (CursorType.NON_TAILABLE, CursorType.TAILABLE,
-                               CursorType.TAILABLE_AWAIT, CursorType.EXHAUST):
+        if no_cursor_timeout and not self.__explicit_session:
+            warnings.warn(
+                "use an explicit session with no_cursor_timeout=True "
+                "otherwise the cursor may still timeout after "
+                "30 minutes, for more info see "
+                "https://docs.mongodb.com/v4.4/reference/method/"
+                "cursor.noCursorTimeout/"
+                "#session-idle-timeout-overrides-nocursortimeout",
+                UserWarning,
+                stacklevel=2,
+            )
+        if cursor_type not in (
+            CursorType.NON_TAILABLE,
+            CursorType.TAILABLE,
+            CursorType.TAILABLE_AWAIT,
+            CursorType.EXHAUST,
+        ):
             raise ValueError("not a valid value for cursor_type")
         validate_boolean("allow_partial_results", allow_partial_results)
         validate_boolean("oplog_replay", oplog_replay)
         if modifiers is not None:
-            warnings.warn("the 'modifiers' parameter is deprecated",
-                          DeprecationWarning, stacklevel=2)
+            warnings.warn(
+                "the 'modifiers' parameter is deprecated", DeprecationWarning, stacklevel=2
+            )
             validate_is_mapping("modifiers", modifiers)
         if not isinstance(batch_size, integer_types):
             raise TypeError("batch_size must be an integer")
@@ -169,7 +231,6 @@ class Cursor(object):
                 projection = {"_id": 1}
             projection = helpers._fields_list_to_dict(projection, "projection")
 
-        self.__collection = collection
         self.__spec = spec
         self.__projection = projection
         self.__skip = skip
@@ -195,8 +256,7 @@ class Cursor(object):
         # Exhaust cursor support
         if cursor_type == CursorType.EXHAUST:
             if self.__collection.database.client.is_mongos:
-                raise InvalidOperation('Exhaust cursors are '
-                                       'not supported by mongos')
+                raise InvalidOperation("Exhaust cursors are " "not supported by mongos")
             if limit:
                 raise InvalidOperation("Can't use limit and exhaust together.")
             self.__exhaust = True
@@ -239,8 +299,7 @@ class Cursor(object):
 
     @property
     def retrieved(self):
-        """The number of documents retrieved so far.
-        """
+        """The number of documents retrieved so far."""
         return self.__retrieved
 
     def __del__(self):
@@ -255,6 +314,7 @@ class Cursor(object):
         be sent to the server, even if the resultant data has already been
         retrieved by this cursor.
         """
+        self.close()
         self.__data = deque()
         self.__id = None
         self.__address = None
@@ -281,28 +341,48 @@ class Cursor(object):
             else:
                 base = self._clone_base(None)
 
-        values_to_clone = ("spec", "projection", "skip", "limit",
-                           "max_time_ms", "max_await_time_ms", "comment",
-                           "max", "min", "ordering", "explain", "hint",
-                           "batch_size", "max_scan", "manipulate",
-                           "query_flags", "modifiers", "collation", "empty",
-                           "show_record_id", "return_key", "allow_disk_use",
-                           "snapshot", "exhaust")
-        data = dict((k, v) for k, v in iteritems(self.__dict__)
-                    if k.startswith('_Cursor__') and k[9:] in values_to_clone)
+        values_to_clone = (
+            "spec",
+            "projection",
+            "skip",
+            "limit",
+            "max_time_ms",
+            "max_await_time_ms",
+            "comment",
+            "max",
+            "min",
+            "ordering",
+            "explain",
+            "hint",
+            "batch_size",
+            "max_scan",
+            "manipulate",
+            "query_flags",
+            "modifiers",
+            "collation",
+            "empty",
+            "show_record_id",
+            "return_key",
+            "allow_disk_use",
+            "snapshot",
+            "exhaust",
+        )
+        data = dict(
+            (k, v)
+            for k, v in iteritems(self.__dict__)
+            if k.startswith("_Cursor__") and k[9:] in values_to_clone
+        )
         if deepcopy:
             data = self._deepcopy(data)
         base.__dict__.update(data)
         return base
 
     def _clone_base(self, session):
-        """Creates an empty Cursor object for information to be copied into.
-        """
+        """Creates an empty Cursor object for information to be copied into."""
         return self.__class__(self.__collection, session=session)
 
     def __die(self, synchronous=False):
-        """Closes this cursor.
-        """
+        """Closes this cursor."""
         try:
             already_killed = self.__killed
         except AttributeError:
@@ -311,36 +391,30 @@ class Cursor(object):
 
         self.__killed = True
         if self.__id and not already_killed:
-            if self.__exhaust and self.__exhaust_mgr:
-                # If this is an exhaust cursor and we haven't completely
-                # exhausted the result set we *must* close the socket
-                # to stop the server from sending more data.
-                self.__exhaust_mgr.sock.close_socket(
-                    ConnectionClosedReason.ERROR)
-            else:
-                address = _CursorAddress(
-                    self.__address, self.__collection.full_name)
-                if synchronous:
-                    self.__collection.database.client._close_cursor_now(
-                        self.__id, address, session=self.__session)
-                else:
-                    # The cursor will be closed later in a different session.
-                    self.__collection.database.client._close_cursor(
-                        self.__id, address)
-        if self.__exhaust and self.__exhaust_mgr:
-            self.__exhaust_mgr.close()
-        if self.__session and not self.__explicit_session:
-            self.__session._end_session(lock=synchronous)
+            cursor_id = self.__id
+            address = _CursorAddress(self.__address, "%s.%s" % (self.__dbname, self.__collname))
+        else:
+            # Skip killCursors.
+            cursor_id = 0
+            address = None
+        self.__collection.database.client._cleanup_cursor(
+            synchronous,
+            cursor_id,
+            address,
+            self.__sock_mgr,
+            self.__session,
+            self.__explicit_session,
+        )
+        if not self.__explicit_session:
             self.__session = None
+        self.__sock_mgr = None
 
     def close(self):
-        """Explicitly close / kill this cursor.
-        """
+        """Explicitly close / kill this cursor."""
         self.__die(True)
 
     def __query_spec(self):
-        """Get the spec to use for a query.
-        """
+        """Get the spec to use for a query."""
         operators = self.__modifiers.copy()
         if self.__ordering:
             operators["$orderby"] = self.__ordering
@@ -358,19 +432,19 @@ class Cursor(object):
             operators["$max"] = self.__max
         if self.__min:
             operators["$min"] = self.__min
-        if self.__return_key:
+        if self.__return_key is not None:
             operators["$returnKey"] = self.__return_key
-        if self.__show_record_id:
+        if self.__show_record_id is not None:
             # This is upgraded to showRecordId for MongoDB 3.2+ "find" command.
             operators["$showDiskLoc"] = self.__show_record_id
-        if self.__snapshot:
+        if self.__snapshot is not None:
             operators["$snapshot"] = self.__snapshot
 
         if operators:
             # Make a shallow copy so we can cleanly rewind or clone.
             spec = self.__spec.copy()
 
-            # White-listed commands must be wrapped in $query.
+            # Allow-listed commands must be wrapped in $query.
             if "$query" not in spec:
                 # $query has to come first
                 spec = SON([("$query", spec)])
@@ -387,16 +461,15 @@ class Cursor(object):
         # that breaks commands like count and find_and_modify.
         # Checking spec.keys()[0] covers the case that the spec
         # was passed as an instance of SON or OrderedDict.
-        elif ("query" in self.__spec and
-              (len(self.__spec) == 1 or
-               next(iter(self.__spec)) == "query")):
+        elif "query" in self.__spec and (
+            len(self.__spec) == 1 or next(iter(self.__spec)) == "query"
+        ):
             return SON({"$query": self.__spec})
 
         return self.__spec
 
     def __check_okay_to_chain(self):
-        """Check if it is okay to chain more options onto this cursor.
-        """
+        """Check if it is okay to chain more options onto this cursor."""
         if self.__retrieved or self.__id is not None:
             raise InvalidOperation("cannot set options after executing query")
 
@@ -414,8 +487,7 @@ class Cursor(object):
             if self.__limit:
                 raise InvalidOperation("Can't use limit and exhaust together.")
             if self.__collection.database.client.is_mongos:
-                raise InvalidOperation('Exhaust cursors are '
-                                       'not supported by mongos')
+                raise InvalidOperation("Exhaust cursors are " "not supported by mongos")
             self.__exhaust = True
 
         self.__query_flags |= mask
@@ -453,7 +525,7 @@ class Cursor(object):
         .. versionadded:: 3.11
         """
         if not isinstance(allow_disk_use, bool):
-            raise TypeError('allow_disk_use must be a bool')
+            raise TypeError("allow_disk_use must be a bool")
         self.__check_okay_to_chain()
 
         self.__allow_disk_use = allow_disk_use
@@ -470,7 +542,7 @@ class Cursor(object):
         :Parameters:
           - `limit`: the number of results to return
 
-        .. mongodoc:: limit
+        .. seealso:: The MongoDB documentation on `limit <https://dochub.mongodb.org/core/limit>`_.
         """
         if not isinstance(limit, integer_types):
             raise TypeError("limit must be an integer")
@@ -544,8 +616,7 @@ class Cursor(object):
         :Parameters:
           - `max_time_ms`: the time limit after which the operation is aborted
         """
-        if (not isinstance(max_time_ms, integer_types)
-                and max_time_ms is not None):
+        if not isinstance(max_time_ms, integer_types) and max_time_ms is not None:
             raise TypeError("max_time_ms must be an integer or None")
         self.__check_okay_to_chain()
 
@@ -569,8 +640,7 @@ class Cursor(object):
 
         .. versionadded:: 3.2
         """
-        if (not isinstance(max_await_time_ms, integer_types)
-                and max_await_time_ms is not None):
+        if not isinstance(max_await_time_ms, integer_types) and max_await_time_ms is not None:
             raise TypeError("max_await_time_ms must be an integer or None")
         self.__check_okay_to_chain()
 
@@ -583,6 +653,18 @@ class Cursor(object):
     def __getitem__(self, index):
         """Get a single document or a slice of documents from this cursor.
 
+        .. warning:: A :class:`~Cursor` is not a Python :class:`list`. Each
+          index access or slice requires that a new query be run using skip
+          and limit. Do not iterate the cursor using index accesses.
+          The following example is **extremely inefficient** and may return
+          surprising results::
+
+            cursor = db.collection.find()
+            # Warning: This runs a new query for each document.
+            # Don't do this!
+            for idx in range(10):
+                print(cursor[idx])
+
         Raises :class:`~pymongo.errors.InvalidOperation` if this
         cursor has already been used.
 
@@ -618,15 +700,15 @@ class Cursor(object):
             skip = 0
             if index.start is not None:
                 if index.start < 0:
-                    raise IndexError("Cursor instances do not support "
-                                     "negative indices")
+                    raise IndexError("Cursor instances do not support " "negative indices")
                 skip = index.start
 
             if index.stop is not None:
                 limit = index.stop - skip
                 if limit < 0:
-                    raise IndexError("stop index must be greater than start "
-                                     "index for slice %r" % index)
+                    raise IndexError(
+                        "stop index must be greater than start " "index for slice %r" % index
+                    )
                 if limit == 0:
                     self.__empty = True
             else:
@@ -638,8 +720,7 @@ class Cursor(object):
 
         if isinstance(index, integer_types):
             if index < 0:
-                raise IndexError("Cursor instances do not support negative "
-                                 "indices")
+                raise IndexError("Cursor instances do not support negative " "indices")
             clone = self.clone()
             clone.skip(index + self.__skip)
             clone.limit(-1)  # use a hard limit
@@ -647,8 +728,7 @@ class Cursor(object):
             for doc in clone:
                 return doc
             raise IndexError("no such item for Cursor instance")
-        raise TypeError("index %r cannot be applied to Cursor "
-                        "instances" % index)
+        raise TypeError("index %r cannot be applied to Cursor " "instances" % index)
 
     def max_scan(self, max_scan):
         """**DEPRECATED** - Limit the number of documents to scan when
@@ -800,11 +880,13 @@ class Cursor(object):
         .. versionchanged:: 2.8
            The :meth:`~count` method now supports :meth:`~hint`.
         """
-        warnings.warn("count is deprecated. Use Collection.count_documents "
-                      "instead.", DeprecationWarning, stacklevel=2)
+        warnings.warn(
+            "count is deprecated. Use Collection.count_documents " "instead.",
+            DeprecationWarning,
+            stacklevel=2,
+        )
         validate_boolean("with_limit_and_skip", with_limit_and_skip)
-        cmd = SON([("count", self.__collection.name),
-                   ("query", self.__spec)])
+        cmd = SON([("count", self.__collection.name), ("query", self.__spec)])
         if self.__max_time_ms is not None:
             cmd["maxTimeMS"] = self.__max_time_ms
         if self.__comment:
@@ -819,8 +901,7 @@ class Cursor(object):
             if self.__skip:
                 cmd["skip"] = self.__skip
 
-        return self.__collection._count(
-            cmd, self.__collation, session=self.__session)
+        return self.__collection._count(cmd, self.__collation, session=self.__session)
 
     def distinct(self, key):
         """Get a list of distinct values for `key` among all documents
@@ -843,14 +924,13 @@ class Cursor(object):
         if self.__spec:
             options["query"] = self.__spec
         if self.__max_time_ms is not None:
-            options['maxTimeMS'] = self.__max_time_ms
+            options["maxTimeMS"] = self.__max_time_ms
         if self.__comment:
-            options['comment'] = self.__comment
+            options["comment"] = self.__comment
         if self.__collation is not None:
-            options['collation'] = self.__collation
+            options["collation"] = self.__collation
 
-        return self.__collection.distinct(
-            key, session=self.__session, **options)
+        return self.__collection.distinct(key, session=self.__session, **options)
 
     def explain(self):
         """Returns an explain plan record for this cursor.
@@ -862,7 +942,7 @@ class Cursor(object):
           :meth:`~pymongo.database.Database.command` to run the explain
           command directly.
 
-        .. mongodoc:: explain
+        .. seealso:: The MongoDB documentation on `explain <https://dochub.mongodb.org/core/explain>`_.
         """
         c = self.clone()
         c.__explain = True
@@ -992,68 +1072,55 @@ class Cursor(object):
         client = self.__collection.database.client
         # OP_MSG is required to support exhaust cursors with encryption.
         if client._encrypter and self.__exhaust:
-            raise InvalidOperation(
-                "exhaust cursors do not support auto encryption")
+            raise InvalidOperation("exhaust cursors do not support auto encryption")
 
         try:
-            response = client._run_operation_with_response(
-                operation, self._unpack_response, exhaust=self.__exhaust,
-                address=self.__address)
-        except OperationFailure:
-            self.__killed = True
-
-            # Make sure exhaust socket is returned immediately, if necessary.
-            self.__die()
-
+            response = client._run_operation(
+                operation, self._unpack_response, address=self.__address
+            )
+        except OperationFailure as exc:
+            if exc.code in _CURSOR_CLOSED_ERRORS or self.__exhaust:
+                # Don't send killCursors because the cursor is already closed.
+                self.__killed = True
+            self.close()
             # If this is a tailable cursor the error is likely
             # due to capped collection roll over. Setting
             # self.__killed to True ensures Cursor.alive will be
             # False. No need to re-raise.
-            if self.__query_flags & _QUERY_OPTIONS["tailable_cursor"]:
+            if (
+                exc.code in _CURSOR_CLOSED_ERRORS
+                and self.__query_flags & _QUERY_OPTIONS["tailable_cursor"]
+            ):
                 return
             raise
-        except NotMasterError:
-            # Don't send kill cursors to another server after a "not master"
-            # error. It's completely pointless.
-            self.__killed = True
-
-            # Make sure exhaust socket is returned immediately, if necessary.
-            self.__die()
-
-            raise
         except ConnectionFailure:
-            # Don't try to send kill cursors on another socket
-            # or to another server. It can cause a _pinValue
-            # assertion on some server releases if we get here
-            # due to a socket timeout.
+            # Don't send killCursors because the cursor is already closed.
             self.__killed = True
-            self.__die()
+            self.close()
             raise
         except Exception:
-            # Close the cursor
-            self.__die()
+            self.close()
             raise
 
         self.__address = response.address
-        if self.__exhaust and not self.__exhaust_mgr:
-            # 'response' is an ExhaustResponse.
-            self.__exhaust_mgr = _SocketManager(response.socket_info,
-                                                response.pool)
+        if isinstance(response, PinnedResponse):
+            if not self.__sock_mgr:
+                self.__sock_mgr = _SocketManager(response.socket_info, response.more_to_come)
 
         cmd_name = operation.name
         docs = response.docs
         if response.from_command:
             if cmd_name != "explain":
-                cursor = docs[0]['cursor']
-                self.__id = cursor['id']
-                if cmd_name == 'find':
-                    documents = cursor['firstBatch']
+                cursor = docs[0]["cursor"]
+                self.__id = cursor["id"]
+                if cmd_name == "find":
+                    documents = cursor["firstBatch"]
                     # Update the namespace used for future getMore commands.
-                    ns = cursor.get('ns')
+                    ns = cursor.get("ns")
                     if ns:
-                        self.__dbname, self.__collname = ns.split('.', 1)
+                        self.__dbname, self.__collname = ns.split(".", 1)
                 else:
-                    documents = cursor['nextBatch']
+                    documents = cursor["nextBatch"]
                 self.__data = deque(documents)
                 self.__retrieved += len(documents)
             else:
@@ -1066,24 +1133,22 @@ class Cursor(object):
             self.__retrieved += response.data.number_returned
 
         if self.__id == 0:
-            self.__killed = True
             # Don't wait for garbage collection to call __del__, return the
             # socket and the session to the pool now.
-            self.__die()
+            self.close()
 
         if self.__limit and self.__id and self.__limit <= self.__retrieved:
-            self.__die()
+            self.close()
 
-    def _unpack_response(self, response, cursor_id, codec_options,
-                         user_fields=None, legacy_response=False):
-        return response.unpack_response(cursor_id, codec_options, user_fields,
-                                        legacy_response)
+    def _unpack_response(
+        self, response, cursor_id, codec_options, user_fields=None, legacy_response=False
+    ):
+        return response.unpack_response(cursor_id, codec_options, user_fields, legacy_response)
 
     def _read_preference(self):
         if self.__read_preference is None:
             # Save the read preference for getMore commands.
-            self.__read_preference = self.__collection._read_preference_for(
-                self.session)
+            self.__read_preference = self.__collection._read_preference_for(self.session)
         return self.__read_preference
 
     def _refresh(self):
@@ -1101,26 +1166,32 @@ class Cursor(object):
 
         if self.__id is None:  # Query
             if (self.__min or self.__max) and not self.__hint:
-                warnings.warn("using a min/max query operator without "
-                              "specifying a Cursor.hint is deprecated. A "
-                              "hint will be required when using min/max in "
-                              "PyMongo 4.0",
-                              DeprecationWarning, stacklevel=3)
-            q = self._query_class(self.__query_flags,
-                                  self.__collection.database.name,
-                                  self.__collection.name,
-                                  self.__skip,
-                                  self.__query_spec(),
-                                  self.__projection,
-                                  self.__codec_options,
-                                  self._read_preference(),
-                                  self.__limit,
-                                  self.__batch_size,
-                                  self.__read_concern,
-                                  self.__collation,
-                                  self.__session,
-                                  self.__collection.database.client,
-                                  self.__allow_disk_use)
+                warnings.warn(
+                    "using a min/max query operator without "
+                    "specifying a Cursor.hint is deprecated. A "
+                    "hint will be required when using min/max in "
+                    "PyMongo 4.0",
+                    DeprecationWarning,
+                    stacklevel=3,
+                )
+            q = self._query_class(
+                self.__query_flags,
+                self.__collection.database.name,
+                self.__collection.name,
+                self.__skip,
+                self.__query_spec(),
+                self.__projection,
+                self.__codec_options,
+                self._read_preference(),
+                self.__limit,
+                self.__batch_size,
+                self.__read_concern,
+                self.__collation,
+                self.__session,
+                self.__collection.database.client,
+                self.__allow_disk_use,
+                self.__exhaust,
+            )
             self.__send_message(q)
         elif self.__id:  # Get More
             if self.__limit:
@@ -1129,18 +1200,20 @@ class Cursor(object):
                     limit = min(limit, self.__batch_size)
             else:
                 limit = self.__batch_size
-
             # Exhaust cursors don't send getMore messages.
-            g = self._getmore_class(self.__dbname,
-                                    self.__collname,
-                                    limit,
-                                    self.__id,
-                                    self.__codec_options,
-                                    self._read_preference(),
-                                    self.__session,
-                                    self.__collection.database.client,
-                                    self.__max_await_time_ms,
-                                    self.__exhaust_mgr)
+            g = self._getmore_class(
+                self.__dbname,
+                self.__collname,
+                limit,
+                self.__id,
+                self.__codec_options,
+                self._read_preference(),
+                self.__session,
+                self.__collection.database.client,
+                self.__max_await_time_ms,
+                self.__sock_mgr,
+                self.__exhaust,
+            )
             self.__send_message(g)
 
         return len(self.__data)
@@ -1207,8 +1280,7 @@ class Cursor(object):
         if len(self.__data) or self._refresh():
             if self.__manipulate:
                 _db = self.__collection.database
-                return _db._fix_outgoing(self.__data.popleft(),
-                                         self.__collection)
+                return _db._fix_outgoing(self.__data.popleft(), self.__collection)
             else:
                 return self.__data.popleft()
         else:
@@ -1242,7 +1314,7 @@ class Cursor(object):
         Regular expressions cannot be deep copied but as they are immutable we
         don't have to copy them when cloning.
         """
-        if not hasattr(x, 'items'):
+        if not hasattr(x, "items"):
             y, is_list, iterator = [], True, enumerate(x)
         else:
             y, is_list, iterator = {}, False, iteritems(x)
@@ -1282,25 +1354,30 @@ class RawBatchCursor(Cursor):
         see :meth:`~pymongo.collection.Collection.find_raw_batches`
         instead.
 
-        .. mongodoc:: cursors
+        .. seealso:: The MongoDB documentation on `cursors <https://dochub.mongodb.org/core/cursors>`_.
         """
-        manipulate = kwargs.get('manipulate')
-        kwargs['manipulate'] = False
+        manipulate = kwargs.get("manipulate")
+        kwargs["manipulate"] = False
         super(RawBatchCursor, self).__init__(*args, **kwargs)
 
         # Throw only after cursor's initialized, to prevent errors in __del__.
         if manipulate:
-            raise InvalidOperation(
-                "Cannot use RawBatchCursor with manipulate=True")
+            raise InvalidOperation("Cannot use RawBatchCursor with manipulate=True")
 
-    def _unpack_response(self, response, cursor_id, codec_options,
-                         user_fields=None, legacy_response=False):
-        return response.raw_response(cursor_id)
+    def _unpack_response(
+        self, response, cursor_id, codec_options, user_fields=None, legacy_response=False
+    ):
+        raw_response = response.raw_response(cursor_id, user_fields=user_fields)
+        if not legacy_response:
+            # OP_MSG returns firstBatch/nextBatch documents as a BSON array
+            # Re-assemble the array of documents into a document stream
+            _convert_raw_document_lists_to_streams(raw_response[0])
+        return raw_response
 
     def explain(self):
         """Returns an explain plan record for this cursor.
 
-        .. mongodoc:: explain
+        .. seealso:: The MongoDB documentation on `explain <https://dochub.mongodb.org/core/explain>`_.
         """
         clone = self._clone(deepcopy=True, base=Cursor(self.collection))
         return clone.explain()
