@@ -26,44 +26,45 @@ try:
     from pymongocrypt.explicit_encrypter import ExplicitEncrypter
     from pymongocrypt.mongocrypt import MongoCryptOptions
     from pymongocrypt.state_machine import MongoCryptCallback
+
     _HAVE_PYMONGOCRYPT = True
 except ImportError:
     _HAVE_PYMONGOCRYPT = False
     MongoCryptCallback = object
 
 from bson import _dict_to_bson, decode, encode
+from bson.binary import STANDARD, UUID_SUBTYPE, Binary
 from bson.codec_options import CodecOptions
-from bson.binary import (Binary,
-                         STANDARD,
-                         UUID_SUBTYPE)
 from bson.errors import BSONError
-from bson.raw_bson import (DEFAULT_RAW_BSON_OPTIONS,
-                           RawBSONDocument,
-                           _inflate_bson)
+from bson.raw_bson import DEFAULT_RAW_BSON_OPTIONS, RawBSONDocument, _inflate_bson
 from bson.son import SON
-
-from pymongo.errors import (ConfigurationError,
-                            EncryptionError,
-                            InvalidOperation,
-                            ServerSelectionTimeoutError)
+from pymongo.daemon import _spawn_daemon
+from pymongo.errors import (
+    ConfigurationError,
+    EncryptionError,
+    InvalidOperation,
+    ServerSelectionTimeoutError,
+)
 from pymongo.mongo_client import MongoClient
-from pymongo.pool import _configured_socket, PoolOptions
+from pymongo.pool import PoolOptions, _configured_socket
 from pymongo.read_concern import ReadConcern
-from pymongo.ssl_support import get_ssl_context
+from pymongo.ssl_support import HAVE_SSL, get_ssl_context
 from pymongo.uri_parser import parse_host
 from pymongo.write_concern import WriteConcern
-from pymongo.daemon import _spawn_daemon
 
+if HAVE_SSL:
+    from ssl import CERT_REQUIRED
+else:
+    CERT_REQUIRED = None
 
 _HTTPS_PORT = 443
 _KMS_CONNECT_TIMEOUT = 10  # TODO: CDRIVER-3262 will define this value.
-_MONGOCRYPTD_TIMEOUT_MS = 1000
+_MONGOCRYPTD_TIMEOUT_MS = 10000
 
 _DATA_KEY_OPTS = CodecOptions(document_class=SON, uuid_representation=STANDARD)
 # Use RawBSONDocument codec options to avoid needlessly decoding
 # documents from the key vault.
-_KEY_VAULT_OPTS = CodecOptions(document_class=RawBSONDocument,
-                               uuid_representation=STANDARD)
+_KEY_VAULT_OPTS = CodecOptions(document_class=RawBSONDocument, uuid_representation=STANDARD)
 
 
 @contextlib.contextmanager
@@ -89,8 +90,9 @@ class _EncryptionIO(MongoCryptCallback):
             self.client_ref = None
         self.key_vault_coll = key_vault_coll.with_options(
             codec_options=_KEY_VAULT_OPTS,
-            read_concern=ReadConcern(level='majority'),
-            write_concern=WriteConcern(w='majority'))
+            read_concern=ReadConcern(level="majority"),
+            write_concern=WriteConcern(w="majority"),
+        )
         self.mongocryptd_client = mongocryptd_client
         self.opts = opts
         self._spawned = False
@@ -107,15 +109,30 @@ class _EncryptionIO(MongoCryptCallback):
         endpoint = kms_context.endpoint
         message = kms_context.message
         host, port = parse_host(endpoint, _HTTPS_PORT)
-        ctx = get_ssl_context(None, None, None, None, None, None, True, True)
-        opts = PoolOptions(connect_timeout=_KMS_CONNECT_TIMEOUT,
-                           socket_timeout=_KMS_CONNECT_TIMEOUT,
-                           ssl_context=ctx)
+        # Enable strict certificate verification, OCSP, match hostname, and
+        # SNI using the system default CA certificates.
+        ctx = get_ssl_context(
+            None,  # certfile
+            None,  # keyfile
+            None,  # passphrase
+            None,  # ca_certs
+            CERT_REQUIRED,  # cert_reqs
+            None,  # crlfile
+            True,  # match_hostname
+            True,
+        )  # check_ocsp_endpoint
+        opts = PoolOptions(
+            connect_timeout=_KMS_CONNECT_TIMEOUT,
+            socket_timeout=_KMS_CONNECT_TIMEOUT,
+            ssl_context=ctx,
+        )
         conn = _configured_socket((host, port), opts)
         try:
             conn.sendall(message)
             while kms_context.bytes_needed > 0:
                 data = conn.recv(kms_context.bytes_needed)
+                if not data:
+                    raise OSError("KMS connection closed")
                 kms_context.feed(data)
         finally:
             conn.close()
@@ -133,8 +150,7 @@ class _EncryptionIO(MongoCryptCallback):
         :Returns:
           The first document from the listCollections command response as BSON.
         """
-        with self.client_ref()[database].list_collections(
-                filter=RawBSONDocument(filter)) as cursor:
+        with self.client_ref()[database].list_collections(filter=RawBSONDocument(filter)) as cursor:
             for doc in cursor:
                 return _dict_to_bson(doc, False, _DATA_KEY_OPTS)
 
@@ -145,7 +161,7 @@ class _EncryptionIO(MongoCryptCallback):
         successfully.
         """
         self._spawned = True
-        args = [self.opts._mongocryptd_spawn_path or 'mongocryptd']
+        args = [self.opts._mongocryptd_spawn_path or "mongocryptd"]
         args.extend(self.opts._mongocryptd_spawn_args)
         _spawn_daemon(args)
 
@@ -166,15 +182,15 @@ class _EncryptionIO(MongoCryptCallback):
         inflated_cmd = _inflate_bson(cmd, DEFAULT_RAW_BSON_OPTIONS)
         try:
             res = self.mongocryptd_client[database].command(
-                inflated_cmd,
-                codec_options=DEFAULT_RAW_BSON_OPTIONS)
+                inflated_cmd, codec_options=DEFAULT_RAW_BSON_OPTIONS
+            )
         except ServerSelectionTimeoutError:
             if self.opts._mongocryptd_bypass_spawn:
                 raise
             self.spawn()
             res = self.mongocryptd_client[database].command(
-                inflated_cmd,
-                codec_options=DEFAULT_RAW_BSON_OPTIONS)
+                inflated_cmd, codec_options=DEFAULT_RAW_BSON_OPTIONS
+            )
         return res.raw
 
     def fetch_keys(self, filter):
@@ -200,9 +216,9 @@ class _EncryptionIO(MongoCryptCallback):
           The _id of the inserted data key document.
         """
         raw_doc = RawBSONDocument(data_key)
-        data_key_id = raw_doc.get('_id')
+        data_key_id = raw_doc.get("_id")
         if not isinstance(data_key_id, uuid.UUID):
-            raise TypeError('data_key _id must be a UUID')
+            raise TypeError("data_key _id must be a UUID")
 
         self.key_vault_coll.insert_one(raw_doc)
         return Binary(data_key_id.bytes, subtype=UUID_SUBTYPE)
@@ -233,23 +249,57 @@ class _EncryptionIO(MongoCryptCallback):
 
 
 class _Encrypter(object):
-    def __init__(self, io_callbacks, opts):
-        """Encrypts and decrypts MongoDB commands.
+    """Encrypts and decrypts MongoDB commands.
 
-        This class is used to support automatic encryption and decryption of
-        MongoDB commands.
+    This class is used to support automatic encryption and decryption of
+    MongoDB commands."""
+
+    def __init__(self, client, opts):
+        """Create a _Encrypter for a client.
 
         :Parameters:
-          - `io_callbacks`: A :class:`MongoCryptCallback`.
+          - `client`: The encrypted MongoClient.
           - `opts`: The encrypted client's :class:`AutoEncryptionOpts`.
         """
         if opts._schema_map is None:
             schema_map = None
         else:
             schema_map = _dict_to_bson(opts._schema_map, False, _DATA_KEY_OPTS)
-        self._auto_encrypter = AutoEncrypter(io_callbacks, MongoCryptOptions(
-            opts._kms_providers, schema_map))
         self._bypass_auto_encryption = opts._bypass_auto_encryption
+        self._internal_client = None
+
+        def _get_internal_client(encrypter, mongo_client):
+            if mongo_client.options.pool_options.max_pool_size is None:
+                # Unlimited pool size, use the same client.
+                return mongo_client
+            # Else - limited pool size, use an internal client.
+            if encrypter._internal_client is not None:
+                return encrypter._internal_client
+            internal_client = mongo_client._duplicate(minPoolSize=0, auto_encryption_opts=None)
+            encrypter._internal_client = internal_client
+            return internal_client
+
+        if opts._key_vault_client is not None:
+            key_vault_client = opts._key_vault_client
+        else:
+            key_vault_client = _get_internal_client(self, client)
+
+        if opts._bypass_auto_encryption:
+            metadata_client = None
+        else:
+            metadata_client = _get_internal_client(self, client)
+
+        db, coll = opts._key_vault_namespace.split(".", 1)
+        key_vault_coll = key_vault_client[db][coll]
+
+        mongocryptd_client = MongoClient(
+            opts._mongocryptd_uri, connect=False, serverSelectionTimeoutMS=_MONGOCRYPTD_TIMEOUT_MS
+        )
+
+        io_callbacks = _EncryptionIO(metadata_client, key_vault_coll, mongocryptd_client, opts)
+        self._auto_encrypter = AutoEncrypter(
+            io_callbacks, MongoCryptOptions(opts._kms_providers, schema_map)
+        )
         self._closed = False
 
     def encrypt(self, database, cmd, check_keys, codec_options):
@@ -267,15 +317,14 @@ class _Encrypter(object):
         self._check_closed()
         # Workaround for $clusterTime which is incompatible with
         # check_keys.
-        cluster_time = check_keys and cmd.pop('$clusterTime', None)
+        cluster_time = check_keys and cmd.pop("$clusterTime", None)
         encoded_cmd = _dict_to_bson(cmd, check_keys, codec_options)
         with _wrap_encryption_errors():
             encrypted_cmd = self._auto_encrypter.encrypt(database, encoded_cmd)
             # TODO: PYTHON-1922 avoid decoding the encrypted_cmd.
-            encrypt_cmd = _inflate_bson(
-                encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
+            encrypt_cmd = _inflate_bson(encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
             if cluster_time:
-                encrypt_cmd['$clusterTime'] = cluster_time
+                encrypt_cmd["$clusterTime"] = cluster_time
             return encrypt_cmd
 
     def decrypt(self, response):
@@ -299,44 +348,22 @@ class _Encrypter(object):
         """Cleanup resources."""
         self._closed = True
         self._auto_encrypter.close()
-
-    @staticmethod
-    def create(client, opts):
-        """Create a _CommandEncyptor for a client.
-
-        :Parameters:
-          - `client`: The encrypted MongoClient.
-          - `opts`: The encrypted client's :class:`AutoEncryptionOpts`.
-
-        :Returns:
-          A :class:`_CommandEncrypter` for this client.
-        """
-        key_vault_client = opts._key_vault_client or client
-        db, coll = opts._key_vault_namespace.split('.', 1)
-        key_vault_coll = key_vault_client[db][coll]
-
-        mongocryptd_client = MongoClient(
-            opts._mongocryptd_uri, connect=False,
-            serverSelectionTimeoutMS=_MONGOCRYPTD_TIMEOUT_MS)
-
-        io_callbacks = _EncryptionIO(
-            client, key_vault_coll, mongocryptd_client, opts)
-        return _Encrypter(io_callbacks, opts)
+        if self._internal_client:
+            self._internal_client.close()
+            self._internal_client = None
 
 
 class Algorithm(object):
     """An enum that defines the supported encryption algorithms."""
-    AEAD_AES_256_CBC_HMAC_SHA_512_Deterministic = (
-        "AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic")
-    AEAD_AES_256_CBC_HMAC_SHA_512_Random = (
-        "AEAD_AES_256_CBC_HMAC_SHA_512-Random")
+
+    AEAD_AES_256_CBC_HMAC_SHA_512_Deterministic = "AEAD_AES_256_CBC_HMAC_SHA_512-Deterministic"
+    AEAD_AES_256_CBC_HMAC_SHA_512_Random = "AEAD_AES_256_CBC_HMAC_SHA_512-Random"
 
 
 class ClientEncryption(object):
     """Explicit client-side field level encryption."""
 
-    def __init__(self, kms_providers, key_vault_namespace, key_vault_client,
-                 codec_options):
+    def __init__(self, kms_providers, key_vault_namespace, key_vault_client, codec_options):
         """Explicit client-side field level encryption.
 
         The ClientEncryption class encapsulates explicit operations on a key
@@ -357,10 +384,23 @@ class ClientEncryption(object):
 
               - `aws`: Map with "accessKeyId" and "secretAccessKey" as strings.
                 These are the AWS access key ID and AWS secret access key used
-                to generate KMS messages.
-              - `local`: Map with "key" as a 96-byte array or string. "key"
-                is the master key used to encrypt/decrypt data keys. This key
-                should be generated and stored as securely as possible.
+                to generate KMS messages. An optional "sessionToken" may be
+                included to support temporary AWS credentials.
+              - `azure`: Map with "tenantId", "clientId", and "clientSecret" as
+                strings. Additionally, "identityPlatformEndpoint" may also be
+                specified as a string (defaults to 'login.microsoftonline.com').
+                These are the Azure Active Directory credentials used to
+                generate Azure Key Vault messages.
+              - `gcp`: Map with "email" as a string and "privateKey"
+                as `bytes` or a base64 encoded string (unicode on Python 2).
+                Additionally, "endpoint" may also be specified as a string
+                (defaults to 'oauth2.googleapis.com'). These are the
+                credentials used to generate Google Cloud KMS messages.
+              - `local`: Map with "key" as `bytes` (96 bytes in length) or
+                a base64 encoded string (unicode on Python 2) which decodes
+                to 96 bytes. "key" is the master key used to encrypt/decrypt
+                data keys. This key should be generated and stored as securely
+                as possible.
 
           - `key_vault_namespace`: The namespace for the key vault collection.
             The key vault collection contains all data keys used for encryption
@@ -382,26 +422,28 @@ class ClientEncryption(object):
             raise ConfigurationError(
                 "client-side field level encryption requires the pymongocrypt "
                 "library: install a compatible version with: "
-                "python -m pip install 'pymongo[encryption]'")
+                "python -m pip install 'pymongo[encryption]'"
+            )
 
         if not isinstance(codec_options, CodecOptions):
-            raise TypeError("codec_options must be an instance of "
-                            "bson.codec_options.CodecOptions")
+            raise TypeError(
+                "codec_options must be an instance of " "bson.codec_options.CodecOptions"
+            )
 
         self._kms_providers = kms_providers
         self._key_vault_namespace = key_vault_namespace
         self._key_vault_client = key_vault_client
         self._codec_options = codec_options
 
-        db, coll = key_vault_namespace.split('.', 1)
+        db, coll = key_vault_namespace.split(".", 1)
         key_vault_coll = key_vault_client[db][coll]
 
         self._io_callbacks = _EncryptionIO(None, key_vault_coll, None, None)
         self._encryption = ExplicitEncrypter(
-            self._io_callbacks, MongoCryptOptions(kms_providers, None))
+            self._io_callbacks, MongoCryptOptions(kms_providers, None)
+        )
 
-    def create_data_key(self, kms_provider, master_key=None,
-                        key_alt_names=None):
+    def create_data_key(self, kms_provider, master_key=None, key_alt_names=None):
         """Create and insert a new data key into the key vault collection.
 
         :Parameters:
@@ -409,8 +451,10 @@ class ClientEncryption(object):
             "aws" and "local".
           - `master_key`: Identifies a KMS-specific key used to encrypt the
             new data key. If the kmsProvider is "local" the `master_key` is
-            not applicable and may be omitted. If the `kms_provider` is "aws"
-            it is required and has the following fields::
+            not applicable and may be omitted.
+
+            If the `kms_provider` is "aws" it is required and has the
+            following fields::
 
               - `region` (string): Required. The AWS region, e.g. "us-east-1".
               - `key` (string): Required. The Amazon Resource Name (ARN) to
@@ -419,6 +463,26 @@ class ClientEncryption(object):
                 requests to. May include port number, e.g.
                 "kms.us-east-1.amazonaws.com:443".
 
+            If the `kms_provider` is "azure" it is required and has the
+            following fields::
+
+              - `keyVaultEndpoint` (string): Required. Host with optional
+                 port, e.g. "example.vault.azure.net".
+              - `keyName` (string): Required. Key name in the key vault.
+              - `keyVersion` (string): Optional. Version of the key to use.
+
+            If the `kms_provider` is "gcp" it is required and has the
+            following fields::
+
+              - `projectId` (string): Required. The Google cloud project ID.
+              - `location` (string): Required. The GCP location, e.g. "us-east1".
+              - `keyRing` (string): Required. Name of the key ring that contains
+                the key to use.
+              - `keyName` (string): Required. Name of the key to use.
+              - `keyVersion` (string): Optional. Version of the key to use.
+              - `endpoint` (string): Optional. Host with optional port.
+                Defaults to "cloudkms.googleapis.com".
+
           - `key_alt_names` (optional): An optional list of string alternate
             names used to reference a key. If a key is created with alternate
             names, then encryption may refer to the key by the unique alternate
@@ -428,7 +492,7 @@ class ClientEncryption(object):
               client_encryption.create_data_key("local", keyAltNames=["name1"])
               # reference the key with the alternate name
               client_encryption.encrypt("457-55-5462", keyAltName="name1",
-                                        algorithm=Algorithm.Random)
+                                        algorithm=Algorithm.AEAD_AES_256_CBC_HMAC_SHA_512_Random)
 
         :Returns:
           The ``_id`` of the created data key document as a
@@ -438,8 +502,8 @@ class ClientEncryption(object):
         self._check_closed()
         with _wrap_encryption_errors():
             return self._encryption.create_data_key(
-                kms_provider, master_key=master_key,
-                key_alt_names=key_alt_names)
+                kms_provider, master_key=master_key, key_alt_names=key_alt_names
+            )
 
     def encrypt(self, value, algorithm, key_id=None, key_alt_name=None):
         """Encrypt a BSON value with a given key and algorithm.
@@ -460,17 +524,17 @@ class ClientEncryption(object):
           The encrypted value, a :class:`~bson.binary.Binary` with subtype 6.
         """
         self._check_closed()
-        if (key_id is not None and not (
-                isinstance(key_id, Binary) and
-                key_id.subtype == UUID_SUBTYPE)):
-            raise TypeError(
-                'key_id must be a bson.binary.Binary with subtype 4')
+        if key_id is not None and not (
+            isinstance(key_id, Binary) and key_id.subtype == UUID_SUBTYPE
+        ):
+            raise TypeError("key_id must be a bson.binary.Binary with subtype 4")
 
-        doc = encode({'v': value}, codec_options=self._codec_options)
+        doc = encode({"v": value}, codec_options=self._codec_options)
         with _wrap_encryption_errors():
             encrypted_doc = self._encryption.encrypt(
-                doc, algorithm, key_id=key_id, key_alt_name=key_alt_name)
-            return decode(encrypted_doc)['v']
+                doc, algorithm, key_id=key_id, key_alt_name=key_alt_name
+            )
+            return decode(encrypted_doc)["v"]
 
     def decrypt(self, value):
         """Decrypt an encrypted value.
@@ -484,14 +548,12 @@ class ClientEncryption(object):
         """
         self._check_closed()
         if not (isinstance(value, Binary) and value.subtype == 6):
-            raise TypeError(
-                'value to decrypt must be a bson.binary.Binary with subtype 6')
+            raise TypeError("value to decrypt must be a bson.binary.Binary with subtype 6")
 
         with _wrap_encryption_errors():
-            doc = encode({'v': value})
+            doc = encode({"v": value})
             decrypted_doc = self._encryption.decrypt(doc)
-            return decode(decrypted_doc,
-                          codec_options=self._codec_options)['v']
+            return decode(decrypted_doc, codec_options=self._codec_options)["v"]
 
     def __enter__(self):
         return self
