@@ -18,10 +18,9 @@ import atexit
 import threading
 import weakref
 
+from bson.py3compat import PY3
 from pymongo import common, periodic_executor
-from pymongo.errors import (NotMasterError,
-                            OperationFailure,
-                            _OperationCancelled)
+from pymongo.errors import NotPrimaryError, OperationFailure, _OperationCancelled
 from pymongo.ismaster import IsMaster
 from pymongo.monotonic import time as _time
 from pymongo.periodic_executor import _shutdown_executors
@@ -30,6 +29,14 @@ from pymongo.server_description import ServerDescription
 from pymongo.srv_resolver import _SrvResolver
 
 
+def _sanitize(error):
+    """PYTHON-2433 Clear error traceback info."""
+    if PY3:
+        error.__traceback__ = None
+        error.__context__ = None
+        error.__cause__ = None
+
+
 class MonitorBase(object):
     def __init__(self, topology, name, interval, min_interval):
         """Base class to do periodic work on a background thread.
@@ -47,15 +54,13 @@ class MonitorBase(object):
             return True
 
         executor = periodic_executor.PeriodicExecutor(
-            interval=interval,
-            min_interval=min_interval,
-            target=target,
-            name=name)
+            interval=interval, min_interval=min_interval, target=target, name=name
+        )
 
         self._executor = executor
 
         def _on_topology_gc(dummy=None):
-            # This prevents GC from waiting 10 seconds for isMaster to complete
+            # This prevents GC from waiting 10 seconds for 'hello' to complete
             # See test_cleanup_executors_on_client_del.
             monitor = self_ref()
             if monitor:
@@ -94,12 +99,7 @@ class MonitorBase(object):
 
 
 class Monitor(MonitorBase):
-    def __init__(
-            self,
-            server_description,
-            topology,
-            pool,
-            topology_settings):
+    def __init__(self, server_description, topology, pool, topology_settings):
         """Class to monitor a MongoDB server on a background thread.
 
         Pass an initial ServerDescription, a Topology, a Pool, and
@@ -112,21 +112,24 @@ class Monitor(MonitorBase):
             topology,
             "pymongo_server_monitor_thread",
             topology_settings.heartbeat_frequency,
-            common.MIN_HEARTBEAT_INTERVAL)
+            common.MIN_HEARTBEAT_INTERVAL,
+        )
         self._server_description = server_description
         self._pool = pool
         self._settings = topology_settings
-        self._listeners = self._settings._pool_options.event_listeners
+        self._listeners = self._settings._pool_options._event_listeners
         pub = self._listeners is not None
         self._publish = pub and self._listeners.enabled_for_server_heartbeat
         self._cancel_context = None
         self._rtt_monitor = _RttMonitor(
-            topology, topology_settings, topology._create_pool_for_monitor(
-                server_description.address))
+            topology,
+            topology_settings,
+            topology._create_pool_for_monitor(server_description.address),
+        )
         self.heartbeater = None
 
     def cancel_check(self):
-        """Cancel any concurrent isMaster check.
+        """Cancel any concurrent hello check.
 
         Note: this is called from a weakref.proxy callback and MUST NOT take
         any locks.
@@ -169,9 +172,11 @@ class Monitor(MonitorBase):
             try:
                 self._server_description = self._check_server()
             except _OperationCancelled as exc:
+                _sanitize(exc)
                 # Already closed the connection, wait for the next check.
                 self._server_description = ServerDescription(
-                    self._server_description.address, error=exc)
+                    self._server_description.address, error=exc
+                )
                 if prev_sd.is_server_type_known:
                     # Immediately retry since we've already waited 500ms to
                     # discover that we've been cancelled.
@@ -179,11 +184,14 @@ class Monitor(MonitorBase):
                 return
 
             # Update the Topology and clear the server pool on error.
-            self._topology.on_change(self._server_description,
-                                     reset_pool=self._server_description.error)
+            self._topology.on_change(
+                self._server_description, reset_pool=self._server_description.error
+            )
 
-            if (self._server_description.is_server_type_known and
-                     self._server_description.topology_version):
+            if (
+                self._server_description.is_server_type_known
+                and self._server_description.topology_version
+            ):
                 self._start_rtt_monitor()
                 # Immediately check for the next streaming response.
                 self._executor.skip_sleep()
@@ -196,7 +204,7 @@ class Monitor(MonitorBase):
             self.close()
 
     def _check_server(self):
-        """Call isMaster or read the next streaming response.
+        """Call hello or read the next streaming response.
 
         Returns a ServerDescription.
         """
@@ -204,21 +212,20 @@ class Monitor(MonitorBase):
         try:
             try:
                 return self._check_once()
-            except (OperationFailure, NotMasterError) as exc:
-                # Update max cluster time even when isMaster fails.
-                self._topology.receive_cluster_time(
-                    exc.details.get('$clusterTime'))
+            except (OperationFailure, NotPrimaryError) as exc:
+                # Update max cluster time even when hello fails.
+                self._topology.receive_cluster_time(exc.details.get("$clusterTime"))
                 raise
         except ReferenceError:
             raise
         except Exception as error:
+            _sanitize(error)
             sd = self._server_description
             address = sd.address
             duration = _time() - start
             if self._publish:
                 awaited = sd.is_server_type_known and sd.topology_version
-                self._listeners.publish_server_heartbeat_failed(
-                    address, duration, error, awaited)
+                self._listeners.publish_server_heartbeat_failed(address, duration, error, awaited)
             self._reset_connection()
             if isinstance(error, _OperationCancelled):
                 raise
@@ -227,7 +234,7 @@ class Monitor(MonitorBase):
             return ServerDescription(address, error=error)
 
     def _check_once(self):
-        """A single attempt to call ismaster.
+        """A single attempt to call hello.
 
         Returns a ServerDescription, or raises an exception.
         """
@@ -243,34 +250,34 @@ class Monitor(MonitorBase):
             if not response.awaitable:
                 self._rtt_monitor.add_sample(round_trip_time)
 
-            sd = ServerDescription(address, response,
-                                   self._rtt_monitor.average())
+            sd = ServerDescription(address, response, self._rtt_monitor.average())
             if self._publish:
                 self._listeners.publish_server_heartbeat_succeeded(
-                    address, round_trip_time, response, response.awaitable)
+                    address, round_trip_time, response, response.awaitable
+                )
             return sd
 
     def _check_with_socket(self, conn):
-        """Return (IsMaster, round_trip_time).
+        """Return (Hello, round_trip_time).
 
         Can raise ConnectionFailure or OperationFailure.
         """
         cluster_time = self._topology.max_cluster_time()
         start = _time()
         if conn.more_to_come:
-            # Read the next streaming isMaster (MongoDB 4.4+).
+            # Read the next streaming hello (MongoDB 4.4+).
             response = IsMaster(conn._next_reply(), awaitable=True)
-        elif (conn.performed_handshake and
-              self._server_description.topology_version):
-            # Initiate streaming isMaster (MongoDB 4.4+).
-            response = conn._ismaster(
+        elif conn.performed_handshake and self._server_description.topology_version:
+            # Initiate streaming hello (MongoDB 4.4+).
+            response = conn._hello(
                 cluster_time,
                 self._server_description.topology_version,
                 self._settings.heartbeat_frequency,
-                None)
+                None,
+            )
         else:
-            # New connection handshake or polling isMaster (MongoDB <4.4).
-            response = conn._ismaster(cluster_time, None, None, None)
+            # New connection handshake or polling hello (MongoDB <4.4).
+            response = conn._hello(cluster_time, None, None, None)
         return response, _time() - start
 
 
@@ -286,7 +293,8 @@ class SrvMonitor(MonitorBase):
             topology,
             "pymongo_srv_polling_thread",
             common.MIN_SRV_RESCAN_INTERVAL,
-            topology_settings.heartbeat_frequency)
+            topology_settings.heartbeat_frequency,
+        )
         self._settings = topology_settings
         self._seedlist = self._settings._seeds
         self._fqdn = self._settings.fqdn
@@ -319,8 +327,7 @@ class SrvMonitor(MonitorBase):
             self.request_check()
             return None
         else:
-            self._executor.update_interval(
-                max(ttl, common.MIN_SRV_RESCAN_INTERVAL))
+            self._executor.update_interval(max(ttl, common.MIN_SRV_RESCAN_INTERVAL))
             return seedlist
 
 
@@ -334,7 +341,8 @@ class _RttMonitor(MonitorBase):
             topology,
             "pymongo_server_rtt_thread",
             topology_settings.heartbeat_frequency,
-            common.MIN_HEARTBEAT_INTERVAL)
+            common.MIN_HEARTBEAT_INTERVAL,
+        )
 
         self._pool = pool
         self._moving_average = MovingAverage()
@@ -375,12 +383,12 @@ class _RttMonitor(MonitorBase):
             self._pool.reset()
 
     def _ping(self):
-        """Run an "isMaster" command and return the RTT."""
+        """Run a "hello" command and return the RTT."""
         with self._pool.get_socket({}) as sock_info:
             if self._executor._stopped:
-                raise Exception('_RttMonitor closed')
+                raise Exception("_RttMonitor closed")
             start = _time()
-            sock_info.ismaster()
+            sock_info.hello()
             return _time() - start
 
 
