@@ -183,12 +183,19 @@ will not add that listener to existing client instances.
 from collections import namedtuple
 
 from bson.py3compat import abc
+from pymongo.hello_compat import HelloCompat
 from pymongo.helpers import _handle_exception
 
-_Listeners = namedtuple('Listeners',
-                        ('command_listeners', 'server_listeners',
-                         'server_heartbeat_listeners', 'topology_listeners',
-                         'cmap_listeners'))
+_Listeners = namedtuple(
+    "Listeners",
+    (
+        "command_listeners",
+        "server_listeners",
+        "server_heartbeat_listeners",
+        "topology_listeners",
+        "cmap_listeners",
+    ),
+)
 
 _LISTENERS = _Listeners([], [], [], [], [])
 
@@ -459,10 +466,12 @@ def _validate_event_listeners(option, listeners):
         raise TypeError("%s must be a list or tuple" % (option,))
     for listener in listeners:
         if not isinstance(listener, _EventListener):
-            raise TypeError("Listeners for %s must be either a "
-                            "CommandListener, ServerHeartbeatListener, "
-                            "ServerListener, TopologyListener, or "
-                            "ConnectionPoolListener." % (option,))
+            raise TypeError(
+                "Listeners for %s must be either a "
+                "CommandListener, ServerHeartbeatListener, "
+                "ServerListener, TopologyListener, or "
+                "ConnectionPoolListener." % (option,)
+            )
     return listeners
 
 
@@ -475,10 +484,12 @@ def register(listener):
         :class:`TopologyListener`, or :class:`ConnectionPoolListener`.
     """
     if not isinstance(listener, _EventListener):
-        raise TypeError("Listeners for %s must be either a "
-                        "CommandListener, ServerHeartbeatListener, "
-                        "ServerListener, TopologyListener, or "
-                        "ConnectionPoolListener." % (listener,))
+        raise TypeError(
+            "Listeners for %s must be either a "
+            "CommandListener, ServerHeartbeatListener, "
+            "ServerListener, TopologyListener, or "
+            "ConnectionPoolListener." % (listener,)
+        )
     if isinstance(listener, CommandListener):
         _LISTENERS.command_listeners.append(listener)
     if isinstance(listener, ServerHeartbeatListener):
@@ -490,24 +501,47 @@ def register(listener):
     if isinstance(listener, ConnectionPoolListener):
         _LISTENERS.cmap_listeners.append(listener)
 
+
 # Note - to avoid bugs from forgetting which if these is all lowercase and
 # which are camelCase, and at the same time avoid having to add a test for
 # every command, use all lowercase here and test against command_name.lower().
 _SENSITIVE_COMMANDS = set(
-    ["authenticate", "saslstart", "saslcontinue", "getnonce", "createuser",
-     "updateuser", "copydbgetnonce", "copydbsaslstart", "copydb"])
+    [
+        "authenticate",
+        "saslstart",
+        "saslcontinue",
+        "getnonce",
+        "createuser",
+        "updateuser",
+        "copydbgetnonce",
+        "copydbsaslstart",
+        "copydb",
+    ]
+)
+
+
+# The "hello" command is also deemed sensitive when attempting speculative
+# authentication.
+def _is_speculative_authenticate(command_name, doc):
+    if (
+        command_name.lower() in ("hello", HelloCompat.LEGACY_CMD)
+        and "speculativeAuthenticate" in doc
+    ):
+        return True
+    return False
 
 
 class _CommandEvent(object):
     """Base class for command events."""
 
-    __slots__ = ("__cmd_name", "__rqst_id", "__conn_id", "__op_id")
+    __slots__ = ("__cmd_name", "__rqst_id", "__conn_id", "__op_id", "__service_id")
 
-    def __init__(self, command_name, request_id, connection_id, operation_id):
+    def __init__(self, command_name, request_id, connection_id, operation_id, service_id=None):
         self.__cmd_name = command_name
         self.__rqst_id = request_id
         self.__conn_id = connection_id
         self.__op_id = operation_id
+        self.__service_id = service_id
 
     @property
     def command_name(self):
@@ -524,6 +558,14 @@ class _CommandEvent(object):
         """The address (host, port) of the server this command was sent to."""
         return self.__conn_id
 
+    @property
+    def service_id(self):
+        """The service_id this command was sent to, or ``None``.
+
+        .. versionadded:: 3.12
+        """
+        return self.__service_id
+
     @property
     def operation_id(self):
         """An id for this series of events or None."""
@@ -540,16 +582,23 @@ class CommandStartedEvent(_CommandEvent):
       - `connection_id`: The address (host, port) of the server this command
         was sent to.
       - `operation_id`: An optional identifier for a series of related events.
+      - `service_id`: The service_id this command was sent to, or ``None``.
     """
+
     __slots__ = ("__cmd", "__db")
 
-    def __init__(self, command, database_name, *args):
+    def __init__(
+        self, command, database_name, request_id, connection_id, operation_id, service_id=None
+    ):
         if not command:
             raise ValueError("%r is not a valid command" % (command,))
         # Command name must be first key.
         command_name = next(iter(command))
-        super(CommandStartedEvent, self).__init__(command_name, *args)
-        if command_name.lower() in _SENSITIVE_COMMANDS:
+        super(CommandStartedEvent, self).__init__(
+            command_name, request_id, connection_id, operation_id, service_id=service_id
+        )
+        cmd_name, cmd_doc = command_name.lower(), command[command_name]
+        if cmd_name in _SENSITIVE_COMMANDS or _is_speculative_authenticate(cmd_name, command):
             self.__cmd = {}
         else:
             self.__cmd = command
@@ -566,9 +615,14 @@ class CommandStartedEvent(_CommandEvent):
         return self.__db
 
     def __repr__(self):
-        return "<%s %s db: %r, command: %r, operation_id: %s>" % (
-            self.__class__.__name__, self.connection_id, self.database_name,
-            self.command_name, self.operation_id)
+        return ("<%s %s db: %r, command: %r, operation_id: %s, " "service_id: %s>") % (
+            self.__class__.__name__,
+            self.connection_id,
+            self.database_name,
+            self.command_name,
+            self.operation_id,
+            self.service_id,
+        )
 
 
 class CommandSucceededEvent(_CommandEvent):
@@ -582,15 +636,27 @@ class CommandSucceededEvent(_CommandEvent):
       - `connection_id`: The address (host, port) of the server this command
         was sent to.
       - `operation_id`: An optional identifier for a series of related events.
+      - `service_id`: The service_id this command was sent to, or ``None``.
     """
+
     __slots__ = ("__duration_micros", "__reply")
 
-    def __init__(self, duration, reply, command_name,
-                 request_id, connection_id, operation_id):
+    def __init__(
+        self,
+        duration,
+        reply,
+        command_name,
+        request_id,
+        connection_id,
+        operation_id,
+        service_id=None,
+    ):
         super(CommandSucceededEvent, self).__init__(
-            command_name, request_id, connection_id, operation_id)
+            command_name, request_id, connection_id, operation_id, service_id=service_id
+        )
         self.__duration_micros = _to_micros(duration)
-        if command_name.lower() in _SENSITIVE_COMMANDS:
+        cmd_name = command_name.lower()
+        if cmd_name in _SENSITIVE_COMMANDS or _is_speculative_authenticate(cmd_name, reply):
             self.__reply = {}
         else:
             self.__reply = reply
@@ -606,9 +672,14 @@ class CommandSucceededEvent(_CommandEvent):
         return self.__reply
 
     def __repr__(self):
-        return "<%s %s command: %r, operation_id: %s, duration_micros: %s>" % (
-            self.__class__.__name__, self.connection_id,
-            self.command_name, self.operation_id, self.duration_micros)
+        return ("<%s %s command: %r, operation_id: %s, duration_micros: %s, " "service_id: %s>") % (
+            self.__class__.__name__,
+            self.connection_id,
+            self.command_name,
+            self.operation_id,
+            self.duration_micros,
+            self.service_id,
+        )
 
 
 class CommandFailedEvent(_CommandEvent):
@@ -622,11 +693,24 @@ class CommandFailedEvent(_CommandEvent):
       - `connection_id`: The address (host, port) of the server this command
         was sent to.
       - `operation_id`: An optional identifier for a series of related events.
+      - `service_id`: The service_id this command was sent to, or ``None``.
     """
+
     __slots__ = ("__duration_micros", "__failure")
 
-    def __init__(self, duration, failure, *args):
-        super(CommandFailedEvent, self).__init__(*args)
+    def __init__(
+        self,
+        duration,
+        failure,
+        command_name,
+        request_id,
+        connection_id,
+        operation_id,
+        service_id=None,
+    ):
+        super(CommandFailedEvent, self).__init__(
+            command_name, request_id, connection_id, operation_id, service_id=service_id
+        )
         self.__duration_micros = _to_micros(duration)
         self.__failure = failure
 
@@ -643,13 +727,21 @@ class CommandFailedEvent(_CommandEvent):
     def __repr__(self):
         return (
             "<%s %s command: %r, operation_id: %s, duration_micros: %s, "
-            "failure: %r>" % (
-                self.__class__.__name__, self.connection_id, self.command_name,
-                self.operation_id, self.duration_micros, self.failure))
+            "failure: %r, service_id: %s>"
+        ) % (
+            self.__class__.__name__,
+            self.connection_id,
+            self.command_name,
+            self.operation_id,
+            self.duration_micros,
+            self.failure,
+            self.service_id,
+        )
 
 
 class _PoolEvent(object):
     """Base class for pool events."""
+
     __slots__ = ("__address",)
 
     def __init__(self, address):
@@ -663,7 +755,7 @@ class _PoolEvent(object):
         return self.__address
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, self.__address)
+        return "%s(%r)" % (self.__class__.__name__, self.__address)
 
 
 class PoolCreatedEvent(_PoolEvent):
@@ -675,6 +767,7 @@ class PoolCreatedEvent(_PoolEvent):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ("__options",)
 
     def __init__(self, address, options):
@@ -683,13 +776,11 @@ class PoolCreatedEvent(_PoolEvent):
 
     @property
     def options(self):
-        """Any non-default pool options that were set on this Connection Pool.
-        """
+        """Any non-default pool options that were set on this Connection Pool."""
         return self.__options
 
     def __repr__(self):
-        return '%s(%r, %r)' % (
-            self.__class__.__name__, self.address, self.__options)
+        return "%s(%r, %r)" % (self.__class__.__name__, self.address, self.__options)
 
 
 class PoolClearedEvent(_PoolEvent):
@@ -698,10 +789,29 @@ class PoolClearedEvent(_PoolEvent):
     :Parameters:
      - `address`: The address (host, port) pair of the server this Pool is
        attempting to connect to.
+     - `service_id`: The service_id this command was sent to, or ``None``.
 
     .. versionadded:: 3.9
     """
-    __slots__ = ()
+
+    __slots__ = ("__service_id",)
+
+    def __init__(self, address, service_id=None):
+        super(PoolClearedEvent, self).__init__(address)
+        self.__service_id = service_id
+
+    @property
+    def service_id(self):
+        """Connections with this service_id are cleared.
+
+        When service_id is ``None``, all connections in the pool are cleared.
+
+        .. versionadded:: 3.12
+        """
+        return self.__service_id
+
+    def __repr__(self):
+        return "%s(%r, %r)" % (self.__class__.__name__, self.address, self.__service_id)
 
 
 class PoolClosedEvent(_PoolEvent):
@@ -713,6 +823,7 @@ class PoolClosedEvent(_PoolEvent):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ()
 
 
@@ -723,17 +834,17 @@ class ConnectionClosedReason(object):
     .. versionadded:: 3.9
     """
 
-    STALE = 'stale'
+    STALE = "stale"
     """The pool was cleared, making the connection no longer valid."""
 
-    IDLE = 'idle'
+    IDLE = "idle"
     """The connection became stale by being idle for too long (maxIdleTimeMS).
     """
 
-    ERROR = 'error'
+    ERROR = "error"
     """The connection experienced an error, making it no longer valid."""
 
-    POOL_CLOSED = 'poolClosed'
+    POOL_CLOSED = "poolClosed"
     """The pool was closed, making the connection no longer valid."""
 
 
@@ -744,13 +855,13 @@ class ConnectionCheckOutFailedReason(object):
     .. versionadded:: 3.9
     """
 
-    TIMEOUT = 'timeout'
+    TIMEOUT = "timeout"
     """The connection check out attempt exceeded the specified timeout."""
 
-    POOL_CLOSED = 'poolClosed'
+    POOL_CLOSED = "poolClosed"
     """The pool was previously closed, and cannot provide new connections."""
 
-    CONN_ERROR = 'connectionError'
+    CONN_ERROR = "connectionError"
     """The connection check out attempt experienced an error while setting up
     a new connection.
     """
@@ -758,6 +869,7 @@ class ConnectionCheckOutFailedReason(object):
 
 class _ConnectionEvent(object):
     """Private base class for some connection events."""
+
     __slots__ = ("__address", "__connection_id")
 
     def __init__(self, address, connection_id):
@@ -777,8 +889,7 @@ class _ConnectionEvent(object):
         return self.__connection_id
 
     def __repr__(self):
-        return '%s(%r, %r)' % (
-            self.__class__.__name__, self.__address, self.__connection_id)
+        return "%s(%r, %r)" % (self.__class__.__name__, self.__address, self.__connection_id)
 
 
 class ConnectionCreatedEvent(_ConnectionEvent):
@@ -794,6 +905,7 @@ class ConnectionCreatedEvent(_ConnectionEvent):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ()
 
 
@@ -807,6 +919,7 @@ class ConnectionReadyEvent(_ConnectionEvent):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ()
 
 
@@ -821,6 +934,7 @@ class ConnectionClosedEvent(_ConnectionEvent):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ("__reason",)
 
     def __init__(self, address, connection_id, reason):
@@ -837,9 +951,12 @@ class ConnectionClosedEvent(_ConnectionEvent):
         return self.__reason
 
     def __repr__(self):
-        return '%s(%r, %r, %r)' % (
-            self.__class__.__name__, self.address, self.connection_id,
-            self.__reason)
+        return "%s(%r, %r, %r)" % (
+            self.__class__.__name__,
+            self.address,
+            self.connection_id,
+            self.__reason,
+        )
 
 
 class ConnectionCheckOutStartedEvent(object):
@@ -851,6 +968,7 @@ class ConnectionCheckOutStartedEvent(object):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ("__address",)
 
     def __init__(self, address):
@@ -864,7 +982,7 @@ class ConnectionCheckOutStartedEvent(object):
         return self.__address
 
     def __repr__(self):
-        return '%s(%r)' % (self.__class__.__name__, self.__address)
+        return "%s(%r)" % (self.__class__.__name__, self.__address)
 
 
 class ConnectionCheckOutFailedEvent(object):
@@ -877,6 +995,7 @@ class ConnectionCheckOutFailedEvent(object):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ("__address", "__reason")
 
     def __init__(self, address, reason):
@@ -900,8 +1019,7 @@ class ConnectionCheckOutFailedEvent(object):
         return self.__reason
 
     def __repr__(self):
-        return '%s(%r, %r)' % (
-            self.__class__.__name__, self.__address, self.__reason)
+        return "%s(%r, %r)" % (self.__class__.__name__, self.__address, self.__reason)
 
 
 class ConnectionCheckedOutEvent(_ConnectionEvent):
@@ -914,6 +1032,7 @@ class ConnectionCheckedOutEvent(_ConnectionEvent):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ()
 
 
@@ -927,6 +1046,7 @@ class ConnectionCheckedInEvent(_ConnectionEvent):
 
     .. versionadded:: 3.9
     """
+
     __slots__ = ()
 
 
@@ -951,7 +1071,10 @@ class _ServerEvent(object):
 
     def __repr__(self):
         return "<%s %s topology_id: %s>" % (
-            self.__class__.__name__, self.server_address, self.topology_id)
+            self.__class__.__name__,
+            self.server_address,
+            self.topology_id,
+        )
 
 
 class ServerDescriptionChangedEvent(_ServerEvent):
@@ -960,7 +1083,7 @@ class ServerDescriptionChangedEvent(_ServerEvent):
     .. versionadded:: 3.3
     """
 
-    __slots__ = ('__previous_description', '__new_description')
+    __slots__ = ("__previous_description", "__new_description")
 
     def __init__(self, previous_description, new_description, *args):
         super(ServerDescriptionChangedEvent, self).__init__(*args)
@@ -981,8 +1104,11 @@ class ServerDescriptionChangedEvent(_ServerEvent):
 
     def __repr__(self):
         return "<%s %s changed from: %s, to: %s>" % (
-            self.__class__.__name__, self.server_address,
-            self.previous_description, self.new_description)
+            self.__class__.__name__,
+            self.server_address,
+            self.previous_description,
+            self.new_description,
+        )
 
 
 class ServerOpeningEvent(_ServerEvent):
@@ -1006,7 +1132,7 @@ class ServerClosedEvent(_ServerEvent):
 class TopologyEvent(object):
     """Base class for topology description events."""
 
-    __slots__ = ('__topology_id')
+    __slots__ = "__topology_id"
 
     def __init__(self, topology_id):
         self.__topology_id = topology_id
@@ -1017,8 +1143,7 @@ class TopologyEvent(object):
         return self.__topology_id
 
     def __repr__(self):
-        return "<%s topology_id: %s>" % (
-            self.__class__.__name__, self.topology_id)
+        return "<%s topology_id: %s>" % (self.__class__.__name__, self.topology_id)
 
 
 class TopologyDescriptionChangedEvent(TopologyEvent):
@@ -1027,9 +1152,9 @@ class TopologyDescriptionChangedEvent(TopologyEvent):
     .. versionadded:: 3.3
     """
 
-    __slots__ = ('__previous_description', '__new_description')
+    __slots__ = ("__previous_description", "__new_description")
 
-    def __init__(self, previous_description,  new_description, *args):
+    def __init__(self, previous_description, new_description, *args):
         super(TopologyDescriptionChangedEvent, self).__init__(*args)
         self.__previous_description = previous_description
         self.__new_description = new_description
@@ -1048,8 +1173,11 @@ class TopologyDescriptionChangedEvent(TopologyEvent):
 
     def __repr__(self):
         return "<%s topology_id: %s changed from: %s, to: %s>" % (
-            self.__class__.__name__, self.topology_id,
-            self.previous_description, self.new_description)
+            self.__class__.__name__,
+            self.topology_id,
+            self.previous_description,
+            self.new_description,
+        )
 
 
 class TopologyOpenedEvent(TopologyEvent):
@@ -1073,7 +1201,7 @@ class TopologyClosedEvent(TopologyEvent):
 class _ServerHeartbeatEvent(object):
     """Base class for server heartbeat events."""
 
-    __slots__ = ('__connection_id')
+    __slots__ = "__connection_id"
 
     def __init__(self, connection_id):
         self.__connection_id = connection_id
@@ -1103,7 +1231,7 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
     .. versionadded:: 3.3
     """
 
-    __slots__ = ('__duration', '__reply', '__awaited')
+    __slots__ = ("__duration", "__reply", "__awaited")
 
     def __init__(self, duration, reply, connection_id, awaited=False):
         super(ServerHeartbeatSucceededEvent, self).__init__(connection_id)
@@ -1118,7 +1246,12 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
 
     @property
     def reply(self):
-        """An instance of :class:`~pymongo.ismaster.IsMaster`."""
+        """An instance of :class:`~pymongo.ismaster.IsMaster`.
+
+        .. warning:: :class:`~pymongo.ismaster.IsMaster` is deprecated.
+          Starting with PyMongo 4.0 this attribute will return an instance
+          of :class:`~pymongo.hello.Hello`, which provides the same API.
+        """
         return self.__reply
 
     @property
@@ -1133,8 +1266,12 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
 
     def __repr__(self):
         return "<%s %s duration: %s, awaited: %s, reply: %s>" % (
-            self.__class__.__name__, self.connection_id,
-            self.duration, self.awaited, self.reply)
+            self.__class__.__name__,
+            self.connection_id,
+            self.duration,
+            self.awaited,
+            self.reply,
+        )
 
 
 class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
@@ -1144,7 +1281,7 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
     .. versionadded:: 3.3
     """
 
-    __slots__ = ('__duration', '__reply', '__awaited')
+    __slots__ = ("__duration", "__reply", "__awaited")
 
     def __init__(self, duration, reply, connection_id, awaited=False):
         super(ServerHeartbeatFailedEvent, self).__init__(connection_id)
@@ -1174,8 +1311,12 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
 
     def __repr__(self):
         return "<%s %s duration: %s, awaited: %s, reply: %r>" % (
-            self.__class__.__name__, self.connection_id,
-            self.duration, self.awaited, self.reply)
+            self.__class__.__name__,
+            self.connection_id,
+            self.duration,
+            self.awaited,
+            self.reply,
+        )
 
 
 class _EventListeners(object):
@@ -1186,6 +1327,7 @@ class _EventListeners(object):
     :Parameters:
       - `listeners`: A list of event listeners.
     """
+
     def __init__(self, listeners):
         self.__command_listeners = _LISTENERS.command_listeners[:]
         self.__server_listeners = _LISTENERS.server_listeners[:]
@@ -1207,8 +1349,7 @@ class _EventListeners(object):
                     self.__cmap_listeners.append(lst)
         self.__enabled_for_commands = bool(self.__command_listeners)
         self.__enabled_for_server = bool(self.__server_listeners)
-        self.__enabled_for_server_heartbeat = bool(
-            self.__server_heartbeat_listeners)
+        self.__enabled_for_server_heartbeat = bool(self.__server_heartbeat_listeners)
         self.__enabled_for_topology = bool(self.__topology_listeners)
         self.__enabled_for_cmap = bool(self.__cmap_listeners)
 
@@ -1239,13 +1380,16 @@ class _EventListeners(object):
 
     def event_listeners(self):
         """List of registered event listeners."""
-        return (self.__command_listeners[:],
-                self.__server_heartbeat_listeners[:],
-                self.__server_listeners[:],
-                self.__topology_listeners[:])
+        return (
+            self.__command_listeners +
+            self.__server_heartbeat_listeners +
+            self.__server_listeners +
+            self.__topology_listeners
+        )
 
-    def publish_command_start(self, command, database_name,
-                              request_id, connection_id, op_id=None):
+    def publish_command_start(
+        self, command, database_name, request_id, connection_id, op_id=None, service_id=None
+    ):
         """Publish a CommandStartedEvent to all command listeners.
 
         :Parameters:
@@ -1256,19 +1400,30 @@ class _EventListeners(object):
           - `connection_id`: The address (host, port) of the server this
             command was sent to.
           - `op_id`: The (optional) operation id for this operation.
+          - `service_id`: The service_id this command was sent to, or ``None``.
         """
         if op_id is None:
             op_id = request_id
         event = CommandStartedEvent(
-            command, database_name, request_id, connection_id, op_id)
+            command, database_name, request_id, connection_id, op_id, service_id=service_id
+        )
         for subscriber in self.__command_listeners:
             try:
                 subscriber.started(event)
             except Exception:
                 _handle_exception()
 
-    def publish_command_success(self, duration, reply, command_name,
-                                request_id, connection_id, op_id=None):
+    def publish_command_success(
+        self,
+        duration,
+        reply,
+        command_name,
+        request_id,
+        connection_id,
+        op_id=None,
+        service_id=None,
+        speculative_hello=False,
+    ):
         """Publish a CommandSucceededEvent to all command listeners.
 
         :Parameters:
@@ -1279,19 +1434,34 @@ class _EventListeners(object):
           - `connection_id`: The address (host, port) of the server this
             command was sent to.
           - `op_id`: The (optional) operation id for this operation.
+          - `service_id`: The service_id this command was sent to, or ``None``.
+          - `speculative_hello`: Was the command sent with speculative auth?
         """
         if op_id is None:
             op_id = request_id
+        if speculative_hello:
+            # Redact entire response when the command started contained
+            # speculativeAuthenticate.
+            reply = {}
         event = CommandSucceededEvent(
-            duration, reply, command_name, request_id, connection_id, op_id)
+            duration, reply, command_name, request_id, connection_id, op_id, service_id
+        )
         for subscriber in self.__command_listeners:
             try:
                 subscriber.succeeded(event)
             except Exception:
                 _handle_exception()
 
-    def publish_command_failure(self, duration, failure, command_name,
-                                request_id, connection_id, op_id=None):
+    def publish_command_failure(
+        self,
+        duration,
+        failure,
+        command_name,
+        request_id,
+        connection_id,
+        op_id=None,
+        service_id=None,
+    ):
         """Publish a CommandFailedEvent to all command listeners.
 
         :Parameters:
@@ -1303,11 +1473,13 @@ class _EventListeners(object):
           - `connection_id`: The address (host, port) of the server this
             command was sent to.
           - `op_id`: The (optional) operation id for this operation.
+          - `service_id`: The service_id this command was sent to, or ``None``.
         """
         if op_id is None:
             op_id = request_id
         event = CommandFailedEvent(
-            duration, failure, command_name, request_id, connection_id, op_id)
+            duration, failure, command_name, request_id, connection_id, op_id, service_id=service_id
+        )
         for subscriber in self.__command_listeners:
             try:
                 subscriber.failed(event)
@@ -1328,8 +1500,7 @@ class _EventListeners(object):
             except Exception:
                 _handle_exception()
 
-    def publish_server_heartbeat_succeeded(self, connection_id, duration,
-                                           reply, awaited):
+    def publish_server_heartbeat_succeeded(self, connection_id, duration, reply, awaited):
         """Publish a ServerHeartbeatSucceededEvent to all server heartbeat
         listeners.
 
@@ -1339,17 +1510,15 @@ class _EventListeners(object):
             resolution for the platform.
          - `reply`: The command reply.
          - `awaited`: True if the response was awaited.
-         """
-        event = ServerHeartbeatSucceededEvent(duration, reply, connection_id,
-                                              awaited)
+        """
+        event = ServerHeartbeatSucceededEvent(duration, reply, connection_id, awaited)
         for subscriber in self.__server_heartbeat_listeners:
             try:
                 subscriber.succeeded(event)
             except Exception:
                 _handle_exception()
 
-    def publish_server_heartbeat_failed(self, connection_id, duration, reply,
-                                        awaited):
+    def publish_server_heartbeat_failed(self, connection_id, duration, reply, awaited):
         """Publish a ServerHeartbeatFailedEvent to all server heartbeat
         listeners.
 
@@ -1359,9 +1528,8 @@ class _EventListeners(object):
             resolution for the platform.
          - `reply`: The command reply.
          - `awaited`: True if the response was awaited.
-         """
-        event = ServerHeartbeatFailedEvent(duration, reply, connection_id,
-                                           awaited)
+        """
+        event = ServerHeartbeatFailedEvent(duration, reply, connection_id, awaited)
         for subscriber in self.__server_heartbeat_listeners:
             try:
                 subscriber.failed(event)
@@ -1398,9 +1566,9 @@ class _EventListeners(object):
             except Exception:
                 _handle_exception()
 
-    def publish_server_description_changed(self, previous_description,
-                                           new_description, server_address,
-                                           topology_id):
+    def publish_server_description_changed(
+        self, previous_description, new_description, server_address, topology_id
+    ):
         """Publish a ServerDescriptionChangedEvent to all server listeners.
 
         :Parameters:
@@ -1410,9 +1578,9 @@ class _EventListeners(object):
          - `topology_id`: A unique identifier for the topology this server
            is a part of.
         """
-        event = ServerDescriptionChangedEvent(previous_description,
-                                              new_description, server_address,
-                                              topology_id)
+        event = ServerDescriptionChangedEvent(
+            previous_description, new_description, server_address, topology_id
+        )
         for subscriber in self.__server_listeners:
             try:
                 subscriber.description_changed(event)
@@ -1447,8 +1615,9 @@ class _EventListeners(object):
             except Exception:
                 _handle_exception()
 
-    def publish_topology_description_changed(self, previous_description,
-                                             new_description, topology_id):
+    def publish_topology_description_changed(
+        self, previous_description, new_description, topology_id
+    ):
         """Publish a TopologyDescriptionChangedEvent to all topology listeners.
 
         :Parameters:
@@ -1457,8 +1626,7 @@ class _EventListeners(object):
          - `topology_id`: A unique identifier for the topology this server
            is a part of.
         """
-        event = TopologyDescriptionChangedEvent(previous_description,
-                                                new_description, topology_id)
+        event = TopologyDescriptionChangedEvent(previous_description, new_description, topology_id)
         for subscriber in self.__topology_listeners:
             try:
                 subscriber.description_changed(event)
@@ -1466,8 +1634,7 @@ class _EventListeners(object):
                 _handle_exception()
 
     def publish_pool_created(self, address, options):
-        """Publish a :class:`PoolCreatedEvent` to all pool listeners.
-        """
+        """Publish a :class:`PoolCreatedEvent` to all pool listeners."""
         event = PoolCreatedEvent(address, options)
         for subscriber in self.__cmap_listeners:
             try:
@@ -1475,10 +1642,9 @@ class _EventListeners(object):
             except Exception:
                 _handle_exception()
 
-    def publish_pool_cleared(self, address):
-        """Publish a :class:`PoolClearedEvent` to all pool listeners.
-        """
-        event = PoolClearedEvent(address)
+    def publish_pool_cleared(self, address, service_id):
+        """Publish a :class:`PoolClearedEvent` to all pool listeners."""
+        event = PoolClearedEvent(address, service_id)
         for subscriber in self.__cmap_listeners:
             try:
                 subscriber.pool_cleared(event)
@@ -1486,8 +1652,7 @@ class _EventListeners(object):
                 _handle_exception()
 
     def publish_pool_closed(self, address):
-        """Publish a :class:`PoolClosedEvent` to all pool listeners.
-        """
+        """Publish a :class:`PoolClosedEvent` to all pool listeners."""
         event = PoolClosedEvent(address)
         for subscriber in self.__cmap_listeners:
             try:
@@ -1507,8 +1672,7 @@ class _EventListeners(object):
                 _handle_exception()
 
     def publish_connection_ready(self, address, connection_id):
-        """Publish a :class:`ConnectionReadyEvent` to all connection listeners.
-        """
+        """Publish a :class:`ConnectionReadyEvent` to all connection listeners."""
         event = ConnectionReadyEvent(address, connection_id)
         for subscriber in self.__cmap_listeners:
             try:
