@@ -15,7 +15,9 @@
 """Time. Monotonic if possible.
 """
 
-__all__ = ['time']
+from __future__ import absolute_import
+
+__all__ = ["time"]
 
 try:
     # Patches standard time module.
@@ -25,8 +27,12 @@ except ImportError:
     pass
 
 try:
-    # Monotime or Python 3.3+.
-    from time import monotonic as time
+    # From https://pypi.python.org/pypi/monotonic.
+    from monotonic import monotonic as time
 except ImportError:
-    # Not monotonic.
-    from time import time
+    try:
+        # Monotime or Python 3.
+        from time import monotonic as time
+    except ImportError:
+        # Not monotonic.
+        from time import time
