@@ -27,29 +27,33 @@ from pymongo.topology_description import TOPOLOGY_TYPE
 
 
 class TopologySettings(object):
-    def __init__(self,
-                 seeds=None,
-                 replica_set_name=None,
-                 pool_class=None,
-                 pool_options=None,
-                 monitor_class=None,
-                 condition_class=None,
-                 local_threshold_ms=LOCAL_THRESHOLD_MS,
-                 server_selection_timeout=SERVER_SELECTION_TIMEOUT,
-                 heartbeat_frequency=common.HEARTBEAT_FREQUENCY,
-                 server_selector=None,
-                 fqdn=None,
-                 direct_connection=None):
+    def __init__(
+        self,
+        seeds=None,
+        replica_set_name=None,
+        pool_class=None,
+        pool_options=None,
+        monitor_class=None,
+        condition_class=None,
+        local_threshold_ms=LOCAL_THRESHOLD_MS,
+        server_selection_timeout=SERVER_SELECTION_TIMEOUT,
+        heartbeat_frequency=common.HEARTBEAT_FREQUENCY,
+        server_selector=None,
+        fqdn=None,
+        direct_connection=None,
+        load_balanced=None,
+    ):
         """Represent MongoClient's configuration.
 
         Take a list of (host, port) pairs and optional replica set name.
         """
         if heartbeat_frequency < common.MIN_HEARTBEAT_INTERVAL:
             raise ConfigurationError(
-                "heartbeatFrequencyMS cannot be less than %d" % (
-                    common.MIN_HEARTBEAT_INTERVAL * 1000,))
+                "heartbeatFrequencyMS cannot be less than %d"
+                % (common.MIN_HEARTBEAT_INTERVAL * 1000,)
+            )
 
-        self._seeds = seeds or [('localhost', 27017)]
+        self._seeds = seeds or [("localhost", 27017)]
         self._replica_set_name = replica_set_name
         self._pool_class = pool_class or pool.Pool
         self._pool_options = pool_options or PoolOptions()
@@ -62,14 +66,15 @@ class TopologySettings(object):
         self._heartbeat_frequency = heartbeat_frequency
 
         if direct_connection is None:
-            self._direct = (len(self._seeds) == 1 and not self.replica_set_name)
+            self._direct = len(self._seeds) == 1 and not self.replica_set_name
         else:
             self._direct = direct_connection
+        self._load_balanced = load_balanced
 
         self._topology_id = ObjectId()
         # Store the allocation traceback to catch unclosed clients in the
         # test suite.
-        self._stack = ''.join(traceback.format_stack())
+        self._stack = "".join(traceback.format_stack())
 
     @property
     def seeds(self):
@@ -124,8 +129,15 @@ class TopologySettings(object):
         """
         return self._direct
 
+    @property
+    def load_balanced(self):
+        """True if the client was configured to connect to a load balancer."""
+        return self._load_balanced
+
     def get_topology_type(self):
-        if self.direct:
+        if self.load_balanced:
+            return TOPOLOGY_TYPE.LoadBalanced
+        elif self.direct:
             return TOPOLOGY_TYPE.Single
         elif self.replica_set_name is not None:
             return TOPOLOGY_TYPE.ReplicaSetNoPrimary
@@ -134,6 +146,4 @@ class TopologySettings(object):
 
     def get_server_descriptions(self):
         """Initial dict of (address, ServerDescription) for all seeds."""
-        return dict([
-            (address, ServerDescription(address))
-            for address in self.seeds])
+        return dict([(address, ServerDescription(address)) for address in self.seeds])
