@@ -15,13 +15,14 @@
 """Utilities for multi-threading support."""
 
 import threading
+
 try:
     from time import monotonic as _time
 except ImportError:
     from time import time as _time
 
-from pymongo.monotonic import time as _time
 from pymongo.errors import ExceededMaxWaiters
+from pymongo.monotonic import time as _time
 
 
 ### Begin backport from CPython 3.2 for timeout support for Semaphore.acquire
@@ -74,6 +75,7 @@ class Semaphore:
 
 class BoundedSemaphore(Semaphore):
     """Semaphore that checks that # releases is <= # acquires"""
+
     def __init__(self, value=1):
         Semaphore.__init__(self, value)
         self._initial_value = value
@@ -82,6 +84,8 @@ class BoundedSemaphore(Semaphore):
         if self._value >= self._initial_value:
             raise ValueError("Semaphore released too many times")
         return Semaphore.release(self)
+
+
 ### End backport from CPython 3.2
 
 
@@ -115,8 +119,7 @@ class MaxWaitersBoundedSemaphore(object):
 
 class MaxWaitersBoundedSemaphoreThread(MaxWaitersBoundedSemaphore):
     def __init__(self, value=1, max_waiters=1):
-        MaxWaitersBoundedSemaphore.__init__(
-            self, BoundedSemaphore, value, max_waiters)
+        MaxWaitersBoundedSemaphore.__init__(self, BoundedSemaphore, value, max_waiters)
 
 
 def create_semaphore(max_size, max_waiters):
