@@ -20,37 +20,45 @@ import threading
 import warnings
 import weakref
 
-from bson.py3compat import itervalues, PY3
+from bson.py3compat import PY3, itervalues
+
 if PY3:
     import queue as Queue
 else:
     import Queue
 
-from pymongo import (common,
-                     helpers,
-                     periodic_executor)
-from pymongo.pool import PoolOptions
-from pymongo.topology_description import (updated_topology_description,
-                                          _updated_topology_description_srv_polling,
-                                          TopologyDescription,
-                                          SRV_POLLING_TOPOLOGIES, TOPOLOGY_TYPE)
-from pymongo.errors import (ConnectionFailure,
-                            ConfigurationError,
-                            NetworkTimeout,
-                            NotMasterError,
-                            OperationFailure,
-                            ServerSelectionTimeoutError)
+from pymongo import common, helpers, periodic_executor
+from pymongo.client_session import _ServerSessionPool
+from pymongo.errors import (
+    ConfigurationError,
+    ConnectionFailure,
+    NetworkTimeout,
+    NotPrimaryError,
+    OperationFailure,
+    ServerSelectionTimeoutError,
+    WriteError,
+)
+from pymongo.ismaster import IsMaster
 from pymongo.monitor import SrvMonitor
 from pymongo.monotonic import time as _time
+from pymongo.pool import PoolOptions
 from pymongo.server import Server
 from pymongo.server_description import ServerDescription
-from pymongo.server_selectors import (any_server_selector,
-                                      arbiter_server_selector,
-                                      secondary_server_selector,
-                                      readable_server_selector,
-                                      writable_server_selector,
-                                      Selection)
-from pymongo.client_session import _ServerSessionPool
+from pymongo.server_selectors import (
+    Selection,
+    any_server_selector,
+    arbiter_server_selector,
+    readable_server_selector,
+    secondary_server_selector,
+    writable_server_selector,
+)
+from pymongo.topology_description import (
+    SRV_POLLING_TOPOLOGIES,
+    TOPOLOGY_TYPE,
+    TopologyDescription,
+    _updated_topology_description_srv_polling,
+    updated_topology_description,
+)
 
 
 def process_events_queue(queue_ref):
@@ -72,9 +80,10 @@ def process_events_queue(queue_ref):
 
 class Topology(object):
     """Monitor a topology of one or more servers."""
+
     def __init__(self, topology_settings):
         self._topology_id = topology_settings._topology_id
-        self._listeners = topology_settings._pool_options.event_listeners
+        self._listeners = topology_settings._pool_options._event_listeners
         pub = self._listeners is not None
         self._publish_server = pub and self._listeners.enabled_for_server
         self._publish_tp = pub and self._listeners.enabled_for_topology
@@ -87,8 +96,7 @@ class Topology(object):
             self._events = Queue.Queue(maxsize=100)
 
         if self._publish_tp:
-            self._events.put((self._listeners.publish_topology_opened,
-                             (self._topology_id,)))
+            self._events.put((self._listeners.publish_topology_opened, (self._topology_id,)))
         self._settings = topology_settings
         topology_description = TopologyDescription(
             topology_settings.get_topology_type(),
@@ -96,20 +104,24 @@ class Topology(object):
             topology_settings.replica_set_name,
             None,
             None,
-            topology_settings)
+            topology_settings,
+        )
 
         self._description = topology_description
         if self._publish_tp:
-            initial_td = TopologyDescription(TOPOLOGY_TYPE.Unknown, {}, None,
-                                             None, None, self._settings)
-            self._events.put((
-                self._listeners.publish_topology_description_changed,
-                (initial_td, self._description, self._topology_id)))
+            initial_td = TopologyDescription(
+                TOPOLOGY_TYPE.Unknown, {}, None, None, None, self._settings
+            )
+            self._events.put(
+                (
+                    self._listeners.publish_topology_description_changed,
+                    (initial_td, self._description, self._topology_id),
+                )
+            )
 
         for seed in topology_settings.seeds:
             if self._publish_server:
-                self._events.put((self._listeners.publish_server_opened,
-                                 (seed, self._topology_id)))
+                self._events.put((self._listeners.publish_server_opened, (seed, self._topology_id)))
 
         # Store the seed list to help diagnose errors in _error_message().
         self._seed_addresses = list(topology_description.server_descriptions())
@@ -122,6 +134,7 @@ class Topology(object):
         self._session_pool = _ServerSessionPool()
 
         if self._publish_server or self._publish_tp:
+
             def target():
                 return process_events_queue(weak)
 
@@ -129,7 +142,8 @@ class Topology(object):
                 interval=common.EVENTS_QUEUE_FREQUENCY,
                 min_interval=0.5,
                 target=target,
-                name="pymongo_events_thread")
+                name="pymongo_events_thread",
+            )
 
             # We strongly reference the executor and it weakly references
             # the queue via this closure. When the topology is freed, stop
@@ -139,7 +153,7 @@ class Topology(object):
             executor.open()
 
         self._srv_monitor = None
-        if self._settings.fqdn is not None:
+        if self._settings.fqdn is not None and not self._settings.load_balanced:
             self._srv_monitor = SrvMonitor(self, self._settings)
 
     def open(self):
@@ -162,7 +176,8 @@ class Topology(object):
                     "MongoClient opened before fork. Create MongoClient only "
                     "after forking. See PyMongo's documentation for details: "
                     "https://pymongo.readthedocs.io/en/stable/faq.html#"
-                    "is-pymongo-fork-safe")
+                    "is-pymongo-fork-safe"
+                )
                 with self._lock:
                     # Reset the session pool to avoid duplicate sessions in
                     # the child process.
@@ -171,10 +186,7 @@ class Topology(object):
         with self._lock:
             self._ensure_opened()
 
-    def select_servers(self,
-                       selector,
-                       server_selection_timeout=None,
-                       address=None):
+    def select_servers(self, selector, server_selection_timeout=None, address=None):
         """Return a list of Servers matching selector, or time out.
 
         :Parameters:
@@ -196,25 +208,25 @@ class Topology(object):
             server_timeout = server_selection_timeout
 
         with self._lock:
-            server_descriptions = self._select_servers_loop(
-                selector, server_timeout, address)
+            server_descriptions = self._select_servers_loop(selector, server_timeout, address)
 
-            return [self.get_server_by_address(sd.address)
-                    for sd in server_descriptions]
+            return [self.get_server_by_address(sd.address) for sd in server_descriptions]
 
     def _select_servers_loop(self, selector, timeout, address):
         """select_servers() guts. Hold the lock when calling this."""
         now = _time()
         end_time = now + timeout
         server_descriptions = self._description.apply_selector(
-            selector, address, custom_selector=self._settings.server_selector)
+            selector, address, custom_selector=self._settings.server_selector
+        )
 
         while not server_descriptions:
             # No suitable servers.
             if timeout == 0 or now > end_time:
                 raise ServerSelectionTimeoutError(
-                    "%s, Timeout: %ss, Topology Description: %r" %
-                    (self._error_message(selector), timeout, self.description))
+                    "%s, Timeout: %ss, Topology Description: %r"
+                    % (self._error_message(selector), timeout, self.description)
+                )
 
             self._ensure_opened()
             self._request_check_all()
@@ -227,23 +239,17 @@ class Topology(object):
             self._description.check_compatible()
             now = _time()
             server_descriptions = self._description.apply_selector(
-                selector, address,
-                custom_selector=self._settings.server_selector)
+                selector, address, custom_selector=self._settings.server_selector
+            )
 
         self._description.check_compatible()
         return server_descriptions
 
-    def select_server(self,
-                      selector,
-                      server_selection_timeout=None,
-                      address=None):
+    def select_server(self, selector, server_selection_timeout=None, address=None):
         """Like select_servers, but choose a random server if several match."""
-        return random.choice(self.select_servers(selector,
-                                                 server_selection_timeout,
-                                                 address))
+        return random.choice(self.select_servers(selector, server_selection_timeout, address))
 
-    def select_server_by_address(self, address,
-                                 server_selection_timeout=None):
+    def select_server_by_address(self, address, server_selection_timeout=None):
         """Return a Server for "address", reconnecting if necessary.
 
         If the server's type is not known, request an immediate check of all
@@ -261,9 +267,7 @@ class Topology(object):
         Raises exc:`ServerSelectionTimeoutError` after
         `server_selection_timeout` if no matching servers are found.
         """
-        return self.select_server(any_server_selector,
-                                  server_selection_timeout,
-                                  address)
+        return self.select_server(any_server_selector, server_selection_timeout, address)
 
     def _process_change(self, server_description, reset_pool=False):
         """Process a new ServerDescription on an opened topology.
@@ -273,34 +277,38 @@ class Topology(object):
         td_old = self._description
         sd_old = td_old._server_descriptions[server_description.address]
         if _is_stale_server_description(sd_old, server_description):
-            # This is a stale isMaster response. Ignore it.
+            # This is a stale hello response. Ignore it.
             return
 
-        suppress_event = ((self._publish_server or self._publish_tp)
-                          and sd_old == server_description)
+        suppress_event = (self._publish_server or self._publish_tp) and sd_old == server_description
         if self._publish_server and not suppress_event:
-            self._events.put((
-                self._listeners.publish_server_description_changed,
-                (sd_old, server_description,
-                 server_description.address, self._topology_id)))
+            self._events.put(
+                (
+                    self._listeners.publish_server_description_changed,
+                    (sd_old, server_description, server_description.address, self._topology_id),
+                )
+            )
 
-        self._description = updated_topology_description(
-            self._description, server_description)
+        self._description = updated_topology_description(self._description, server_description)
 
         self._update_servers()
         self._receive_cluster_time_no_lock(server_description.cluster_time)
 
         if self._publish_tp and not suppress_event:
-            self._events.put((
-                self._listeners.publish_topology_description_changed,
-                (td_old, self._description, self._topology_id)))
+            self._events.put(
+                (
+                    self._listeners.publish_topology_description_changed,
+                    (td_old, self._description, self._topology_id),
+                )
+            )
 
         # Shutdown SRV polling for unsupported cluster types.
         # This is only applicable if the old topology was Unknown, and the
         # new one is something other than Unknown or Sharded.
-        if self._srv_monitor and (td_old.topology_type == TOPOLOGY_TYPE.Unknown
-                                  and self._description.topology_type not in
-                                  SRV_POLLING_TOPOLOGIES):
+        if self._srv_monitor and (
+            td_old.topology_type == TOPOLOGY_TYPE.Unknown
+            and self._description.topology_type not in SRV_POLLING_TOPOLOGIES
+        ):
             self._srv_monitor.close()
 
         # Clear the pool from a failed heartbeat.
@@ -313,10 +321,10 @@ class Topology(object):
         self._condition.notify_all()
 
     def on_change(self, server_description, reset_pool=False):
-        """Process a new ServerDescription after an ismaster call completes."""
+        """Process a new ServerDescription after a hello call completes."""
         # We do no I/O holding the lock.
         with self._lock:
-            # Monitors may continue working on ismaster calls for some time
+            # Monitors may continue working on hello calls for some time
             # after a call to Topology.close, so this method may be called at
             # any time. Ensure the topology is open before processing the
             # change.
@@ -324,8 +332,7 @@ class Topology(object):
             # once. Check if it's still in the description or if some state-
             # change removed it. E.g., we got a host list from the primary
             # that didn't include this server.
-            if (self._opened and
-                    self._description.has_server(server_description.address)):
+            if self._opened and self._description.has_server(server_description.address):
                 self._process_change(server_description, reset_pool)
 
     def _process_srv_update(self, seedlist):
@@ -333,15 +340,19 @@ class Topology(object):
         Hold the lock when calling this.
         """
         td_old = self._description
-        self._description = _updated_topology_description_srv_polling(
-            self._description, seedlist)
+        if td_old.topology_type not in SRV_POLLING_TOPOLOGIES:
+            return
+        self._description = _updated_topology_description_srv_polling(self._description, seedlist)
 
         self._update_servers()
 
         if self._publish_tp:
-            self._events.put((
-                self._listeners.publish_topology_description_changed,
-                (td_old, self._description, self._topology_id)))
+            self._events.put(
+                (
+                    self._listeners.publish_topology_description_changed,
+                    (td_old, self._description, self._topology_id),
+                )
+            )
 
     def on_srv_update(self, seedlist):
         """Process a new list of nodes obtained from scanning SRV records."""
@@ -378,8 +389,10 @@ class Topology(object):
         # Implemented here in Topology instead of MongoClient, so it can lock.
         with self._lock:
             topology_type = self._description.topology_type
-            if topology_type not in (TOPOLOGY_TYPE.ReplicaSetWithPrimary,
-                                     TOPOLOGY_TYPE.ReplicaSetNoPrimary):
+            if topology_type not in (
+                TOPOLOGY_TYPE.ReplicaSetWithPrimary,
+                TOPOLOGY_TYPE.ReplicaSetNoPrimary,
+            ):
                 return set()
 
             return set([sd.address for sd in selector(self._new_selection())])
@@ -405,9 +418,10 @@ class Topology(object):
         # value of the clusterTime embedded field."
         if cluster_time:
             # ">" uses bson.timestamp.Timestamp's comparison operator.
-            if (not self._max_cluster_time
-                or cluster_time['clusterTime'] >
-                    self._max_cluster_time['clusterTime']):
+            if (
+                not self._max_cluster_time
+                or cluster_time["clusterTime"] > self._max_cluster_time["clusterTime"]
+            ):
                 self._max_cluster_time = cluster_time
 
     def receive_cluster_time(self, cluster_time):
@@ -422,23 +436,33 @@ class Topology(object):
 
     def handle_getlasterror(self, address, error_msg):
         """Clear our pool for a server, mark it Unknown, and check it soon."""
-        error = NotMasterError(error_msg, {'code': 10107, 'errmsg': error_msg})
+        error = NotPrimaryError(error_msg, {"code": 10107, "errmsg": error_msg})
         with self._lock:
             server = self._servers.get(address)
             if server:
-                self._process_change(
-                    ServerDescription(address, error=error), True)
+                self._process_change(ServerDescription(address, error=error), True)
                 server.request_check()
 
+    def data_bearing_servers(self):
+        """Return a list of all data-bearing servers.
+
+        This includes any server that might be selected for an operation.
+        """
+        if self._description.topology_type == TOPOLOGY_TYPE.Single:
+            return self._description.known_servers
+        return self._description.readable_servers
+
     def update_pool(self, all_credentials):
         # Remove any stale sockets and add new sockets if pool is too small.
         servers = []
         with self._lock:
-            for server in self._servers.values():
-                servers.append((server, server._pool.generation))
+            # Only update pools for data-bearing servers.
+            for sd in self.data_bearing_servers():
+                server = self._servers[sd.address]
+                servers.append((server, server.pool.gen.get_overall()))
 
         for server, generation in servers:
-            server._pool.remove_stale_sockets(generation, all_credentials)
+            server.pool.remove_stale_sockets(generation, all_credentials)
 
     def close(self):
         """Clear pools and terminate monitors. Topology reopens on demand."""
@@ -460,8 +484,7 @@ class Topology(object):
 
         # Publish only after releasing the lock.
         if self._publish_tp:
-            self._events.put((self._listeners.publish_topology_closed,
-                              (self._topology_id,)))
+            self._events.put((self._listeners.publish_topology_closed, (self._topology_id,)))
         if self._publish_server or self._publish_tp:
             self.__events_executor.close()
 
@@ -474,39 +497,43 @@ class Topology(object):
         with self._lock:
             return self._session_pool.pop_all()
 
+    def _check_session_support(self):
+        """Internal check for session support on non-load balanced clusters."""
+        session_timeout = self._description.logical_session_timeout_minutes
+        if session_timeout is None:
+            # Maybe we need an initial scan? Can raise ServerSelectionError.
+            if self._description.topology_type == TOPOLOGY_TYPE.Single:
+                if not self._description.has_known_servers:
+                    self._select_servers_loop(
+                        any_server_selector, self._settings.server_selection_timeout, None
+                    )
+            elif not self._description.readable_servers:
+                self._select_servers_loop(
+                    readable_server_selector, self._settings.server_selection_timeout, None
+                )
+
+            session_timeout = self._description.logical_session_timeout_minutes
+            if session_timeout is None:
+                raise ConfigurationError("Sessions are not supported by this MongoDB deployment")
+        return session_timeout
+
     def get_server_session(self):
         """Start or resume a server session, or raise ConfigurationError."""
         with self._lock:
-            session_timeout = self._description.logical_session_timeout_minutes
-            if session_timeout is None:
-                # Maybe we need an initial scan? Can raise ServerSelectionError.
-                if self._description.topology_type == TOPOLOGY_TYPE.Single:
-                    if not self._description.has_known_servers:
-                        self._select_servers_loop(
-                            any_server_selector,
-                            self._settings.server_selection_timeout,
-                            None)
-                elif not self._description.readable_servers:
-                    self._select_servers_loop(
-                        readable_server_selector,
-                        self._settings.server_selection_timeout,
-                        None)
-
-            session_timeout = self._description.logical_session_timeout_minutes
-            if session_timeout is None:
-                raise ConfigurationError(
-                    "Sessions are not supported by this MongoDB deployment")
-
+            # Sessions are always supported in load balanced mode.
+            if not self._settings.load_balanced:
+                session_timeout = self._check_session_support()
+            else:
+                # Sessions never time out in load balanced mode.
+                session_timeout = float("inf")
             return self._session_pool.get_server_session(session_timeout)
 
     def return_server_session(self, server_session, lock):
         if lock:
             with self._lock:
-                session_timeout = \
-                    self._description.logical_session_timeout_minutes
-                if session_timeout is not None:
-                    self._session_pool.return_server_session(server_session,
-                                                             session_timeout)
+                self._session_pool.return_server_session(
+                    server_session, self._description.logical_session_timeout_minutes
+                )
         else:
             # Called from a __del__ method, can't use a lock.
             self._session_pool.return_server_session_no_lock(server_session)
@@ -532,10 +559,18 @@ class Topology(object):
                 self.__events_executor.open()
 
             # Start the SRV polling thread.
-            if self._srv_monitor and (self.description.topology_type in
-                                      SRV_POLLING_TOPOLOGIES):
+            if self._srv_monitor and (self.description.topology_type in SRV_POLLING_TOPOLOGIES):
                 self._srv_monitor.open()
 
+            if self._settings.load_balanced:
+                # Emit initial SDAM events for load balancer mode.
+                self._process_change(
+                    ServerDescription(
+                        self._seed_addresses[0],
+                        IsMaster({"ok": 1, "serviceId": self._topology_id, "maxWireVersion": 13}),
+                    )
+                )
+
         # Ensure that the monitors are open.
         for server in itervalues(self._servers):
             server.open()
@@ -546,7 +581,7 @@ class Topology(object):
             # Another thread removed this server from the topology.
             return True
 
-        if err_ctx.sock_generation != server._pool.generation:
+        if server._pool.stale_generation(err_ctx.sock_generation, err_ctx.service_id):
             # This is an outdated error from a previous pool version.
             return True
 
@@ -554,9 +589,9 @@ class Topology(object):
         cur_tv = server.description.topology_version
         error = err_ctx.error
         error_tv = None
-        if error and hasattr(error, 'details'):
+        if error and hasattr(error, "details"):
             if isinstance(error.details, dict):
-                error_tv = error.details.get('topologyVersion')
+                error_tv = error.details.get("topologyVersion")
 
         return _is_stale_error_topology_version(cur_tv, error_tv)
 
@@ -567,47 +602,53 @@ class Topology(object):
         server = self._servers[address]
         error = err_ctx.error
         exc_type = type(error)
-        if (issubclass(exc_type, NetworkTimeout) and
-                err_ctx.completed_handshake):
+        service_id = err_ctx.service_id
+        if issubclass(exc_type, NetworkTimeout) and err_ctx.completed_handshake:
             # The socket has been closed. Don't reset the server.
             # Server Discovery And Monitoring Spec: "When an application
             # operation fails because of any network error besides a socket
             # timeout...."
             return
-        elif issubclass(exc_type, NotMasterError):
+        elif issubclass(exc_type, WriteError):
+            # Ignore writeErrors.
+            return
+        elif issubclass(exc_type, NotPrimaryError):
             # As per the SDAM spec if:
-            #   - the server sees a "not master" error, and
+            #   - the server sees a "not primary" error, and
             #   - the server is not shutting down, and
             #   - the server version is >= 4.2, then
             # we keep the existing connection pool, but mark the server type
             # as Unknown and request an immediate check of the server.
             # Otherwise, we clear the connection pool, mark the server as
             # Unknown and request an immediate check of the server.
-            err_code = error.details.get('code', -1)
+            err_code = error.details.get("code", -1)
             is_shutting_down = err_code in helpers._SHUTDOWN_CODES
             # Mark server Unknown, clear the pool, and request check.
-            self._process_change(ServerDescription(address, error=error))
+            if not self._settings.load_balanced:
+                self._process_change(ServerDescription(address, error=error))
             if is_shutting_down or (err_ctx.max_wire_version <= 7):
                 # Clear the pool.
-                server.reset()
+                server.reset(service_id)
             server.request_check()
         elif issubclass(exc_type, ConnectionFailure):
             # "Client MUST replace the server's description with type Unknown
             # ... MUST NOT request an immediate check of the server."
-            self._process_change(ServerDescription(address, error=error))
+            if not self._settings.load_balanced:
+                self._process_change(ServerDescription(address, error=error))
             # Clear the pool.
-            server.reset()
+            server.reset(service_id)
             # "When a client marks a server Unknown from `Network error when
-            # reading or writing`_, clients MUST cancel the isMaster check on
+            # reading or writing`_, clients MUST cancel the hello check on
             # that server and close the current monitoring connection."
             server._monitor.cancel_check()
         elif issubclass(exc_type, OperationFailure):
             # Do not request an immediate check since the server is likely
             # shutting down.
             if error.code in helpers._NOT_MASTER_CODES:
-                self._process_change(ServerDescription(address, error=error))
+                if not self._settings.load_balanced:
+                    self._process_change(ServerDescription(address, error=error))
                 # Clear the pool.
-                server.reset()
+                server.reset(service_id)
 
     def handle_error(self, address, err_ctx):
         """Handle an application error.
@@ -634,7 +675,8 @@ class Topology(object):
                     server_description=sd,
                     topology=self,
                     pool=self._create_pool_for_monitor(address),
-                    topology_settings=self._settings)
+                    topology_settings=self._settings,
+                )
 
                 weak = None
                 if self._publish_server:
@@ -645,7 +687,8 @@ class Topology(object):
                     monitor=monitor,
                     topology_id=self._topology_id,
                     listeners=self._listeners,
-                    events=weak)
+                    events=weak,
+                )
 
                 self._servers[address] = server
                 server.open()
@@ -656,8 +699,7 @@ class Topology(object):
                 self._servers[address].description = sd
                 # Update is_writable value of the pool, if it changed.
                 if was_writable != sd.is_writable:
-                    self._servers[address].pool.update_is_writable(
-                        sd.is_writable)
+                    self._servers[address].pool.update_is_writable(sd.is_writable)
 
         for address, server in list(self._servers.items()):
             if not self._description.has_server(address):
@@ -676,14 +718,15 @@ class Topology(object):
         monitor_pool_options = PoolOptions(
             connect_timeout=options.connect_timeout,
             socket_timeout=options.connect_timeout,
-            ssl_context=options.ssl_context,
+            ssl_context=options._ssl_context,
             ssl_match_hostname=options.ssl_match_hostname,
-            event_listeners=options.event_listeners,
+            event_listeners=options._event_listeners,
             appname=options.appname,
-            driver=options.driver)
+            driver=options.driver,
+            server_api=options.server_api,
+        )
 
-        return self._settings.pool_class(address, monitor_pool_options,
-                                         handshake=False)
+        return self._settings.pool_class(address, monitor_pool_options, handshake=False)
 
     def _error_message(self, selector):
         """Format an error message if server selection fails.
@@ -692,22 +735,23 @@ class Topology(object):
         """
         is_replica_set = self._description.topology_type in (
             TOPOLOGY_TYPE.ReplicaSetWithPrimary,
-            TOPOLOGY_TYPE.ReplicaSetNoPrimary)
+            TOPOLOGY_TYPE.ReplicaSetNoPrimary,
+        )
 
         if is_replica_set:
-            server_plural = 'replica set members'
+            server_plural = "replica set members"
         elif self._description.topology_type == TOPOLOGY_TYPE.Sharded:
-            server_plural = 'mongoses'
+            server_plural = "mongoses"
         else:
-            server_plural = 'servers'
+            server_plural = "servers"
 
         if self._description.known_servers:
             # We've connected, but no servers match the selector.
             if selector is writable_server_selector:
                 if is_replica_set:
-                    return 'No primary available for writes'
+                    return "No primary available for writes"
                 else:
-                    return 'No %s available for writes' % server_plural
+                    return "No %s available for writes" % server_plural
             else:
                 return 'No %s match selector "%s"' % (server_plural, selector)
         else:
@@ -717,9 +761,11 @@ class Topology(object):
                 if is_replica_set:
                     # We removed all servers because of the wrong setName?
                     return 'No %s available for replica set name "%s"' % (
-                        server_plural, self._settings.replica_set_name)
+                        server_plural,
+                        self._settings.replica_set_name,
+                    )
                 else:
-                    return 'No %s available' % server_plural
+                    return "No %s available" % server_plural
 
             # 1 or more servers, all Unknown. Are they unknown for one reason?
             error = servers[0].error
@@ -727,45 +773,44 @@ class Topology(object):
             if same:
                 if error is None:
                     # We're still discovering.
-                    return 'No %s found yet' % server_plural
+                    return "No %s found yet" % server_plural
 
-                if (is_replica_set and not
-                        set(addresses).intersection(self._seed_addresses)):
+                if is_replica_set and not set(addresses).intersection(self._seed_addresses):
                     # We replaced our seeds with new hosts but can't reach any.
                     return (
-                        'Could not reach any servers in %s. Replica set is'
-                        ' configured with internal hostnames or IPs?' %
-                        addresses)
+                        "Could not reach any servers in %s. Replica set is"
+                        " configured with internal hostnames or IPs?" % addresses
+                    )
 
                 return str(error)
             else:
-                return ','.join(str(server.error) for server in servers
-                                if server.error)
+                return ",".join(str(server.error) for server in servers if server.error)
 
     def __repr__(self):
-        msg = ''
+        msg = ""
         if not self._opened:
-            msg = 'CLOSED '
-        return '<%s %s%r>' % (self.__class__.__name__, msg, self._description)
+            msg = "CLOSED "
+        return "<%s %s%r>" % (self.__class__.__name__, msg, self._description)
 
 
 class _ErrorContext(object):
     """An error with context for SDAM error handling."""
-    def __init__(self, error, max_wire_version, sock_generation,
-                 completed_handshake):
+
+    def __init__(self, error, max_wire_version, sock_generation, completed_handshake, service_id):
         self.error = error
         self.max_wire_version = max_wire_version
         self.sock_generation = sock_generation
         self.completed_handshake = completed_handshake
+        self.service_id = service_id
 
 
 def _is_stale_error_topology_version(current_tv, error_tv):
     """Return True if the error's topologyVersion is <= current."""
     if current_tv is None or error_tv is None:
         return False
-    if current_tv['processId'] != error_tv['processId']:
+    if current_tv["processId"] != error_tv["processId"]:
         return False
-    return current_tv['counter'] >= error_tv['counter']
+    return current_tv["counter"] >= error_tv["counter"]
 
 
 def _is_stale_server_description(current_sd, new_sd):
@@ -773,6 +818,6 @@ def _is_stale_server_description(current_sd, new_sd):
     current_tv, new_tv = current_sd.topology_version, new_sd.topology_version
     if current_tv is None or new_tv is None:
         return False
-    if current_tv['processId'] != new_tv['processId']:
+    if current_tv["processId"] != new_tv["processId"]:
         return False
-    return current_tv['counter'] > new_tv['counter']
+    return current_tv["counter"] > new_tv["counter"]
