@@ -23,24 +23,33 @@ from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import Selection
 from pymongo.server_type import SERVER_TYPE
 
-
 # Enumeration for various kinds of MongoDB cluster topologies.
-TOPOLOGY_TYPE = namedtuple('TopologyType', ['Single', 'ReplicaSetNoPrimary',
-                                            'ReplicaSetWithPrimary', 'Sharded',
-                                            'Unknown'])(*range(5))
+TOPOLOGY_TYPE = namedtuple(
+    "TopologyType",
+    [
+        "Single",
+        "ReplicaSetNoPrimary",
+        "ReplicaSetWithPrimary",
+        "Sharded",
+        "Unknown",
+        "LoadBalanced",
+    ],
+)(*range(6))
 
 # Topologies compatible with SRV record polling.
 SRV_POLLING_TOPOLOGIES = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.Sharded)
 
 
 class TopologyDescription(object):
-    def __init__(self,
-                 topology_type,
-                 server_descriptions,
-                 replica_set_name,
-                 max_set_version,
-                 max_election_id,
-                 topology_settings):
+    def __init__(
+        self,
+        topology_type,
+        server_descriptions,
+        replica_set_name,
+        max_set_version,
+        max_election_id,
+        topology_settings,
+    ):
         """Representation of a deployment of MongoDB servers.
 
         :Parameters:
@@ -63,7 +72,28 @@ class TopologyDescription(object):
 
         # Is PyMongo compatible with all servers' wire protocols?
         self._incompatible_err = None
+        if self._topology_type != TOPOLOGY_TYPE.LoadBalanced:
+            self._init_incompatible_err()
 
+        # Server Discovery And Monitoring Spec: Whenever a client updates the
+        # TopologyDescription from a hello response, it MUST set
+        # TopologyDescription.logicalSessionTimeoutMinutes to the smallest
+        # logicalSessionTimeoutMinutes value among ServerDescriptions of all
+        # data-bearing server types. If any have a null
+        # logicalSessionTimeoutMinutes, then
+        # TopologyDescription.logicalSessionTimeoutMinutes MUST be set to null.
+        readable_servers = self.readable_servers
+        if not readable_servers:
+            self._ls_timeout_minutes = None
+        elif any(s.logical_session_timeout_minutes is None for s in readable_servers):
+            self._ls_timeout_minutes = None
+        else:
+            self._ls_timeout_minutes = min(
+                s.logical_session_timeout_minutes for s in readable_servers
+            )
+
+    def _init_incompatible_err(self):
+        """Internal compatibility check for non-load balanced topologies."""
         for s in self._server_descriptions.values():
             if not s.is_server_type_known:
                 continue
@@ -73,48 +103,42 @@ class TopologyDescription(object):
             server_too_new = (
                 # Server too new.
                 s.min_wire_version is not None
-                and s.min_wire_version > common.MAX_SUPPORTED_WIRE_VERSION)
+                and s.min_wire_version > common.MAX_SUPPORTED_WIRE_VERSION
+            )
 
             server_too_old = (
                 # Server too old.
                 s.max_wire_version is not None
-                and s.max_wire_version < common.MIN_SUPPORTED_WIRE_VERSION)
+                and s.max_wire_version < common.MIN_SUPPORTED_WIRE_VERSION
+            )
 
             if server_too_new:
                 self._incompatible_err = (
                     "Server at %s:%d requires wire version %d, but this "
                     "version of PyMongo only supports up to %d."
-                    % (s.address[0], s.address[1],
-                       s.min_wire_version, common.MAX_SUPPORTED_WIRE_VERSION))
+                    % (
+                        s.address[0],
+                        s.address[1],
+                        s.min_wire_version,
+                        common.MAX_SUPPORTED_WIRE_VERSION,
+                    )
+                )
 
             elif server_too_old:
                 self._incompatible_err = (
                     "Server at %s:%d reports wire version %d, but this "
                     "version of PyMongo requires at least %d (MongoDB %s)."
-                    % (s.address[0], s.address[1],
-                       s.max_wire_version,
-                       common.MIN_SUPPORTED_WIRE_VERSION,
-                       common.MIN_SUPPORTED_SERVER_VERSION))
+                    % (
+                        s.address[0],
+                        s.address[1],
+                        s.max_wire_version,
+                        common.MIN_SUPPORTED_WIRE_VERSION,
+                        common.MIN_SUPPORTED_SERVER_VERSION,
+                    )
+                )
 
                 break
 
-        # Server Discovery And Monitoring Spec: Whenever a client updates the
-        # TopologyDescription from an ismaster response, it MUST set
-        # TopologyDescription.logicalSessionTimeoutMinutes to the smallest
-        # logicalSessionTimeoutMinutes value among ServerDescriptions of all
-        # data-bearing server types. If any have a null
-        # logicalSessionTimeoutMinutes, then
-        # TopologyDescription.logicalSessionTimeoutMinutes MUST be set to null.
-        readable_servers = self.readable_servers
-        if not readable_servers:
-            self._ls_timeout_minutes = None
-        elif any(s.logical_session_timeout_minutes is None
-                 for s in readable_servers):
-            self._ls_timeout_minutes = None
-        else:
-            self._ls_timeout_minutes = min(s.logical_session_timeout_minutes
-                                           for s in readable_servers)
-
     def check_compatible(self):
         """Raise ConfigurationError if any server is incompatible.
 
@@ -140,8 +164,7 @@ class TopologyDescription(object):
             topology_type = self._topology_type
 
         # The default ServerDescription's type is Unknown.
-        sds = dict((address, ServerDescription(address))
-                   for address in self._server_descriptions)
+        sds = dict((address, ServerDescription(address)) for address in self._server_descriptions)
 
         return TopologyDescription(
             topology_type,
@@ -149,7 +172,8 @@ class TopologyDescription(object):
             self._replica_set_name,
             self._max_set_version,
             self._max_election_id,
-            self._topology_settings)
+            self._topology_settings,
+        )
 
     def server_descriptions(self):
         """Dict of (address,
@@ -192,14 +216,12 @@ class TopologyDescription(object):
     @property
     def known_servers(self):
         """List of Servers of types besides Unknown."""
-        return [s for s in self._server_descriptions.values()
-                if s.is_server_type_known]
+        return [s for s in self._server_descriptions.values() if s.is_server_type_known]
 
     @property
     def has_known_servers(self):
         """Whether there are any Servers of types besides Unknown."""
-        return any(s for s in self._server_descriptions.values()
-                   if s.is_server_type_known)
+        return any(s for s in self._server_descriptions.values() if s.is_server_type_known)
 
     @property
     def readable_servers(self):
@@ -220,7 +242,6 @@ class TopologyDescription(object):
         return self._topology_settings.heartbeat_frequency
 
     def apply_selector(self, selector, address, custom_selector=None):
-
         def apply_local_threshold(selection):
             if not selection:
                 return []
@@ -228,23 +249,24 @@ class TopologyDescription(object):
             settings = self._topology_settings
 
             # Round trip time in seconds.
-            fastest = min(
-                s.round_trip_time for s in selection.server_descriptions)
+            fastest = min(s.round_trip_time for s in selection.server_descriptions)
             threshold = settings.local_threshold_ms / 1000.0
-            return [s for s in selection.server_descriptions
-                    if (s.round_trip_time - fastest) <= threshold]
+            return [
+                s
+                for s in selection.server_descriptions
+                if (s.round_trip_time - fastest) <= threshold
+            ]
 
-        if getattr(selector, 'min_wire_version', 0):
+        if getattr(selector, "min_wire_version", 0):
             common_wv = self.common_wire_version
             if common_wv and common_wv < selector.min_wire_version:
                 raise ConfigurationError(
                     "%s requires min wire version %d, but topology's min"
-                    " wire version is %d" % (selector,
-                                             selector.min_wire_version,
-                                             common_wv))
+                    " wire version is %d" % (selector, selector.min_wire_version, common_wv)
+                )
 
-        if self.topology_type == TOPOLOGY_TYPE.Single:
-            # Ignore selectors for standalone.
+        if self.topology_type in (TOPOLOGY_TYPE.Single, TOPOLOGY_TYPE.LoadBalanced):
+            # Ignore selectors for standalone and load balancer mode.
             return self.known_servers
         elif address:
             # Ignore selectors when explicit address is requested.
@@ -259,7 +281,8 @@ class TopologyDescription(object):
         # Apply custom selector followed by localThresholdMS.
         if custom_selector is not None and selection:
             selection = selection.with_server_descriptions(
-                custom_selector(selection.server_descriptions))
+                custom_selector(selection.server_descriptions)
+            )
         return apply_local_threshold(selection)
 
     def has_readable_server(self, read_preference=ReadPreference.PRIMARY):
@@ -291,14 +314,16 @@ class TopologyDescription(object):
 
     def __repr__(self):
         # Sort the servers by address.
-        servers = sorted(self._server_descriptions.values(),
-                         key=lambda sd: sd.address)
+        servers = sorted(self._server_descriptions.values(), key=lambda sd: sd.address)
         return "<%s id: %s, topology_type: %s, servers: %r>" % (
-            self.__class__.__name__, self._topology_settings._topology_id,
-            self.topology_type_name, servers)
+            self.__class__.__name__,
+            self._topology_settings._topology_id,
+            self.topology_type_name,
+            servers,
+        )
 
 
-# If topology type is Unknown and we receive an ismaster response, what should
+# If topology type is Unknown and we receive a hello response, what should
 # the new topology type be?
 _SERVER_TYPE_TO_TOPOLOGY_TYPE = {
     SERVER_TYPE.Mongos: TOPOLOGY_TYPE.Sharded,
@@ -306,6 +331,7 @@ _SERVER_TYPE_TO_TOPOLOGY_TYPE = {
     SERVER_TYPE.RSSecondary: TOPOLOGY_TYPE.ReplicaSetNoPrimary,
     SERVER_TYPE.RSArbiter: TOPOLOGY_TYPE.ReplicaSetNoPrimary,
     SERVER_TYPE.RSOther: TOPOLOGY_TYPE.ReplicaSetNoPrimary,
+    # Note: SERVER_TYPE.LoadBalancer and Unknown are intentionally left out.
 }
 
 
@@ -315,9 +341,9 @@ def updated_topology_description(topology_description, server_description):
     :Parameters:
       - `topology_description`: the current TopologyDescription
       - `server_description`: a new ServerDescription that resulted from
-        an ismaster call
+        a hello call
 
-    Called after attempting (successfully or not) to call ismaster on the
+    Called after attempting (successfully or not) to call hello on the
     server at server_description.address. Does not modify topology_description.
     """
     address = server_description.address
@@ -338,12 +364,12 @@ def updated_topology_description(topology_description, server_description):
 
     if topology_type == TOPOLOGY_TYPE.Single:
         # Set server type to Unknown if replica set name does not match.
-        if (set_name is not None and
-                set_name != server_description.replica_set_name):
+        if set_name is not None and set_name != server_description.replica_set_name:
             error = ConfigurationError(
                 "client is configured to connect to a replica set named "
-                "'%s' but this node belongs to a set named '%s'" % (
-                    set_name, server_description.replica_set_name))
+                "'%s' but this node belongs to a set named '%s'"
+                % (set_name, server_description.replica_set_name)
+            )
             sds[address] = server_description.to_unknown(error=error)
         # Single type never changes.
         return TopologyDescription(
@@ -352,10 +378,11 @@ def updated_topology_description(topology_description, server_description):
             set_name,
             max_set_version,
             max_election_id,
-            topology_description._topology_settings)
+            topology_description._topology_settings,
+        )
 
     if topology_type == TOPOLOGY_TYPE.Unknown:
-        if server_type == SERVER_TYPE.Standalone:
+        if server_type in (SERVER_TYPE.Standalone, SERVER_TYPE.LoadBalancer):
             if len(topology_description._topology_settings.seeds) == 1:
                 topology_type = TOPOLOGY_TYPE.Single
             else:
@@ -373,21 +400,14 @@ def updated_topology_description(topology_description, server_description):
             sds.pop(address)
 
         elif server_type == SERVER_TYPE.RSPrimary:
-            (topology_type,
-             set_name,
-             max_set_version,
-             max_election_id) = _update_rs_from_primary(sds,
-                                                        set_name,
-                                                        server_description,
-                                                        max_set_version,
-                                                        max_election_id)
+            (topology_type, set_name, max_set_version, max_election_id) = _update_rs_from_primary(
+                sds, set_name, server_description, max_set_version, max_election_id
+            )
 
-        elif server_type in (
-                SERVER_TYPE.RSSecondary,
-                SERVER_TYPE.RSArbiter,
-                SERVER_TYPE.RSOther):
+        elif server_type in (SERVER_TYPE.RSSecondary, SERVER_TYPE.RSArbiter, SERVER_TYPE.RSOther):
             topology_type, set_name = _update_rs_no_primary_from_member(
-                sds, set_name, server_description)
+                sds, set_name, server_description
+            )
 
     elif topology_type == TOPOLOGY_TYPE.ReplicaSetWithPrimary:
         if server_type in (SERVER_TYPE.Standalone, SERVER_TYPE.Mongos):
@@ -395,33 +415,26 @@ def updated_topology_description(topology_description, server_description):
             topology_type = _check_has_primary(sds)
 
         elif server_type == SERVER_TYPE.RSPrimary:
-            (topology_type,
-             set_name,
-             max_set_version,
-             max_election_id) = _update_rs_from_primary(sds,
-                                                        set_name,
-                                                        server_description,
-                                                        max_set_version,
-                                                        max_election_id)
+            (topology_type, set_name, max_set_version, max_election_id) = _update_rs_from_primary(
+                sds, set_name, server_description, max_set_version, max_election_id
+            )
 
-        elif server_type in (
-                SERVER_TYPE.RSSecondary,
-                SERVER_TYPE.RSArbiter,
-                SERVER_TYPE.RSOther):
-            topology_type = _update_rs_with_primary_from_member(
-                sds, set_name, server_description)
+        elif server_type in (SERVER_TYPE.RSSecondary, SERVER_TYPE.RSArbiter, SERVER_TYPE.RSOther):
+            topology_type = _update_rs_with_primary_from_member(sds, set_name, server_description)
 
         else:
             # Server type is Unknown or RSGhost: did we just lose the primary?
             topology_type = _check_has_primary(sds)
 
     # Return updated copy.
-    return TopologyDescription(topology_type,
-                               sds,
-                               set_name,
-                               max_set_version,
-                               max_election_id,
-                               topology_description._topology_settings)
+    return TopologyDescription(
+        topology_type,
+        sds,
+        set_name,
+        max_set_version,
+        max_election_id,
+        topology_description._topology_settings,
+    )
 
 
 def _updated_topology_description_srv_polling(topology_description, seedlist):
@@ -430,8 +443,9 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
     :Parameters:
       - `topology_description`: the current TopologyDescription
       - `seedlist`: a list of new seeds new ServerDescription that resulted from
-        an ismaster call
+        a hello call
     """
+    assert topology_description.topology_type in SRV_POLLING_TOPOLOGIES
     # Create a copy of the server descriptions.
     sds = topology_description.server_descriptions()
 
@@ -455,16 +469,14 @@ def _updated_topology_description_srv_polling(topology_description, seedlist):
         topology_description.replica_set_name,
         topology_description.max_set_version,
         topology_description.max_election_id,
-        topology_description._topology_settings)
+        topology_description._topology_settings,
+    )
 
 
 def _update_rs_from_primary(
-        sds,
-        replica_set_name,
-        server_description,
-        max_set_version,
-        max_election_id):
-    """Update topology description from a primary's ismaster response.
+    sds, replica_set_name, server_description, max_set_version, max_election_id
+):
+    """Update topology description from a primary's hello response.
 
     Pass in a dict of ServerDescriptions, current replica set name, the
     ServerDescription we are processing, and the TopologyDescription's
@@ -480,35 +492,33 @@ def _update_rs_from_primary(
         # We found a primary but it doesn't have the replica_set_name
         # provided by the user.
         sds.pop(server_description.address)
-        return (_check_has_primary(sds),
-                replica_set_name,
-                max_set_version,
-                max_election_id)
+        return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
 
     max_election_tuple = max_set_version, max_election_id
     if None not in server_description.election_tuple:
-        if (None not in max_election_tuple and
-                max_election_tuple > server_description.election_tuple):
+        if (
+            None not in max_election_tuple
+            and max_election_tuple > server_description.election_tuple
+        ):
 
             # Stale primary, set to type Unknown.
             sds[server_description.address] = server_description.to_unknown()
-            return (_check_has_primary(sds),
-                    replica_set_name,
-                    max_set_version,
-                    max_election_id)
+            return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
 
         max_election_id = server_description.election_id
 
-    if (server_description.set_version is not None and
-        (max_set_version is None or
-            server_description.set_version > max_set_version)):
+    if server_description.set_version is not None and (
+        max_set_version is None or server_description.set_version > max_set_version
+    ):
 
         max_set_version = server_description.set_version
 
     # We've heard from the primary. Is it the same primary as before?
     for server in sds.values():
-        if (server.server_type is SERVER_TYPE.RSPrimary
-                and server.address != server_description.address):
+        if (
+            server.server_type is SERVER_TYPE.RSPrimary
+            and server.address != server_description.address
+        ):
 
             # Reset old primary's type to Unknown.
             sds[server.address] = server.to_unknown()
@@ -527,16 +537,10 @@ def _update_rs_from_primary(
 
     # If the host list differs from the seed list, we may not have a primary
     # after all.
-    return (_check_has_primary(sds),
-            replica_set_name,
-            max_set_version,
-            max_election_id)
+    return (_check_has_primary(sds), replica_set_name, max_set_version, max_election_id)
 
 
-def _update_rs_with_primary_from_member(
-        sds,
-        replica_set_name,
-        server_description):
+def _update_rs_with_primary_from_member(sds, replica_set_name, server_description):
     """RS with known primary. Process a response from a non-primary.
 
     Pass in a dict of ServerDescriptions, current replica set name, and the
@@ -548,18 +552,14 @@ def _update_rs_with_primary_from_member(
 
     if replica_set_name != server_description.replica_set_name:
         sds.pop(server_description.address)
-    elif (server_description.me and
-          server_description.address != server_description.me):
+    elif server_description.me and server_description.address != server_description.me:
         sds.pop(server_description.address)
 
     # Had this member been the primary?
     return _check_has_primary(sds)
 
 
-def _update_rs_no_primary_from_member(
-        sds,
-        replica_set_name,
-        server_description):
+def _update_rs_no_primary_from_member(sds, replica_set_name, server_description):
     """RS without known primary. Update from a non-primary's response.
 
     Pass in a dict of ServerDescriptions, current replica set name, and the
@@ -581,8 +581,7 @@ def _update_rs_no_primary_from_member(
         if address not in sds:
             sds[address] = ServerDescription(address)
 
-    if (server_description.me and
-            server_description.address != server_description.me):
+    if server_description.me and server_description.address != server_description.me:
         sds.pop(server_description.address)
 
     return topology_type, replica_set_name
