@@ -15,19 +15,18 @@
 """Tools for mocking parts of PyMongo to test other parts."""
 
 import contextlib
-from functools import partial
 import weakref
+from functools import partial
+from test import client_context
 
-from pymongo import common
-from pymongo import MongoClient
+from pymongo import MongoClient, common
 from pymongo.errors import AutoReconnect, NetworkTimeout
-from pymongo.ismaster import IsMaster
+from pymongo.hello import Hello
+from pymongo.hello_compat import HelloCompat
 from pymongo.monitor import Monitor
 from pymongo.pool import Pool
 from pymongo.server_description import ServerDescription
 
-from test import client_context
-
 
 class MockPool(Pool):
     def __init__(self, client, pair, *args, **kwargs):
@@ -40,58 +39,54 @@ class MockPool(Pool):
         Pool.__init__(self, (client_context.host, client_context.port), *args, **kwargs)
 
     @contextlib.contextmanager
-    def get_socket(self, all_credentials, checkout=False):
+    def get_socket(self, all_credentials, handler=None):
         client = self.client
-        host_and_port = '%s:%s' % (self.mock_host, self.mock_port)
+        host_and_port = "%s:%s" % (self.mock_host, self.mock_port)
         if host_and_port in client.mock_down_hosts:
-            raise AutoReconnect('mock error')
+            raise AutoReconnect("mock error")
 
         assert host_and_port in (
-            client.mock_standalones
-            + client.mock_members
-            + client.mock_mongoses), "bad host: %s" % host_and_port
+            client.mock_standalones + client.mock_members + client.mock_mongoses
+        ), ("bad host: %s" % host_and_port)
 
-        with Pool.get_socket(self, all_credentials, checkout) as sock_info:
+        with Pool.get_socket(self, all_credentials, handler) as sock_info:
             sock_info.mock_host = self.mock_host
             sock_info.mock_port = self.mock_port
             yield sock_info
 
 
 class MockMonitor(Monitor):
-    def __init__(
-            self,
-            client,
-            server_description,
-            topology,
-            pool,
-            topology_settings):
+    def __init__(self, client, server_description, topology, pool, topology_settings):
         # MockMonitor gets a 'client' arg, regular monitors don't. Weakref it
         # to avoid cycles.
         self.client = weakref.proxy(client)
-        Monitor.__init__(
-            self,
-            server_description,
-            topology,
-            pool,
-            topology_settings)
+        Monitor.__init__(self, server_description, topology, pool, topology_settings)
 
     def _check_once(self):
         client = self.client
         address = self._server_description.address
-        response, rtt = client.mock_is_master('%s:%d' % address)
-        return ServerDescription(address, IsMaster(response), rtt)
+        response, rtt = client.mock_hello("%s:%d" % address)
+        return ServerDescription(address, Hello(response), rtt)
 
 
 class MockClient(MongoClient):
     def __init__(
-            self, standalones, members, mongoses, ismaster_hosts=None,
-            *args, **kwargs):
+        self,
+        standalones,
+        members,
+        mongoses,
+        hello_hosts=None,
+        arbiters=None,
+        down_hosts=None,
+        *args,
+        **kwargs
+    ):
         """A MongoClient connected to the default server, with a mock topology.
 
-        standalones, members, mongoses determine the configuration of the
-        topology. They are formatted like ['a:1', 'b:2']. ismaster_hosts
-        provides an alternative host list for the server's mocked ismaster
-        response; see test_connect_with_internal_ips.
+        standalones, members, mongoses, arbiters, and down_hosts determine the
+        configuration of the topology. They are formatted like ['a:1', 'b:2'].
+        hello_hosts provides an alternative host list for the server's
+        mocked hello response; see test_connect_with_internal_ips.
         """
         self.mock_standalones = standalones[:]
         self.mock_members = members[:]
@@ -101,15 +96,15 @@ class MockClient(MongoClient):
         else:
             self.mock_primary = None
 
-        if ismaster_hosts is not None:
-            self.mock_ismaster_hosts = ismaster_hosts
-        else:
-            self.mock_ismaster_hosts = members[:]
+        # Hosts that should be considered an arbiter.
+        self.mock_arbiters = arbiters[:] if arbiters else []
+
+        self.mock_hello_hosts = hello_hosts or members[:]
 
         self.mock_mongoses = mongoses[:]
 
         # Hosts that should raise socket errors.
-        self.mock_down_hosts = []
+        self.mock_down_hosts = down_hosts[:] if down_hosts else []
 
         # Hostname -> (min wire version, max wire version)
         self.mock_wire_versions = {}
@@ -120,8 +115,8 @@ class MockClient(MongoClient):
         # Hostname -> round trip time
         self.mock_rtts = {}
 
-        kwargs['_pool_class'] = partial(MockPool, self)
-        kwargs['_monitor_class'] = partial(MockMonitor, self)
+        kwargs["_pool_class"] = partial(MockPool, self)
+        kwargs["_monitor_class"] = partial(MockMonitor, self)
 
         client_options = client_context.default_client_options.copy()
         client_options.update(kwargs)
@@ -142,8 +137,8 @@ class MockClient(MongoClient):
     def set_max_write_batch_size(self, host, size):
         self.mock_max_write_batch_sizes[host] = size
 
-    def mock_is_master(self, host):
-        """Return mock ismaster response (a dict) and round trip time."""
+    def mock_hello(self, host):
+        """Return mock hello response (a dict) and round trip time."""
         if host in self.mock_wire_versions:
             min_wire_version, max_wire_version = self.mock_wire_versions[host]
         else:
@@ -151,49 +146,57 @@ class MockClient(MongoClient):
             max_wire_version = common.MAX_SUPPORTED_WIRE_VERSION
 
         max_write_batch_size = self.mock_max_write_batch_sizes.get(
-            host, common.MAX_WRITE_BATCH_SIZE)
+            host, common.MAX_WRITE_BATCH_SIZE
+        )
 
         rtt = self.mock_rtts.get(host, 0)
 
         # host is like 'a:1'.
         if host in self.mock_down_hosts:
-            raise NetworkTimeout('mock timeout')
+            raise NetworkTimeout("mock timeout")
 
         elif host in self.mock_standalones:
             response = {
-                'ok': 1,
-                'ismaster': True,
-                'minWireVersion': min_wire_version,
-                'maxWireVersion': max_wire_version,
-                'maxWriteBatchSize': max_write_batch_size}
+                "ok": 1,
+                HelloCompat.LEGACY_CMD: True,
+                "minWireVersion": min_wire_version,
+                "maxWireVersion": max_wire_version,
+                "maxWriteBatchSize": max_write_batch_size,
+            }
         elif host in self.mock_members:
-            ismaster = (host == self.mock_primary)
+            is_writable_primary = host == self.mock_primary
 
             # Simulate a replica set member.
             response = {
-                'ok': 1,
-                'ismaster': ismaster,
-                'secondary': not ismaster,
-                'setName': 'rs',
-                'hosts': self.mock_ismaster_hosts,
-                'minWireVersion': min_wire_version,
-                'maxWireVersion': max_wire_version,
-                'maxWriteBatchSize': max_write_batch_size}
+                "ok": 1,
+                HelloCompat.LEGACY_CMD: is_writable_primary,
+                "secondary": not is_writable_primary,
+                "setName": "rs",
+                "hosts": self.mock_hello_hosts,
+                "minWireVersion": min_wire_version,
+                "maxWireVersion": max_wire_version,
+                "maxWriteBatchSize": max_write_batch_size,
+            }
 
             if self.mock_primary:
-                response['primary'] = self.mock_primary
+                response["primary"] = self.mock_primary
+
+            if host in self.mock_arbiters:
+                response["arbiterOnly"] = True
+                response["secondary"] = False
         elif host in self.mock_mongoses:
             response = {
-                'ok': 1,
-                'ismaster': True,
-                'minWireVersion': min_wire_version,
-                'maxWireVersion': max_wire_version,
-                'msg': 'isdbgrid',
-                'maxWriteBatchSize': max_write_batch_size}
+                "ok": 1,
+                HelloCompat.LEGACY_CMD: True,
+                "minWireVersion": min_wire_version,
+                "maxWireVersion": max_wire_version,
+                "msg": "isdbgrid",
+                "maxWriteBatchSize": max_write_batch_size,
+            }
         else:
             # In test_internal_ips(), we try to connect to a host listed
-            # in ismaster['hosts'] but not publicly accessible.
-            raise AutoReconnect('Unknown host: %s' % host)
+            # in hello['hosts'] but not publicly accessible.
+            raise AutoReconnect("Unknown host: %s" % host)
 
         return response, rtt
 
