@@ -17,35 +17,38 @@
 """Tests for the gridfs package.
 """
 import sys
+
 sys.path[0:0] = [""]
 
 import datetime
 import threading
 import time
-import gridfs
+from test import IntegrationTest, client_context, unittest
+from test.test_replica_set_client import TestReplicaSetClientBase
+from test.utils import (
+    ignore_deprecations,
+    joinall,
+    one,
+    rs_client,
+    rs_or_single_client,
+    single_client,
+)
 
+import gridfs
 from bson.binary import Binary
 from bson.py3compat import StringIO, string_type
-from pymongo.mongo_client import MongoClient
-from pymongo.errors import (ConfigurationError,
-                            ConnectionFailure,
-                            ServerSelectionTimeoutError)
-from pymongo.read_preferences import ReadPreference
 from gridfs.errors import CorruptGridFile, FileExists, NoFile
-from test.test_replica_set_client import TestReplicaSetClientBase
-from test import (client_context,
-                  unittest,
-                  IntegrationTest)
-from test.utils import (ignore_deprecations,
-                        joinall,
-                        one,
-                        rs_client,
-                        rs_or_single_client,
-                        single_client)
+from gridfs.grid_file import GridOutCursor
+from pymongo.errors import (
+    ConfigurationError,
+    ConnectionFailure,
+    ServerSelectionTimeoutError,
+)
+from pymongo.mongo_client import MongoClient
+from pymongo.read_preferences import ReadPreference
 
 
 class JustWrite(threading.Thread):
-
     def __init__(self, fs, n):
         threading.Thread.__init__(self)
         self.fs = fs
@@ -60,7 +63,6 @@ class JustWrite(threading.Thread):
 
 
 class JustRead(threading.Thread):
-
     def __init__(self, fs, n, results):
         threading.Thread.__init__(self)
         self.fs = fs
@@ -77,7 +79,6 @@ class JustRead(threading.Thread):
 
 
 class TestGridfsNoConnect(unittest.TestCase):
-
     @classmethod
     def setUpClass(cls):
         cls.db = MongoClient(connect=False).pymongo_test
@@ -88,7 +89,6 @@ class TestGridfsNoConnect(unittest.TestCase):
 
 
 class TestGridfs(IntegrationTest):
-
     @classmethod
     def setUpClass(cls):
         super(TestGridfs, cls).setUpClass()
@@ -143,8 +143,7 @@ class TestGridfs(IntegrationTest):
         self.fs.put(b"foo", filename="test")
         self.fs.put(b"", filename="hello world")
 
-        self.assertEqual(set(["mike", "test", "hello world"]),
-                         set(self.fs.list()))
+        self.assertEqual(set(["mike", "test", "hello world"]), set(self.fs.list()))
 
     def test_empty_file(self):
         oid = self.fs.put(b"")
@@ -160,9 +159,8 @@ class TestGridfs(IntegrationTest):
         self.assertTrue(isinstance(raw["md5"], string_type))
 
     def test_corrupt_chunk(self):
-        files_id = self.fs.put(b'foobar')
-        self.db.fs.chunks.update_one({'files_id': files_id},
-                                     {'$set': {'data': Binary(b'foo', 0)}})
+        files_id = self.fs.put(b"foobar")
+        self.db.fs.chunks.update_one({"files_id": files_id}, {"$set": {"data": Binary(b"foo", 0)}})
         try:
             out = self.fs.get(files_id)
             self.assertRaises(CorruptGridFile, out.read)
@@ -175,18 +173,24 @@ class TestGridfs(IntegrationTest):
     def test_put_ensures_index(self):
         # setUp has dropped collections.
         names = self.db.list_collection_names()
-        self.assertFalse([name for name in names if name.startswith('fs')])
+        self.assertFalse([name for name in names if name.startswith("fs")])
 
         chunks = self.db.fs.chunks
         files = self.db.fs.files
         self.fs.put(b"junk")
 
-        self.assertTrue(any(
-            info.get('key') == [('files_id', 1), ('n', 1)]
-            for info in chunks.index_information().values()))
-        self.assertTrue(any(
-            info.get('key') == [('filename', 1), ('uploadDate', 1)]
-            for info in files.index_information().values()))
+        self.assertTrue(
+            any(
+                info.get("key") == [("files_id", 1), ("n", 1)]
+                for info in chunks.index_information().values()
+            )
+        )
+        self.assertTrue(
+            any(
+                info.get("key") == [("filename", 1), ("uploadDate", 1)]
+                for info in files.index_information().values()
+            )
+        )
 
     def test_alt_collection(self):
         oid = self.alt.put(b"hello world")
@@ -208,8 +212,7 @@ class TestGridfs(IntegrationTest):
         self.alt.put(b"foo", filename="test")
         self.alt.put(b"", filename="hello world")
 
-        self.assertEqual(set(["mike", "test", "hello world"]),
-                         set(self.alt.list()))
+        self.assertEqual(set(["mike", "test", "hello world"]), set(self.alt.list()))
 
     def test_threaded_reads(self):
         self.fs.put(b"hello", _id="test")
@@ -222,10 +225,7 @@ class TestGridfs(IntegrationTest):
 
         joinall(threads)
 
-        self.assertEqual(
-            100 * [b'hello'],
-            results
-        )
+        self.assertEqual(100 * [b"hello"], results)
 
     def test_threaded_writes(self):
         threads = []
@@ -239,10 +239,7 @@ class TestGridfs(IntegrationTest):
         self.assertEqual(f.read(), b"hello")
 
         # Should have created 100 versions of 'test' file
-        self.assertEqual(
-            100,
-            self.db.fs.files.count_documents({'filename': 'test'})
-        )
+        self.assertEqual(100, self.db.fs.files.count_documents({"filename": "test"}))
 
     def test_get_last_version(self):
         one = self.fs.put(b"foo", filename="test")
@@ -312,11 +309,21 @@ class TestGridfs(IntegrationTest):
         time.sleep(0.01)
         three = self.fs.put(b"baz", filename="test", author="author2")
 
-        self.assertEqual(b"foo", self.fs.get_version(filename="test", author="author1", version=-2).read())
-        self.assertEqual(b"bar", self.fs.get_version(filename="test", author="author1", version=-1).read())
-        self.assertEqual(b"foo", self.fs.get_version(filename="test", author="author1", version=0).read())
-        self.assertEqual(b"bar", self.fs.get_version(filename="test", author="author1", version=1).read())
-        self.assertEqual(b"baz", self.fs.get_version(filename="test", author="author2", version=0).read())
+        self.assertEqual(
+            b"foo", self.fs.get_version(filename="test", author="author1", version=-2).read()
+        )
+        self.assertEqual(
+            b"bar", self.fs.get_version(filename="test", author="author1", version=-1).read()
+        )
+        self.assertEqual(
+            b"foo", self.fs.get_version(filename="test", author="author1", version=0).read()
+        )
+        self.assertEqual(
+            b"bar", self.fs.get_version(filename="test", author="author1", version=1).read()
+        )
+        self.assertEqual(
+            b"baz", self.fs.get_version(filename="test", author="author2", version=0).read()
+        )
         self.assertEqual(b"baz", self.fs.get_version(filename="test", version=-1).read())
         self.assertEqual(b"baz", self.fs.get_version(filename="test", version=2).read())
 
@@ -341,7 +348,7 @@ class TestGridfs(IntegrationTest):
         one.close()
 
         two = self.fs.new_file(_id=123)
-        self.assertRaises(FileExists, two.write, b'x' * 262146)
+        self.assertRaises(FileExists, two.write, b"x" * 262146)
 
     def test_exists(self):
         oid = self.fs.put(b"hello")
@@ -395,8 +402,7 @@ class TestGridfs(IntegrationTest):
         self.assertTrue(iterate_file(f))
 
     def test_gridfs_lazy_connect(self):
-        client = MongoClient('badhost', connect=False,
-                             serverSelectionTimeoutMS=10)
+        client = MongoClient("badhost", connect=False, serverSelectionTimeoutMS=10)
         db = client.db
         gfs = gridfs.GridFS(db)
         self.assertRaises(ServerSelectionTimeoutError, gfs.list)
@@ -416,8 +422,7 @@ class TestGridfs(IntegrationTest):
         self.fs.put(b"test2++", filename="two")
         self.assertEqual(3, self.fs.find({"filename": "two"}).count())
         self.assertEqual(4, self.fs.find().count())
-        cursor = self.fs.find(
-            no_cursor_timeout=False).sort("uploadDate", -1).skip(1).limit(2)
+        cursor = self.fs.find(no_cursor_timeout=False).sort("uploadDate", -1).skip(1).limit(2)
         gout = next(cursor)
         self.assertEqual(b"test1", gout.read())
         cursor.rewind()
@@ -429,29 +434,35 @@ class TestGridfs(IntegrationTest):
         cursor.close()
         self.assertRaises(TypeError, self.fs.find, {}, {"_id": True})
 
+    def test_delete_not_initialized(self):
+        # Creating a cursor with invalid arguments will not run __init__
+        # but will still call __del__.
+        cursor = GridOutCursor.__new__(GridOutCursor)  # Skip calling __init__
+        with self.assertRaises(TypeError):
+            cursor.__init__(self.db.fs.files, {}, {"_id": True})
+        cursor.__del__()  # no error
+
     def test_gridfs_find_one(self):
         self.assertEqual(None, self.fs.find_one())
 
-        id1 = self.fs.put(b'test1', filename='file1')
-        self.assertEqual(b'test1', self.fs.find_one().read())
+        id1 = self.fs.put(b"test1", filename="file1")
+        self.assertEqual(b"test1", self.fs.find_one().read())
 
-        id2 = self.fs.put(b'test2', filename='file2', meta='data')
-        self.assertEqual(b'test1', self.fs.find_one(id1).read())
-        self.assertEqual(b'test2', self.fs.find_one(id2).read())
+        id2 = self.fs.put(b"test2", filename="file2", meta="data")
+        self.assertEqual(b"test1", self.fs.find_one(id1).read())
+        self.assertEqual(b"test2", self.fs.find_one(id2).read())
 
-        self.assertEqual(b'test1',
-                         self.fs.find_one({'filename': 'file1'}).read())
+        self.assertEqual(b"test1", self.fs.find_one({"filename": "file1"}).read())
 
-        self.assertEqual('data', self.fs.find_one(id2).meta)
+        self.assertEqual("data", self.fs.find_one(id2).meta)
 
     def test_grid_in_non_int_chunksize(self):
         # Lua, and perhaps other buggy GridFS clients, store size as a float.
-        data = b'data'
-        self.fs.put(data, filename='f')
-        self.db.fs.files.update_one({'filename': 'f'},
-                                    {'$set': {'chunkSize': 100.0}})
+        data = b"data"
+        self.fs.put(data, filename="f")
+        self.db.fs.files.update_one({"filename": "f"}, {"$set": {"chunkSize": 100.0}})
 
-        self.assertEqual(data, self.fs.get_version('f').read())
+        self.assertEqual(data, self.fs.get_version("f").read())
 
     def test_unacknowledged(self):
         # w=0 is prohibited.
@@ -488,7 +499,6 @@ class TestGridfs(IntegrationTest):
 
 
 class TestGridfsReplicaSet(TestReplicaSetClientBase):
-
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
@@ -496,21 +506,19 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
 
     @classmethod
     def tearDownClass(cls):
-        client_context.client.drop_database('gfsreplica')
+        client_context.client.drop_database("gfsreplica")
 
     def test_gridfs_replica_set(self):
-        rsc = rs_client(
-            w=self.w,
-            read_preference=ReadPreference.SECONDARY)
+        rsc = rs_client(w=self.w, read_preference=ReadPreference.SECONDARY)
 
-        fs = gridfs.GridFS(rsc.gfsreplica, 'gfsreplicatest')
+        fs = gridfs.GridFS(rsc.gfsreplica, "gfsreplicatest")
 
         gin = fs.new_file()
         self.assertEqual(gin._coll.read_preference, ReadPreference.PRIMARY)
 
-        oid = fs.put(b'foo')
+        oid = fs.put(b"foo")
         content = fs.get(oid).read()
-        self.assertEqual(b'foo', content)
+        self.assertEqual(b"foo", content)
 
     def test_gridfs_secondary(self):
         primary_host, primary_port = self.primary
@@ -518,32 +526,30 @@ class TestGridfsReplicaSet(TestReplicaSetClientBase):
 
         secondary_host, secondary_port = one(self.secondaries)
         secondary_connection = single_client(
-            secondary_host, secondary_port,
-            read_preference=ReadPreference.SECONDARY)
+            secondary_host, secondary_port, read_preference=ReadPreference.SECONDARY
+        )
 
         # Should detect it's connected to secondary and not attempt to
         # create index
-        fs = gridfs.GridFS(secondary_connection.gfsreplica, 'gfssecondarytest')
+        fs = gridfs.GridFS(secondary_connection.gfsreplica, "gfssecondarytest")
 
         # This won't detect secondary, raises error
-        self.assertRaises(ConnectionFailure, fs.put, b'foo')
+        self.assertRaises(ConnectionFailure, fs.put, b"foo")
 
     def test_gridfs_secondary_lazy(self):
         # Should detect it's connected to secondary and not attempt to
         # create index.
         secondary_host, secondary_port = one(self.secondaries)
         client = single_client(
-            secondary_host,
-            secondary_port,
-            read_preference=ReadPreference.SECONDARY,
-            connect=False)
+            secondary_host, secondary_port, read_preference=ReadPreference.SECONDARY, connect=False
+        )
 
         # Still no connection.
-        fs = gridfs.GridFS(client.gfsreplica, 'gfssecondarylazytest')
+        fs = gridfs.GridFS(client.gfsreplica, "gfssecondarylazytest")
 
         # Connects, doesn't create index.
         self.assertRaises(NoFile, fs.get_last_version)
-        self.assertRaises(ConnectionFailure, fs.put, 'data')
+        self.assertRaises(ConnectionFailure, fs.put, "data")
 
 
 if __name__ == "__main__":
