@@ -16,43 +16,39 @@
 import copy
 import datetime
 import os
-import sys
 import re
-
+import sys
 from json import loads
 
 import gridfs
 
 sys.path[0:0] = [""]
 
+from test import IntegrationTest, unittest
+
 from bson import Binary
 from bson.int64 import Int64
 from bson.json_util import object_hook
 from bson.py3compat import bytes_from_hex
-from gridfs.errors import NoFile, CorruptGridFile
-from test import (unittest,
-                  IntegrationTest)
+from gridfs.errors import CorruptGridFile, NoFile
 
 # Commands.
-_COMMANDS = {"delete": lambda coll, doc: [coll.delete_many(d["q"])
-                                          for d in doc['deletes']],
-             "insert": lambda coll, doc: coll.insert_many(doc['documents']),
-             "update": lambda coll, doc: [coll.update_many(u["q"], u["u"])
-                                          for u in doc['updates']]
-            }
+_COMMANDS = {
+    "delete": lambda coll, doc: [coll.delete_many(d["q"]) for d in doc["deletes"]],
+    "insert": lambda coll, doc: coll.insert_many(doc["documents"]),
+    "update": lambda coll, doc: [coll.update_many(u["q"], u["u"]) for u in doc["updates"]],
+}
 
 # Location of JSON test specifications.
-_TEST_PATH = os.path.join(
-    os.path.dirname(os.path.realpath(__file__)),
-    'gridfs')
+_TEST_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "gridfs")
 
 
 def camel_to_snake(camel):
     # Regex to convert CamelCase to snake_case. Special case for _id.
     if camel == "id":
         return "file_id"
-    snake = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', camel)
-    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', snake).lower()
+    snake = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", camel)
+    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", snake).lower()
 
 
 class TestAllScenarios(IntegrationTest):
@@ -64,7 +60,8 @@ class TestAllScenarios(IntegrationTest):
             "upload": cls.fs.upload_from_stream,
             "download": cls.fs.open_download_stream,
             "delete": cls.fs.delete,
-            "download_by_name": cls.fs.open_download_stream_by_name}
+            "download_by_name": cls.fs.open_download_stream_by_name,
+        }
 
     def init_db(self, data, test):
         self.db.drop_collection("fs.files")
@@ -73,16 +70,16 @@ class TestAllScenarios(IntegrationTest):
         self.db.drop_collection("expected.chunks")
 
         # Read in data.
-        if data['files']:
-            self.db.fs.files.insert_many(data['files'])
-            self.db.expected.files.insert_many(data['files'])
-        if data['chunks']:
-            self.db.fs.chunks.insert_many(data['chunks'])
-            self.db.expected.chunks.insert_many(data['chunks'])
+        if data["files"]:
+            self.db.fs.files.insert_many(data["files"])
+            self.db.expected.files.insert_many(data["files"])
+        if data["chunks"]:
+            self.db.fs.chunks.insert_many(data["chunks"])
+            self.db.expected.chunks.insert_many(data["chunks"])
 
         # Make initial modifications.
         if "arrange" in test:
-            for cmd in test['arrange'].get('data', []):
+            for cmd in test["arrange"].get("data", []):
                 for key in cmd.keys():
                     if key in _COMMANDS:
                         coll = self.db.get_collection(cmd[key])
@@ -90,11 +87,11 @@ class TestAllScenarios(IntegrationTest):
 
     def init_expected_db(self, test, result):
         # Modify outcome DB.
-        for cmd in test['assert'].get('data', []):
+        for cmd in test["assert"].get("data", []):
             for key in cmd.keys():
                 if key in _COMMANDS:
                     # Replace wildcards in inserts.
-                    for doc in cmd.get('documents', []):
+                    for doc in cmd.get("documents", []):
                         keylist = doc.keys()
                         for dockey in copy.deepcopy(list(keylist)):
                             if "result" in str(doc[dockey]):
@@ -107,8 +104,8 @@ class TestAllScenarios(IntegrationTest):
                     coll = self.db.get_collection(cmd[key])
                     _COMMANDS[key](coll, cmd)
 
-        if test['assert'].get('result') == "&result":
-            test['assert']['result'] = result
+        if test["assert"].get("result") == "&result":
+            test["assert"]["result"] = result
 
     def sorted_list(self, coll, ignore_id):
         to_sort = []
@@ -129,30 +126,28 @@ def create_test(scenario_def):
     def run_scenario(self):
 
         # Run tests.
-        self.assertTrue(scenario_def['tests'], "tests cannot be empty")
-        for test in scenario_def['tests']:
-            self.init_db(scenario_def['data'], test)
+        self.assertTrue(scenario_def["tests"], "tests cannot be empty")
+        for test in scenario_def["tests"]:
+            self.init_db(scenario_def["data"], test)
 
             # Run GridFs Operation.
-            operation = self.str_to_cmd[test['act']['operation']]
-            args = test['act']['arguments']
+            operation = self.str_to_cmd[test["act"]["operation"]]
+            args = test["act"]["arguments"]
             extra_opts = args.pop("options", {})
             if "contentType" in extra_opts:
-                extra_opts["metadata"] = {
-                    "contentType": extra_opts.pop("contentType")}
+                extra_opts["metadata"] = {"contentType": extra_opts.pop("contentType")}
 
             args.update(extra_opts)
 
-            converted_args = dict((camel_to_snake(c), v)
-                                  for c, v in args.items())
+            converted_args = dict((camel_to_snake(c), v) for c, v in args.items())
 
-            expect_error = test['assert'].get("error", False)
+            expect_error = test["assert"].get("error", False)
             result = None
             error = None
             try:
                 result = operation(**converted_args)
 
-                if 'download' in test['act']['operation']:
+                if "download" in test["act"]["operation"]:
                     result = Binary(result.read())
             except Exception as exc:
                 if not expect_error:
@@ -162,47 +157,51 @@ def create_test(scenario_def):
             self.init_expected_db(test, result)
 
             # Asserts.
-            errors = {"FileNotFound": NoFile,
-                      "ChunkIsMissing": CorruptGridFile,
-                      "ExtraChunk": CorruptGridFile,
-                      "ChunkIsWrongSize": CorruptGridFile,
-                      "RevisionNotFound": NoFile}
+            errors = {
+                "FileNotFound": NoFile,
+                "ChunkIsMissing": CorruptGridFile,
+                "ExtraChunk": CorruptGridFile,
+                "ChunkIsWrongSize": CorruptGridFile,
+                "RevisionNotFound": NoFile,
+            }
 
             if expect_error:
                 self.assertIsNotNone(error)
-                self.assertIsInstance(error, errors[test['assert']['error']],
-                                      test['description'])
+                self.assertIsInstance(error, errors[test["assert"]["error"]], test["description"])
             else:
                 self.assertIsNone(error)
 
-            if 'result' in test['assert']:
-                if test['assert']['result'] == 'void':
-                    test['assert']['result'] = None
-                self.assertEqual(result, test['assert'].get('result'))
+            if "result" in test["assert"]:
+                if test["assert"]["result"] == "void":
+                    test["assert"]["result"] = None
+                self.assertEqual(result, test["assert"].get("result"))
 
-            if 'data' in test['assert']:
+            if "data" in test["assert"]:
                 # Create alphabetized list
                 self.assertEqual(
                     set(self.sorted_list(self.db.fs.chunks, True)),
-                    set(self.sorted_list(self.db.expected.chunks, True)))
+                    set(self.sorted_list(self.db.expected.chunks, True)),
+                )
 
                 self.assertEqual(
                     set(self.sorted_list(self.db.fs.files, False)),
-                    set(self.sorted_list(self.db.expected.files, False)))
+                    set(self.sorted_list(self.db.expected.files, False)),
+                )
 
     return run_scenario
 
+
 def _object_hook(dct):
-    if 'length' in dct:
-        dct['length'] = Int64(dct['length'])
+    if "length" in dct:
+        dct["length"] = Int64(dct["length"])
     return object_hook(dct)
 
+
 def create_tests():
     for dirpath, _, filenames in os.walk(_TEST_PATH):
         for filename in filenames:
             with open(os.path.join(dirpath, filename)) as scenario_stream:
-                scenario_def = loads(
-                    scenario_stream.read(), object_hook=_object_hook)
+                scenario_def = loads(scenario_stream.read(), object_hook=_object_hook)
 
             # Because object_hook is already defined by bson.json_util,
             # and everything is named 'data'
@@ -210,7 +209,7 @@ def create_tests():
                 for key, val in jsn.items():
                     if key in ("data", "source", "result"):
                         if "$hex" in val:
-                            jsn[key] = Binary(bytes_from_hex(val['$hex']))
+                            jsn[key] = Binary(bytes_from_hex(val["$hex"]))
                     if isinstance(jsn[key], dict):
                         str2hex(jsn[key])
                     if isinstance(jsn[key], list):
@@ -221,8 +220,7 @@ def create_tests():
 
             # Construct test from scenario.
             new_test = create_test(scenario_def)
-            test_name = 'test_%s' % (
-                os.path.splitext(filename)[0])
+            test_name = "test_%s" % (os.path.splitext(filename)[0])
             new_test.__name__ = test_name
             setattr(TestAllScenarios, new_test.__name__, new_test)
 
