@@ -22,14 +22,21 @@ import uuid
 
 sys.path[0:0] = [""]
 
-from bson import json_util, EPOCH_AWARE, SON
-from bson.json_util import (DatetimeRepresentation,
-                            STRICT_JSON_OPTIONS)
-from bson.binary import (ALL_UUID_REPRESENTATIONS, Binary, MD5_SUBTYPE,
-                         USER_DEFINED_SUBTYPE, UuidRepresentation, STANDARD)
+from test import IntegrationTest, unittest
+
+from bson import EPOCH_AWARE, SON, json_util
+from bson.binary import (
+    ALL_UUID_REPRESENTATIONS,
+    MD5_SUBTYPE,
+    STANDARD,
+    USER_DEFINED_SUBTYPE,
+    Binary,
+    UuidRepresentation,
+)
 from bson.code import Code
 from bson.dbref import DBRef
 from bson.int64 import Int64
+from bson.json_util import STRICT_JSON_OPTIONS, DatetimeRepresentation
 from bson.max_key import MaxKey
 from bson.min_key import MinKey
 from bson.objectid import ObjectId
@@ -37,8 +44,6 @@ from bson.regex import Regex
 from bson.timestamp import Timestamp
 from bson.tz_util import FixedOffset, utc
 
-from test import unittest, IntegrationTest
-
 PY3 = sys.version_info[0] == 3
 
 
@@ -53,14 +58,10 @@ class TestJsonUtil(unittest.TestCase):
         self.round_trip({"hello": "world"})
 
     def test_json_options_with_options(self):
-        opts = json_util.JSONOptions(
-            datetime_representation=DatetimeRepresentation.NUMBERLONG)
-        self.assertEqual(
-            opts.datetime_representation, DatetimeRepresentation.NUMBERLONG)
-        opts2 = opts.with_options(
-            datetime_representation=DatetimeRepresentation.ISO8601)
-        self.assertEqual(
-            opts2.datetime_representation, DatetimeRepresentation.ISO8601)
+        opts = json_util.JSONOptions(datetime_representation=DatetimeRepresentation.NUMBERLONG)
+        self.assertEqual(opts.datetime_representation, DatetimeRepresentation.NUMBERLONG)
+        opts2 = opts.with_options(datetime_representation=DatetimeRepresentation.ISO8601)
+        self.assertEqual(opts2.datetime_representation, DatetimeRepresentation.ISO8601)
 
         opts = json_util.JSONOptions(strict_number_long=True)
         self.assertEqual(opts.strict_number_long, True)
@@ -68,16 +69,12 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual(opts2.strict_number_long, False)
 
         opts = json_util.CANONICAL_JSON_OPTIONS
-        self.assertNotEqual(
-            opts.uuid_representation, UuidRepresentation.JAVA_LEGACY)
-        opts2 = opts.with_options(
-            uuid_representation=UuidRepresentation.JAVA_LEGACY)
-        self.assertEqual(
-            opts2.uuid_representation, UuidRepresentation.JAVA_LEGACY)
+        self.assertNotEqual(opts.uuid_representation, UuidRepresentation.JAVA_LEGACY)
+        opts2 = opts.with_options(uuid_representation=UuidRepresentation.JAVA_LEGACY)
+        self.assertEqual(opts2.uuid_representation, UuidRepresentation.JAVA_LEGACY)
         self.assertEqual(opts2.document_class, dict)
         opts3 = opts2.with_options(document_class=SON)
-        self.assertEqual(
-            opts3.uuid_representation, UuidRepresentation.JAVA_LEGACY)
+        self.assertEqual(opts3.uuid_representation, UuidRepresentation.JAVA_LEGACY)
         self.assertEqual(opts3.document_class, SON)
 
     def test_objectid(self):
@@ -91,12 +88,12 @@ class TestJsonUtil(unittest.TestCase):
         # Check order.
         self.assertEqual(
             '{"$ref": "collection", "$id": 1, "$db": "db"}',
-            json_util.dumps(DBRef('collection', 1, 'db')))
+            json_util.dumps(DBRef("collection", 1, "db")),
+        )
 
     def test_datetime(self):
         # only millis, not micros
-        self.round_trip({"date": datetime.datetime(2009, 12, 9, 15,
-                                                   49, 45, 191000, utc)})
+        self.round_trip({"date": datetime.datetime(2009, 12, 9, 15, 49, 45, 191000, utc)})
 
         jsn = '{"dt": { "$date" : "1970-01-01T00:00:00.000+0000"}}'
         self.assertEqual(EPOCH_AWARE, json_util.loads(jsn)["dt"])
@@ -153,75 +150,84 @@ class TestJsonUtil(unittest.TestCase):
         # Test dumps format
         pre_epoch = {"dt": datetime.datetime(1, 1, 1, 1, 1, 1, 10000, utc)}
         post_epoch = {"dt": datetime.datetime(1972, 1, 1, 1, 1, 1, 10000, utc)}
-        self.assertEqual(
-            '{"dt": {"$date": -62135593138990}}',
-            json_util.dumps(pre_epoch))
-        self.assertEqual(
-            '{"dt": {"$date": 63075661010}}',
-            json_util.dumps(post_epoch))
+        self.assertEqual('{"dt": {"$date": -62135593138990}}', json_util.dumps(pre_epoch))
+        self.assertEqual('{"dt": {"$date": 63075661010}}', json_util.dumps(post_epoch))
         self.assertEqual(
             '{"dt": {"$date": {"$numberLong": "-62135593138990"}}}',
-            json_util.dumps(pre_epoch, json_options=STRICT_JSON_OPTIONS))
+            json_util.dumps(pre_epoch, json_options=STRICT_JSON_OPTIONS),
+        )
         self.assertEqual(
             '{"dt": {"$date": "1972-01-01T01:01:01.010Z"}}',
-            json_util.dumps(post_epoch, json_options=STRICT_JSON_OPTIONS))
+            json_util.dumps(post_epoch, json_options=STRICT_JSON_OPTIONS),
+        )
 
         number_long_options = json_util.JSONOptions(
-            datetime_representation=DatetimeRepresentation.NUMBERLONG)
+            datetime_representation=DatetimeRepresentation.NUMBERLONG
+        )
         self.assertEqual(
             '{"dt": {"$date": {"$numberLong": "63075661010"}}}',
-            json_util.dumps(post_epoch, json_options=number_long_options))
+            json_util.dumps(post_epoch, json_options=number_long_options),
+        )
         self.assertEqual(
             '{"dt": {"$date": {"$numberLong": "-62135593138990"}}}',
-            json_util.dumps(pre_epoch, json_options=number_long_options))
+            json_util.dumps(pre_epoch, json_options=number_long_options),
+        )
 
         # ISO8601 mode assumes naive datetimes are UTC
         pre_epoch_naive = {"dt": datetime.datetime(1, 1, 1, 1, 1, 1, 10000)}
-        post_epoch_naive = {
-            "dt": datetime.datetime(1972, 1, 1, 1, 1, 1, 10000)}
+        post_epoch_naive = {"dt": datetime.datetime(1972, 1, 1, 1, 1, 1, 10000)}
         self.assertEqual(
             '{"dt": {"$date": {"$numberLong": "-62135593138990"}}}',
-            json_util.dumps(pre_epoch_naive, json_options=STRICT_JSON_OPTIONS))
+            json_util.dumps(pre_epoch_naive, json_options=STRICT_JSON_OPTIONS),
+        )
         self.assertEqual(
             '{"dt": {"$date": "1972-01-01T01:01:01.010Z"}}',
-            json_util.dumps(post_epoch_naive,
-                            json_options=STRICT_JSON_OPTIONS))
+            json_util.dumps(post_epoch_naive, json_options=STRICT_JSON_OPTIONS),
+        )
 
         # Test tz_aware and tzinfo options
         self.assertEqual(
             datetime.datetime(1972, 1, 1, 1, 1, 1, 10000, utc),
-            json_util.loads(
-                '{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}')["dt"])
+            json_util.loads('{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}')["dt"],
+        )
         self.assertEqual(
             datetime.datetime(1972, 1, 1, 1, 1, 1, 10000, utc),
             json_util.loads(
                 '{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}',
-                json_options=json_util.JSONOptions(tz_aware=True,
-                                                   tzinfo=utc))["dt"])
+                json_options=json_util.JSONOptions(tz_aware=True, tzinfo=utc),
+            )["dt"],
+        )
         self.assertEqual(
             datetime.datetime(1972, 1, 1, 1, 1, 1, 10000),
             json_util.loads(
                 '{"dt": {"$date": "1972-01-01T01:01:01.010+0000"}}',
-                json_options=json_util.JSONOptions(tz_aware=False))["dt"])
-        self.round_trip(pre_epoch_naive, json_options=json_util.JSONOptions(
-            tz_aware=False))
+                json_options=json_util.JSONOptions(tz_aware=False),
+            )["dt"],
+        )
+        self.round_trip(pre_epoch_naive, json_options=json_util.JSONOptions(tz_aware=False))
 
         # Test a non-utc timezone
-        pacific = FixedOffset(-8 * 60, 'US/Pacific')
-        aware_datetime = {"dt": datetime.datetime(2002, 10, 27, 6, 0, 0, 10000,
-                                                  pacific)}
+        pacific = FixedOffset(-8 * 60, "US/Pacific")
+        aware_datetime = {"dt": datetime.datetime(2002, 10, 27, 6, 0, 0, 10000, pacific)}
         self.assertEqual(
             '{"dt": {"$date": "2002-10-27T06:00:00.010-0800"}}',
-            json_util.dumps(aware_datetime, json_options=STRICT_JSON_OPTIONS))
-        self.round_trip(aware_datetime, json_options=json_util.JSONOptions(
-            tz_aware=True, tzinfo=pacific))
-        self.round_trip(aware_datetime, json_options=json_util.JSONOptions(
-            datetime_representation=DatetimeRepresentation.ISO8601,
-            tz_aware=True, tzinfo=pacific))
+            json_util.dumps(aware_datetime, json_options=STRICT_JSON_OPTIONS),
+        )
+        self.round_trip(
+            aware_datetime, json_options=json_util.JSONOptions(tz_aware=True, tzinfo=pacific)
+        )
+        self.round_trip(
+            aware_datetime,
+            json_options=json_util.JSONOptions(
+                datetime_representation=DatetimeRepresentation.ISO8601,
+                tz_aware=True,
+                tzinfo=pacific,
+            ),
+        )
 
     def test_regex_object_hook(self):
         # Extended JSON format regular expression.
-        pat = 'a*b'
+        pat = "a*b"
         json_re = '{"$regex": "%s", "$options": "u"}' % pat
         loaded = json_util.object_hook(json.loads(json_re))
         self.assertTrue(isinstance(loaded, Regex))
@@ -229,9 +235,7 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual(re.U, loaded.flags)
 
     def test_regex(self):
-        for regex_instance in (
-                re.compile("a*b", re.IGNORECASE),
-                Regex("a*b", re.IGNORECASE)):
+        for regex_instance in (re.compile("a*b", re.IGNORECASE), Regex("a*b", re.IGNORECASE)):
             res = self.round_tripped({"r": regex_instance})["r"]
 
             self.assertEqual("a*b", res.pattern)
@@ -239,28 +243,27 @@ class TestJsonUtil(unittest.TestCase):
             self.assertEqual("a*b", res.pattern)
             self.assertEqual(re.IGNORECASE, res.flags)
 
-        unicode_options = re.I|re.M|re.S|re.U|re.X
+        unicode_options = re.I | re.M | re.S | re.U | re.X
         regex = re.compile("a*b", unicode_options)
         res = self.round_tripped({"r": regex})["r"]
         self.assertEqual(unicode_options, res.flags)
 
         # Some tools may not add $options if no flags are set.
-        res = json_util.loads('{"r": {"$regex": "a*b"}}')['r']
+        res = json_util.loads('{"r": {"$regex": "a*b"}}')["r"]
         self.assertEqual(0, res.flags)
 
         self.assertEqual(
-            Regex('.*', 'ilm'),
-            json_util.loads(
-                '{"r": {"$regex": ".*", "$options": "ilm"}}')['r'])
+            Regex(".*", "ilm"), json_util.loads('{"r": {"$regex": ".*", "$options": "ilm"}}')["r"]
+        )
 
         # Check order.
         self.assertEqual(
-            '{"$regex": ".*", "$options": "mx"}',
-            json_util.dumps(Regex('.*', re.M | re.X)))
+            '{"$regex": ".*", "$options": "mx"}', json_util.dumps(Regex(".*", re.M | re.X))
+        )
 
         self.assertEqual(
-            '{"$regex": ".*", "$options": "mx"}',
-            json_util.dumps(re.compile(b'.*', re.M | re.X)))
+            '{"$regex": ".*", "$options": "mx"}', json_util.dumps(re.compile(b".*", re.M | re.X))
+        )
 
     def test_regex_validation(self):
         non_str_types = [10, {}, []]
@@ -285,64 +288,64 @@ class TestJsonUtil(unittest.TestCase):
         self.assertEqual('{"ts": {"$timestamp": {"t": 4, "i": 13}}}', res)
 
     def test_uuid(self):
-        doc = {'uuid': uuid.UUID('f47ac10b-58cc-4372-a567-0e02b2c3d479')}
+        doc = {"uuid": uuid.UUID("f47ac10b-58cc-4372-a567-0e02b2c3d479")}
         self.round_trip(doc)
         self.assertEqual(
-            '{"uuid": {"$uuid": "f47ac10b58cc4372a5670e02b2c3d479"}}',
-            json_util.dumps(doc))
+            '{"uuid": {"$uuid": "f47ac10b58cc4372a5670e02b2c3d479"}}', json_util.dumps(doc)
+        )
         self.assertEqual(
-            '{"uuid": '
-            '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}',
+            '{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}',
+            json_util.dumps(doc, json_options=json_util.STRICT_JSON_OPTIONS),
+        )
+        self.assertEqual(
+            '{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
             json_util.dumps(
-                doc, json_options=json_util.STRICT_JSON_OPTIONS))
+                doc,
+                json_options=json_util.JSONOptions(strict_uuid=True, uuid_representation=STANDARD),
+            ),
+        )
         self.assertEqual(
-            '{"uuid": '
-            '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
-            json_util.dumps(
-                doc, json_options=json_util.JSONOptions(
-                    strict_uuid=True, uuid_representation=STANDARD)))
-        self.assertEqual(
-            doc, json_util.loads(
-                '{"uuid": '
-                '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}'))
-        for uuid_representation in (set(ALL_UUID_REPRESENTATIONS) -
-                                    {UuidRepresentation.UNSPECIFIED}):
+            doc,
+            json_util.loads('{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "03"}}'),
+        )
+        for uuid_representation in set(ALL_UUID_REPRESENTATIONS) - {UuidRepresentation.UNSPECIFIED}:
             options = json_util.JSONOptions(
-                strict_uuid=True, uuid_representation=uuid_representation)
+                strict_uuid=True, uuid_representation=uuid_representation
+            )
             self.round_trip(doc, json_options=options)
             # Ignore UUID representation when decoding BSON binary subtype 4.
-            self.assertEqual(doc, json_util.loads(
-                '{"uuid": '
-                '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
-                json_options=options))
+            self.assertEqual(
+                doc,
+                json_util.loads(
+                    '{"uuid": ' '{"$binary": "9HrBC1jMQ3KlZw4CssPUeQ==", "$type": "04"}}',
+                    json_options=options,
+                ),
+            )
 
     def test_uuid_uuid_rep_unspecified(self):
         _uuid = uuid.uuid4()
         options = json_util.JSONOptions(
-            strict_uuid=True,
-            uuid_representation=UuidRepresentation.UNSPECIFIED)
+            strict_uuid=True, uuid_representation=UuidRepresentation.UNSPECIFIED
+        )
 
         # Cannot directly encode native UUIDs with UNSPECIFIED.
-        doc = {'uuid': _uuid}
+        doc = {"uuid": _uuid}
         with self.assertRaises(ValueError):
             json_util.dumps(doc, json_options=options)
 
         # All UUID subtypes are decoded as Binary with UNSPECIFIED.
         # subtype 3
-        doc = {'uuid': Binary(_uuid.bytes, subtype=3)}
+        doc = {"uuid": Binary(_uuid.bytes, subtype=3)}
         ext_json_str = json_util.dumps(doc)
-        self.assertEqual(
-            doc, json_util.loads(ext_json_str, json_options=options))
+        self.assertEqual(doc, json_util.loads(ext_json_str, json_options=options))
         # subtype 4
-        doc = {'uuid': Binary(_uuid.bytes, subtype=4)}
+        doc = {"uuid": Binary(_uuid.bytes, subtype=4)}
         ext_json_str = json_util.dumps(doc)
-        self.assertEqual(
-            doc, json_util.loads(ext_json_str, json_options=options))
+        self.assertEqual(doc, json_util.loads(ext_json_str, json_options=options))
         # $uuid-encoded fields
-        doc = {'uuid': Binary(_uuid.bytes, subtype=4)}
-        ext_json_str = json_util.dumps({'uuid': _uuid})
-        self.assertEqual(
-            doc, json_util.loads(ext_json_str, json_options=options))
+        doc = {"uuid": Binary(_uuid.bytes, subtype=4)}
+        ext_json_str = json_util.dumps({"uuid": _uuid})
+        self.assertEqual(doc, json_util.loads(ext_json_str, json_options=options))
 
     def test_binary(self):
         if PY3:
@@ -350,8 +353,8 @@ class TestJsonUtil(unittest.TestCase):
         else:
             bin_type_dict = {"bin": Binary(b"\x00\x01\x02\x03\x04")}
         md5_type_dict = {
-            "md5": Binary(b' n7\x18\xaf\t/\xd1\xd1/\x80\xca\xe7q\xcc\xac',
-                MD5_SUBTYPE)}
+            "md5": Binary(b" n7\x18\xaf\t/\xd1\xd1/\x80\xca\xe7q\xcc\xac", MD5_SUBTYPE)
+        }
         custom_type_dict = {"custom": Binary(b"hello", USER_DEFINED_SUBTYPE)}
 
         self.round_trip(bin_type_dict)
@@ -359,8 +362,7 @@ class TestJsonUtil(unittest.TestCase):
         self.round_trip(custom_type_dict)
 
         # Binary with subtype 0 is decoded into bytes in Python 3.
-        bin = json_util.loads(
-            '{"bin": {"$binary": "AAECAwQ=", "$type": "00"}}')['bin']
+        bin = json_util.loads('{"bin": {"$binary": "AAECAwQ=", "$type": "00"}}')["bin"]
         if PY3:
             self.assertEqual(type(bin), bytes)
         else:
@@ -369,33 +371,41 @@ class TestJsonUtil(unittest.TestCase):
         # PYTHON-443 ensure old type formats are supported
         json_bin_dump = json_util.dumps(bin_type_dict)
         self.assertTrue('"$type": "00"' in json_bin_dump)
-        self.assertEqual(bin_type_dict,
-            json_util.loads('{"bin": {"$type": 0, "$binary": "AAECAwQ="}}'))
+        self.assertEqual(
+            bin_type_dict, json_util.loads('{"bin": {"$type": 0, "$binary": "AAECAwQ="}}')
+        )
         json_bin_dump = json_util.dumps(md5_type_dict)
         # Check order.
         self.assertEqual(
-            '{"md5": {"$binary": "IG43GK8JL9HRL4DK53HMrA==",'
-            + ' "$type": "05"}}',
-            json_bin_dump)
+            '{"md5": {"$binary": "IG43GK8JL9HRL4DK53HMrA==",' + ' "$type": "05"}}', json_bin_dump
+        )
 
-        self.assertEqual(md5_type_dict,
-            json_util.loads('{"md5": {"$type": 5, "$binary":'
-                            ' "IG43GK8JL9HRL4DK53HMrA=="}}'))
+        self.assertEqual(
+            md5_type_dict,
+            json_util.loads('{"md5": {"$type": 5, "$binary":' ' "IG43GK8JL9HRL4DK53HMrA=="}}'),
+        )
 
         json_bin_dump = json_util.dumps(custom_type_dict)
         self.assertTrue('"$type": "80"' in json_bin_dump)
-        self.assertEqual(custom_type_dict,
-            json_util.loads('{"custom": {"$type": 128, "$binary":'
-                            ' "aGVsbG8="}}'))
+        self.assertEqual(
+            custom_type_dict,
+            json_util.loads('{"custom": {"$type": 128, "$binary":' ' "aGVsbG8="}}'),
+        )
 
         # Handle mongoexport where subtype >= 128
-        self.assertEqual(128,
-            json_util.loads('{"custom": {"$type": "ffffff80", "$binary":'
-                            ' "aGVsbG8="}}')['custom'].subtype)
+        self.assertEqual(
+            128,
+            json_util.loads('{"custom": {"$type": "ffffff80", "$binary":' ' "aGVsbG8="}}')[
+                "custom"
+            ].subtype,
+        )
 
-        self.assertEqual(255,
-            json_util.loads('{"custom": {"$type": "ffffffff", "$binary":'
-                            ' "aGVsbG8="}}')['custom'].subtype)
+        self.assertEqual(
+            255,
+            json_util.loads('{"custom": {"$type": "ffffffff", "$binary":' ' "aGVsbG8="}}')[
+                "custom"
+            ].subtype,
+        )
 
     def test_code(self):
         self.round_trip({"code": Code("function x() { return 1; }")})
@@ -407,33 +417,34 @@ class TestJsonUtil(unittest.TestCase):
         # Check order.
         self.assertEqual('{"$code": "return z", "$scope": {"z": 2}}', res)
 
-        no_scope = Code('function() {}')
-        self.assertEqual(
-            '{"$code": "function() {}"}', json_util.dumps(no_scope))
+        no_scope = Code("function() {}")
+        self.assertEqual('{"$code": "function() {}"}', json_util.dumps(no_scope))
 
     def test_undefined(self):
         jsn = '{"name": {"$undefined": true}}'
-        self.assertIsNone(json_util.loads(jsn)['name'])
+        self.assertIsNone(json_util.loads(jsn)["name"])
 
     def test_numberlong(self):
         jsn = '{"weight": {"$numberLong": "65535"}}'
-        self.assertEqual(json_util.loads(jsn)['weight'],
-                         Int64(65535))
-        self.assertEqual(json_util.dumps({"weight": Int64(65535)}),
-                         '{"weight": 65535}')
+        self.assertEqual(json_util.loads(jsn)["weight"], Int64(65535))
+        self.assertEqual(json_util.dumps({"weight": Int64(65535)}), '{"weight": 65535}')
         json_options = json_util.JSONOptions(strict_number_long=True)
-        self.assertEqual(json_util.dumps({"weight": Int64(65535)},
-                                         json_options=json_options),
-                         jsn)
+        self.assertEqual(json_util.dumps({"weight": Int64(65535)}, json_options=json_options), jsn)
 
     def test_loads_document_class(self):
         # document_class dict should always work
-        self.assertEqual({"foo": "bar"}, json_util.loads(
-            '{"foo": "bar"}',
-            json_options=json_util.JSONOptions(document_class=dict)))
-        self.assertEqual(SON([("foo", "bar"), ("b", 1)]), json_util.loads(
-            '{"foo": "bar", "b": 1}',
-            json_options=json_util.JSONOptions(document_class=SON)))
+        self.assertEqual(
+            {"foo": "bar"},
+            json_util.loads(
+                '{"foo": "bar"}', json_options=json_util.JSONOptions(document_class=dict)
+            ),
+        )
+        self.assertEqual(
+            SON([("foo", "bar"), ("b", 1)]),
+            json_util.loads(
+                '{"foo": "bar", "b": 1}', json_options=json_util.JSONOptions(document_class=SON)
+            ),
+        )
 
 
 class TestJsonUtilRoundtrip(IntegrationTest):
@@ -442,12 +453,11 @@ class TestJsonUtilRoundtrip(IntegrationTest):
 
         db.drop_collection("test")
         docs = [
-            {'foo': [1, 2]},
-            {'bar': {'hello': 'world'}},
-            {'code': Code("function x() { return 1; }")},
-            {'bin': Binary(b"\x00\x01\x02\x03\x04", USER_DEFINED_SUBTYPE)},
-            {'dbref': {'_ref': DBRef('simple',
-                               ObjectId('509b8db456c02c5ab7e63c34'))}}
+            {"foo": [1, 2]},
+            {"bar": {"hello": "world"}},
+            {"code": Code("function x() { return 1; }")},
+            {"bin": Binary(b"\x00\x01\x02\x03\x04", USER_DEFINED_SUBTYPE)},
+            {"dbref": {"_ref": DBRef("simple", ObjectId("509b8db456c02c5ab7e63c34"))}},
         ]
 
         db.test.insert_many(docs)
@@ -455,5 +465,6 @@ class TestJsonUtilRoundtrip(IntegrationTest):
         for doc in docs:
             self.assertTrue(doc in reloaded_docs)
 
+
 if __name__ == "__main__":
     unittest.main()
