@@ -21,18 +21,16 @@ import warnings
 
 sys.path[0:0] = [""]
 
-from pymongo import MongoClient
-from pymongo.errors import ConfigurationError
-from pymongo.server_selectors import writable_server_selector
-
 from test import client_context, unittest
 from test.utils import rs_or_single_client
 from test.utils_selection_tests import create_selection_tests
 
+from pymongo import MongoClient
+from pymongo.errors import ConfigurationError
+from pymongo.server_selectors import writable_server_selector
+
 # Location of JSON test specifications.
-_TEST_PATH = os.path.join(
-    os.path.dirname(os.path.realpath(__file__)),
-    'max_staleness')
+_TEST_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "max_staleness")
 
 
 class TestAllScenarios(create_selection_tests(_TEST_PATH)):
@@ -54,26 +52,21 @@ class TestMaxStaleness(unittest.TestCase):
 
         with self.assertRaises(ConfigurationError):
             # Read pref "primary" can't be used with max staleness.
-            MongoClient("mongodb://a/?readPreference=primary&"
-                        "maxStalenessSeconds=120")
+            MongoClient("mongodb://a/?readPreference=primary&" "maxStalenessSeconds=120")
 
         client = MongoClient("mongodb://host/?maxStalenessSeconds=-1")
         self.assertEqual(-1, client.read_preference.max_staleness)
 
-        client = MongoClient("mongodb://host/?readPreference=primary&"
-                             "maxStalenessSeconds=-1")
+        client = MongoClient("mongodb://host/?readPreference=primary&" "maxStalenessSeconds=-1")
         self.assertEqual(-1, client.read_preference.max_staleness)
 
-        client = MongoClient("mongodb://host/?readPreference=secondary&"
-                             "maxStalenessSeconds=120")
+        client = MongoClient("mongodb://host/?readPreference=secondary&" "maxStalenessSeconds=120")
         self.assertEqual(120, client.read_preference.max_staleness)
 
-        client = MongoClient("mongodb://a/?readPreference=secondary&"
-                             "maxStalenessSeconds=1")
+        client = MongoClient("mongodb://a/?readPreference=secondary&" "maxStalenessSeconds=1")
         self.assertEqual(1, client.read_preference.max_staleness)
 
-        client = MongoClient("mongodb://a/?readPreference=secondary&"
-                             "maxStalenessSeconds=-1")
+        client = MongoClient("mongodb://a/?readPreference=secondary&" "maxStalenessSeconds=-1")
         self.assertEqual(-1, client.read_preference.max_staleness)
 
         client = MongoClient(maxStalenessSeconds=-1, readPreference="nearest")
@@ -85,15 +78,15 @@ class TestMaxStaleness(unittest.TestCase):
 
     def test_max_staleness_float(self):
         with self.assertRaises(TypeError) as ctx:
-            rs_or_single_client(maxStalenessSeconds=1.5,
-                                readPreference="nearest")
+            rs_or_single_client(maxStalenessSeconds=1.5, readPreference="nearest")
 
         self.assertIn("must be an integer", str(ctx.exception))
 
         with warnings.catch_warnings(record=True) as ctx:
             warnings.simplefilter("always")
-            client = MongoClient("mongodb://host/?maxStalenessSeconds=1.5"
-                                 "&readPreference=nearest")
+            client = MongoClient(
+                "mongodb://host/?maxStalenessSeconds=1.5" "&readPreference=nearest"
+            )
 
             # Option was ignored.
             self.assertEqual(-1, client.read_preference.max_staleness)
@@ -102,15 +95,13 @@ class TestMaxStaleness(unittest.TestCase):
     def test_max_staleness_zero(self):
         # Zero is too small.
         with self.assertRaises(ValueError) as ctx:
-            rs_or_single_client(maxStalenessSeconds=0,
-                                readPreference="nearest")
+            rs_or_single_client(maxStalenessSeconds=0, readPreference="nearest")
 
         self.assertIn("must be a positive integer", str(ctx.exception))
 
         with warnings.catch_warnings(record=True) as ctx:
             warnings.simplefilter("always")
-            client = MongoClient("mongodb://host/?maxStalenessSeconds=0"
-                                 "&readPreference=nearest")
+            client = MongoClient("mongodb://host/?maxStalenessSeconds=0" "&readPreference=nearest")
 
             # Option was ignored.
             self.assertEqual(-1, client.read_preference.max_staleness)
@@ -145,5 +136,6 @@ class TestMaxStaleness(unittest.TestCase):
         sd = client._topology.select_server(writable_server_selector)
         self.assertIsNone(sd.description.last_write_date)
 
+
 if __name__ == "__main__":
     unittest.main()
