@@ -21,30 +21,30 @@ import sys
 import threading
 import time
 
-from bson.son import SON
 from bson.codec_options import DEFAULT_CODEC_OPTIONS
-
+from bson.son import SON
 from pymongo import MongoClient, message
-from pymongo.errors import (AutoReconnect,
-                            ConnectionFailure,
-                            DuplicateKeyError,
-                            ExceededMaxWaiters)
+from pymongo.errors import (
+    AutoReconnect,
+    ConnectionFailure,
+    DuplicateKeyError,
+    ExceededMaxWaiters,
+)
 
 sys.path[0:0] = [""]
 
+from test import IntegrationTest, client_context, unittest
+from test.utils import delay, get_pool, joinall, rs_or_single_client
+
 from pymongo.pool import Pool, PoolOptions
 from pymongo.socket_checker import SocketChecker
-from test import client_context, unittest
-from test.utils import (get_pool,
-                        joinall,
-                        delay,
-                        rs_or_single_client)
 
 
 @client_context.require_connection
 def setUpModule():
     pass
 
+
 N = 10
 DB = "pymongo-pooling-tests"
 
@@ -63,6 +63,7 @@ def gc_collect_until_done(threads, timeout=60):
 
 class MongoThread(threading.Thread):
     """A thread that uses a MongoClient."""
+
     def __init__(self, client):
         super(MongoThread, self).__init__()
         self.daemon = True  # Don't hang whole test if thread hangs.
@@ -116,20 +117,22 @@ class SocketGetter(MongoThread):
     test_no_wait_queue_timeout, test_wait_queue_multiple, and
     test_no_wait_queue_multiple.
     """
+
     def __init__(self, client, pool):
         super(SocketGetter, self).__init__(client)
-        self.state = 'init'
+        self.state = "init"
         self.pool = pool
         self.sock = None
 
     def run_mongo_thread(self):
-        self.state = 'get_socket'
+        self.state = "get_socket"
 
-        # Pass 'checkout' so we can hold the socket.
-        with self.pool.get_socket({}, checkout=True) as sock:
+        # Call 'pin_cursor' so we can hold the socket.
+        with self.pool.get_socket({}) as sock:
+            sock.pin_cursor()
             self.sock = sock
 
-        self.state = 'sock'
+        self.state = "sock"
 
     def __del__(self):
         if self.sock:
@@ -153,10 +156,11 @@ def run_cases(client, cases):
         assert t.passed, "%s.run() threw an exception" % repr(t)
 
 
-class _TestPoolingBase(unittest.TestCase):
+class _TestPoolingBase(IntegrationTest):
     """Base class for all connection-pool tests."""
 
     def setUp(self):
+        super(_TestPoolingBase, self).setUp()
         self.c = rs_or_single_client()
         db = self.c[DB]
         db.unique.drop()
@@ -166,30 +170,26 @@ class _TestPoolingBase(unittest.TestCase):
 
     def tearDown(self):
         self.c.close()
+        super(_TestPoolingBase, self).tearDown()
 
-    def create_pool(
-            self,
-            pair=(client_context.host, client_context.port),
-            *args,
-            **kwargs):
+    def create_pool(self, pair=(client_context.host, client_context.port), *args, **kwargs):
         # Start the pool with the correct ssl options.
         pool_options = client_context.client._topology_settings.pool_options
-        kwargs['ssl_context'] = pool_options.ssl_context
-        kwargs['ssl_match_hostname'] = pool_options.ssl_match_hostname
+        kwargs["ssl_context"] = pool_options._ssl_context
+        kwargs["ssl_match_hostname"] = pool_options.ssl_match_hostname
+        kwargs["server_api"] = pool_options.server_api
         return Pool(pair, PoolOptions(*args, **kwargs))
 
 
 class TestPooling(_TestPoolingBase):
     def test_max_pool_size_validation(self):
         host, port = client_context.host, client_context.port
-        self.assertRaises(
-            ValueError, MongoClient, host=host, port=port, maxPoolSize=-1)
+        self.assertRaises(ValueError, MongoClient, host=host, port=port, maxPoolSize=-1)
 
-        self.assertRaises(
-            ValueError, MongoClient, host=host, port=port, maxPoolSize='foo')
+        self.assertRaises(ValueError, MongoClient, host=host, port=port, maxPoolSize="foo")
 
         c = MongoClient(host=host, port=port, maxPoolSize=100, connect=False)
-        self.assertEqual(c.max_pool_size, 100)
+        self.assertEqual(c.options.pool_options.max_pool_size, 100)
 
     def test_no_disconnect(self):
         run_cases(self.c, [NonUnique, Unique, InsertOneAndFind])
@@ -269,27 +269,27 @@ class TestPooling(_TestPoolingBase):
         # Socket has nothing to read.
         self.assertFalse(socket_checker.select(s, read=True))
         self.assertFalse(socket_checker.select(s, read=True, timeout=0))
-        self.assertFalse(socket_checker.select(s, read=True, timeout=.05))
+        self.assertFalse(socket_checker.select(s, read=True, timeout=0.05))
         # Socket is writable.
         self.assertTrue(socket_checker.select(s, write=True, timeout=None))
         self.assertTrue(socket_checker.select(s, write=True))
         self.assertTrue(socket_checker.select(s, write=True, timeout=0))
-        self.assertTrue(socket_checker.select(s, write=True, timeout=.05))
+        self.assertTrue(socket_checker.select(s, write=True, timeout=0.05))
         # Make the socket readable
         _, msg, _ = message.query(
-            0, 'admin.$cmd', 0, -1, SON([('isMaster', 1)]), None,
-            DEFAULT_CODEC_OPTIONS)
+            0, "admin.$cmd", 0, -1, SON([("ping", 1)]), None, DEFAULT_CODEC_OPTIONS
+        )
         s.sendall(msg)
         # Block until the socket is readable.
         self.assertTrue(socket_checker.select(s, read=True, timeout=None))
         self.assertTrue(socket_checker.select(s, read=True))
         self.assertTrue(socket_checker.select(s, read=True, timeout=0))
-        self.assertTrue(socket_checker.select(s, read=True, timeout=.05))
+        self.assertTrue(socket_checker.select(s, read=True, timeout=0.05))
         # Socket is still writable.
         self.assertTrue(socket_checker.select(s, write=True, timeout=None))
         self.assertTrue(socket_checker.select(s, write=True))
         self.assertTrue(socket_checker.select(s, write=True, timeout=0))
-        self.assertTrue(socket_checker.select(s, write=True, timeout=.05))
+        self.assertTrue(socket_checker.select(s, write=True, timeout=0.05))
         s.close()
         self.assertTrue(socket_checker.socket_closed(s))
 
@@ -304,9 +304,7 @@ class TestPooling(_TestPoolingBase):
     def test_pool_check(self):
         # Test that Pool recovers from two connection failures in a row.
         # This exercises code at the end of Pool._check().
-        cx_pool = self.create_pool(max_pool_size=1,
-                                   connect_timeout=1,
-                                   wait_queue_timeout=1)
+        cx_pool = self.create_pool(max_pool_size=1, connect_timeout=1, wait_queue_timeout=1)
         cx_pool._check_interval_seconds = 0  # Always check.
         self.addCleanup(cx_pool.close)
 
@@ -316,22 +314,19 @@ class TestPooling(_TestPoolingBase):
             sock_info.sock.close()
 
         # Swap pool's address with a bad one.
-        address, cx_pool.address = cx_pool.address, ('foo.com', 1234)
+        address, cx_pool.address = cx_pool.address, ("foo.com", 1234)
         with self.assertRaises(AutoReconnect):
             with cx_pool.get_socket({}):
                 pass
 
         # Back to normal, semaphore was correctly released.
         cx_pool.address = address
-        with cx_pool.get_socket({}, checkout=True) as sock_info:
+        with cx_pool.get_socket({}):
             pass
 
-        sock_info.close_socket(None)
-
     def test_wait_queue_timeout(self):
         wait_queue_timeout = 2  # Seconds
-        pool = self.create_pool(
-            max_pool_size=1, wait_queue_timeout=wait_queue_timeout)
+        pool = self.create_pool(max_pool_size=1, wait_queue_timeout=wait_queue_timeout)
         self.addCleanup(pool.close)
 
         with pool.get_socket({}) as sock_info:
@@ -343,9 +338,8 @@ class TestPooling(_TestPoolingBase):
         duration = time.time() - start
         self.assertTrue(
             abs(wait_queue_timeout - duration) < 1,
-            "Waited %.2f seconds for a socket, expected %f" % (
-                duration, wait_queue_timeout))
-
+            "Waited %.2f seconds for a socket, expected %f" % (duration, wait_queue_timeout),
+        )
 
     def test_no_wait_queue_timeout(self):
         # Verify get_socket() with no wait_queue_timeout blocks forever.
@@ -356,22 +350,21 @@ class TestPooling(_TestPoolingBase):
         with pool.get_socket({}) as s1:
             t = SocketGetter(self.c, pool)
             t.start()
-            while t.state != 'get_socket':
+            while t.state != "get_socket":
                 time.sleep(0.1)
 
             time.sleep(1)
-            self.assertEqual(t.state, 'get_socket')
+            self.assertEqual(t.state, "get_socket")
 
-        while t.state != 'sock':
+        while t.state != "sock":
             time.sleep(0.1)
 
-        self.assertEqual(t.state, 'sock')
+        self.assertEqual(t.state, "sock")
         self.assertEqual(t.sock, s1)
 
     def test_wait_queue_multiple(self):
         wait_queue_multiple = 3
-        pool = self.create_pool(
-            max_pool_size=2, wait_queue_multiple=wait_queue_multiple)
+        pool = self.create_pool(max_pool_size=2, wait_queue_multiple=wait_queue_multiple)
 
         # Reach max_size sockets.
         with pool.get_socket({}):
@@ -386,7 +379,7 @@ class TestPooling(_TestPoolingBase):
 
                 time.sleep(1)
                 for t in threads:
-                    self.assertEqual(t.state, 'get_socket')
+                    self.assertEqual(t.state, "get_socket")
 
                 with self.assertRaises(ExceededMaxWaiters):
                     with pool.get_socket({}):
@@ -397,8 +390,9 @@ class TestPooling(_TestPoolingBase):
 
         socks = []
         for _ in range(2):
-            # Pass 'checkout' so we can hold the socket.
-            with pool.get_socket({}, checkout=True) as sock:
+            # Call 'pin_cursor' so we can hold the socket.
+            with pool.get_socket({}) as sock:
+                sock.pin_cursor()
                 socks.append(sock)
 
         threads = []
@@ -408,7 +402,7 @@ class TestPooling(_TestPoolingBase):
             threads.append(t)
         time.sleep(1)
         for t in threads:
-            self.assertEqual(t.state, 'get_socket')
+            self.assertEqual(t.state, "get_socket")
 
         for socket_info in socks:
             socket_info.close_socket(None)
@@ -418,6 +412,7 @@ class TestPoolMaxSize(_TestPoolingBase):
     def test_max_pool_size(self):
         max_pool_size = 4
         c = rs_or_single_client(maxPoolSize=max_pool_size)
+        self.addCleanup(c.close)
         collection = c[DB].test
 
         # Need one document.
@@ -435,7 +430,7 @@ class TestPoolMaxSize(_TestPoolingBase):
 
         def f():
             for _ in range(5):
-                collection.find_one({'$where': delay(0.1)})
+                collection.find_one({"$where": delay(0.1)})
                 assert len(cx_pool.sockets) <= max_pool_size
 
             with lock:
@@ -453,6 +448,7 @@ class TestPoolMaxSize(_TestPoolingBase):
 
     def test_max_pool_size_none(self):
         c = rs_or_single_client(maxPoolSize=None)
+        self.addCleanup(c.close)
         collection = c[DB].test
 
         # Need one document.
@@ -467,7 +463,7 @@ class TestPoolMaxSize(_TestPoolingBase):
 
         def f():
             for _ in range(5):
-                collection.find_one({'$where': delay(0.1)})
+                collection.find_one({"$where": delay(0.1)})
 
             with lock:
                 self.n_passed += 1
@@ -489,26 +485,22 @@ class TestPoolMaxSize(_TestPoolingBase):
         # The pool acquires its semaphore before attempting to connect; ensure
         # it releases the semaphore on connection failure.
         test_pool = Pool(
-            ('somedomainthatdoesntexist.org', 27017),
-            PoolOptions(
-                max_pool_size=1,
-                connect_timeout=1,
-                socket_timeout=1,
-                wait_queue_timeout=1))
+            ("somedomainthatdoesntexist.org", 27017),
+            PoolOptions(max_pool_size=1, connect_timeout=1, socket_timeout=1, wait_queue_timeout=1),
+        )
 
         # First call to get_socket fails; if pool doesn't release its semaphore
         # then the second call raises "ConnectionFailure: Timed out waiting for
         # socket from pool" instead of AutoReconnect.
         for i in range(2):
             with self.assertRaises(AutoReconnect) as context:
-                with test_pool.get_socket({}, checkout=True):
+                with test_pool.get_socket({}):
                     pass
 
             # Testing for AutoReconnect instead of ConnectionFailure, above,
             # is sufficient right *now* to catch a semaphore leak. But that
             # seems error-prone, so check the message too.
-            self.assertNotIn('waiting for socket from pool',
-                             str(context.exception))
+            self.assertNotIn("waiting for socket from pool", str(context.exception))
 
 
 if __name__ == "__main__":
