@@ -15,17 +15,18 @@
 """Test the pymongo module itself."""
 
 import sys
+
 sys.path[0:0] = [""]
 
-import pymongo
 from test import unittest
 
+import pymongo
+
 
 class TestPyMongo(unittest.TestCase):
     def test_mongo_client_alias(self):
         # Testing that pymongo module imports mongo_client.MongoClient
-        self.assertEqual(pymongo.MongoClient,
-                         pymongo.mongo_client.MongoClient)
+        self.assertEqual(pymongo.MongoClient, pymongo.mongo_client.MongoClient)
 
 
 if __name__ == "__main__":
