@@ -23,56 +23,59 @@ import warnings
 
 sys.path[0:0] = [""]
 
+from test import IntegrationTest, SkipTest, client_context, db_pwd, db_user, unittest
+from test.test_replica_set_client import TestReplicaSetClientBase
+from test.utils import (
+    OvertCommandListener,
+    connected,
+    ignore_deprecations,
+    one,
+    rs_client,
+    single_client,
+    wait_until,
+)
+from test.version import Version
+
 from bson.py3compat import MAXSIZE
 from bson.son import SON
 from pymongo.errors import ConfigurationError, OperationFailure
 from pymongo.message import _maybe_add_read_preference
 from pymongo.mongo_client import MongoClient
-from pymongo.read_preferences import (ReadPreference, MovingAverage,
-                                      Primary, PrimaryPreferred,
-                                      Secondary, SecondaryPreferred,
-                                      Nearest)
+from pymongo.read_preferences import (
+    MovingAverage,
+    Nearest,
+    Primary,
+    PrimaryPreferred,
+    ReadPreference,
+    Secondary,
+    SecondaryPreferred,
+)
 from pymongo.server_description import ServerDescription
-from pymongo.server_selectors import readable_server_selector, Selection
+from pymongo.server_selectors import Selection, readable_server_selector
 from pymongo.server_type import SERVER_TYPE
 from pymongo.write_concern import WriteConcern
 
-from test import (SkipTest,
-                  client_context,
-                  unittest,
-                  db_user,
-                  db_pwd)
-from test.test_replica_set_client import TestReplicaSetClientBase
-from test.utils import (connected,
-                        ignore_deprecations,
-                        one,
-                        OvertCommandListener,
-                        rs_client,
-                        single_client,
-                        wait_until)
-from test.version import Version
-
-
-class TestSelections(unittest.TestCase):
 
+class TestSelections(IntegrationTest):
     @client_context.require_connection
     def test_bool(self):
         client = single_client()
 
         wait_until(lambda: client.address, "discover primary")
-        selection = Selection.from_topology_description(
-            client._topology.description)
+        selection = Selection.from_topology_description(client._topology.description)
 
         self.assertTrue(selection)
         self.assertFalse(selection.with_server_descriptions([]))
 
 
 class TestReadPreferenceObjects(unittest.TestCase):
-    prefs = [Primary(),
-             PrimaryPreferred(),
-             Secondary(),
-             Nearest(tag_sets=[{'a': 1}, {'b': 2}]),
-             SecondaryPreferred(max_staleness=30)]
+    prefs = [
+        Primary(),
+        PrimaryPreferred(),
+        Secondary(),
+        Nearest(tag_sets=[{"a": 1}, {"b": 2}]),
+        SecondaryPreferred(max_staleness=30),
+    ]
 
     def test_pickle(self):
         for pref in self.prefs:
@@ -88,7 +91,6 @@ class TestReadPreferenceObjects(unittest.TestCase):
 
 
 class TestReadPreferencesBase(TestReplicaSetClientBase):
-
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
@@ -99,47 +101,41 @@ class TestReadPreferencesBase(TestReplicaSetClientBase):
         # Insert some data so we can use cursors in read_from_which_host
         self.client.pymongo_test.test.drop()
         self.client.get_database(
-            "pymongo_test",
-            write_concern=WriteConcern(w=self.w)).test.insert_many(
-                [{'_id': i} for i in range(10)])
+            "pymongo_test", write_concern=WriteConcern(w=self.w)
+        ).test.insert_many([{"_id": i} for i in range(10)])
 
         self.addCleanup(self.client.pymongo_test.test.drop)
 
     def read_from_which_host(self, client):
-        """Do a find() on the client and return which host was used
-        """
+        """Do a find() on the client and return which host was used"""
         cursor = client.pymongo_test.test.find()
         next(cursor)
         return cursor.address
 
     def read_from_which_kind(self, client):
         """Do a find() on the client and return 'primary' or 'secondary'
-           depending on which the client used.
+        depending on which the client used.
         """
         address = self.read_from_which_host(client)
         if address == client.primary:
-            return 'primary'
+            return "primary"
         elif address in client.secondaries:
-            return 'secondary'
+            return "secondary"
         else:
             self.fail(
-                'Cursor used address %s, expected either primary '
-                '%s or secondaries %s' % (
-                    address, client.primary, client.secondaries))
+                "Cursor used address %s, expected either primary "
+                "%s or secondaries %s" % (address, client.primary, client.secondaries)
+            )
 
     def assertReadsFrom(self, expected, **kwargs):
         c = rs_client(**kwargs)
-        wait_until(
-            lambda: len(c.nodes - c.arbiters) == self.w,
-            "discovered all nodes")
+        wait_until(lambda: len(c.nodes - c.arbiters) == self.w, "discovered all nodes")
 
         used = self.read_from_which_kind(c)
-        self.assertEqual(expected, used, 'Cursor used %s, expected %s' % (
-            used, expected))
+        self.assertEqual(expected, used, "Cursor used %s, expected %s" % (used, expected))
 
 
-class TestSingleSlaveOk(TestReadPreferencesBase):
-
+class TestSingleSecondaryOk(TestReadPreferencesBase):
     def test_reads_from_secondary(self):
 
         host, port = next(iter(self.client.secondaries))
@@ -176,66 +172,47 @@ class TestSingleSlaveOk(TestReadPreferencesBase):
 
 
 class TestReadPreferences(TestReadPreferencesBase):
-
     def test_mode_validation(self):
-        for mode in (ReadPreference.PRIMARY,
-                     ReadPreference.PRIMARY_PREFERRED,
-                     ReadPreference.SECONDARY,
-                     ReadPreference.SECONDARY_PREFERRED,
-                     ReadPreference.NEAREST):
-            self.assertEqual(
-                mode,
-                rs_client(read_preference=mode).read_preference)
+        for mode in (
+            ReadPreference.PRIMARY,
+            ReadPreference.PRIMARY_PREFERRED,
+            ReadPreference.SECONDARY,
+            ReadPreference.SECONDARY_PREFERRED,
+            ReadPreference.NEAREST,
+        ):
+            self.assertEqual(mode, rs_client(read_preference=mode).read_preference)
 
-        self.assertRaises(
-            TypeError,
-            rs_client, read_preference='foo')
+        self.assertRaises(TypeError, rs_client, read_preference="foo")
 
     def test_tag_sets_validation(self):
         S = Secondary(tag_sets=[{}])
-        self.assertEqual(
-            [{}],
-            rs_client(read_preference=S).read_preference.tag_sets)
+        self.assertEqual([{}], rs_client(read_preference=S).read_preference.tag_sets)
 
-        S = Secondary(tag_sets=[{'k': 'v'}])
-        self.assertEqual(
-            [{'k': 'v'}],
-            rs_client(read_preference=S).read_preference.tag_sets)
+        S = Secondary(tag_sets=[{"k": "v"}])
+        self.assertEqual([{"k": "v"}], rs_client(read_preference=S).read_preference.tag_sets)
 
-        S = Secondary(tag_sets=[{'k': 'v'}, {}])
-        self.assertEqual(
-            [{'k': 'v'}, {}],
-            rs_client(read_preference=S).read_preference.tag_sets)
+        S = Secondary(tag_sets=[{"k": "v"}, {}])
+        self.assertEqual([{"k": "v"}, {}], rs_client(read_preference=S).read_preference.tag_sets)
 
         self.assertRaises(ValueError, Secondary, tag_sets=[])
 
         # One dict not ok, must be a list of dicts
-        self.assertRaises(TypeError, Secondary, tag_sets={'k': 'v'})
+        self.assertRaises(TypeError, Secondary, tag_sets={"k": "v"})
 
-        self.assertRaises(TypeError, Secondary, tag_sets='foo')
+        self.assertRaises(TypeError, Secondary, tag_sets="foo")
 
-        self.assertRaises(TypeError, Secondary, tag_sets=['foo'])
+        self.assertRaises(TypeError, Secondary, tag_sets=["foo"])
 
     def test_threshold_validation(self):
-        self.assertEqual(17, rs_client(
-            localThresholdMS=17
-        ).local_threshold_ms)
+        self.assertEqual(17, rs_client(localThresholdMS=17).options.local_threshold_ms)
 
-        self.assertEqual(42, rs_client(
-            localThresholdMS=42
-        ).local_threshold_ms)
+        self.assertEqual(42, rs_client(localThresholdMS=42).options.local_threshold_ms)
 
-        self.assertEqual(666, rs_client(
-            localthresholdms=666
-        ).local_threshold_ms)
+        self.assertEqual(666, rs_client(localthresholdms=666).options.local_threshold_ms)
 
-        self.assertEqual(0, rs_client(
-            localthresholdms=0
-        ).local_threshold_ms)
+        self.assertEqual(0, rs_client(localthresholdms=0).options.local_threshold_ms)
 
-        self.assertRaises(ValueError,
-                          rs_client,
-                          localthresholdms=-1)
+        self.assertRaises(ValueError, rs_client, localthresholdms=-1)
 
     def test_zero_latency(self):
         ping_times = set()
@@ -245,11 +222,8 @@ class TestReadPreferences(TestReadPreferencesBase):
         for ping_time, host in zip(ping_times, self.client.nodes):
             ServerDescription._host_to_round_trip_time[host] = ping_time
         try:
-            client = connected(
-                rs_client(readPreference='nearest', localThresholdMS=0))
-            wait_until(
-                lambda: client.nodes == self.client.nodes,
-                "discovered all nodes")
+            client = connected(rs_client(readPreference="nearest", localThresholdMS=0))
+            wait_until(lambda: client.nodes == self.client.nodes, "discovered all nodes")
             host = self.read_from_which_host(client)
             for _ in range(5):
                 self.assertEqual(host, self.read_from_which_host(client))
@@ -257,33 +231,25 @@ class TestReadPreferences(TestReadPreferencesBase):
             ServerDescription._host_to_round_trip_time.clear()
 
     def test_primary(self):
-        self.assertReadsFrom(
-            'primary', read_preference=ReadPreference.PRIMARY)
+        self.assertReadsFrom("primary", read_preference=ReadPreference.PRIMARY)
 
     def test_primary_with_tags(self):
         # Tags not allowed with PRIMARY
-        self.assertRaises(
-            ConfigurationError,
-            rs_client, tag_sets=[{'dc': 'ny'}])
+        self.assertRaises(ConfigurationError, rs_client, tag_sets=[{"dc": "ny"}])
 
     def test_primary_preferred(self):
-        self.assertReadsFrom(
-            'primary', read_preference=ReadPreference.PRIMARY_PREFERRED)
+        self.assertReadsFrom("primary", read_preference=ReadPreference.PRIMARY_PREFERRED)
 
     def test_secondary(self):
-        self.assertReadsFrom(
-            'secondary', read_preference=ReadPreference.SECONDARY)
+        self.assertReadsFrom("secondary", read_preference=ReadPreference.SECONDARY)
 
     def test_secondary_preferred(self):
-        self.assertReadsFrom(
-            'secondary', read_preference=ReadPreference.SECONDARY_PREFERRED)
+        self.assertReadsFrom("secondary", read_preference=ReadPreference.SECONDARY_PREFERRED)
 
     def test_nearest(self):
         # With high localThresholdMS, expect to read from any
         # member
-        c = rs_client(
-            read_preference=ReadPreference.NEAREST,
-            localThresholdMS=10000)  # 10 seconds
+        c = rs_client(read_preference=ReadPreference.NEAREST, localThresholdMS=10000)  # 10 seconds
 
         data_members = set(self.hosts).difference(set(self.arbiters))
 
@@ -299,16 +265,16 @@ class TestReadPreferences(TestReadPreferencesBase):
             i += 1
 
         not_used = data_members.difference(used)
-        latencies = ', '.join(
-            '%s: %dms' % (server.description.address,
-                          server.description.round_trip_time)
-            for server in c._get_topology().select_servers(
-                readable_server_selector))
+        latencies = ", ".join(
+            "%s: %dms" % (server.description.address, server.description.round_trip_time)
+            for server in c._get_topology().select_servers(readable_server_selector)
+        )
 
         self.assertFalse(
             not_used,
             "Expected to use primary and all secondaries for mode NEAREST,"
-            " but didn't use %s\nlatencies: %s" % (not_used, latencies))
+            " but didn't use %s\nlatencies: %s" % (not_used, latencies),
+        )
 
 
 class ReadPrefTester(MongoClient):
@@ -320,36 +286,35 @@ class ReadPrefTester(MongoClient):
 
     @contextlib.contextmanager
     def _socket_for_reads(self, read_preference, session):
-        context = super(ReadPrefTester, self)._socket_for_reads(
-            read_preference, session)
-        with context as (sock_info, slave_ok):
+        context = super(ReadPrefTester, self)._socket_for_reads(read_preference, session)
+        with context as (sock_info, secondary_ok):
             self.record_a_read(sock_info.address)
-            yield sock_info, slave_ok
+            yield sock_info, secondary_ok
 
     @contextlib.contextmanager
-    def _slaveok_for_server(self, read_preference, server, session,
-                            exhaust=False):
-        context = super(ReadPrefTester, self)._slaveok_for_server(
-            read_preference, server, session, exhaust=exhaust)
-        with context as (sock_info, slave_ok):
+    def _secondaryok_for_server(self, read_preference, server, session):
+        context = super(ReadPrefTester, self)._secondaryok_for_server(
+            read_preference, server, session
+        )
+        with context as (sock_info, secondary_ok):
             self.record_a_read(sock_info.address)
-            yield sock_info, slave_ok
+            yield sock_info, secondary_ok
 
     def record_a_read(self, address):
         server = self._get_topology().select_server_by_address(address, 0)
         self.has_read_from.add(server)
 
+
 _PREF_MAP = [
     (Primary, SERVER_TYPE.RSPrimary),
     (PrimaryPreferred, SERVER_TYPE.RSPrimary),
     (Secondary, SERVER_TYPE.RSSecondary),
     (SecondaryPreferred, SERVER_TYPE.RSSecondary),
-    (Nearest, 'any')
+    (Nearest, "any"),
 ]
 
 
 class TestCommandAndReadPreference(TestReplicaSetClientBase):
-
     @classmethod
     @client_context.require_secondaries_count(1)
     def setUpClass(cls):
@@ -358,18 +323,18 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
             client_context.pair,
             replicaSet=cls.name,
             # Ignore round trip times, to test ReadPreference modes only.
-            localThresholdMS=1000*1000)
+            localThresholdMS=1000 * 1000,
+        )
         if client_context.auth_enabled:
             cls.c.admin.authenticate(db_user, db_pwd)
         cls.client_version = Version.from_client(cls.c)
         # mapReduce and group fail with no collection
-        coll = cls.c.pymongo_test.get_collection(
-            'test', write_concern=WriteConcern(w=cls.w))
+        coll = cls.c.pymongo_test.get_collection("test", write_concern=WriteConcern(w=cls.w))
         coll.insert_one({})
 
     @classmethod
     def tearDownClass(cls):
-        cls.c.drop_database('pymongo_test')
+        cls.c.drop_database("pymongo_test")
         cls.c.close()
 
     def executed_on_which_server(self, client, fn, *args, **kwargs):
@@ -381,12 +346,13 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
 
     def assertExecutedOn(self, server_type, client, fn, *args, **kwargs):
         server = self.executed_on_which_server(client, fn, *args, **kwargs)
-        self.assertEqual(SERVER_TYPE._fields[server_type],
-                         SERVER_TYPE._fields[server.description.server_type])
+        self.assertEqual(
+            SERVER_TYPE._fields[server_type], SERVER_TYPE._fields[server.description.server_type]
+        )
 
     def _test_fn(self, server_type, fn):
         for _ in range(10):
-            if server_type == 'any':
+            if server_type == "any":
                 used = set()
                 for _ in range(1000):
                     server = self.executed_on_which_server(self.c, fn)
@@ -395,13 +361,9 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
                         # Success
                         break
 
-                unused = self.c.secondaries.union(
-                    set([self.c.primary])
-                ).difference(used)
+                unused = self.c.secondaries.union(set([self.c.primary])).difference(used)
                 if unused:
-                    self.fail(
-                        "Some members not used for NEAREST: %s" % (
-                            unused))
+                    self.fail("Some members not used for NEAREST: %s" % (unused))
             else:
                 self.assertExecutedOn(server_type, self.c, fn)
 
@@ -422,8 +384,7 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
         # Test that the generic command helper obeys the read preference
         # passed to it.
         for mode, server_type in _PREF_MAP:
-            func = lambda: self.c.pymongo_test.command('dbStats',
-                                                       read_preference=mode())
+            func = lambda: self.c.pymongo_test.command("dbStats", read_preference=mode())
             self._test_fn(server_type, func)
 
     def test_create_collection(self):
@@ -431,49 +392,58 @@ class TestCommandAndReadPreference(TestReplicaSetClientBase):
         # the collection already exists.
         self._test_primary_helper(
             lambda: self.c.pymongo_test.create_collection(
-                'some_collection%s' % random.randint(0, MAXSIZE)))
+                "some_collection%s" % random.randint(0, MAXSIZE)
+            )
+        )
 
     @client_context.require_version_max(4, 1, 0, -1)
     def test_group(self):
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
-            self._test_coll_helper(True, self.c.pymongo_test.test, 'group',
-                                   {'a': 1}, {}, {}, 'function() { }')
+            self._test_coll_helper(
+                True, self.c.pymongo_test.test, "group", {"a": 1}, {}, {}, "function() { }"
+            )
 
     def test_map_reduce(self):
-        self._test_coll_helper(False, self.c.pymongo_test.test, 'map_reduce',
-                               'function() { }', 'function() { }',
-                               {'inline': 1})
+        self._test_coll_helper(
+            False,
+            self.c.pymongo_test.test,
+            "map_reduce",
+            "function() { }",
+            "function() { }",
+            {"inline": 1},
+        )
 
     def test_inline_map_reduce(self):
-        self._test_coll_helper(True, self.c.pymongo_test.test,
-                               'inline_map_reduce',
-                               'function() { }', 'function() { }')
+        self._test_coll_helper(
+            True, self.c.pymongo_test.test, "inline_map_reduce", "function() { }", "function() { }"
+        )
 
     @ignore_deprecations
     def test_count(self):
-        self._test_coll_helper(True, self.c.pymongo_test.test, 'count')
+        self._test_coll_helper(True, self.c.pymongo_test.test, "count")
 
     def test_count_documents(self):
-        self._test_coll_helper(
-            True, self.c.pymongo_test.test, 'count_documents', {})
+        self._test_coll_helper(True, self.c.pymongo_test.test, "count_documents", {})
 
     def test_estimated_document_count(self):
-        self._test_coll_helper(
-            True, self.c.pymongo_test.test, 'estimated_document_count')
+        self._test_coll_helper(True, self.c.pymongo_test.test, "estimated_document_count")
 
     def test_distinct(self):
-        self._test_coll_helper(True, self.c.pymongo_test.test, 'distinct', 'a')
+        self._test_coll_helper(True, self.c.pymongo_test.test, "distinct", "a")
 
     def test_aggregate(self):
-        self._test_coll_helper(True, self.c.pymongo_test.test,
-                               'aggregate',
-                               [{'$project': {'_id': 1}}])
+        self._test_coll_helper(
+            True, self.c.pymongo_test.test, "aggregate", [{"$project": {"_id": 1}}]
+        )
 
     def test_aggregate_write(self):
-        self._test_coll_helper(False, self.c.pymongo_test.test,
-                               'aggregate',
-                               [{'$project': {'_id': 1}}, {'$out': "agg_write_test"}])
+        self._test_coll_helper(
+            False,
+            self.c.pymongo_test.test,
+            "aggregate",
+            [{"$project": {"_id": 1}}, {"$out": "agg_write_test"}],
+        )
 
 
 class TestMovingAverage(unittest.TestCase):
@@ -487,78 +457,50 @@ class TestMovingAverage(unittest.TestCase):
         avg.add_sample(30)
         self.assertAlmostEqual(15.6, avg.get())
 
-class TestMongosAndReadPreference(unittest.TestCase):
 
+class TestMongosAndReadPreference(IntegrationTest):
     def test_read_preference_document(self):
 
         pref = Primary()
-        self.assertEqual(
-            pref.document,
-            {'mode': 'primary'})
+        self.assertEqual(pref.document, {"mode": "primary"})
 
         pref = PrimaryPreferred()
+        self.assertEqual(pref.document, {"mode": "primaryPreferred"})
+        pref = PrimaryPreferred(tag_sets=[{"dc": "sf"}])
+        self.assertEqual(pref.document, {"mode": "primaryPreferred", "tags": [{"dc": "sf"}]})
+        pref = PrimaryPreferred(tag_sets=[{"dc": "sf"}], max_staleness=30)
         self.assertEqual(
             pref.document,
-            {'mode': 'primaryPreferred'})
-        pref = PrimaryPreferred(tag_sets=[{'dc': 'sf'}])
-        self.assertEqual(
-            pref.document,
-            {'mode': 'primaryPreferred', 'tags': [{'dc': 'sf'}]})
-        pref = PrimaryPreferred(
-            tag_sets=[{'dc': 'sf'}], max_staleness=30)
-        self.assertEqual(
-            pref.document,
-            {'mode': 'primaryPreferred',
-             'tags': [{'dc': 'sf'}],
-             'maxStalenessSeconds': 30})
+            {"mode": "primaryPreferred", "tags": [{"dc": "sf"}], "maxStalenessSeconds": 30},
+        )
 
         pref = Secondary()
+        self.assertEqual(pref.document, {"mode": "secondary"})
+        pref = Secondary(tag_sets=[{"dc": "sf"}])
+        self.assertEqual(pref.document, {"mode": "secondary", "tags": [{"dc": "sf"}]})
+        pref = Secondary(tag_sets=[{"dc": "sf"}], max_staleness=30)
         self.assertEqual(
-            pref.document,
-            {'mode': 'secondary'})
-        pref = Secondary(tag_sets=[{'dc': 'sf'}])
-        self.assertEqual(
-            pref.document,
-            {'mode': 'secondary', 'tags': [{'dc': 'sf'}]})
-        pref = Secondary(
-            tag_sets=[{'dc': 'sf'}], max_staleness=30)
-        self.assertEqual(
-            pref.document,
-            {'mode': 'secondary',
-             'tags': [{'dc': 'sf'}],
-             'maxStalenessSeconds': 30})
+            pref.document, {"mode": "secondary", "tags": [{"dc": "sf"}], "maxStalenessSeconds": 30}
+        )
 
         pref = SecondaryPreferred()
+        self.assertEqual(pref.document, {"mode": "secondaryPreferred"})
+        pref = SecondaryPreferred(tag_sets=[{"dc": "sf"}])
+        self.assertEqual(pref.document, {"mode": "secondaryPreferred", "tags": [{"dc": "sf"}]})
+        pref = SecondaryPreferred(tag_sets=[{"dc": "sf"}], max_staleness=30)
         self.assertEqual(
             pref.document,
-            {'mode': 'secondaryPreferred'})
-        pref = SecondaryPreferred(tag_sets=[{'dc': 'sf'}])
-        self.assertEqual(
-            pref.document,
-            {'mode': 'secondaryPreferred', 'tags': [{'dc': 'sf'}]})
-        pref = SecondaryPreferred(
-            tag_sets=[{'dc': 'sf'}], max_staleness=30)
-        self.assertEqual(
-            pref.document,
-            {'mode': 'secondaryPreferred',
-             'tags': [{'dc': 'sf'}],
-             'maxStalenessSeconds': 30})
+            {"mode": "secondaryPreferred", "tags": [{"dc": "sf"}], "maxStalenessSeconds": 30},
+        )
 
         pref = Nearest()
+        self.assertEqual(pref.document, {"mode": "nearest"})
+        pref = Nearest(tag_sets=[{"dc": "sf"}])
+        self.assertEqual(pref.document, {"mode": "nearest", "tags": [{"dc": "sf"}]})
+        pref = Nearest(tag_sets=[{"dc": "sf"}], max_staleness=30)
         self.assertEqual(
-            pref.document,
-            {'mode': 'nearest'})
-        pref = Nearest(tag_sets=[{'dc': 'sf'}])
-        self.assertEqual(
-            pref.document,
-            {'mode': 'nearest', 'tags': [{'dc': 'sf'}]})
-        pref = Nearest(
-            tag_sets=[{'dc': 'sf'}], max_staleness=30)
-        self.assertEqual(
-            pref.document,
-            {'mode': 'nearest',
-             'tags': [{'dc': 'sf'}],
-             'maxStalenessSeconds': 30})
+            pref.document, {"mode": "nearest", "tags": [{"dc": "sf"}], "maxStalenessSeconds": 30}
+        )
 
         with self.assertRaises(TypeError):
             Nearest(max_staleness=1.5)  # Float is prohibited.
@@ -571,70 +513,66 @@ class TestMongosAndReadPreference(unittest.TestCase):
 
     def test_read_preference_document_hedge(self):
         cases = {
-            'primaryPreferred': PrimaryPreferred,
-            'secondary': Secondary,
-            'secondaryPreferred': SecondaryPreferred,
-            'nearest': Nearest,
+            "primaryPreferred": PrimaryPreferred,
+            "secondary": Secondary,
+            "secondaryPreferred": SecondaryPreferred,
+            "nearest": Nearest,
         }
         for mode, cls in cases.items():
             with self.assertRaises(TypeError):
                 cls(hedge=[])
 
             pref = cls(hedge={})
-            self.assertEqual(pref.document, {'mode': mode})
+            self.assertEqual(pref.document, {"mode": mode})
             out = _maybe_add_read_preference({}, pref)
             if cls == SecondaryPreferred:
                 # SecondaryPreferred without hedge doesn't add $readPreference.
                 self.assertEqual(out, {})
             else:
-                self.assertEqual(
-                    out,
-                    SON([("$query", {}), ("$readPreference", pref.document)]))
+                self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
-            hedge = {'enabled': True}
+            hedge = {"enabled": True}
             pref = cls(hedge=hedge)
-            self.assertEqual(pref.document, {'mode': mode, 'hedge': hedge})
+            self.assertEqual(pref.document, {"mode": mode, "hedge": hedge})
             out = _maybe_add_read_preference({}, pref)
-            self.assertEqual(
-                out, SON([("$query", {}), ("$readPreference", pref.document)]))
+            self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
-            hedge = {'enabled': False}
+            hedge = {"enabled": False}
             pref = cls(hedge=hedge)
-            self.assertEqual(pref.document, {'mode': mode, 'hedge': hedge})
+            self.assertEqual(pref.document, {"mode": mode, "hedge": hedge})
             out = _maybe_add_read_preference({}, pref)
-            self.assertEqual(
-                out, SON([("$query", {}), ("$readPreference", pref.document)]))
+            self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
-            hedge = {'enabled': False, 'extra': 'option'}
+            hedge = {"enabled": False, "extra": "option"}
             pref = cls(hedge=hedge)
-            self.assertEqual(pref.document, {'mode': mode, 'hedge': hedge})
+            self.assertEqual(pref.document, {"mode": mode, "hedge": hedge})
             out = _maybe_add_read_preference({}, pref)
-            self.assertEqual(
-                out, SON([("$query", {}), ("$readPreference", pref.document)]))
+            self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
     # Require OP_MSG so that $readPreference is visible in the command event.
     @client_context.require_version_min(3, 6)
     def test_send_hedge(self):
         cases = {
-            'primaryPreferred': PrimaryPreferred,
-            'secondary': Secondary,
-            'secondaryPreferred': SecondaryPreferred,
-            'nearest': Nearest,
+            "primaryPreferred": PrimaryPreferred,
+            "secondaryPreferred": SecondaryPreferred,
+            "nearest": Nearest,
         }
+        if client_context.supports_secondary_read_pref:
+            cases["secondary"] = Secondary
         listener = OvertCommandListener()
         client = rs_client(event_listeners=[listener])
         self.addCleanup(client.close)
-        client.admin.command('ping')
+        client.admin.command("ping")
         for mode, cls in cases.items():
-            pref = cls(hedge={'enabled': True})
-            coll = client.test.get_collection('test', read_preference=pref)
+            pref = cls(hedge={"enabled": True})
+            coll = client.test.get_collection("test", read_preference=pref)
             listener.reset()
             coll.find_one()
-            started = listener.results['started']
+            started = listener.results["started"]
             self.assertEqual(len(started), 1, started)
             cmd = started[0].command
-            self.assertIn('$readPreference', cmd)
-            self.assertEqual(cmd['$readPreference'], pref.document)
+            self.assertIn("$readPreference", cmd)
+            self.assertEqual(cmd["$readPreference"], pref.document)
 
     def test_maybe_add_read_preference(self):
 
@@ -644,70 +582,72 @@ class TestMongosAndReadPreference(unittest.TestCase):
 
         pref = PrimaryPreferred()
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
-        pref = PrimaryPreferred(tag_sets=[{'dc': 'nyc'}])
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        pref = PrimaryPreferred(tag_sets=[{"dc": "nyc"}])
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
         pref = Secondary()
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
-        pref = Secondary(tag_sets=[{'dc': 'nyc'}])
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        pref = Secondary(tag_sets=[{"dc": "nyc"}])
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
         # SecondaryPreferred without tag_sets or max_staleness doesn't add
         # $readPreference
         pref = SecondaryPreferred()
         out = _maybe_add_read_preference({}, pref)
         self.assertEqual(out, {})
-        pref = SecondaryPreferred(tag_sets=[{'dc': 'nyc'}])
+        pref = SecondaryPreferred(tag_sets=[{"dc": "nyc"}])
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
         pref = SecondaryPreferred(max_staleness=120)
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
         pref = Nearest()
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
-        pref = Nearest(tag_sets=[{'dc': 'nyc'}])
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        pref = Nearest(tag_sets=[{"dc": "nyc"}])
         out = _maybe_add_read_preference({}, pref)
-        self.assertEqual(
-            out, SON([("$query", {}), ("$readPreference", pref.document)]))
+        self.assertEqual(out, SON([("$query", {}), ("$readPreference", pref.document)]))
 
         criteria = SON([("$query", {}), ("$orderby", SON([("_id", 1)]))])
         pref = Nearest()
         out = _maybe_add_read_preference(criteria, pref)
         self.assertEqual(
             out,
-            SON([("$query", {}),
-                 ("$orderby", SON([("_id", 1)])),
-                 ("$readPreference", pref.document)]))
-        pref = Nearest(tag_sets=[{'dc': 'nyc'}])
+            SON(
+                [
+                    ("$query", {}),
+                    ("$orderby", SON([("_id", 1)])),
+                    ("$readPreference", pref.document),
+                ]
+            ),
+        )
+        pref = Nearest(tag_sets=[{"dc": "nyc"}])
         out = _maybe_add_read_preference(criteria, pref)
         self.assertEqual(
             out,
-            SON([("$query", {}),
-                 ("$orderby", SON([("_id", 1)])),
-                 ("$readPreference", pref.document)]))
+            SON(
+                [
+                    ("$query", {}),
+                    ("$orderby", SON([("_id", 1)])),
+                    ("$readPreference", pref.document),
+                ]
+            ),
+        )
 
     @client_context.require_mongos
     def test_mongos(self):
-        shard = client_context.client.config.shards.find_one()['host']
-        num_members = shard.count(',') + 1
+        shard = client_context.client.config.shards.find_one()["host"]
+        num_members = shard.count(",") + 1
         if num_members == 1:
             raise SkipTest("Need a replica set shard to test.")
         coll = client_context.client.pymongo_test.get_collection(
-            "test",
-            write_concern=WriteConcern(w=num_members))
+            "test", write_concern=WriteConcern(w=num_members)
+        )
         coll.drop()
         res = coll.insert_many([{} for _ in range(5)])
         first_id = res.inserted_ids[0]
@@ -715,11 +655,7 @@ class TestMongosAndReadPreference(unittest.TestCase):
 
         # Note - this isn't a perfect test since there's no way to
         # tell what shard member a query ran on.
-        for pref in (Primary(),
-                     PrimaryPreferred(),
-                     Secondary(),
-                     SecondaryPreferred(),
-                     Nearest()):
+        for pref in (Primary(), PrimaryPreferred(), Secondary(), SecondaryPreferred(), Nearest()):
             qcoll = coll.with_options(read_preference=pref)
             results = list(qcoll.find().sort([("_id", 1)]))
             self.assertEqual(first_id, results[0]["_id"])
@@ -733,12 +669,14 @@ class TestMongosAndReadPreference(unittest.TestCase):
     def test_mongos_max_staleness(self):
         # Sanity check that we're sending maxStalenessSeconds
         coll = client_context.client.pymongo_test.get_collection(
-            "test", read_preference=SecondaryPreferred(max_staleness=120))
+            "test", read_preference=SecondaryPreferred(max_staleness=120)
+        )
         # No error
         coll.find_one()
 
         coll = client_context.client.pymongo_test.get_collection(
-            "test", read_preference=SecondaryPreferred(max_staleness=10))
+            "test", read_preference=SecondaryPreferred(max_staleness=10)
+        )
         try:
             coll.find_one()
         except OperationFailure as exc:
@@ -747,14 +685,14 @@ class TestMongosAndReadPreference(unittest.TestCase):
             self.fail("mongos accepted invalid staleness")
 
         coll = single_client(
-            readPreference='secondaryPreferred',
-            maxStalenessSeconds=120).pymongo_test.test
+            readPreference="secondaryPreferred", maxStalenessSeconds=120
+        ).pymongo_test.test
         # No error
         coll.find_one()
 
         coll = single_client(
-            readPreference='secondaryPreferred',
-            maxStalenessSeconds=10).pymongo_test.test
+            readPreference="secondaryPreferred", maxStalenessSeconds=10
+        ).pymongo_test.test
         try:
             coll.find_one()
         except OperationFailure as exc:
@@ -762,5 +700,6 @@ class TestMongosAndReadPreference(unittest.TestCase):
         else:
             self.fail("mongos accepted invalid staleness")
 
+
 if __name__ == "__main__":
     unittest.main()
