@@ -18,12 +18,13 @@ import sys
 
 sys.path[0:0] = [""]
 
-from pymongo.errors import ConnectionFailure, AutoReconnect
-from pymongo import ReadPreference
-from test import unittest, client_context, client_knobs, MockClientTest
+from test import MockClientTest, client_context, client_knobs, unittest
 from test.pymongo_mocks import MockClient
 from test.utils import wait_until
 
+from pymongo import ReadPreference
+from pymongo.errors import AutoReconnect, ConnectionFailure
+
 
 @client_context.require_connection
 @client_context.require_no_load_balancer
@@ -38,56 +39,55 @@ class TestSecondaryBecomesStandalone(MockClientTest):
     def test_client(self):
         c = MockClient(
             standalones=[],
-            members=['a:1', 'b:2', 'c:3'],
+            members=["a:1", "b:2", "c:3"],
             mongoses=[],
-            host='a:1,b:2,c:3',
-            replicaSet='rs',
-            serverSelectionTimeoutMS=100)
+            host="a:1,b:2,c:3",
+            replicaSet="rs",
+            serverSelectionTimeoutMS=100,
+        )
         self.addCleanup(c.close)
 
         # MongoClient connects to primary by default.
-        wait_until(lambda: c.address is not None, 'connect to primary')
-        self.assertEqual(c.address, ('a', 1))
+        wait_until(lambda: c.address is not None, "connect to primary")
+        self.assertEqual(c.address, ("a", 1))
 
         # C is brought up as a standalone.
-        c.mock_members.remove('c:3')
-        c.mock_standalones.append('c:3')
+        c.mock_members.remove("c:3")
+        c.mock_standalones.append("c:3")
 
         # Fail over.
-        c.kill_host('a:1')
-        c.kill_host('b:2')
+        c.kill_host("a:1")
+        c.kill_host("b:2")
 
         # Force reconnect.
         c.close()
 
         with self.assertRaises(AutoReconnect):
-            c.db.command('ping')
+            c.db.command("ping")
 
         self.assertEqual(c.address, None)
 
     def test_replica_set_client(self):
         c = MockClient(
             standalones=[],
-            members=['a:1', 'b:2', 'c:3'],
+            members=["a:1", "b:2", "c:3"],
             mongoses=[],
-            host='a:1,b:2,c:3',
-            replicaSet='rs')
+            host="a:1,b:2,c:3",
+            replicaSet="rs",
+        )
         self.addCleanup(c.close)
 
-        wait_until(lambda: ('b', 2) in c.secondaries,
-                   'discover host "b"')
+        wait_until(lambda: ("b", 2) in c.secondaries, 'discover host "b"')
 
-        wait_until(lambda: ('c', 3) in c.secondaries,
-                   'discover host "c"')
+        wait_until(lambda: ("c", 3) in c.secondaries, 'discover host "c"')
 
         # C is brought up as a standalone.
-        c.mock_members.remove('c:3')
-        c.mock_standalones.append('c:3')
+        c.mock_members.remove("c:3")
+        c.mock_standalones.append("c:3")
 
-        wait_until(lambda: set([('b', 2)]) == c.secondaries,
-                   'update the list of secondaries')
+        wait_until(lambda: set([("b", 2)]) == c.secondaries, "update the list of secondaries")
 
-        self.assertEqual(('a', 1), c.primary)
+        self.assertEqual(("a", 1), c.primary)
 
 
 class TestSecondaryRemoved(MockClientTest):
@@ -96,21 +96,21 @@ class TestSecondaryRemoved(MockClientTest):
     def test_replica_set_client(self):
         c = MockClient(
             standalones=[],
-            members=['a:1', 'b:2', 'c:3'],
+            members=["a:1", "b:2", "c:3"],
             mongoses=[],
-            host='a:1,b:2,c:3',
-            replicaSet='rs')
+            host="a:1,b:2,c:3",
+            replicaSet="rs",
+        )
         self.addCleanup(c.close)
 
-        wait_until(lambda: ('b', 2) in c.secondaries, 'discover host "b"')
-        wait_until(lambda: ('c', 3) in c.secondaries, 'discover host "c"')
+        wait_until(lambda: ("b", 2) in c.secondaries, 'discover host "b"')
+        wait_until(lambda: ("c", 3) in c.secondaries, 'discover host "c"')
 
         # C is removed.
-        c.mock_hello_hosts.remove('c:3')
-        wait_until(lambda: set([('b', 2)]) == c.secondaries,
-                   'update list of secondaries')
+        c.mock_hello_hosts.remove("c:3")
+        wait_until(lambda: set([("b", 2)]) == c.secondaries, "update list of secondaries")
 
-        self.assertEqual(('a', 1), c.primary)
+        self.assertEqual(("a", 1), c.primary)
 
 
 class TestSocketError(MockClientTest):
@@ -119,21 +119,22 @@ class TestSocketError(MockClientTest):
         with client_knobs(heartbeat_frequency=999999):
             c = MockClient(
                 standalones=[],
-                members=['a:1', 'b:2'],
+                members=["a:1", "b:2"],
                 mongoses=[],
-                host='a:1',
-                replicaSet='rs',
-                serverSelectionTimeoutMS=100)
+                host="a:1",
+                replicaSet="rs",
+                serverSelectionTimeoutMS=100,
+            )
             self.addCleanup(c.close)
 
-            wait_until(lambda: len(c.nodes) == 2, 'discover both nodes')
+            wait_until(lambda: len(c.nodes) == 2, "discover both nodes")
 
             # b now raises socket.error.
-            c.mock_down_hosts.append('b:2')
+            c.mock_down_hosts.append("b:2")
             self.assertRaises(
                 ConnectionFailure,
-                c.db.collection.with_options(
-                    read_preference=ReadPreference.SECONDARY).find_one)
+                c.db.collection.with_options(read_preference=ReadPreference.SECONDARY).find_one,
+            )
 
             self.assertEqual(1, len(c.nodes))
 
@@ -141,52 +142,45 @@ class TestSocketError(MockClientTest):
 class TestSecondaryAdded(MockClientTest):
     def test_client(self):
         c = MockClient(
-            standalones=[],
-            members=['a:1', 'b:2'],
-            mongoses=[],
-            host='a:1',
-            replicaSet='rs')
+            standalones=[], members=["a:1", "b:2"], mongoses=[], host="a:1", replicaSet="rs"
+        )
         self.addCleanup(c.close)
 
-        wait_until(lambda: len(c.nodes) == 2, 'discover both nodes')
+        wait_until(lambda: len(c.nodes) == 2, "discover both nodes")
 
         # MongoClient connects to primary by default.
-        self.assertEqual(c.address, ('a', 1))
-        self.assertEqual(set([('a', 1), ('b', 2)]), c.nodes)
+        self.assertEqual(c.address, ("a", 1))
+        self.assertEqual(set([("a", 1), ("b", 2)]), c.nodes)
 
         # C is added.
-        c.mock_members.append('c:3')
-        c.mock_hello_hosts.append('c:3')
+        c.mock_members.append("c:3")
+        c.mock_hello_hosts.append("c:3")
 
         c.close()
-        c.db.command('ping')
+        c.db.command("ping")
 
-        self.assertEqual(c.address, ('a', 1))
+        self.assertEqual(c.address, ("a", 1))
 
-        wait_until(lambda: set([('a', 1), ('b', 2), ('c', 3)]) == c.nodes,
-                   'reconnect to both secondaries')
+        wait_until(
+            lambda: set([("a", 1), ("b", 2), ("c", 3)]) == c.nodes, "reconnect to both secondaries"
+        )
 
     def test_replica_set_client(self):
         c = MockClient(
-            standalones=[],
-            members=['a:1', 'b:2'],
-            mongoses=[],
-            host='a:1',
-            replicaSet='rs')
+            standalones=[], members=["a:1", "b:2"], mongoses=[], host="a:1", replicaSet="rs"
+        )
         self.addCleanup(c.close)
 
-        wait_until(lambda: ('a', 1) == c.primary, 'discover the primary')
-        wait_until(lambda: set([('b', 2)]) == c.secondaries,
-                   'discover the secondary')
+        wait_until(lambda: ("a", 1) == c.primary, "discover the primary")
+        wait_until(lambda: set([("b", 2)]) == c.secondaries, "discover the secondary")
 
         # C is added.
-        c.mock_members.append('c:3')
-        c.mock_hello_hosts.append('c:3')
+        c.mock_members.append("c:3")
+        c.mock_hello_hosts.append("c:3")
 
-        wait_until(lambda: set([('b', 2), ('c', 3)]) == c.secondaries,
-                   'discover the new secondary')
+        wait_until(lambda: set([("b", 2), ("c", 3)]) == c.secondaries, "discover the new secondary")
 
-        self.assertEqual(('a', 1), c.primary)
+        self.assertEqual(("a", 1), c.primary)
 
 
 if __name__ == "__main__":
