@@ -21,9 +21,10 @@ import sys
 
 sys.path[0:0] = [""]
 
+from test import SkipTest, unittest
+
 from bson.py3compat import b
 from bson.son import SON
-from test import SkipTest, unittest
 
 
 class TestSON(unittest.TestCase):
@@ -32,9 +33,9 @@ class TestSON(unittest.TestCase):
         a1["hello"] = "world"
         a1["mike"] = "awesome"
         a1["hello_"] = "mike"
-        self.assertEqual(list(a1.items()), [("hello", "world"),
-                                            ("mike", "awesome"),
-                                            ("hello_", "mike")])
+        self.assertEqual(
+            list(a1.items()), [("hello", "world"), ("mike", "awesome"), ("hello_", "mike")]
+        )
 
         b2 = SON({"hello": "world"})
         self.assertEqual(b2["hello"], "world")
@@ -42,38 +43,28 @@ class TestSON(unittest.TestCase):
 
     def test_equality(self):
         a1 = SON({"hello": "world"})
-        b2 = SON((('hello', 'world'), ('mike', 'awesome'), ('hello_', 'mike')))
+        b2 = SON((("hello", "world"), ("mike", "awesome"), ("hello_", "mike")))
 
         self.assertEqual(a1, SON({"hello": "world"}))
-        self.assertEqual(b2, SON((('hello', 'world'),
-                                  ('mike', 'awesome'),
-                                  ('hello_', 'mike'))))
-        self.assertEqual(b2, dict((('hello_', 'mike'),
-                                   ('mike', 'awesome'),
-                                   ('hello', 'world'))))
+        self.assertEqual(b2, SON((("hello", "world"), ("mike", "awesome"), ("hello_", "mike"))))
+        self.assertEqual(b2, dict((("hello_", "mike"), ("mike", "awesome"), ("hello", "world"))))
 
         self.assertNotEqual(a1, b2)
-        self.assertNotEqual(b2, SON((('hello_', 'mike'),
-                                     ('mike', 'awesome'),
-                                     ('hello', 'world'))))
+        self.assertNotEqual(b2, SON((("hello_", "mike"), ("mike", "awesome"), ("hello", "world"))))
 
         # Explicitly test inequality
         self.assertFalse(a1 != SON({"hello": "world"}))
-        self.assertFalse(b2 != SON((('hello', 'world'),
-                                    ('mike', 'awesome'),
-                                    ('hello_', 'mike'))))
-        self.assertFalse(b2 != dict((('hello_', 'mike'),
-                                     ('mike', 'awesome'),
-                                     ('hello', 'world'))))
+        self.assertFalse(b2 != SON((("hello", "world"), ("mike", "awesome"), ("hello_", "mike"))))
+        self.assertFalse(b2 != dict((("hello_", "mike"), ("mike", "awesome"), ("hello", "world"))))
 
         # Embedded SON.
-        d4 = SON([('blah', {'foo': SON()})])
-        self.assertEqual(d4, {'blah': {'foo': {}}})
-        self.assertEqual(d4, {'blah': {'foo': SON()}})
-        self.assertNotEqual(d4, {'blah': {'foo': []}})
+        d4 = SON([("blah", {"foo": SON()})])
+        self.assertEqual(d4, {"blah": {"foo": {}}})
+        self.assertEqual(d4, {"blah": {"foo": SON()}})
+        self.assertNotEqual(d4, {"blah": {"foo": []}})
 
         # Original data unaffected.
-        self.assertEqual(SON, d4['blah']['foo'].__class__)
+        self.assertEqual(SON, d4["blah"]["foo"].__class__)
 
     def test_to_dict(self):
         a1 = SON()
@@ -90,19 +81,17 @@ class TestSON(unittest.TestCase):
         self.assertEqual(dict, d4.to_dict()["blah"]["foo"].__class__)
 
         # Original data unaffected.
-        self.assertEqual(SON, d4['blah']['foo'].__class__)
+        self.assertEqual(SON, d4["blah"]["foo"].__class__)
 
     def test_pickle(self):
 
         simple_son = SON([])
-        complex_son = SON([('son', simple_son),
-                           ('list', [simple_son, simple_son])])
+        complex_son = SON([("son", simple_son), ("list", [simple_son, simple_son])])
 
         for protocol in range(pickle.HIGHEST_PROTOCOL + 1):
-            pickled = pickle.loads(pickle.dumps(complex_son,
-                                                protocol=protocol))
-            self.assertEqual(pickled['son'], pickled['list'][0])
-            self.assertEqual(pickled['son'], pickled['list'][1])
+            pickled = pickle.loads(pickle.dumps(complex_son, protocol=protocol))
+            self.assertEqual(pickled["son"], pickled["list"][0])
+            self.assertEqual(pickled["son"], pickled["list"][1])
 
     def test_pickle_backwards_compatability(self):
         # This string was generated by pickling a SON object in pymongo
@@ -117,10 +106,9 @@ class TestSON(unittest.TestCase):
 
     def test_copying(self):
         simple_son = SON([])
-        complex_son = SON([('son', simple_son),
-                           ('list', [simple_son, simple_son])])
+        complex_son = SON([("son", simple_son), ("list", [simple_son, simple_son])])
         regex_son = SON([("x", re.compile("^hello.*"))])
-        reflexive_son = SON([('son', simple_son)])
+        reflexive_son = SON([("son", simple_son)])
         reflexive_son["reflexive"] = reflexive_son
 
         simple_son1 = copy.copy(simple_son)
@@ -191,5 +179,6 @@ class TestSON(unittest.TestCase):
         test_son.popitem()
         self.assertEqual(2, len(test_son))
 
+
 if __name__ == "__main__":
     unittest.main()
