@@ -15,14 +15,9 @@
 """Test that pymongo is thread safe."""
 
 import threading
+from test import IntegrationTest, client_context, db_pwd, db_user, unittest
+from test.utils import joinall, rs_or_single_client, rs_or_single_client_noauth
 
-from test import (client_context,
-                  db_user,
-                  db_pwd,
-                  IntegrationTest,
-                  unittest)
-from test.utils import rs_or_single_client_noauth, rs_or_single_client
-from test.utils import joinall
 from pymongo.errors import OperationFailure
 
 
@@ -32,7 +27,6 @@ def setUpModule():
 
 
 class AutoAuthenticateThreads(threading.Thread):
-
     def __init__(self, collection, num):
         threading.Thread.__init__(self)
         self.coll = collection
@@ -42,14 +36,13 @@ class AutoAuthenticateThreads(threading.Thread):
 
     def run(self):
         for i in range(self.num):
-            self.coll.insert_one({'num': i})
-            self.coll.find_one({'num': i})
+            self.coll.insert_one({"num": i})
+            self.coll.find_one({"num": i})
 
         self.success = True
 
 
 class SaveAndFind(threading.Thread):
-
     def __init__(self, collection):
         threading.Thread.__init__(self)
         self.collection = collection
@@ -66,7 +59,6 @@ class SaveAndFind(threading.Thread):
 
 
 class Insert(threading.Thread):
-
     def __init__(self, collection, n, expect_exception):
         threading.Thread.__init__(self)
         self.collection = collection
@@ -90,7 +82,6 @@ class Insert(threading.Thread):
 
 
 class Update(threading.Thread):
-
     def __init__(self, collection, n, expect_exception):
         threading.Thread.__init__(self)
         self.collection = collection
@@ -103,8 +94,7 @@ class Update(threading.Thread):
             error = True
 
             try:
-                self.collection.update_one({"test": "unique"},
-                                           {"$set": {"test": "update"}})
+                self.collection.update_one({"test": "unique"}, {"$set": {"test": "update"}})
                 error = False
             except:
                 if not self.expect_exception:
@@ -115,7 +105,6 @@ class Update(threading.Thread):
 
 
 class Disconnect(threading.Thread):
-
     def __init__(self, client, n):
         threading.Thread.__init__(self)
         self.client = client
