@@ -24,36 +24,29 @@ import shutil
 import sys
 import threading
 import time
+import unittest
 import warnings
-
 from collections import defaultdict
 from functools import partial
+from test import client_context, db_pwd, db_user
 
 from bson import json_util, py3compat
 from bson.objectid import ObjectId
 from bson.py3compat import abc, iteritems, string_type
 from bson.son import SON
-
-from pymongo import (MongoClient,
-                     monitoring, operations, read_preferences)
+from pymongo import MongoClient, monitoring, operations, read_preferences
 from pymongo.collection import ReturnDocument
 from pymongo.errors import ConfigurationError, OperationFailure
 from pymongo.hello_compat import HelloCompat
 from pymongo.monitoring import _SENSITIVE_COMMANDS
-from pymongo.pool import (_CancellationContext,
-                          PoolOptions,
-                          _PoolGeneration)
+from pymongo.pool import PoolOptions, _CancellationContext, _PoolGeneration
 from pymongo.read_concern import ReadConcern
 from pymongo.read_preferences import ReadPreference
-from pymongo.server_selectors import (any_server_selector,
-                                      writable_server_selector)
+from pymongo.server_selectors import any_server_selector, writable_server_selector
 from pymongo.server_type import SERVER_TYPE
+from pymongo.uri_parser import parse_uri
 from pymongo.write_concern import WriteConcern
 
-from test import (client_context,
-                  db_user,
-                  db_pwd)
-
 if sys.version_info[0] < 3:
     # Python 2.7, use our backport.
     from test.barrier import Barrier
@@ -90,12 +83,10 @@ class BaseListener(object):
 
     def wait_for_event(self, event, count):
         """Wait for a number of events to be published, or fail."""
-        wait_until(lambda: self.event_count(event) >= count,
-                   'find %s %s event(s)' % (count, event))
+        wait_until(lambda: self.event_count(event) >= count, "find %s %s event(s)" % (count, event))
 
 
 class CMAPListener(BaseListener, monitoring.ConnectionPoolListener):
-
     def connection_created(self, event):
         self.add_event(event)
 
@@ -128,22 +119,21 @@ class CMAPListener(BaseListener, monitoring.ConnectionPoolListener):
 
 
 class EventListener(monitoring.CommandListener):
-
     def __init__(self):
         self.results = defaultdict(list)
 
     def started(self, event):
-        self.results['started'].append(event)
+        self.results["started"].append(event)
 
     def succeeded(self, event):
-        self.results['succeeded'].append(event)
+        self.results["succeeded"].append(event)
 
     def failed(self, event):
-        self.results['failed'].append(event)
+        self.results["failed"].append(event)
 
     def started_command_names(self):
         """Return list of command names started."""
-        return [event.command_name for event in self.results['started']]
+        return [event.command_name for event in self.results["started"]]
 
     def reset(self):
         """Reset the state of this listener."""
@@ -155,40 +145,40 @@ class TopologyEventListener(monitoring.TopologyListener):
         self.results = defaultdict(list)
 
     def closed(self, event):
-        self.results['closed'].append(event)
+        self.results["closed"].append(event)
 
     def description_changed(self, event):
-        self.results['description_changed'].append(event)
+        self.results["description_changed"].append(event)
 
     def opened(self, event):
-        self.results['opened'].append(event)
+        self.results["opened"].append(event)
 
     def reset(self):
         """Reset the state of this listener."""
         self.results.clear()
 
 
-class WhiteListEventListener(EventListener):
-
+class AllowListEventListener(EventListener):
     def __init__(self, *commands):
         self.commands = set(commands)
-        super(WhiteListEventListener, self).__init__()
+        super(AllowListEventListener, self).__init__()
 
     def started(self, event):
         if event.command_name in self.commands:
-            super(WhiteListEventListener, self).started(event)
+            super(AllowListEventListener, self).started(event)
 
     def succeeded(self, event):
         if event.command_name in self.commands:
-            super(WhiteListEventListener, self).succeeded(event)
+            super(AllowListEventListener, self).succeeded(event)
 
     def failed(self, event):
         if event.command_name in self.commands:
-            super(WhiteListEventListener, self).failed(event)
+            super(AllowListEventListener, self).failed(event)
 
 
 class OvertCommandListener(EventListener):
     """A CommandListener that ignores sensitive commands."""
+
     def started(self, event):
         if event.command_name.lower() not in _SENSITIVE_COMMANDS:
             super(OvertCommandListener, self).started(event)
@@ -226,13 +216,11 @@ class _ServerEventListener(object):
         self.results = []
 
 
-class ServerEventListener(_ServerEventListener,
-                          monitoring.ServerListener):
+class ServerEventListener(_ServerEventListener, monitoring.ServerListener):
     """Listens to Server events."""
 
 
-class ServerAndTopologyEventListener(ServerEventListener,
-                                     monitoring.TopologyListener):
+class ServerAndTopologyEventListener(ServerEventListener, monitoring.TopologyListener):
     """Listens to Server and Topology events."""
 
 
@@ -298,6 +286,7 @@ class MockPool(object):
 
 class ScenarioDict(dict):
     """Dict that returns {} for any unknown key, recursively."""
+
     def __init__(self, data):
         def convert(v):
             if isinstance(v, abc.Mapping):
@@ -320,6 +309,7 @@ class ScenarioDict(dict):
 
 class CompareType(object):
     """Class that compares equal to any object of the given type."""
+
     def __init__(self, type):
         self.type = type
 
@@ -333,6 +323,7 @@ class CompareType(object):
 
 class FunctionCallRecorder(object):
     """Utility class to wrap a callable and record its invocations."""
+
     def __init__(self, function):
         self._function = function
         self._call_list = []
@@ -357,6 +348,7 @@ class FunctionCallRecorder(object):
 
 class TestCreator(object):
     """Class to create test cases from specifications."""
+
     def __init__(self, create_test, test_class, test_path):
         """Create a TestCreator object.
 
@@ -370,7 +362,7 @@ class TestCreator(object):
             test case.
           - `test_path`: path to the directory containing the JSON files with
             the test specifications.
-            """
+        """
         self._create_test = create_test
         self._test_class = test_class
         self.test_path = test_path
@@ -378,67 +370,96 @@ class TestCreator(object):
     def _ensure_min_max_server_version(self, scenario_def, method):
         """Test modifier that enforces a version range for the server on a
         test case."""
-        if 'minServerVersion' in scenario_def:
-            min_ver = tuple(
-                int(elt) for
-                elt in scenario_def['minServerVersion'].split('.'))
+        if "minServerVersion" in scenario_def:
+            min_ver = tuple(int(elt) for elt in scenario_def["minServerVersion"].split("."))
             if min_ver is not None:
                 method = client_context.require_version_min(*min_ver)(method)
 
-        if 'maxServerVersion' in scenario_def:
-            max_ver = tuple(
-                int(elt) for
-                elt in scenario_def['maxServerVersion'].split('.'))
+        if "maxServerVersion" in scenario_def:
+            max_ver = tuple(int(elt) for elt in scenario_def["maxServerVersion"].split("."))
             if max_ver is not None:
                 method = client_context.require_version_max(*max_ver)(method)
 
+        if "serverless" in scenario_def:
+            serverless = scenario_def["serverless"]
+            if serverless == "require":
+                serverless_satisfied = client_context.serverless
+            elif serverless == "forbid":
+                serverless_satisfied = not client_context.serverless
+            else:  # unset or "allow"
+                serverless_satisfied = True
+            method = unittest.skipUnless(
+                serverless_satisfied, "Serverless requirement not satisfied"
+            )(method)
+
         return method
 
     @staticmethod
     def valid_topology(run_on_req):
         return client_context.is_topology_type(
-            run_on_req.get('topology', ['single', 'replicaset', 'sharded',
-                                        'load-balanced']))
+            run_on_req.get("topology", ["single", "replicaset", "sharded", "load-balanced"])
+        )
 
     @staticmethod
     def min_server_version(run_on_req):
-        version = run_on_req.get('minServerVersion')
+        version = run_on_req.get("minServerVersion")
         if version:
-            min_ver = tuple(int(elt) for elt in version.split('.'))
+            min_ver = tuple(int(elt) for elt in version.split("."))
             return client_context.version >= min_ver
         return True
 
     @staticmethod
     def max_server_version(run_on_req):
-        version = run_on_req.get('maxServerVersion')
+        version = run_on_req.get("maxServerVersion")
         if version:
-            max_ver = tuple(int(elt) for elt in version.split('.'))
+            max_ver = tuple(int(elt) for elt in version.split("."))
             return client_context.version <= max_ver
         return True
 
+    @staticmethod
+    def valid_auth_enabled(run_on_req):
+        if "authEnabled" in run_on_req:
+            if run_on_req["authEnabled"]:
+                return client_context.auth_enabled
+            return not client_context.auth_enabled
+        return True
+
+    @staticmethod
+    def serverless_ok(run_on_req):
+        serverless = run_on_req["serverless"]
+        if serverless == "require":
+            return client_context.serverless
+        elif serverless == "forbid":
+            return not client_context.serverless
+        else:  # unset or "allow"
+            return True
+
     def should_run_on(self, scenario_def):
-        run_on = scenario_def.get('runOn', [])
+        run_on = scenario_def.get("runOn", [])
         if not run_on:
             # Always run these tests.
             return True
 
         for req in run_on:
-            if (self.valid_topology(req) and
-                    self.min_server_version(req) and
-                    self.max_server_version(req)):
+            if (
+                self.valid_topology(req)
+                and self.min_server_version(req)
+                and self.max_server_version(req)
+                and self.valid_auth_enabled(req)
+                and self.serverless_ok(req)
+            ):
                 return True
         return False
 
     def ensure_run_on(self, scenario_def, method):
         """Test modifier that enforces a 'runOn' on a test case."""
         return client_context._require(
-            lambda: self.should_run_on(scenario_def),
-            "runOn not satisfied",
-            method)
+            lambda: self.should_run_on(scenario_def), "runOn not satisfied", method
+        )
 
     def tests(self, scenario_def):
         """Allow CMAP spec test to override the location of test."""
-        return scenario_def['tests']
+        return scenario_def["tests"]
 
     def create_tests(self):
         for dirpath, _, filenames in os.walk(self.test_path):
@@ -450,59 +471,61 @@ class TestCreator(object):
                     # dates.
                     opts = json_util.JSONOptions(tz_aware=False)
                     scenario_def = ScenarioDict(
-                        json_util.loads(scenario_stream.read(),
-                                        json_options=opts))
+                        json_util.loads(scenario_stream.read(), json_options=opts)
+                    )
 
                 test_type = os.path.splitext(filename)[0]
 
                 # Construct test from scenario.
                 for test_def in self.tests(scenario_def):
-                    test_name = 'test_%s_%s_%s' % (
+                    test_name = "test_%s_%s_%s" % (
                         dirname,
-                        test_type.replace("-", "_").replace('.', '_'),
-                        str(test_def['description'].replace(" ", "_").replace(
-                            '.', '_')))
+                        test_type.replace("-", "_").replace(".", "_"),
+                        str(test_def["description"].replace(" ", "_").replace(".", "_")),
+                    )
 
-                    new_test = self._create_test(
-                        scenario_def, test_def, test_name)
-                    new_test = self._ensure_min_max_server_version(
-                        scenario_def, new_test)
-                    new_test = self.ensure_run_on(
-                        scenario_def, new_test)
+                    new_test = self._create_test(scenario_def, test_def, test_name)
+                    new_test = self._ensure_min_max_server_version(scenario_def, new_test)
+                    new_test = self.ensure_run_on(scenario_def, new_test)
 
                     new_test.__name__ = test_name
                     setattr(self._test_class, new_test.__name__, new_test)
 
 
-def _connection_string(h, authenticate):
-    if h.startswith("mongodb://"):
+def _connection_string(h):
+    if h.startswith("mongodb://") or h.startswith("mongodb+srv://"):
         return h
-    elif client_context.auth_enabled and authenticate:
-        return "mongodb://%s:%s@%s" % (db_user, db_pwd, str(h))
-    else:
-        return "mongodb://%s" % (str(h),)
+    return "mongodb://%s" % (str(h),)
 
 
-def _mongo_client(host, port, authenticate=True, directConnection=False,
-                  **kwargs):
+def _mongo_client(host, port, authenticate=True, directConnection=False, **kwargs):
     """Create a new client over SSL/TLS if necessary."""
     host = host or client_context.host
     port = port or client_context.port
     client_options = client_context.default_client_options.copy()
     if client_context.replica_set_name and not directConnection:
-        client_options['replicaSet'] = client_context.replica_set_name
+        client_options["replicaSet"] = client_context.replica_set_name
     client_options.update(kwargs)
 
-    client = MongoClient(_connection_string(host, authenticate), port,
-                         **client_options)
+    uri = _connection_string(host)
+    if client_context.auth_enabled and authenticate:
+        # Only add the default username or password if one is not provided.
+        res = parse_uri(uri)
+        if (
+            not res["username"]
+            and not res["password"]
+            and "username" not in client_options
+            and "password" not in client_options
+        ):
+            client_options["username"] = db_user
+            client_options["password"] = db_pwd
 
-    return client
+    return MongoClient(uri, port, **client_options)
 
 
 def single_client_noauth(h=None, p=None, **kwargs):
     """Make a direct connection. Don't authenticate."""
-    return _mongo_client(h, p, authenticate=False,
-                         directConnection=True, **kwargs)
+    return _mongo_client(h, p, authenticate=False, directConnection=True, **kwargs)
 
 
 def single_client(h=None, p=None, **kwargs):
@@ -545,17 +568,16 @@ def ensure_all_connected(client):
     that are configured on the client.
     """
     hello = client.admin.command(HelloCompat.LEGACY_CMD)
-    if 'setName' not in hello:
+    if "setName" not in hello:
         raise ConfigurationError("cluster is not a replica set")
 
-    target_host_list = set(hello['hosts'])
-    connected_host_list = set([hello['me']])
-    admindb = client.get_database('admin')
+    target_host_list = set(hello["hosts"])
+    connected_host_list = set([hello["me"]])
+    admindb = client.get_database("admin")
 
     # Run legacy hello until we have connected to each host at least once.
     while connected_host_list != target_host_list:
-        hello = admindb.command(HelloCompat.LEGACY_CMD,
-                                   read_preference=ReadPreference.SECONDARY)
+        hello = admindb.command(HelloCompat.LEGACY_CMD, read_preference=ReadPreference.SECONDARY)
         connected_host_list.update([hello["me"]])
 
 
@@ -572,19 +594,19 @@ def oid_generated_on_process(oid):
 
 
 def delay(sec):
-    return '''function() { sleep(%f * 1000); return true; }''' % sec
+    return """function() { sleep(%f * 1000); return true; }""" % sec
 
 
 def get_command_line(client):
-    command_line = client.admin.command('getCmdLineOpts')
-    assert command_line['ok'] == 1, "getCmdLineOpts() failed"
+    command_line = client.admin.command("getCmdLineOpts")
+    assert command_line["ok"] == 1, "getCmdLineOpts() failed"
     return command_line
 
 
 def camel_to_snake(camel):
     # Regex to convert CamelCase to snake_case.
-    snake = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', camel)
-    return re.sub('([a-z0-9])([A-Z])', r'\1_\2', snake).lower()
+    snake = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", camel)
+    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", snake).lower()
 
 
 def camel_to_upper_camel(camel):
@@ -600,21 +622,18 @@ def camel_to_snake_args(arguments):
 
 def snake_to_camel(snake):
     # Regex to convert snake_case to lowerCamelCase.
-    return re.sub(r'_([a-z])', lambda m: m.group(1).upper(), snake)
+    return re.sub(r"_([a-z])", lambda m: m.group(1).upper(), snake)
 
 
 def parse_collection_options(opts):
-    if 'readPreference' in opts:
-        opts['read_preference'] = parse_read_preference(
-            opts.pop('readPreference'))
+    if "readPreference" in opts:
+        opts["read_preference"] = parse_read_preference(opts.pop("readPreference"))
 
-    if 'writeConcern' in opts:
-        opts['write_concern'] = WriteConcern(
-            **dict(opts.pop('writeConcern')))
+    if "writeConcern" in opts:
+        opts["write_concern"] = WriteConcern(**dict(opts.pop("writeConcern")))
 
-    if 'readConcern' in opts:
-        opts['read_concern'] = ReadConcern(
-            **dict(opts.pop('readConcern')))
+    if "readConcern" in opts:
+        opts["read_concern"] = ReadConcern(**dict(opts.pop("readConcern")))
     return opts
 
 
@@ -626,11 +645,11 @@ def server_started_with_option(client, cmdline_opt, config_opt):
       - `config_opt`: The config file option (i.e. nojournal)
     """
     command_line = get_command_line(client)
-    if 'parsed' in command_line:
-        parsed = command_line['parsed']
+    if "parsed" in command_line:
+        parsed = command_line["parsed"]
         if config_opt in parsed:
             return parsed[config_opt]
-    argv = command_line['argv']
+    argv = command_line["argv"]
     return cmdline_opt in argv
 
 
@@ -638,53 +657,51 @@ def server_started_with_auth(client):
     try:
         command_line = get_command_line(client)
     except OperationFailure as e:
-        msg = e.details.get('errmsg', '')
-        if e.code == 13 or 'unauthorized' in msg or 'login' in msg:
+        msg = e.details.get("errmsg", "")
+        if e.code == 13 or "unauthorized" in msg or "login" in msg:
             # Unauthorized.
             return True
         raise
 
     # MongoDB >= 2.0
-    if 'parsed' in command_line:
-        parsed = command_line['parsed']
+    if "parsed" in command_line:
+        parsed = command_line["parsed"]
         # MongoDB >= 2.6
-        if 'security' in parsed:
-            security = parsed['security']
+        if "security" in parsed:
+            security = parsed["security"]
             # >= rc3
-            if 'authorization' in security:
-                return security['authorization'] == 'enabled'
+            if "authorization" in security:
+                return security["authorization"] == "enabled"
             # < rc3
-            return security.get('auth', False) or bool(security.get('keyFile'))
-        return parsed.get('auth', False) or bool(parsed.get('keyFile'))
+            return security.get("auth", False) or bool(security.get("keyFile"))
+        return parsed.get("auth", False) or bool(parsed.get("keyFile"))
     # Legacy
-    argv = command_line['argv']
-    return '--auth' in argv or '--keyFile' in argv
+    argv = command_line["argv"]
+    return "--auth" in argv or "--keyFile" in argv
 
 
 def server_started_with_nojournal(client):
     command_line = get_command_line(client)
 
     # MongoDB 2.6.
-    if 'parsed' in command_line:
-        parsed = command_line['parsed']
-        if 'storage' in parsed:
-            storage = parsed['storage']
-            if 'journal' in storage:
-                return not storage['journal']['enabled']
+    if "parsed" in command_line:
+        parsed = command_line["parsed"]
+        if "storage" in parsed:
+            storage = parsed["storage"]
+            if "journal" in storage:
+                return not storage["journal"]["enabled"]
 
-    return server_started_with_option(client, '--nojournal', 'nojournal')
+    return server_started_with_option(client, "--nojournal", "nojournal")
 
 
 def drop_collections(db):
     # Drop all non-system collections in this database.
-    for coll in db.list_collection_names(
-            filter={"name": {"$regex": r"^(?!system\.)"}}):
+    for coll in db.list_collection_names(filter={"name": {"$regex": r"^(?!system\.)"}}):
         db.drop_collection(coll)
 
 
 def remove_all_users(db):
-    db.command("dropAllUsersFromDatabase", 1,
-               writeConcern={"w": client_context.w})
+    db.command("dropAllUsersFromDatabase", 1, writeConcern={"w": client_context.w})
 
 
 def joinall(threads):
@@ -700,7 +717,7 @@ def connected(client):
         # Ignore warning that "ping" is always routed to primary even
         # if client's read preference isn't PRIMARY.
         warnings.simplefilter("ignore", UserWarning)
-        client.admin.command('ping')  # Force connection.
+        client.admin.command("ping")  # Force connection.
 
     return client
 
@@ -719,7 +736,7 @@ def wait_until(predicate, success_description, timeout=10):
     Returns the predicate's first true value.
     """
     start = time.time()
-    interval = min(float(timeout)/100, 0.1)
+    interval = min(float(timeout) / 100, 0.1)
     while True:
         retval = predicate()
         if retval:
@@ -733,17 +750,17 @@ def wait_until(predicate, success_description, timeout=10):
 
 def repl_set_step_down(client, **kwargs):
     """Run replSetStepDown, first unfreezing a secondary with replSetFreeze."""
-    cmd = SON([('replSetStepDown', 1)])
+    cmd = SON([("replSetStepDown", 1)])
     cmd.update(kwargs)
 
     # Unfreeze a secondary to ensure a speedy election.
-    client.admin.command(
-        'replSetFreeze', 0, read_preference=ReadPreference.SECONDARY)
+    client.admin.command("replSetFreeze", 0, read_preference=ReadPreference.SECONDARY)
     client.admin.command(cmd)
 
+
 def is_mongos(client):
     res = client.admin.command(HelloCompat.LEGACY_CMD)
-    return res.get('msg', '') == 'isdbgrid'
+    return res.get("msg", "") == "isdbgrid"
 
 
 def assertRaisesExactly(cls, fn, *args, **kwargs):
@@ -755,8 +772,7 @@ def assertRaisesExactly(cls, fn, *args, **kwargs):
     try:
         fn(*args, **kwargs)
     except Exception as e:
-        assert e.__class__ == cls, "got %s, expected %s" % (
-            e.__class__.__name__, cls.__name__)
+        assert e.__class__ == cls, "got %s, expected %s" % (e.__class__.__name__, cls.__name__)
     else:
         raise AssertionError("%s not raised" % cls)
 
@@ -771,6 +787,7 @@ def _ignore_deprecations():
 def ignore_deprecations(wrapped=None):
     """A context manager or a decorator."""
     if wrapped:
+
         @functools.wraps(wrapped)
         def wrapper(*args, **kwargs):
             with _ignore_deprecations():
@@ -783,7 +800,6 @@ def ignore_deprecations(wrapped=None):
 
 
 class DeprecationFilter(object):
-
     def __init__(self, action="ignore"):
         """Start filtering deprecations."""
         self.warn_context = warnings.catch_warnings()
@@ -805,9 +821,7 @@ def get_pool(client):
 
 def get_pools(client):
     """Get all pools."""
-    return [
-        server.pool for server in
-        client._get_topology().select_servers(any_server_selector)]
+    return [server.pool for server in client._get_topology().select_servers(any_server_selector)]
 
 
 # Constants for run_threads and lazy_client_trial.
@@ -837,8 +851,8 @@ def run_threads(collection, target):
 def frequent_thread_switches():
     """Make concurrency bugs more likely to manifest."""
     interval = None
-    if not sys.platform.startswith('java'):
-        if hasattr(sys, 'getswitchinterval'):
+    if not sys.platform.startswith("java"):
+        if hasattr(sys, "getswitchinterval"):
             interval = sys.getswitchinterval()
             sys.setswitchinterval(1e-6)
         else:
@@ -848,8 +862,8 @@ def frequent_thread_switches():
     try:
         yield
     finally:
-        if not sys.platform.startswith('java'):
-            if hasattr(sys, 'setswitchinterval'):
+        if not sys.platform.startswith("java"):
+            if hasattr(sys, "setswitchinterval"):
                 sys.setswitchinterval(interval)
             else:
                 sys.setcheckinterval(interval)
@@ -884,7 +898,9 @@ def gevent_monkey_patched():
         warnings.simplefilter("ignore", ImportWarning)
         try:
             import socket
+
             import gevent.socket
+
             return socket.socket is gevent.socket.socket
         except ImportError:
             return False
@@ -892,13 +908,9 @@ def gevent_monkey_patched():
 
 def eventlet_monkey_patched():
     """Check if eventlet's monkey patching is active."""
-    try:
-        import threading
-        import eventlet
-        return (threading.current_thread.__module__ ==
-                'eventlet.green.threading')
-    except ImportError:
-        return False
+    import threading
+
+    return threading.current_thread.__module__ == "eventlet.green.threading"
 
 
 def is_greenthread_patched():
@@ -909,7 +921,9 @@ def cdecimal_patched():
     """Check if Python 2.7 cdecimal patching is active."""
     try:
         import decimal
+
         import cdecimal
+
         return decimal is cdecimal
     except ImportError:
         return False
@@ -919,20 +933,19 @@ def disable_replication(client):
     """Disable replication on all secondaries, requires MongoDB 3.2."""
     for host, port in client.secondaries:
         secondary = single_client(host, port)
-        secondary.admin.command('configureFailPoint', 'stopReplProducer',
-                                mode='alwaysOn')
+        secondary.admin.command("configureFailPoint", "stopReplProducer", mode="alwaysOn")
 
 
 def enable_replication(client):
     """Enable replication on all secondaries, requires MongoDB 3.2."""
     for host, port in client.secondaries:
         secondary = single_client(host, port)
-        secondary.admin.command('configureFailPoint', 'stopReplProducer',
-                                mode='off')
+        secondary.admin.command("configureFailPoint", "stopReplProducer", mode="off")
 
 
 class ExceptionCatchingThread(threading.Thread):
     """A thread that stores any exception encountered from run()."""
+
     def __init__(self, *args, **kwargs):
         self.exc = None
         super(ExceptionCatchingThread, self).__init__(*args, **kwargs)
@@ -947,13 +960,14 @@ class ExceptionCatchingThread(threading.Thread):
 
 def parse_read_preference(pref):
     # Make first letter lowercase to match read_pref's modes.
-    mode_string = pref.get('mode', 'primary')
+    mode_string = pref.get("mode", "primary")
     mode_string = mode_string[:1].lower() + mode_string[1:]
     mode = read_preferences.read_pref_mode_from_name(mode_string)
-    max_staleness = pref.get('maxStalenessSeconds', -1)
-    tag_sets = pref.get('tag_sets')
+    max_staleness = pref.get("maxStalenessSeconds", -1)
+    tag_sets = pref.get("tag_sets")
     return read_preferences.make_read_preference(
-        mode, tag_sets=tag_sets, max_staleness=max_staleness)
+        mode, tag_sets=tag_sets, max_staleness=max_staleness
+    )
 
 
 def server_name_to_type(name):
@@ -961,16 +975,16 @@ def server_name_to_type(name):
     # Special case, some tests in the spec include the PossiblePrimary
     # type, but only single-threaded drivers need that type. We call
     # possible primaries Unknown.
-    if name == 'PossiblePrimary':
+    if name == "PossiblePrimary":
         return SERVER_TYPE.Unknown
     return getattr(SERVER_TYPE, name)
 
 
 def cat_files(dest, *sources):
     """Cat multiple files into dest."""
-    with open(dest, 'wb') as fdst:
+    with open(dest, "wb") as fdst:
         for src in sources:
-            with open(src, 'rb') as fsrc:
+            with open(src, "rb") as fsrc:
                 shutil.copyfileobj(fsrc, fdst)
 
 
@@ -980,63 +994,59 @@ def assertion_context(msg):
     try:
         yield
     except AssertionError as exc:
-        msg = '%s (%s)' % (exc, msg)
+        msg = "%s (%s)" % (exc, msg)
         py3compat.reraise(type(exc), msg, sys.exc_info()[2])
 
 
 def parse_spec_options(opts):
-    if 'readPreference' in opts:
-        opts['read_preference'] = parse_read_preference(
-            opts.pop('readPreference'))
+    if "readPreference" in opts:
+        opts["read_preference"] = parse_read_preference(opts.pop("readPreference"))
 
-    if 'writeConcern' in opts:
-        opts['write_concern'] = WriteConcern(
-            **dict(opts.pop('writeConcern')))
+    if "writeConcern" in opts:
+        opts["write_concern"] = WriteConcern(**dict(opts.pop("writeConcern")))
 
-    if 'readConcern' in opts:
-        opts['read_concern'] = ReadConcern(
-            **dict(opts.pop('readConcern')))
+    if "readConcern" in opts:
+        opts["read_concern"] = ReadConcern(**dict(opts.pop("readConcern")))
 
-    if 'maxTimeMS' in opts:
-        opts['max_time_ms'] = opts.pop('maxTimeMS')
+    if "maxTimeMS" in opts:
+        opts["max_time_ms"] = opts.pop("maxTimeMS")
 
-    if 'maxCommitTimeMS' in opts:
-        opts['max_commit_time_ms'] = opts.pop('maxCommitTimeMS')
+    if "maxCommitTimeMS" in opts:
+        opts["max_commit_time_ms"] = opts.pop("maxCommitTimeMS")
 
-    if 'hint' in opts:
-        hint = opts.pop('hint')
+    if "hint" in opts:
+        hint = opts.pop("hint")
         if not isinstance(hint, string_type):
             hint = list(iteritems(hint))
-        opts['hint'] = hint
+        opts["hint"] = hint
 
     # Properly format 'hint' arguments for the Bulk API tests.
-    if 'requests' in opts:
-        reqs = opts.pop('requests')
+    if "requests" in opts:
+        reqs = opts.pop("requests")
         for req in reqs:
-            if 'name' in req:
+            if "name" in req:
                 # CRUD v2 format
-                args = req.pop('arguments', {})
-                if 'hint' in args:
-                    hint = args.pop('hint')
+                args = req.pop("arguments", {})
+                if "hint" in args:
+                    hint = args.pop("hint")
                     if not isinstance(hint, string_type):
                         hint = list(iteritems(hint))
-                    args['hint'] = hint
-                req['arguments'] = args
+                    args["hint"] = hint
+                req["arguments"] = args
             else:
                 # Unified test format
                 bulk_model, spec = next(iteritems(req))
-                if 'hint' in spec:
-                    hint = spec.pop('hint')
+                if "hint" in spec:
+                    hint = spec.pop("hint")
                     if not isinstance(hint, string_type):
                         hint = list(iteritems(hint))
-                    spec['hint'] = hint
-        opts['requests'] = reqs
+                    spec["hint"] = hint
+        opts["requests"] = reqs
 
     return dict(opts)
 
 
-def prepare_spec_arguments(spec, arguments, opname, entity_map,
-                           with_txn_callback):
+def prepare_spec_arguments(spec, arguments, opname, entity_map, with_txn_callback):
     for arg_name in list(arguments):
         c2s = camel_to_snake(arg_name)
         # PyMongo accepts sort as list of tuples.
@@ -1047,8 +1057,7 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map,
         if arg_name == "fieldName":
             arguments["key"] = arguments.pop(arg_name)
         # Aggregate uses "batchSize", while find uses batch_size.
-        elif ((arg_name == "batchSize" or arg_name == "allowDiskUse") and
-              opname == "aggregate"):
+        elif (arg_name == "batchSize" or arg_name == "allowDiskUse") and opname == "aggregate":
             continue
         # Requires boolean returnDocument.
         elif arg_name == "returnDocument":
@@ -1057,7 +1066,7 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map,
             # Parse each request into a bulk write model.
             requests = []
             for request in arguments["requests"]:
-                if 'name' in request:
+                if "name" in request:
                     # CRUD v2 format
                     bulk_model = camel_to_upper_camel(request["name"])
                     bulk_class = getattr(operations, bulk_model)
@@ -1070,39 +1079,37 @@ def prepare_spec_arguments(spec, arguments, opname, entity_map,
                 requests.append(bulk_class(**dict(bulk_arguments)))
             arguments["requests"] = requests
         elif arg_name == "session":
-            arguments['session'] = entity_map[arguments['session']]
-        elif (opname in ('command', 'run_admin_command') and
-              arg_name == 'command'):
+            arguments["session"] = entity_map[arguments["session"]]
+        elif opname in ("command", "run_admin_command") and arg_name == "command":
             # Ensure the first key is the command name.
-            ordered_command = SON([(spec['command_name'], 1)])
-            ordered_command.update(arguments['command'])
-            arguments['command'] = ordered_command
-        elif opname == 'open_download_stream' and arg_name == 'id':
-            arguments['file_id'] = arguments.pop(arg_name)
-        elif opname != 'find' and c2s == 'max_time_ms':
+            ordered_command = SON([(spec["command_name"], 1)])
+            ordered_command.update(arguments["command"])
+            arguments["command"] = ordered_command
+        elif opname == "open_download_stream" and arg_name == "id":
+            arguments["file_id"] = arguments.pop(arg_name)
+        elif opname != "find" and c2s == "max_time_ms":
             # find is the only method that accepts snake_case max_time_ms.
             # All other methods take kwargs which must use the server's
             # camelCase maxTimeMS. See PYTHON-1855.
-            arguments['maxTimeMS'] = arguments.pop('max_time_ms')
-        elif opname == 'with_transaction' and arg_name == 'callback':
-            if 'operations' in arguments[arg_name]:
+            arguments["maxTimeMS"] = arguments.pop("max_time_ms")
+        elif opname == "with_transaction" and arg_name == "callback":
+            if "operations" in arguments[arg_name]:
                 # CRUD v2 format
-                callback_ops = arguments[arg_name]['operations']
+                callback_ops = arguments[arg_name]["operations"]
             else:
                 # Unified test format
                 callback_ops = arguments[arg_name]
-            arguments['callback'] = lambda _: with_txn_callback(
-                copy.deepcopy(callback_ops))
-        elif opname == 'drop_collection' and arg_name == 'collection':
-            arguments['name_or_collection'] = arguments.pop(arg_name)
-        elif opname == 'create_collection':
-            if arg_name == 'collection':
-                arguments['name'] = arguments.pop(arg_name)
+            arguments["callback"] = lambda _: with_txn_callback(copy.deepcopy(callback_ops))
+        elif opname == "drop_collection" and arg_name == "collection":
+            arguments["name_or_collection"] = arguments.pop(arg_name)
+        elif opname == "create_collection":
+            if arg_name == "collection":
+                arguments["name"] = arguments.pop(arg_name)
             # Any other arguments to create_collection are passed through
             # **kwargs.
-        elif opname == 'create_index' and arg_name == 'keys':
-            arguments['keys'] = list(arguments.pop(arg_name).items())
-        elif opname == 'drop_index' and arg_name == 'name':
-            arguments['index_or_name'] = arguments.pop(arg_name)
+        elif opname == "create_index" and arg_name == "keys":
+            arguments["keys"] = list(arguments.pop(arg_name).items())
+        elif opname == "drop_index" and arg_name == "name":
+            arguments["index_or_name"] = arguments.pop(arg_name)
         else:
             arguments[c2s] = arguments.pop(arg_name)
